#ifndef G4INCLCrossSections_hh
#define G4INCLCrossSections_hh 1

#include "G4INCLICrossSections.hh"
#include "G4INCLConfig.hh"

namespace G4INCL {
  namespace CrossSections {
      double elastic(Particle const * const p1, Particle const * const p2);
      double total(Particle const * const p1, Particle const * const p2);

      double NDeltaToNN(Particle const * const p1, Particle const * const p2);
      double NNToNDelta(Particle const * const p1, Particle const * const p2);
      double NNToxPiNN(const int xpi, Particle const * const p1, Particle const * const p2);
      double piNToDelta(Particle const * const p1, Particle const * const p2);
      double piNToxPiN(const int xpi, Particle const * const p1, Particle const * const p2);

      /** \brief Calculate the slope of the NN DDXS.
       *
       * \param energyCM energy in the CM frame, in MeV
       * \param iso total isospin of the system
       *
       * \return the slope of the angular distribution, in (GeV/c)^(-2)
       */
      double calculateNNAngularSlope(double energyCM, int iso);

      /** \brief Compute the "interaction distance".
       *
       * Defined on the basis of the average value of the N-N cross sections at
       * the given kinetic energy.
       *
       * \return the interaction distance
       */
      double interactionDistanceNN(const ParticleSpecies &aSpecies, const double kineticEnergy);

      /** \brief Compute the "interaction distance".
       *
       * Defined on the basis of the average value of the pi-N cross sections at
       * the given kinetic energy.
       *
       * \return the interaction distance
       */
      double interactionDistancePiN(const double projectileKineticEnergy);

      void setCrossSections(ICrossSections *c);

      void deleteCrossSections();

      void initialize(Config const * const theConfig);

  }
}

#endif
