/** \file G4INCLCrossSectionsINCL46.hh
 * \brief Cross sections used in INCL4.6
 *
 * \date 25nd October 2013
 * \author Davide Mancusi
 */

#ifndef G4INCLCROSSSECTIONSINCL46_HH
#define G4INCLCROSSSECTIONSINCL46_HH

#include "G4INCLICrossSections.hh"

namespace G4INCL {
  /// \brief Cross sections used in INCL4.6

  class CrossSectionsINCL46 : public ICrossSections{
    public:

      /// \brief Elastic particle-particle cross section
      virtual double elastic(Particle const * const p1, Particle const * const p2);

      /// \brief Total (elastic+inelastic) particle-particle cross section
      virtual double total(Particle const * const p1, Particle const * const p2);

      /// \brief Cross section for NDelta->NN
      virtual double NDeltaToNN(Particle const * const p1, Particle const * const p2);

      /// \brief Cross section for NN->NDelta
      virtual double NNToNDelta(Particle const * const p1, Particle const * const p2);

      /// \brief Cross section for piN->NDelta
      virtual double piNToDelta(Particle const * const p1, Particle const * const p2);

      /// \brief Cross section for X pion production - NN Channel
      virtual double NNToxPiNN(const int xpi, Particle const * const p1, Particle const * const p2);

      /// \brief Cross section for X pion production - piN Channel
      virtual double piNToxPiN(const int xpi, Particle const * const p1, Particle const * const p2);

      /** \brief Calculate the slope of the NN DDXS.
       *
       * \param energyCM energy in the CM frame, in MeV
       * \param iso total isospin of the system
       *
       * \return the slope of the angular distribution
       */
      virtual double calculateNNAngularSlope(double energyCM, int iso);

    protected:

      /// \brief Internal implementation of the elastic cross section
      double elasticNNLegacy(Particle const * const part1, Particle const * const part2);

      /// \brief Internal function for the delta-production cross section
      double deltaProduction(const int isospin, const double pLab);

      double spnPiPlusPHE(const double x);
      double spnPiMinusPHE(const double x);

  };
}

#endif
