INCL++ / Liège Intranuclear Cascade model                           {#mainpage}
=========================================

General information                                                     {#main}
===================

[TOC]

This directory contains the C++ version of the Liège Intranuclear Cascade
model ([Ref. 1](#references)), developed by the following authors:

A. Boudard¹, J. Cugnon², J.-C. David¹, P. Kaitaniemi¹, S. Leray¹, D. Mancusi¹

¹ CEA-Saclay, IRFU/SPhN, F-91191 Gif-sur-Yvette, France

² Fundamental Interactions in Physics and Astrophysics, University of Liège,
  allée du 6 août 17, bât. B5, B-4000 Liège 1, Belgium

The reference paper for INCL++ is [Ref. 2](#references). Please make sure you
cite it whenever you publish work done using INCL++.



Quick-start guide (for those in a hurry)                         {#quick-start}
========================================

1.  Create a working directory somewhere (say `/home/jdoe/inclxx`) and `cd`
    there.
2.  Put the source tarball in the working directory and untar it. This will
    create a new subdirectory (e.g.
    `/home/jdoe/inclxx/inclxx-XXXXX-0123456`) containing the INCL++
    source files (details [here](#obtaining)).
3.  Create a directory for building INCL++ (e.g. `/home/jdoe/inclxx/build`).
4.  Install the dependencies:
    * CMake (>=2.8)
    * ROOT (>=5.20/00)
    * Boost `program_options` (>=1.35.0)
    * Boost `date_time` (>=1.35.0, optional)
    * Boost `timer` (>=1.48.0, optional)
    * Boost `system` (>=1.48.0, optional)

    (details [here](#prerequisites))
5.  Compile:

        $ cd /home/jdoe/inclxx/build
        $ cmake ../inclxx-XXXXX-0123456
        $ make

    (details [here](#compilation))
6.  Run the code. For example, 50k events for 1-GeV proton + Pb208 with
    INCL++/ABLA07:

        $ ./INCLCascade -pp -tPb208 -E1000 -N50000 -dabla07

    (details [here](#running))
7.  Analyse the resulting ROOT tree (details [here](#analysis)).

Problems? Read on.



History of the model                                                 {#history}
====================

The original INCL code was written in Fortran in the beginning of the 80s and,
as of May 2014, is still under continuous development. However (or perhaps,
*because* of this), it was felt that maintenance of the code was growing more
and more difficult; thus, it was decided to completely redesign and rewrite the
code in C++. The task was carried out by Ph.D. student Pekka Kaitaniemi, with
the assistance of Davide Mancusi.



Scope of the model                                                     {#scope}
==================

The INCL++ model can in principle simulate reactions induced by nucleons, pions
and light ion on any nucleus. The model has been well tested for projectiles up
to A=18, energies from ~10 AMeV to ~3 AGeV and target nuclei close to the
stability valley, with mass numbers between 4 and 250. Starting with version v5.2,
INCL++ is equipped to handle higher projectile energies -- up to 15-20 GeV for
nucleon- and pion-induced reactions. Most of the validation that we perfomed
above 3 GeV concerned cross sections for pion production, plus a few cross
sections for the production of residues. The coverage, however, is nowhere as
comprehensive as below 3 GeV, mostly because few experimental data sets are
available.

The model might work (read: "run without crashing") even outside the limits
indicated above, but your mileage (read: the quality of your results) may vary.
If you find that the model cannot simulate your reaction, please report it to
the model authors (see the ["Questions and bug reports" section](#questions)
below).

Limits of applicability                                               {#limits}
-----------------------

There is no lower limit on the projectile energy, although it is commonly
believed that intranuclear-cascade models should not be applied below ~150 AMeV
(models however seem to give reasonable results down to a few tens of AMeV).
For nucleon-induced reactions below a few MeV, the model is very inefficient at
generating inelastic events and probably does not make much sense.

In low-energy reactions induced by *composite projectiles*, INCL++ falls back
to a fusion model if unphysical cascade conditions are met. The transition
between fusion and intranuclear cascade is smooth, so there should be no jumps
in the excitation functions. Thanks to transition to fusion, the INCL++ package
is able to capture some essential aspects of very low-energy nucleus-nucleus
reactions (see reference 1 for details) and can therefore be of some use even
in the few-AMeV regime.

For reactions induced by negative pions, it should be possible to use the model
down to arbitrarily small energies, although this has never been tested.

The upper energy limit for INCL++ v5.2 is about 15-20 GeV for pion- and
nucleon-induced reactions, thanks to the integration of the multipion extension
[(Refs. 3 and 4)](#references). Light-ion-induced reactions above a few AGeV
*should* run without crashing, but have never really been tested; be warned
that de-excitation models might choke on the very high excitation energies
produced by cascade. At *very* high energy, the model is not guaranteed to run
at all.

The upper limit on the projectile mass number in light-ion-induced reactions
(up to mass 18) is purely indicative and based on the expected validity of the
physical approximations made. There is nothing in the code that will prevent
you from using it with heavier projectiles. However, be aware of the fact that
1) the physical assumptions of the model become really shaky for symmetric
nucleus-nucleus collisions, and 2) the calculation time might become very long.



Obtaining the code                                                 {#obtaining}
==================

If you are reading this, you have probably already obtained a copy of the code.
For the most recent version, just drop a line to one of the authors (see the
["Questions and bug reports" section](#questions) below).

INCL++ is typically shipped in the form of a compressed tarball, with a name
that looks like the following:

    inclxx-XXXXX-0123456.tar.gz

Here `XXXXX` is the human-readable version number, and `0123456` is an
internal unique version identifier (it's actually a `git` hash, if you know
what that is). You normally uncompress the tarball with

    $ tar xzf inclxx-XXXXX-0123456.tar.gz

That will create a new directory called `inclxx-XXXXX-0123456` containing
all the INCL++ source files.

Private `git` repository
------------------------

We keep a private `git` repository of the INCL++ source tree at the following
URL:

    ssh://git@bitbucket.org:arekfu/incl.git

`git` is a free and fast version-control system that is widely used across the
Free Software world. Most Linux distributions out there come with a `git`
package; on Debian, Ubuntu and most apt-based Linux distributions, you should
be able to install it by running

    $ apt-get install git

as root.

If you wish to access the code repository, you need to be granted read-only
access to the INCL++ repository by one of the authors (see the [contact
section](#questions) below). Once this is done, you can clone the repository by
running

    $ git clone ssh://git@bitbucket.org:arekfu/incl.git

A freshly cloned INCL++ `git` repository will contain source files for the
latest stable release, which is the *recommended* version for external users.
If this is what you need, you may skip to the ["Compiling the code"
section](#compiling) for further instructions. Otherwise, read on.

Accessing the development history                                  {#accessing}
---------------------------------

Cloning the INCL++ `git` repository gives you access to the latest stable and
development versions, as well as all the code development history. You can
check out previous INCL++ versions by issuing e.g.

    $ git checkout v5.1.1

where `v5.1.1` can be any existing version number. This command will populate
the INCL++ tree with source files for the desired version. To go back to the
latest stable release, do

    $ git checkout origin/master

The latest *development* version can be checked out by running

    $ git checkout origin/next

*WARNING*: note that the latest development version might well be buggy. It
might not even compile! Don't use it unless you know what you are doing. We
recommend that you always use the latest stable version.

Downloading updates                                                  {#updates}
-------------------

If you already have an INCL++ `git` repository around, you can download any
updates by running

    $ git fetch

This command will connect to the public INCL++ repository and download any
updates to your local repository. Note that it will *not* modify the source
files in your directory; you must explicitly tell `git` which version you would
like to check out with e.g.

    $ git checkout origin/master

or any of the checkout commands in the [previous section](#accessing).

Learning more about git                                                  {#git}
-----------------------

If you want to learn more about `git`, there are plenty of excellent tutorials
online. A good place to start is the official `git` documentation [web
site](http://git-scm.com/documentation).



Compiling the code                                                 {#compiling}
==================

Prerequisites                                                  {#prerequisites}
-------------

The standalone version of the INCL++ code has the following external
dependencies:

* CMake (>=2.8)
* ROOT (>=5.20/00)
* Boost `program_options` (>=1.35.0)
* Boost `date_time` (>=1.35.0, optional)
* Boost `timer` (>=1.48.0, optional)
* Boost `system` (>=1.48.0, optional)

The ROOT dependency is only required if you want to produce ROOT trees as an
output. Alternatively, INCL++ has an ASCII output module (disabled by default)
which does not require any external library.

The Boost `date_time`, `timer` and `system` dependencies are only required for
the timing of the INCL++ calculation.

On Debian, Ubuntu and other apt-based Linux distributions, you can install the
necessary libraries by issuing the following command (on Debian systems, you
might need to add special apt sources for ROOT):

    $ apt-get install cmake root-system libboost-program-options-dev libboost-date-time-dev libboost-timer-dev libboost-system-dev

You might also want to install some GUI for CMake, which helps in the
configuration of the build options. You can either pick `cmake-curses-gui`
(text interface) or `cmake-qt-gui` (graphical interface).

Compilation                                                      {#compilation}
-----------

The code is configured and built by CMake. It is recommended to build the
program in an external directory (CMake calls this an "out-of-tree build").

    $ cd /home/jdoe/inclxx
    $ mkdir build
    $ cd build
    $ cmake ../inclxx-XXXXX-0123456   # this will generate the Makefiles
    $ make                                 # this will compile the code

This will compile an optimised version of INCL++, coupled to the de-excitation
codes that were included in the distribution.  If you want to simulate the full
reaction, INCL++ **needs** to be coupled to a de-excitation code.  See the
["Coupling INCL++ to de-excitation codes" section](#deexcitation) below for
more information.

If Boost and ROOT are installed in standard locations, CMake will find them and
properly set up the build environment, without any required action from the
user. If, on the other hand, external libraries are installed in a
non-conventional location, CMake will have trouble finding them. You should
help it by specifying the library paths using environment variables `ROOTSYS`
and/or `BOOST_ROOT`, e.g.:

    $ cd /home/jdoe/inclxx
    $ mkdir build
    $ cd build
    $ ROOTSYS=/scratch/root-5.34 BOOST_ROOT=/scratch/boost1.40 cmake ../inclxx-XXXXX-0123456
    $ make

*Important*: make sure you start from a fresh `build` directory if you need to
specify custom library locations.

Useful configuration variables                                   {#config-vars}
------------------------------

The CMake configuration system allows the user to customise compilation using
some variables. The easiest way to set them is to use a GUI tool such as
`ccmake` (text interface) or `cmake-gui` (graphical interface); see the
["Prerequisites" section](#prerequisites) above for information about how to
install these programs. Alternatively, you can set variables when you call
CMake, as follows:

    $ cmake -DVARIABLE_NAME=<VARIABLE_VALUE> ..

Make sure you recompile your code (by running `make`) if you modify any of the
compilation options.

A list of useful compile-time flags follows. Possible values are `ON` and
`OFF`.

*   `INCL_DEEXCITATION_ABLA07`

    `INCL_DEEXCITATION_ABLAXX`

    `INCL_DEEXCITATION_SMM`

    `INCL_DEEXCITATION_GEMINIXX`

    Compile and link INCL++ to the relevant de-excitation module. See the
    [de-excitation section](#deexcitation) below for more information.

    Default: ON if the model is present in the distributed package, OFF
    otherwise.

*   `INCL_FPE_DEBUG`

    Emit a floating-point exception (FPE) if illegal mathematical operations
    are attempted. It might be necessary to turn it off if you are coupling
    INCL++ with some not-so-well-behaved de-excitation model. INCL++ is
    supposed to run without any problem with this option on.

    Default: ON



Coupling INCL++ to de-excitation codes                          {#deexcitation}
======================================

De-excitation models are used to describe the de-excitation of the excited
remnant left at the end of the intranuclear-cascade reaction stage. Coupling
INCL++ to a de-excitation code is mandatory in order to correctly describe most
of the reaction observables, including (but not limited to) double-differential
particle-production cross sections, residue yields and fission cross sections.

INCL++ comes with a set of interfaces to nuclear de-excitation models. For the
moment, we provide interfaces to the following codes:

* ABLA07   (A. Kelic-Heil et al., GSI, Darmstadt, Germany)
* ABLA++   (A. Kelic-Heil et al., GSI, Darmstadt, Germany; converted to C++ by
            C. Schmitt, GANIL, France)
* GEMINI++ (R. J. Charity, Washington University, St. Louis, MO, USA)
* SMM      (A. Botvina, Institute for Nuclear Research, Moscow, Russia)

More interfaces can be built, if required. You are anyway encouraged to take a
look at the existing interfaces and try to do the job yourself.

Note that INCL++ comes with a Fermi break-up model (which is essentially
Geant4's model). This model is automatically used for remnants below a certain
mass (A≤16 by default), *regardless* of the de-excitation model chosen on the
command line. If you really wish to use one of the de-excitation models above
for light remnants, you need to pass the `--max-mass-fermi-breakup=0` option
(see [below](#run) for more information about run options).

Note also that *not* all the de-excitation codes above are necessarily shipped
with INCL++; that depends on your agreement with the authors and on how you
obtained the source package.

You need to obtain some de-excitation code separately if you downloaded the
source code via `git`. Once you do, unpack the code source in

    <INCLXX_ROOT>/de-excitation/<MODEL_NAME>/upstream

You will then need to switch on the relevant `INCL_DEEXCITATION_<MODEL_NAME>`
flag, either by using , or by running e.g.

    $ cmake -DINCL_DEEXCITATION_ABLA07=ON ..

from your `build` directory. Make sure you recompile the code with `make`.  See
[the previous section](#config-vars) for a list of all available
`INCL_DEEXCITATION_*` flags.

Unfortunately, the number of de-excitation-code variants is increasing all the
time and the interfaces they expose are not always consistent. Moreover, most
authors keep poor version numbering, or none at all. For these reasons, there
are nine chances in ten that dumping random upstream code in the directory
above will not work out of the box. Some manual adjustment of the build system
will most probably be necessary, especially if you are planning to link to
Fortran code (as you need to do for most of the codes above).

Some de-excitation codes (as well as INCL++) need external tables to run. The
location of these files is hard-wired into the `INCLCascade` executable at
compilation time, but it can be overridden through command-line options and/or
a configuration file if need be (see [the section](#inclxxrc) below).  


Running the code                                                     {#running}
================

The INCL++ executable is called `INCLCascade`. If you followed the build
instructions above, you will find it in your `build` directory.

Options can be provided to `INCLCascade` in three ways, in decreasing order of
priority:

1. command line;
2. input file;
3. configuration file.

Options provided on the command line can override options in input files and
configuration files. Options in an input file can override options in
configuration files.

Options in input and configuration files have the same format:

    option = value

You can add comments in input and configuration files by using a `#` character,
much as in shell scripts.

Per-user configuration                                              {#inclxxrc}
----------------------

If you move or rename your source directory after compilation, `INCLCascade`
will not be able to find the required data files any more. In this case, you
should either recompile `INCLCascade` from the new path, or you can instruct
your executable about the new path by creating a simple configuration file.
When it is started, `INCLCascade` reads any options defined in the

    ${HOME}/.inclxxrc

file, if it exists. This is a good place to define the paths to the tables for
INCL++ or your de-excitation models. For this purpose, you should define an
option called `<MODEL_NAME>-datafile-path` for each de-excitation code. An
example `.inclxxrc` file follows:

    inclxx-datafile-path=/home/jdoe/inclxx/inclxx-XXXXX-0123456/data/
    geminixx-datafile-path=/home/jdoe/inclxx/inclxx-XXXXX-0123456/de-excitation/geminixx/upstream/
    ablav3p-cxx-datafile-path=/home/jdoe/inclxx/inclxx-XXXXX-0123456/de-excitation/ablaxx/upstream/data/G4ABLA3.0/
    abla07-datafile-path=/home/jdoe/inclxx/inclxx-XXXXX-0123456/de-excitation/abla07/upstream/tables/

Defining a run                                                           {#run}
--------------

Options defining a run (such as projectile type and energy, target type, number
of shots, etc.) are better defined in an input file or on the command line. For
example, the following input file starts a 100k-event INCL++/ABLA07 calculation
for 1-GeV proton + Pb208:

    projectile = p
    target = Pb208
    energy = 1000
    number-shots = 100000
    de-excitation = abla07

Assuming you saved the file above as `input.in`, you can start the calculation
by running:

    $ ./INCLCascade /path/to/input.in

The same calculation can also be started with the following command line:

    $ ./INCLCascade -pp -tPb208 -E1000 -N100000 -dabla07

Both commands will produce a log file and a ROOT file (if ROOT was enabled at
compilation time). The log file contains all available options, along with the
values that were taken for that run. Note that you can re-use snippets of the
log file as an input file.

A full listing of the available command-line and input-file options can be
obtained by running:

    $ ./INCLCascade --help



Analysing the code results                                          {#analysis}
==========================

If you have compiled the code with ROOT support, the INCLCascade executable
will produce a ROOT file containing the result of your calculation, under the
form of `TTree` objects.

Global tree                                                      {#global-tree}
-----------

The `gt` tree (*global tree*) contains information about the run. It can
contain several entries, but in practice only the last one is relevant. The
most important variables are the following:

* `nShots` (type `Int_t`): number of requested shots. Corresponds to the
  argument to the `-N` command-line option. Note that not all the shots result
  in a reaction; if the projectile does not collide with any of the target
  nucleons, the event is considered as *transparent*. This allows INCL++ to do
  absolute predictions about the reaction cross section.
* `nTransparents` (`Int_t`): number of transparent events.
* `geometricCrossSection` (`Float_t`): geometric cross section (in mb), useful
  for normalisation (see [below](#normalisation)).
* `reactionCrossSection` (`Float_t`): predicted reaction cross section (mb).
* `errorReactionCrossSection` (`Float_t`): statistical error on the predicted
  reaction cross section, (mb).
* `Ap`, `Zp`, `At`, `Zt` (`Short_t`): projectile and target mass and charge
  numbers.
* `Ep` (`Float_t`): projectile energy (MeV).

Event tree                                                        {#event-tree}
----------

The `et` tree (*event tree*) contains the information about the generated
reaction events. Each entry represents a reaction event. The most important
variables are the following:

* `nParticles` (`Short_t`): number of particles in the final state. This
  includes particles emitted during both the cascade and de-excitation stages.
* `nRemnants` (`Short_t`): number of remnants (sometimes called
  "pre-fragments") produced by the cascade stage. This is always 1 in nucleon-
  and pion-nucleus reactions. In nucleus-nucleus reactions, it can be 1 or 2.

The tree contains some arrays that describe the particles in the final state.
For a given event, the array indices range from 0 to `nParticles-1` (included).
The most important arrays indexed by `nParticles` are:

* `A`, `Z` (`Short_t` arrays): baryon and charge numbers of the particles in
  the final state. *
* `EKin` (`Float_t` array): kinetic energy (MeV). *
* `px`, `py`, `pz` (`Float_t` arrays): cartesian components of the particle
  momentum (MeV/c). *
* `theta` (`Float_t` array): polar angle of the momentum vector (degrees). *
* `phi` (`Float_t` array): azimuthal angle of the momentum vector (degrees). *
* `origin` (`Short_t` array): equals -1 if the particle was emitted during the
  cascade stage; otherwise, it is the index of the remnant that emitted the
  particle during de-excitation (see the `nRemnants`-indexed arrays below). *

If the `--inverse-kinematics` flag was activated on the command line, the
following variables are also available (see [below](#inverse-kinematics)):

* `EKinPrime` (`Float_t` array): kinetic energy of the particles in the
  inverse-kinematics frame (MeV). *
* `pzPrime` (`Float_t` array): z component of the particle momentum in the
  inverse-kinematics frame (MeV/c). *
* `thetaPrime` (`Float_t` array): polar angle of the particle momentum vector
  in the inverse-kinematics frame (degrees). *

Additionally, the `et` tree contains arrays that describe the cascade
remnant(s), indexed by `nRemnants`. In nucleon- and pion-nucleus reactions,
these arrays always contain only one element (index 0). The most important
arrays are:

* `ARem`, `ZRem` (`Short_t` arrays): baryon and charge numbers of the cascade
  remnant(s).
* `EStarRem` (`Float_t` array): excitation energy (MeV).
* `JRem` (`Float_t` array): spin (hbar).
* `EKinRem` (`Float_t` array): kinetic (recoil) energy (MeV).
* `pxRem`, `pyRem`, `pzRem` (`Float_t` arrays): cartesian components of the
  remnant recoil momentum (MeV/c).
* `thetaRem` (`Float_t` array): polar angle of the momentum vector (degrees).
* `phiRem` (`Float_t` array): azimuthal angle of the momentum vector (degrees).


Concise event trees                                                  {#concise}
-------------------

If you want to study rare events (small cross sections), you will typically
need to perform runs with a very large number of incident particles.  In such
high-statistics runs, the ROOT output files quickly unwieldily large, thereby
taking up a lot of space on disk and slowing down the analysis.  There are two
ways to work around this problem.

First, it is possible to save some space by using the `--concise-root-tree`
command-line flag.  This flag tells INCL++ to generate an abridged output ROOT
tree, which will contain only quantities that are directly observable by the
experiment; they are marked with a star in the previous section.

If this does not suffice, or if one is interested in the values of
non-observable values in rare events, it is possible to use the
`--root-selection` option to to specify a cut on the events that should be
included in the output file.  This option takes a ROOT cut string as an
argument; the output event tree will only contain events that pass the cut.
For instance, passing `--root-selection="theta<1 && A==1 && Z==0"` will produce
a ROOT tree containing only events with at neutron emitted between 0° and 1°.
Note that the *full* event is included in the output.


Inverse kinematics                                        {#inverse-kinematics}
------------------

The `INCLCascade` program always performs the requested simulation in the rest
frame of the specified target.  Sometimes, however, the experimental data one
wishes to compare to were taken in "inverse kinematics"; for instance, a 1-AGeV
U238 + H reaction should be simulated as 1-GeV p + U238; the appropriate
reference-frame transformation should then be applied.

`INCLCascade` will actually perform the transformation for you if you specify
the `--inverse-kinematics` command-line option. This will add extra variables
to the output ROOT tree that describe the final state of the reaction in
inverse kinematics.  Reactions are always simulated in direct kinematics
(projectile moving with velocity *v* in the direction of the positive z axis,
target at rest); in the inverse-kinematics frame, the target moves with
velocity *v* in the direction of the positive z-axis, while the projectile is
at rest.

You can recognize them in the output ROOT tree because their names end in
"Prime".


Normalising counts to cross sections                           {#normalisation}
------------------------------------

A typical way to analyse the INCL++ result would be to loop over the event
trees and fill histograms. However, you might want to convert your counts to
cross sections. The correct way to do that is to use the geometric cross
section and the number of shots contained in the global tree. The formula is:

    cross_section = counts * geometric_cross_section / number_of_shots

You can programmatically calculate the normalisation factor (e.g. in a ROOT
script) as follows:

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~{.cc}
    TFile *file = new TFile("output.root");
    TTree *gt = (TTree *) file->Get("gt");
    gt->GetEntry(gt->GetEntries()-1);
    Float_t normalisation = gt->GetLeaf("geometricCrossSection")->GetValue()
                            / gt->GetLeaf("nShots")->GetValue();
    Float_t cross_section = counts * normalisation;
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

The statistical uncertainty on the calculated cross section is given by

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~{.cc}
    #include <cmath>
    Float_t err_cross_section = std::sqrt(counts) * normalisation;
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



Code documentation                                             {#documentation}
==================

The code is fairly well documented and uses Doxygen markup. If `doxygen` and
`dot` (from the `graphviz` package) are installed, you can run the `make doc`
command from the build directory and generate HTML documentation in the `doc/`
subdirectory. Launch your favourite web browser and type

    file://<absolute_path_to_the_build_folder>/doc/html/main.html

in the address bar.



Questions, bug reports and contact information                     {#questions}
==============================================

If you have any problems with the code, please check that you went through all
the steps described in the ["Compiling the code"](#compiling) and ["Running the
code"](#running) sections above. If you still have problems, or if you have any
questions or bug reports, please visit the [official INCL web
site](http://irfu.cea.fr/Sphn/Spallation/incl.html).



References                                                        {#references}
==========

1. A. Boudard, J. Cugnon, J.-C. David, S. Leray and D. Mancusi, **Phys. Rev.
   C87** (2013) 014606. DOI:
   [10.1103/PhysRevC.87.014606](http://dx.doi.org/10.1103/PhysRevC.87.014606).
2. D. Mancusi, A. Boudard, J. Cugnon, J.-C. David, P. Kaitaniemi and S. Leray,
   **Phys. Rev. C90** (2014) 054602. DOI:
   [10.1103/PhysRevC.90.054602](http://dx.doi.org/10.1103/PhysRevC.90.054602).
3. S. Pedoux, Ph.D. thesis (2011), University of Liège, Liège, Belgium.
4. S. Pedoux and J. Cugnon, **Nucl. Phys. A866** (2011) 16.  DOI:
   [10.1016/j.nuclphysa.2011.07.004](http://dx.doi.org/10.1016/j.nuclphysa.2011.07.004).
