      SUBROUTINE BURST(APRF,EEFACTOR,FESTRETCH)

C  This version of BURST was tested by S. Lukic in Jan 2006 to be the best
C  in reproducing the nuclide distributions

c          If either the spallated mass is smaller than half the projectile mass
C          or the target is smaller than 3 nucleons,
C          the "abrasion model" does not work.
C          Instead we use the "statistical burst model". In case of
C          of deuteron target, the particularly broad mass distribution
c          is considered.

C         On input:
C           EEFACTOR    Factor for excitation-energy distribution
C           All other input values are provided by the common block ABRAMAIN
C
C         On output:
C           APRF        Mass of prefragment due to the burst
C           FESTRETCH   Factor to stretch the mean excitation energy of 13.3 MeV per abraded particle
C           B           Impact parameter


c       The value of 0.99 would correspond to the diffuseness, but it does
c       not fit the data.
c       DIFF = 0.99D0

      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     &                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL

      REAL*4 poissonHaz, haz, gausshaz
      REAL*8 DIFFPATH,U_AMEAN

      REAL*8 DIFF,B_TAIL,R_P_DIFF,BURST_COEFF_A,BURST_COEFF_E,BMAX_D,
     & XB,XPROB,B_0,A_1,A_2,A_MEAN,AR,APRF,EEXCMEAN,EEFACTOR,FESTRETCH,
     & DELTA,U_ALPHA,B_1,B_2,A1_MEAN,A2_MEAN,A1R,A2R

      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     &       R_P,R_T,PI,BFPRO,SNPRO,SPPRO,SHELL
      INTEGER*4 IMAX,IRNDM

        DIFF = 0.99D0
        B_tail = R_P - DIFF
        R_P_DIFF = R_P + 3.D0 * DIFF

c       if (AT.EQ.1.D0) then
      

        BMAX_D = R_P_DIFF


530     CONTINUE
        XB = DBLE(HAZ(1))
        XPROB = DBLE(HAZ(1))
        IF(XPROB.GT.XB) GO TO 530
        B_0 = XB*BMAX_D
c       IF (B_0.GT.R_T) GO TO 530

c       Full treatment with diffuseness:
        IF (B_0.LT.B_TAIL) THEN
         A_1 = 2.D0 *DSQRT(R_P**2 - B_0**2)
c          Write(6,*) 'B', B_0,A_1
        ELSE
          A_1 = DiffPath(R_P,Diff,B_0)
c          Write(6,*) 'B diff', B_0,A_1
        ENDIF

C       New parameterisation, August 2006, KHS:
        A_MEAN = U_AMEAN(AP,AT,A_1,EAP)

531     AR = dble(PoissonHaz(1,sngl(A_mean)))
        IF(AR .GT. AP) GOTO 531

c       Poission statistic of first collisions;

c         write(6,*) K,A_mean,AR


532     APRF = AP - AR

C       New parameterisation, March 2005, KHS:
        Eexcmean = 27.D0 * AR

        IF (AR.LE.0.) GOTO 530

        FESTRETCH = (Eexcmean/(AR * 13.3D0))/EEFACTOR

       END

C  ----------------------------------------------------------------

C      Effective length in traversing a diffuse nucleus
       Function DiffPath(R_P,Diff,B_0)
       Real*8 R_P,Diff,B_0,Diffpath,argument,delta
       real*4 gausshaz
       Integer I

         argument = 0.D0

         do I=-8,12
         delta = 0.25D0 * diff * dble(I)
           If (B_0-delta.lt.R_P) then
             argument = argument +
     c       2.D0 *DSQRT(R_P**2 - (B_0-delta)**2) *
     c          gausshaz(0.D0,Diff,delta)
           endif
         enddo
         DiffPath = diff * 0.25D0 * argument * 1.15D0
         Return
       End
C
C  ----------------------------------------------------------------
C
        DOUBLE PRECISION FUNCTION U_AMEAN(U_AP,U_AT,U_A1,U_EAP)
        REAL*8 U_AP,U_AT,U_A1,U_EAP
          U_AMEAN = U_A1 * 0.4D-2 * U_EAP**0.75D0 * (U_AT)**0.667D0
     &      * 1.2D0
     &      * 1.08D0 / (1.D0 + U_AP / 1500.D0)
        END
