      Subroutine Fission_Distri(A,Z,E,JPRF,INUM,A1,Z1,E1,V1,A2,Z2,E2,V2,
     &           NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,
     &           N6HE_POST,imode,
     &           VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC)

C  Last update:
C             28/10/13 - J.L.R.S. - Add deformation from abrablav4 (AK)


C  Authors: K.-H. Schmidt, S. Zhdanov, A. Grewe, J. Benlliure, K. Helariutta,
C           T. Enqvist, A. Kelic, M. V. Ricciardi   (1995 - 2009)

C  On input: A, Z, E (mass, atomic number and exc. energy of compound nucleus
C                     before fission)
C  On output: Ai, Zi, Ei (mass, atomic number and (absolute) exc. energy of
c                         fragment 1 and 2 after fission)

c  /* This program calculates isotopic distributions of fission fragments    */
c  /* with a semiempirical model                                             */

c  /* The width and eventually a shift in N/Z (polarization) follows the     */
c  /* following rules:                                                       */
c  /*                                                                        */
c  /* The line N/Z following UCD has an angle of atan(Zcn/Ncn)               */
c  /* to the horizontal axis on a chart of nuclides.                         */
c  /* (For 238U the angle is 32.2 deg.)                                      */
c  /*                                                                        */
c  /*   The following relations hold: (from Armbruster)
c
c    sigma(N) (A=const) = sigma(Z) (A=const)
c    sigma(A) (N=const) = sigma(Z) (N=const)
c    sigma(A) (Z=const) = sigma(N) (Z=const)
c
c   From this we get:
c    sigma(Z) (N=const) * N = sigma(N) (Z=const) * Z
c    sigma(A) (Z=const) = sigma(Z) (A=const) * A/Z
c    sigma(N) (Z=const) = sigma(Z) (A=const) * A/Z
c    Z*sigma(N) (Z=const) = N*sigma(Z) (N=const) = A*sigma(Z) (A=const)     */
c

c  /*   Input parameters: */
      REAL*8 Z,A,E,JPRF

C  /*   Output parameters: */
      REAL*8 Z1,A1,E1,Z2,A2,E2

c  /*   Model parameters: */
C     These parameters have been adjusted to the compound nucleus 239U.
c     For the fission of another compound nucleus, it might be
c     necessary to slightly adjust some parameter values.
c     The most important ones are
C      Delta_U1_shell_max and
c      Delta_u2_shell.

      REAL*8 Nheavy1_in   !  'position of shell for Standard 1'
      PARAMETER (Nheavy1_in = 83.D0)

      REAL*8 Zheavy1_in   !  'position of shell for Standard 1'
      PARAMETER (Zheavy1_in = 50.D0)

      REAL*8 Nheavy2   !  'position of heavy peak valley 2'
      PARAMETER (Nheavy2 = 89.0D0)

      REAL*8 Delta_U1_shell_max  !  'Shell effect for valley 1'
      PARAMETER (Delta_U1_shell_max = -2.45D0)

      REAL*8 U1NZ_SLOPE  ! Reduction of shell effect with distance to 132Sn
      PARAMETER (U1NZ_SLOPE = 0.2D0)

      REAL*8 DELTA_U2_shell  !  'Shell effect for valley 2'
      PARAMETER (Delta_U2_shell = -2.45D0)

      REAL*8 X_s2s    !  'Ratio (C_sad/C_scis) of curvature of potential'
      PARAMETER (X_s2s = 0.8D0)

      REAL*8 hbom1,hbom2,hbom3  !  'Curvature of potential at saddle'
      PARAMETER (hbom1 = 0.2D0)  ! hbom1 is hbar * omega1 / (2 pi) !!!
      PARAMETER (hbom2 = 0.2D0)  ! hbom2 is hbar * omega2 / (2 pi) !!!
      PARAMETER (hbom3 = 0.2D0)  ! hbom3 is hbar * omega3 / (2 pi) !!!

      REAL*8 Fwidth_asymm1,Fwidth_asymm2,Fwidth_symm
c         'Factors for widths of distr. valley 1 and 2'
      PARAMETER (Fwidth_asymm1 = 0.65D0)
      PARAMETER (Fwidth_asymm2 = 0.65D0)
      PARAMETER (Fwidth_symm   = 1.16D0)

      REAL*8 xLevdens   ! 'Parameter x: a = A/x'
      PARAMETER (xLevdens = 10.75D0)
c     The value of 1/0.093 = 10.75 is consistent with the
c     systematics of the mass widths of Ref. (RuI97).

      REAL*8 FGAMMA ! 'Factor to gamma'
      Parameter (Fgamma = 1.D0)  ! Theoretical expectation, not adjusted to data.
C     Additional factor to attenuation coefficient of shell effects
c     with increasing excitation energy

      REAL*8 FGAMMA1 ! 'Factor to gamma_heavy1'
      Parameter (FGAMMA1 = 2.D0)
c     Adjusted to reduce the weight of Standard 1 with increasing
c     excitation energies, as required by experimental data.

      REAL*8 FREDSHELL
      PARAMETER(FREDSHELL = 0.D0)
c     Adjusted to the reduced attenuation of shells in the superfluid region.
C     If FGAMMA is modified,
c     FGAMMA * FREADSHELL should remain constant (0.65) to keep
c     the attenuation of the shell effects below the critical
c     pairing energy ECRIT unchanged, which has been carefully
c     adjusted to the mass yields of Vives and Zoeller in this
c     energy range. A high value of FGAMMA leads ot a stronger
c     attenuation of shell effects above the superfluid region.

      REAL*8 ECRIT
      PARAMETER(ECRIT = 5.D0)
c     The value of ECRIT determines the transition from a weak
c     decrease of the shell effect below ECRIT to a stronger
c     decrease above the superfluid range.

      REAL*8 d   ! 'Surface distance of scission configuration'
      PARAMETER (D = 2.0D0)

C  /* Charge polarisation from Wagemanns p. 397: */
      REAL*8 cpol1 ! 'Charge polarisation standard I'
      PARAMETER(cpol1 = 0.35D0)  ! calculated internally with shells

      REAL*8 cpol2  ! 'Charge polarisation standard II'
      PARAMETER(cpol2 = 0.0D0)  ! calculated internally from LDM

      REAL*8 Friction_factor
      PARAMETER (Friction_factor = 1.D0)

      REAL*8 Zheavy1,Nheavy1   ! 'position of valley St 1 in Z and N'
      REAL*8 DELTA_U1,DELTA_U2  ! 'used shell effects
      REAL*8 cN_asymm1_shell, cN_asymm2_shell
c      PARAMETER(cN_asymm1_shell = 0.7D0) ! curvature of asymmetric valley 1
c      PARAMETER(cN_asymm2_shell = 0.08D0) ! curvature of asymmetric valley 2
      REAL*8 gamma,gamma_heavy1,gamma_heavy2 ! fading of shells
      REAL*8 E_saddle_scission  ! friction from saddle to scission
c      INTEGER*4 IMODE(3)  ! Internal counter for different modes
      REAL*8 Ysymm   ! Yield of symmetric mode
      REAL*8 Yasymm1  ! Yield of asymmetric mode 1
      REAL*8 Yasymm2  ! Yield of asymmetric mode 2
      REAL*8 Nheavy1_eff  ! Effective position of valley 1
      REAL*8 Nheavy2_eff ! Effective position of valley 2
      REAL*8 Eexc1_saddle ! Excitation energy above saddle 1
      REAL*8 Eexc2_saddle ! Excitation energy above saddle 2
      REAL*8 Eexc_max  ! Excitation energy above lowest saddle
      REAL*8 Aheavy1_mean  ! Effective mass mode 1
      REAL*8 Aheavy2_mean  ! Effective mass mode 2
      REAL*8 WAsymm_saddle ! Width of symmetric mode
      REAL*8 WAheavy1_saddle  ! Width of asymmetric mode 1
      REAL*8 WAheavy2_saddle  ! Width of asymmetric mode 2
      REAL*8 WAsymm  ! Width of symmetric mode
      REAL*8 WAheavy1  ! Width of asymmetric mode 1
      REAL*8 WAheavy2  ! Width of asymmetric mode 2
      REAL*8 R_E_O  ! Even-odd effect in Z
      REAL*8 cN_symm  ! Curvature of symmetric valley
      REAL*8 CN  ! Curvature of mass distribution for fixed Z
      REAL*8 CZ  ! Curvature of Z distribution for fixed A
      REAL*8 Nheavy2_NZ  ! Position of Shell 2, combined N and Z
      REAL*8 N
      REAL*8 Nlight1,Nlight2
      REAL*8 Aheavy1,Alight1,Aheavy2,Alight2
      REAL*8 Eheavy1,Elight1,Eheavy2,Elight2
      REAL*8 Zlight1,Zlight2
      REAL*8 Sasymm1,Sasymm2,Ssymm,Ysum,Yasymm
      REAL*8 Ssymm_mode1,Ssymm_mode2
      REAL*8 CZ_asymm1_saddle,CZ_asymm2_saddle
C     /* Curvature at saddle, modified by ld-potential */
      REAL*8 WZasymm1_saddle, WZasymm2_saddle, WZsymm_saddle
      REAL*8 WZasymm1_scission, WZasymm2_scission, WZsymm_scission
      REAL*8 WZasymm1,WZasymm2,WZsymm
      REAL*8 wNasymm1_saddle, wNasymm2_saddle, wNsymm_saddle
      REAL*8 wNasymm1_scissionw, wNasymm2_scission, wNsymm_scission
      REAL*8 wNasymm1, WNasymm2, wNsymm
      REAL*8 Nlight1_eff, Nlight2_eff
      INTEGER*4 IMODE
      REAL*8  Rmode
      REAL*8 Z1mean, Z2mean, Z1width, Z2width,ZA1width
      REAL*8 N1R,N2R,A1R,A2R,N1,N2
      REAL*8 NE_MIN,NE_M1,NE_M2,W_LOW,W_HIGH,W_HEAVY
      REAL*8 NE_M1_HIGH,NE_M2_HIGH,NE_M1_LOW,NE_M2_LOW
      REAL*8 Zsymm,Nsymm,Asymm
      REAL*8 N1mean, N2mean, N1width, N2width
      REAL*8 dUeff
c      /* effective shell effect at lowest barrier */
      REAL*8 Eld
c      /* Excitation energy with respect to ld barrier */
      REAL*8 re1,re2,re3
      REAL*8 eps1,eps2
      REAL*8 N1ucd,N2ucd,Z1ucd,Z2ucd
      REAL*8 beta,beta1,beta2
      REAL*8 betacomplement
      REAL*8 DN1_POL
C     /* shift of most probable neutron number for given Z,
C            according to polarization */
      REAL*8 R_E_O_MAX
      REAL*8 Bf_mean
      REAL*8 E_PAIR
      INTEGER*4 I_HELP
      INTEGER*4 I_ITERATE
      REAL*8 pi
      PARAMETER(pi = 3.141593D0)
c    /* Parameters of the semiempirical fission model */
      REAL*8 A_levdens
C            /* level-density parameter */
      REAL*8 A_levdens_light1,A_levdens_light2
      REAL*8 A_levdens_heavy1,A_levdens_heavy2

      REAL*8 R0
      PARAMETER(R0 = 1.16D0)

      REAL*8 epsilon_1_saddle,epsilon0_1_saddle
      REAL*8 epsilon_2_saddle,epsilon0_2_saddle,epsilon_symm_saddle
      REAL*8 epsilon_1_scission,epsilon0_1_scission
      REAL*8 epsilon_2_scission,epsilon0_2_scission
      REAL*8 epsilon_symm_scission
c                                    /* modified energy */
      REAL*4 E_eff1_saddle,E_eff2_saddle
      REAL*8 Epot0_mode1_saddle,Epot0_mode2_saddle,Epot0_symm_saddle
      REAL*8 Epot_mode1_saddle,Epot_mode2_saddle,Epot_symm_saddle
      REAL*8 E_defo,E_defo1,E_defo2,E_scission_pre,E_scission_post
      REAL*8 E_asym
      REAL*8 E1exc,E2exc
      REAL*8 E1exc_sigma,E2exc_sigma
      REAL*8 E1final,E2final
      REAL*8 BN1,BN2
      REAL*8 TKE,TKER,VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC
      REAL*8 Ekin1,Ekin2,EkinR1,EkinR2,V1,V2
      INTEGER*4 icz,k

      REAL*8 MassCurv_scis, MassCurv_sadd
      REAL*8 cN_symm_sadd

      REAL*8 NHEAVY1_SHELL,NHEAVY2_shell
      REAL*8 WNASYMM1_SCISSION
      REAL*8 ALIGHT1_EFF,ALIGHT2_EFF,AHEAVY1_EFF,AHEAVY2_EFF
      REAL*8 Z1RR,Z1R,Z2R
      REAL*8 MASSCURV
      REAL*8 CZ_SYMM
      REAL*8 REPS1,REPS2,RN1_POL
      REAL*8 E_HELP
      INTEGER*2 IEVAP1,IEVAP2
      INTEGER*2 IFISS_MODE
      REAL*8 Z_scission,N_scission,A_Scission
      REAL*8 ATEMP,ZTEMP,ETEMP,VTEMP
      REAL*8 Z2_over_A_eff,JFACT

      REAL*8 SIGZMIN   ! 'Minimum neutron width for constant Z'
      Integer*4 ICOUNT, I_MODE
      REAL*8 DSN132,Delta_U1_shell,uwash,E_eff0_saddle,frldm,ecoul,
     &       e_scission
      REAL*4 haz,gausshaz,rtest

C Number of particles emitted between saddle and scission:
      REAL*8 NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,
     &       N6HE_POST
C
      REAL*8 EV_TAB_SSC(100,5)
      INTEGER*4 IEV_TAB_SSC, INUM,IS
      COMMON/EV_TAB_STS/EV_TAB_SSC,IEV_TAB_SSC
      REAL*8 ECGNZ1(0:153,0:98),ECFNZ1(0:153,0:98),
     &       VGSLD1(0:153,0:98),BETA21(0:250,0:136),
     &       BETA41(0:250,0:136),BETA2GS,BETA1GS,BETAGS
      COMMON /ECLD1/ ECGNZ1,ECFNZ1,VGSLD1,BETA21,BETA41

      k = 0
      SIGZMIN = 0.5D0
      N = A - Z ! /*  neutron number of the fissioning nucleus  */

      cN_asymm1_shell = 0.70D0 * N/Z
      cN_asymm2_shell = 0.04D0 * N/Z  ! oli 0.080

       ICOUNT = ICOUNT + 1

C*********************************************************************

      I_MODE = 0

      DSN132 = Nheavy1_in - N/Z * Zheavy1_in
      Aheavy1 = Nheavy1_in + Zheavy1_in + 0.34D0 * DSN132
c     /* Neutron number of valley Standard 1 */
c     /* It is assumed that the 82-neutron shell effect is stronger than
c         the 50-proton shell effect. Therefore, the deviation in N/Z of
c         the fissioning nucleus from the N/Z of 132Sn will
c         change the position of the combined shell in mass. For neutron-
c         deficient fissioning nuclei, the mass will increase and vice
c         versa.  */

      Delta_U1_shell = Delta_U1_shell_max + U1NZ_SLOPE * DABS(DSN132)
      Delta_U1_shell = DMIN1(0.0D0,Delta_U1_shell)
C      /* Empirical reduction of shell effect with distance in N/Z of CN to 132Sn */
C      /* Fits (239U,n)f and 226Th e.-m.-induced fission */

      Nheavy1 = N/A * Aheavy1   !/* UCD */
      Aheavy2 = Nheavy2 * A/N

      Nlight1 = N - Nheavy1
      Nlight2 = N - Nheavy2

      Zsymm  = Z / 2.D0 !/* proton number in symmetric fission (centre) */
      Nsymm  = N / 2.D0
      Asymm = Nsymm + Zsymm

      A_levdens = A / xLevdens
      gamma = A_levdens / (0.4D0 * A**1.3333D0) * FGAMMA
      A_levdens_heavy1 = Aheavy1 / xLevdens
      gamma_heavy1 = A_levdens_heavy1 / (0.4D0 * Aheavy1**1.3333D0)
     &               * FGAMMA * FGAMMA1
      A_levdens_heavy2 = Aheavy2 / xLevdens
      gamma_heavy2 = A_levdens_heavy2 / (0.4D0 * Aheavy2**1.3333D0)
     &               * FGAMMA

      k = 0
      icz = 0


c     Energy dissipated from saddle to scission
c     F. Rejmund et al., Nucl. Phys. A 678 (2000) 215, fig. 4 b    */
      E_saddle_scission = (-24.D0 + 0.02227D0 * Z**2/(A**0.33333D0))
     &                     * Friction_factor
      E_saddle_scission = DMAX1( 0.0D0, E_saddle_scission )

C     Fit to experimental result on curvature of potential at saddle
c     Parametrization of T. Enqvist according to Mulgin et al. 1998
c     MassCurv taken at scission.    */

       Z2_over_A_eff = Z**2/A

      if( Z**2/A .lt. 34.0D0 ) then
        MassCurv_scis =  10.D0 ** ( -1.093364D0 + 0.082933D0 * Z**2/A
     &                          - 0.0002602D0 * Z**4 / (A**2) )
      else
        MassCurv_scis = 10.D0 ** ( 3.053536D0 - 0.056477D0 * Z**2/A
     &                          + 0.0002454D0 * Z**4 / (A**2) )
      endif


c     to do:
c     fix the X with the channel intensities of 226Th (KHS at SEYSSINS,
c     1998)
c     replace then (all) cN_symm by cN_symm_saddle (at least for Yields)
      MassCurv_sadd = X_s2s * MassCurv_scis

      cN_symm      = 8.D0 / N**2 * MassCurv_scis
      cN_symm_sadd = 8.D0 / N**2 * MassCurv_sadd

      Nheavy1_shell = Nheavy1

      IF (E .LT. 100.D0) THEN
      Nheavy1_eff = (cN_symm_sadd*Nsymm +
     &                  cN_asymm1_shell *
     &                Uwash(E/A*Aheavy1,Ecrit,FREDSHELL,gamma_heavy1) *
     &                  Nheavy1_shell)
     &                / (cN_symm_sadd +
     &                  cN_asymm1_shell *
     &                Uwash(E/A*Aheavy1,Ecrit,FREDSHELL,gamma_heavy1))
      ELSE
      Nheavy1_eff = (cN_symm_sadd*Nsymm +
     &               cN_asymm1_shell*Nheavy1_shell)
     &             / (cN_symm_sadd +
     &               cN_asymm1_shell)
      ENDIF
      Nlight1_eff = N - Nheavy1_eff

C     /* Position of Standard II defined by neutron shell */
      Nheavy2_NZ = Nheavy2


      Nheavy2_shell = Nheavy2_NZ
      IF (E .LT. 100.D0) THEN
      Nheavy2_eff = (cN_symm_sadd*Nsymm +
     &               cN_asymm2_shell*
     &               Uwash(E/A*Aheavy2,Ecrit,FREDSHELL,gamma_heavy2) *
     &               Nheavy2_shell)
     &             / (cN_symm_sadd +
     &               cN_asymm2_shell*
     &               Uwash(E/A*Aheavy2,Ecrit,FREDSHELL,gamma_heavy2))
      ELSE
      Nheavy2_eff = (cN_symm_sadd*Nsymm +
     &               cN_asymm2_shell*Nheavy2_shell)
     &             / (cN_symm_sadd +
     &               cN_asymm2_shell)
      ENDIF

c      Write (*,*) 'A',A,' Z',Z,' E',E
c      Write (*,*) 'cN_symm_sadd',CN_symm_sadd,
c     &     '  CN_asymm2_shell',CN_asymm2_shell
c      Write (*,*) 'Aheavy2',Aheavy2,'  gamma',gamma
c      Write (*,*) 'gamma_heavy1',gamma_heavy1,
c     &  '  gamma_heavy2',gamma_heavy2
c      Write (*,*) 'Nheavy2_shell',Nheavy2_shell
c      Write (*,*) 'Nheavy1_eff',Nheavy1_eff,'  Nheavy2_eff',Nheavy2_eff

      Nlight2_eff = N - Nheavy2_eff

      Delta_U1 = Delta_U1_shell + (Nheavy1_shell - Nheavy1_eff)**2 *
     &           cN_asymm1_shell !/* shell effect in valley of mode 1 */
       Delta_U1 = DMIN1(Delta_U1,0.D0)
      Delta_U2 = Delta_U2_shell + (Nheavy2_shell - Nheavy2_eff)**2 *
     &           cN_asymm2_shell !/* shell effect in valley of mode 2 */
       Delta_U2 = DMIN1(Delta_U2,0.D0)

c     liquid drop energies at the centres of the different shell effects
c     with respect to liquid drop at symmetry
      Epot0_mode1_saddle = (Nheavy1_eff-Nsymm)**2 * cN_symm_sadd
      Epot0_mode2_saddle = (Nheavy2_eff-Nsymm)**2 * cN_symm_sadd
      Epot0_symm_saddle = 0.D0

c     energies including shell effects at the centres of the different
c     shell effects with respect to liquid drop at symmetry  */
      Epot_mode1_saddle = Epot0_mode1_saddle + Delta_U1
      Epot_mode2_saddle = Epot0_mode2_saddle + Delta_U2
      Epot_symm_saddle = Epot0_symm_saddle

c     minimum of potential with respect to ld potential at symmetry */
      dUeff = DMIN1( Epot_mode1_saddle, Epot_mode2_saddle )
      dUeff = DMIN1( dUeff, Epot_symm_saddle )
      dUeff = dUeff - Epot_symm_saddle

      Eld = E + dUeff
c     E   = energy above lowest effective barrier
c     Eld = energy above liquid-drop barrier
c     Due to this treatment the energy E on input means the excitation
c     energy above the lowest saddle.                                  */

c     excitation energies at saddle modes 1 and 2 without shell effect  */
      epsilon0_1_saddle = Eld - Epot0_mode1_saddle
      epsilon0_2_saddle = Eld - Epot0_mode2_saddle

c     excitation energies at saddle modes 1 and 2 with shell effect */
      epsilon_1_saddle = Eld - Epot_mode1_saddle
      epsilon_2_saddle = Eld - Epot_mode2_saddle

      epsilon_symm_saddle = Eld - Epot_symm_saddle

      EEXC1_saddle = epsilon_1_saddle
      EEXC2_saddle = epsilon_2_saddle

c     EEXC_MAX is energy above the lowest saddle */
      EEXC_MAX = DMAX1( Eexc1_saddle, Eexc2_saddle )
      EEXC_MAX = DMAX1( EEXC_MAX, Eld )


c     excitation energies without shell effect  */
      epsilon0_1_scission = Eld + E_saddle_scission - Epot0_mode1_saddle
      epsilon0_2_scission = Eld + E_saddle_scission - Epot0_mode2_saddle

c     excitation energy at scission */
      epsilon_1_scission = Eld + E_saddle_scission - Epot_mode1_saddle
      epsilon_2_scission = Eld + E_saddle_scission - Epot_mode2_saddle

c     excitation energy of symmetric fragment at scission  */
      epsilon_symm_scission = Eld + E_saddle_scission - Epot_symm_saddle

c     calculate widhts at the saddle
      E_eff1_saddle = epsilon0_1_saddle -
     &   Delta_U1 *
     & Uwash(epsilon_1_saddle/A*Aheavy1,Ecrit,FREDSHELL,gamma_heavy1)
      if( E_eff1_saddle .LT. A_levdens * hbom1**2) then
          E_eff1_saddle = A_levdens * hbom1**2
      endif
      wNasymm1_saddle =
     &   DSQRT(0.5D0 * DSQRT(1.D0/A_levdens*E_eff1_saddle) /
     &   (cN_asymm1_shell *
     & Uwash(epsilon_1_saddle/A*Aheavy1,Ecrit,FREDSHELL,gamma_heavy1)+
     &   cN_symm_sadd))

      E_eff2_saddle = epsilon0_2_saddle -
     &    Delta_U2 *
     &  Uwash(epsilon_2_saddle/A*Aheavy2,Ecrit,FREDSHELL,gamma_heavy2)
      If (E_eff2_saddle .LT. A_levdens * hbom2**2) Then
          E_eff2_saddle = A_levdens * hbom2**2
      endif
      wNasymm2_saddle =
     &   DSQRT(0.5D0 * DSQRT(1.D0/A_levdens*E_eff2_saddle) /
     &   (cN_asymm2_shell *
     &  Uwash(epsilon_2_saddle/A*Aheavy2,Ecrit,FREDSHELL,gamma_heavy2)+
     &   cN_symm_sadd))

      E_eff0_saddle = epsilon_symm_saddle
      IF (E_eff0_saddle .LT. A_levdens * hbom3**2) Then
          E_eff0_saddle = A_levdens * hbom3**2
      endif
      wNsymm_saddle =
     &     DSQRT(0.5D0 * DSQRT(1.D0/A_levdens*E_eff0_saddle) /
     &               cN_symm_sadd)

      if(epsilon_symm_scission .GT. 0.D0 ) then
        E_HELP = DMAX1(E_Saddle_scission,epsilon_symm_scission)
        wNsymm_scission =
     &   DSQRT(0.5D0 * DSQRT(1.D0/A_levdens*(E_HELP)) /
     &               cN_symm)
      else
        wNsymm_scission =
     &   DSQRT(0.5D0 * DSQRT(1.D0/A_levdens*E_saddle_scission) /
     &               cN_symm)
      endif


c     /* Calculate widhts at the scission point: */
c     /* fits of ref. Beizin 1991 (Plots by Sergei Zhdanov) */

      if( E_saddle_scission .EQ. 0.D0 ) then
        wNasymm1_scission = wNasymm1_saddle
        wNasymm2_scission = wNasymm2_saddle
      else
        if( Nheavy1_eff .GT. 75.D0 ) then
          wNasymm1_scission = DSQRT(21.D0)*N/A
c          wNasymm2_scission = DSQRT(DMAX1( (70.D0-28.D0)/3.D0 *
c     &                     (Z**2/A - 35.D0) + 28.D0,0.D0))*N/A
          wNasymm2_scission = DMAX1( 12.8D0 - 1.D0 *
     &          (92.D0 - Nheavy2_eff),1.D0) * N/A

        else
           wNasymm1_scission = wNasymm1_saddle
           wNasymm2_scission = wNasymm2_saddle
        endIF
      endIF

      wNasymm1_scission = DMAX1( wNasymm1_scission, wNasymm1_saddle )
      wNasymm2_scission = DMAX1( wNasymm2_scission, wNasymm2_saddle )

      wNasymm1 = wNasymm1_scission * Fwidth_asymm1
      wNasymm2 = wNasymm2_scission * Fwidth_asymm2
      wNsymm   = wNsymm_scission * Fwidth_symm

      wAsymm   = wNsymm * A/N
      wAheavy1 = wNasymm1 * A/N
      wAheavy2 = wNasymm2 * A/N

      wAsymm_saddle   = wNsymm_saddle * A/N
      wAheavy1_saddle = wNasymm1_saddle * A/N
      wAheavy2_saddle = wNasymm2_saddle * A/N

c     mass and charge of fragments using UCD, needed for level densities */
      Alight1_eff = Nlight1_eff * A/N
      Alight2_eff = Nlight2_eff * A/N
      Aheavy1_eff = Nheavy1_eff * A/N
      Aheavy2_eff = Nheavy2_eff * A/N

      A_levdens_light1 = Alight1_eff / xLevdens
      A_levdens_light2 = Alight2_eff / xLevdens
      A_levdens_heavy1 = Aheavy1_eff / xLevdens
      A_levdens_heavy2 = Aheavy2_eff / xLevdens
      gamma_heavy1 = A_levdens_heavy1 / (0.4D0 * Aheavy1_eff**1.3333D0)
     &               * FGAMMA * FGAMMA1
      gamma_heavy2 = A_levdens_heavy2 / (0.4D0 * Aheavy2_eff**1.3333D0)
     &               * FGAMMA

c      WRITE (*,*) ' '
c      Write (*,*) 'gamma_heavy1',gamma_heavy1,
c     &  '  gamma_heavy2',gamma_heavy2
c      Write (*,*) 'Aheavy1_eff',Aheavy1_eff
c      Write (*,*) 'Aheavy2_eff',Aheavy2_eff
c      WRITE (*,*) ' '

      if( epsilon_symm_saddle .LT. A_levdens * hbom3**2) then
        Ssymm = 2.D0 * DSQRT(A_levdens**2 * hbom3**2) +
     &        (epsilon_symm_saddle - A_levdens * hbom3**2)/hbom3
      else
        Ssymm = 2.D0 * DSQRT( A_levdens*epsilon_symm_saddle )
      endif
      Ysymm = 1.D0

      if( epsilon0_1_saddle .LT. A_levdens * hbom1**2 ) then
        Ssymm_mode1 = 2.D0 * DSQRT(A_levdens**2 * hbom1**2) +
     &        (epsilon0_1_saddle - A_levdens * hbom1**2)/hbom1
      else
        Ssymm_mode1 = 2.D0 * DSQRT( A_levdens*epsilon0_1_saddle )
      endif

      if( epsilon0_2_saddle .LT. A_Levdens * hbom2**2 ) then
        Ssymm_mode2 = 2.D0 * DSQRT(A_levdens**2 * hbom2**2) +
     &        (epsilon0_2_saddle - A_levdens * hbom2**2)/hbom2
      else
        Ssymm_mode2 = 2.D0 * DSQRT( A_levdens*epsilon0_2_saddle )
      endif

      if( epsilon0_1_saddle -
     &    Delta_U1*
     &  Uwash(epsilon_1_saddle/A*Aheavy1,Ecrit,FREDSHELL,gamma_heavy1)
     &      .LT. A_levdens * hbom1**2 ) then
        Sasymm1 = 2.D0 * DSQRT( A_levdens**2 * hbom1**2 ) +
     &    (epsilon0_1_saddle - Delta_U1 *
     &  Uwash(epsilon_1_saddle/A*Aheavy1,Ecrit,FREDSHELL,gamma_heavy1)
     &    - A_levdens * hbom1**2)/hbom1
      else
        Sasymm1 = 2.D0 *
     &        DSQRT( A_levdens*(epsilon0_1_saddle - Delta_U1 *
     &  Uwash(epsilon_1_saddle/A*Aheavy1,Ecrit,FREDSHELL,gamma_heavy1)))
      endif

      if( epsilon0_2_saddle -
     &   Delta_U2*
     &   Uwash(epsilon_2_saddle/A*Aheavy2,Ecrit,FREDSHELL,gamma_heavy2)
     &   .LT. A_levdens * hbom2**2 ) then
        Sasymm2 = 2.D0 * DSQRT( A_levdens**2 * hbom2**2 ) +
     &    (epsilon0_1_saddle-Delta_U1 *
     &    Uwash(epsilon_2_saddle/A*Aheavy2,Ecrit,FREDSHELL,gamma_heavy2)
     &    - A_levdens * hbom2**2)/hbom2
      else
        Sasymm2 = 2.D0 *
     &        DSQRT( A_levdens*(epsilon0_2_saddle - Delta_U2 *
     &  Uwash(epsilon_2_saddle/A*Aheavy2,Ecrit,FREDSHELL,gamma_heavy2)))

      endif

      Yasymm1 = ( DEXP(Sasymm1 - Ssymm) - DEXP(Ssymm_mode1 - Ssymm) ) *
     &            wNasymm1_saddle / wNsymm_saddle * 2.D0

      Yasymm2 = ( DEXP(Sasymm2 - Ssymm) - DEXP(Ssymm_mode2 - Ssymm) ) *
     &            wNasymm2_saddle / wNsymm_saddle * 2.D0


      Ysum = Ysymm + Yasymm1 + Yasymm2  ! /* normalize */

      if( Ysum .GT. 0.0D0 ) then
        Ysymm = Ysymm / Ysum
        Yasymm1 = Yasymm1 / Ysum
        Yasymm2 = Yasymm2 / Ysum
        Yasymm = Yasymm1 + Yasymm2
      else
        Ysymm = 0.D0
        Yasymm1 = 0.D0
        Yasymm2 = 0.D0
c       search minimum threshold and attribute all events to this mode */
        if( (epsilon_symm_saddle .LT. epsilon_1_saddle) .AND.
     &       (epsilon_symm_saddle .LT. epsilon_2_saddle) ) then
          Ysymm = 1.D0
        else
          if( epsilon_1_saddle .LT. epsilon_2_saddle ) then
            Yasymm1 = 1.D0
          else
            Yasymm2 = 1.D0
          endif
        endif
      endif

c    /* even-odd effect */
c    /* Parametrization from Rejmund et al. */
      IF (MOD(Z,2.D0) .EQ. 0.D0) THEN
        R_E_O = 10.D0 ** (-0.017D0 * (E_saddle_scission + Eld)**2)
      ELSE
           R_E_O = 0.D0
      endif


c     -------------------------------------------------------
c     selecting the fission mode using the yields at scission
c     -------------------------------------------------------
c     random decision: symmetric or asymmetric
c     IMODE = 1 means asymmetric fission, mode 1
c     IMODE = 2 means asymmetric fission, mode 2
c     IMODE = 3 means symmetric fission
c     testcase: 238U, E*= 6 MeV :    6467   8781   4752   (20000)
c                                  127798 176480  95722  (400000)
c                                  319919 440322 239759 (1000000)
c                     E*=12 MeV :  153407 293063 553530 (1000000) */

 321   rmode = DBLE(HAZ(k))
      if( rmode .LT. Yasymm1 ) then
         imode = 1
      else
         if( (rmode .GT. Yasymm1) .AND.
     &         (rmode .LT. Yasymm1+Yasymm2) ) then
            imode = 2
         else
            imode = 3
         endif
      endif

c     determine parameters of the neutron distribution of each mode
c     at scission  */

      if( imode .EQ. 1) then
         N1mean = Nheavy1_eff
         N1width = wNasymm1
      endif
      if( imode .EQ. 2 ) then
         N1mean = Nheavy2_eff
         N1width = wNasymm2
      endif
      if( imode .EQ. 3 ) then
         N1mean = Nsymm
         N1width = wNsymm
      endif

c     N2mean needed by CZ below */
      N2mean = N - N1mean

c     fission mode found, then the determination of the
c     neutron numbers N1 and N2 at scission by randon decision */
      N1r = 1.D0
      N2r = 1.D0
      Do while( N1r .LT. 5.D0 .OR. N2r .LT. 5.D0 )
         N1r = DBLE(GaussHaz(k,sngl(N1mean), sngl(N1width) ))
         N2r = N - N1r
      enddo



c     --------------------------------------------------
c     first approximation of fission fragments using UCD at saddle
c     -------------------------------------------------- */
      Z1UCD = Z/N * N1r
      Z2UCD = Z/N * N2r
      A1r = A/N * N1r
      A2r = A/N * N2r


c     --------------------------
c     deformations: starting ...
c     --------------------------  */
      if( imode .EQ. 1 ) then
c ---   N = 82  */
        E_scission_pre = DMAX1( epsilon_1_scission, 1.D0 )
c   ! Eexc at scission, neutron evaporation from saddle to scission not considered */
        if( N1mean .GT. N*0.5D0 ) then
          beta1  = 0.D0                !  /*   1. fragment is spherical */
          beta2  = 0.55D0                !  /*   2. fragment is deformed  0.5*/
        else
          beta1  = 0.55D0                !   /*  1. fragment is deformed 0.5*/
          beta2  = 0.0D0                !   /*  2. fragment is spherical */
        endif
      endif
      if( imode .EQ. 2 ) then
c ---   N appr. 86  */
        E_scission_pre = DMAX1( epsilon_2_scission, 1.D0 )
        if( N1mean .GT. N*0.5D0 ) then
          beta1  = (N1r - 92.D0) * 0.03D0 + 0.6D0

          beta1gs = BETA21(IDNINT(N1r),IDNINT(Z1UCD))
          beta2gs = BETA21(IDNINT(N2r),IDNINT(Z2UCD))

          beta1 = DMAX1(beta1,beta1gs)
          beta2  = 1.D0 - beta1
          beta2 = DMAX1(beta2,beta2gs)
        else

          beta1gs = BETA21(IDNINT(N1r),IDNINT(Z1UCD))
          beta2gs = BETA21(IDNINT(N2r),IDNINT(Z2UCD))

          beta2  = (N2r -92.D0) * 0.03D0 + 0.6D0
          beta2 = DMAX1(beta2,beta2gs)
          beta1 = 1.D0 - beta2
          beta1 = DMAX1(beta1,beta1gs)
        endif
      endif
      beta = 0.D0
      if( imode .EQ. 3 ) then
c ---   Symmetric fission channel
c       the fit function for beta is the deformation for optimum energy
c       at the scission point, d = 2
c       beta  : deformation of symmetric fragments
c       beta1 : deformation of first fragment
c       beta2 : deformation of second fragment  */
        betags = BETA21(IDNINT(Nsymm),IDNINT(Zsymm))
        beta1gs = BETA21(IDNINT(N1r),IDNINT(Z1UCD))
        beta2gs = BETA21(IDNINT(N2r),IDNINT(Z2UCD))
        beta  = DMAX1(0.177963D0+0.0153241D0*Zsymm-1.62037D-4*Zsymm**2,
     &                betags)
        beta1 = DMAX1(0.177963D0+0.0153241D0*Z1UCD-1.62037D-4*Z1UCD**2,
     &                beta1gs)
        beta2 = DMAX1(0.177963D0+0.0153241D0*Z2UCD-1.62037D-4*Z2UCD**2,
     &                beta2gs)

        E_asym = frldm( Z1UCD, N1r, beta1 ) +
     &         frldm( Z2UCD, N2r, beta2 ) +
     &         ecoul( Z1UCD, N1r, beta1, Z2UCD, N2r, beta2, 2.D0 ) -
     &         2.D0 * frldm( Zsymm, Nsymm, beta ) -
     &           ecoul( Zsymm, Nsymm, beta, Zsymm, Nsymm, beta, 2.D0 )
        E_scission_pre = DMAX1( epsilon_symm_scission - E_asym, 1.D0 )
      endif
c     -----------------------
c     ... end of deformations
c     -----------------------


c     ------------------------------------------
c     evaporation from saddle to scission ...
c     ------------------------------------------        */


        call evap_postsaddle(A,Z,E_scission_pre,INUM,E_scission_post,
     &     A_scission,Z_scission,VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC,
     &     NN_POST,NP_POST,ND_POST,NT_POST,
     &     NHE_POST,NA_POST,N6HE_POST)


c      A_scission = A
c      Z_scission = Z
c      E_scission_post = E_scission_pre

      N_scission = A_scission - Z_scission

c     ---------------------------------------------------
c     second approximation of fission fragments using UCD
c     --------------------------------------------------- */
c      A1r = A1r * A_scission / A
c      A2r = A2r * A_scission / A
      N1r = N1r * N_scission / N
      N2r = N2r * N_scission / N
      Z1UCD = Z1UCD * Z_scission / Z
      Z2UCD = Z2UCD * Z_scission / Z
      A1r = Z1UCD + N1r
      A2r = Z2UCD + N2r

c     ---------------------------------------------------------
c     determination of the charge and mass of the fragments ...
c     ---------------------------------------------------------

c     - CZ is the curvature of charge distribution for fixed mass,
c       common to all modes, gives the width of the charge distribution.
c       The physics picture behind is that the division of the
c       fissioning nucleus in N and Z is slow when mass transport from
c       one nascent fragment to the other is concerned but fast when the
c       N/Z degree of freedom is concernded. In addition, the potential
c       minima in direction of mass transport are broad compared to the
c       potential minimum in N/Z direction.
c          The minima in direction of mass transport are calculated
c          by the liquid-drop (LD) potential (for superlong mode),
c          by LD + N=82 shell (for standard 1 mode) and
c          by LD + N=86 shell (for standard 2 mode).
c          Since the variation of N/Z is fast, it can quickly adjust to
c          the potential and is thus determined close to scission.
c          Thus, we calculate the mean N/Z and its width for fixed mass
c          at scission.
c          For the SL mode, the mean N/Z is calculated by the
c          minimum of the potential at scission as a function of N/Z for
c          fixed mass.
c          For the S1 and S2 modes, this correlation is imposed by the
c          empirical charge polarisation.
c          For the SL mode, the fluctuation in this width is calculated
c          from the curvature of the potential at scission as a function
c          of N/Z. This value is also used for the widths of S1 and S2.


c     Polarisation assumed for standard I and standard II:
c      Z - Zucd = cpol (for A = const);
c      from this we get (see remarks above)
c      Z - Zucd =  Acn/Ncn * cpol (for N = const)   */

      CZ = ( frldm( Z1UCD-1.D0, N1r+1.D0, beta1 ) +
     &        frldm( Z2UCD+1.D0, N2r-1.D0, beta2 ) +
     &        frldm( Z1UCD+1.D0, N1r-1.D0, beta1 ) +
     &        frldm( Z2UCD-1.D0, N2r+1.D0, beta2 ) +
     &        ecoul( Z1UCD-1.D0, N1r+1.D0, beta1,
     &               Z2UCD+1.D0, N2r-1.D0, beta2, 2.D0) +
     &        ecoul( Z1UCD+1.D0, N1r-1.D0, beta1,
     &               Z2UCD-1.D0, N2r+1.D0, beta2, 2.D0) -
     &    2.D0*ecoul( Z1UCD, N1r, beta1, Z2UCD, N2r, beta2, 2.D0) -
     &    2.D0*frldm( Z1UCD, N1r, beta1 ) -
     &    2.D0*frldm( Z2UCD, N2r, beta2) ) * 0.5D0

      IF (1.D0/A_levdens*E_scission_post .LT. 0.D0) THEN
        write (*,*) 'DSQRT 1 < 0',A_LEVDENS,E_SCISSION_post
      ENDif
      IF (0.5D0 * DSQRT(1.0/A_levdens*E_scission_post) / CZ .LT. 0.D0)
     &     THEN
        Write (*,*) 'DSQRT 2 < 0',CZ
        Write (*,*) imode,Z1UCD,N1r,beta1,Z2UCD,n2r,beta2,Z,A,N
        Write (*,*) 'This event was not considered'
        GOTO 321
      ENDIF

      ZA1width = DSQRT( 0.5D0 * DSQRT(1.D0/A_levdens*E_scission_post)
     +                  / CZ )
c  /*  ZA1width = DSQRT( DMAX1(ZA1width**2 - (1.0/12.0), 0.1) ) ; ! Shephard */

c     Minimum width in N/Z imposed. */
c     Value of minimum width taken from 235U(nth,f) data */
c     sigma_Z(A=const) = 0.4 to 0.5  (from Lang paper Nucl Phys. A345 (1980) 34) */
c     sigma_N(Z=const) = 0.45 * A/Z  (= 1.16 for 238U)
c      therefore: SIGZMIN = 1.16                                              */
c /*   Physics; variation in N/Z for fixed A assumed.
c      Thermal energy at scission is reduced by
c         pre-scission neutron evaporation" */

       ZA1width = DMAX1(ZA1width,sigZmin)


      If ( imode .EQ. 1 .AND. CPOL1 .NE. 0.D0 ) then
c       --- asymmetric fission, mode 1 */
        IS = 0
 2801    Z1rr = Z1UCD - cpol1 * A_scission/N_scission
        Z1r = DBLE(GaussHaz(k,sngl(Z1rr), sngl(ZA1width) ))
        IS = IS +1
        IF(IS.GT.100)THEN
         PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING Z1R IN PROFI.FOR. A VALUE WILL BE FORCED'
         Z1R = Z1RR
        ENDIF
        if (Z1r .LT. 1.D0)  goto 2801
        N1r = A1r - Z1r
      ELSE IF ( imode .EQ. 2 .AND. CPOL2 .NE. 0.D0 ) then
c       --- asymmetric fission, mode 2 */
        IS = 0
 2802    Z1rr = Z1UCD - cpol2 * A_scission/N_scission
        Z1r = DBLE(GaussHaz(k,sngl(Z1rr), sngl(ZA1width) ))
        IS = IS +1
        IF(IS.GT.100)THEN
         PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING Z1R IN PROFI.FOR. A VALUE WILL BE FORCED'
         Z1R = Z1RR
        ENDIF
        if( Z1r .LT. 1.D0 ) goto 2802
        N1r = A1r - Z1r
      ELSE
c      Otherwise do; /* Imode = 3 in any case; imode = 1 and 2 for CPOL = 0 */
c       and symmetric case     */
c         We treat a simultaneous split in Z and N to determine
c         polarisation  */

          re1 = frldm( Z1UCD-1.D0, N1r+1.D0, beta1 ) +
     &          frldm( Z2UCD+1.D0, N2r-1.D0, beta2 ) +
     &          ecoul( Z1UCD-1.D0, N1r+1.D0, beta1,
     &                 Z2UCD+1.D0, N2r-1.D0, beta2, d ) ! /* d = 2 fm */
          re2 = frldm( Z1UCD, N1r, beta1) +
     &          frldm( Z2UCD, N2r, beta2 ) +
     &          ecoul( Z1UCD, N1r, beta1,
     &                 Z2UCD, N2r, beta2, d ) ! /*  d = 2 fm */
          re3 = frldm( Z1UCD+1.D0, N1r-1.D0, beta1 ) +
     &          frldm( Z2UCD-1.D0, N2r+1.D0, beta2 ) +
     &          ecoul( Z1UCD+1.D0, N1r-1.D0, beta1,
     &                 Z2UCD-1.D0, N2r+1.D0, beta2, d ) ! /* d = 2 fm */
          eps2 = ( re1 - 2.D0*re2 + re3 ) / 2.D0
          eps1 = ( re3 - re1 ) / 2.D0
          DN1_POL = -eps1 / ( 2.D0 * eps2 )

          Z1rr = Z1UCD + DN1_POL

c       /* Polarization of Standard 1 from shell effects around 132Sn */
          if ( imode .EQ. 1 ) then
            if ( Z1rr .GT. 50.D0 ) then
              DN1_POL = DN1_POL - 0.6D0 *
     &          Uwash(E_scission_post,Ecrit,FREDSHELL,gamma)
              Z1rr = Z1UCD + DN1_POL
              if ( Z1rr .LT. 50.D0 ) Z1rr = 50.D0
            else
              DN1_POL = DN1_POL + 0.6D0 *
     &          Uwash(E_scission_post,Ecrit,FREDSHELL,gamma)
              Z1rr = Z1UCD + DN1_POL
              if ( Z1rr .GT. 50.D0 ) Z1rr = 50.D0
            endif
          endif

           IS = 0
 2803      Z1r = DBLE(GaussHaz(k, sngl(Z1rr), sngl(ZA1width) ))
        IS = IS +1
        IF(IS.GT.100)THEN
         PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING Z1R IN PROFI.FOR. A VALUE WILL BE FORCED'
         Z1R = Z1RR
        ENDIF
          if( (DABS(Z1rr - Z1r) .GT. 3.D0*ZA1width)
     &           .OR. (Z1r .LT. 1.D0) ) then
            goto 2803
          endif

             N1r = A1r - Z1r

      endif

c
c     ------------------------------------------
c     Integer proton number with even-odd effect
c     ------------------------------------------  */
      CALL EVEN_ODD( Z1r, R_E_O, I_HELP )


      Z1 = DFLOAT(I_HELP)
      Z2 = DNINT( Z_SCISSION ) - Z1
      N1 = DNINT( N1r )
      N2 = DNINT( N_SCISSION ) - N1
      A1 = Z1 + N1
c      A2 = A - A1 */
      A2 = Z2 + N2

      if( (Z1 .LT. 0) .or. (Z2 .lt. 0) .or. (A1 .lt. 0) .or.
     &    (A2 .lt. 0) ) then

         write(*,*) ' -------------------------------'
         write(*,*) ' Z, A, N : ', Z, A, N
         write(*,*) ' imode, Z1UCD, N1r, Z2UCD, N2r : ',
     &               imode, Z1UCD, N1r, Z2UCD, N2r
        write(*,*) ' ZA1w, Z1r, Z1, A1, Z2, A2 : ',
     &               ZA1width, Z1r, Z1, A1, Z2, A2
        write(*,*) ' E_scission = ', E_scission_post,
     &             ' A_levdens  = ', A_levdens,
     &             ' CZ = ', CZ

      endif


c
c     -----------------------
c     excitation energies ...
c     -----------------------
c
      if( imode .EQ. 1 ) then
c ----  N = 82
        if( N1mean .GT. N*0.5D0 ) then
c         (a) 1. fragment is spherical and  2. fragment is deformed */
          E_defo = 0.D0
          beta2gs = BETA21(IDNINT(N2),IDNINT(Z2))
          IF(beta2.LT. beta2gs) beta2 = beta2gs
          E1exc = E_scission_pre * A1 / A + E_defo
          E_defo = frldm( Z2, N2, beta2 ) - frldm( Z2, N2, beta2gs )
          E2exc = E_scission_pre * A2 / A + E_defo
        else
c         (b) 1. fragment is deformed and  2. fragment is spherical */
          beta1gs = BETA21(IDNINT(N1),IDNINT(Z1))
          IF(beta1.LT. beta1gs) beta1 = beta1gs
          E_defo = frldm( Z1, N1, beta1 ) - frldm( Z1, N1, beta1gs )
          E1exc = E_scission_pre * A1 / A + E_defo
          E_defo = 0.D0
          E2exc = E_scission_pre * A2 / A + E_defo
        endif
      endif

      if( imode .EQ. 2 ) then
c ---   N appr. 86 */
        if( N1mean .GT. N*0.5D0 ) then      ! /*  2. fragment is spherical */
          beta1gs = BETA21(IDNINT(N1),IDNINT(Z1))
          IF(beta1.LT. beta1gs) beta1 = beta1gs
          E_defo = frldm( Z1, N1, beta1 ) - frldm( Z1, N1, beta1gs )
          E1exc = E_scission_pre * A1 / A + E_defo
          beta2gs = BETA21(IDNINT(N2),IDNINT(Z2))
          IF(beta2.LT. beta2gs) beta2 = beta2gs
          E_defo = frldm( Z2, N2, beta2 ) - frldm( Z2, N2, beta2gs )
          E2exc = E_scission_pre * A2 / A + E_defo
        else                            ! /*  1. fragment is spherical */
          beta2gs = BETA21(IDNINT(N2),IDNINT(Z2))
          IF(beta2.LT. beta2gs) beta2 = beta2gs
          E_defo = frldm( Z2, N2, beta2 ) - frldm( Z2, N2, beta2gs )
          E2exc = E_scission_pre * A2 / A + E_defo
          beta1gs = BETA21(IDNINT(N1),IDNINT(Z1))
          IF(beta1.LT. beta1gs) beta1 = beta1gs
          E_defo = frldm( Z1, N1, beta1 ) - frldm( Z1, N1, beta1gs )
          E1exc = E_scission_pre * A1 / A + E_defo
        endif
      endif

      if( imode .EQ. 3 ) then
c ---   Symmetric fission channel
          beta1gs = BETA21(IDNINT(N1),IDNINT(Z1))
          IF(beta1.LT. beta1gs) beta1 = beta1gs
          beta2gs = BETA21(IDNINT(N2),IDNINT(Z2))
          IF(beta2.LT. beta2gs) beta2 = beta2gs
        E_defo1 = frldm( Z1, N1, beta1 ) - frldm( Z1, N1, beta1gs )
        E_defo2 = frldm( Z2, N2, beta2 ) - frldm( Z2, N2, beta2gs )
        E1exc = E_scission_pre * A1 / A + E_defo1
        E2exc = E_scission_pre * A2 / A + E_defo2
      endif
c      write(*,*) 'imode:  E1exc, E2exc : ', imode, E1exc, E2exc */

c     pre-neutron-emission total kinetic energy */
      TKER = ( Z1 * Z2 * 1.44D0 ) /
     &       ( R0 * A1**0.33333D0 * (1.D0 + 2.D0/3.D0 * beta1 ) +
     &         R0 * A2**0.33333D0 * (1.D0 + 2.D0/3.D0 * beta2 ) + 2.D0 )
c     Pre-neutron-emission kinetic energies of the fragments */
      EkinR1 = TKER * A2 / A
      EkinR2 = TKER * A1 / A
      V1 = DSQRT(EkinR1/A1) * 1.3887D0
      V2 = DSQRT(EkinR2/A2) * 1.3887D0


c     Extracted from Lang et al. Nucl. Phys. A 345 (1980) 34 */
      E1exc_sigma = 5.5D0
      E2exc_sigma = 5.5D0

c     /* ! Enew: */
       IS = 0
 987   E1final = DBLE(GaussHaz(k,sngl(E1exc), sngl(E1exc_sigma) ))
        IS = IS +1
        IF(IS.GT.100)THEN
         PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING E1final IN PROFI.FOR. A VALUE WILL BE FORCED'
         E1final = E1exc
        ENDIF
      if( E1final .LT. 0.D0) goto 987

          IS = 0
 988      E2final = DBLE(GaussHaz(k,sngl(E2exc), sngl(E2exc_sigma) ))
        IS = IS +1
        IF(IS.GT.100)THEN
         PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING E2final IN PROFI.FOR. A VALUE WILL BE FORCED'
         E2final = E2exc
        ENDIF
      if( E2final .LT. 0.D0 ) goto 988
c      write(*,*) ' E1final, E2final : ', E1final, E2final */

      E1 = E1final
      E2 = E2final

       rtest = haz(k)
       IF(rtest.LE.0.5E0)THEN
        ETEMP = E2
        VTEMP = V2
        ATEMP = A2
        ZTEMP = Z2
        E2 = E1
        V2 = V1
        A2 = A1
        Z2 = Z1
        E1 = ETEMP
        V1 = VTEMP
        Z1 = ZTEMP
        A1 = ATEMP
       ENDIF

 4000    ifiss_mode = imode


        return
      END

C     /*=========================================================================*/


C   EVEN_ODD: PROCEDURE(R_ORIGIN,R_EVEN_ODD,I_OUT);
      SUBROUTINE EVEN_ODD(R_ORIGIN,R_EVEN_ODD,I_OUT)

c     /* Procedure to calculate I_OUT from R_IN in a way that         */
c     /* on the average a flat distribution in R_IN results in a      */
c     /* fluctuating distribution in I_OUT with an even-odd effect as */
c     /* given by R_EVEN_ODD                                          */
c
c     /* ------------------------------------------------------------ */
c     /* EXAMPLES :                                                   */
c     /* ------------------------------------------------------------ */
c     /*    If R_EVEN_ODD = 0 :                                       */
c     /*           CEIL(R_IN)  ----                                   */
c     /*                                                              */
c     /*              R_IN ->                                         */
c     /*            (somewhere in between CEIL(R_IN) and FLOOR(R_IN)) */
c     /*                                                              */
c     /*           FLOOR(R_IN) ----       --> I_OUT                   */
c     /* ------------------------------------------------------------ */
c     /*    If R_EVEN_ODD > 0 :                                       */
c     /*      The interval for the above treatment is                 */
c     /*         larger for FLOOR(R_IN) = even and                    */
c     /*         smaller for FLOOR(R_IN) = odd                        */
c     /*    For R_EVEN_ODD < 0 : just opposite treatment              */
c     /* ------------------------------------------------------------ */
c
c     /* ------------------------------------------------------------ */
c     /* On input:   R_ORIGIN    nuclear charge (real number)         */
c     /*             R_EVEN_ODD  requested even-odd effect            */
c     /* Intermediate quantity: R_IN = R_ORIGIN + 0.5                 */
c     /* On output:  I_OUT       nuclear charge (integer)             */
c     /* ------------------------------------------------------------ */

      REAL*8 R_ORIGIN,R_IN,R_EVEN_ODD,R_REST,R_HELP,R_FLOOR,R_MIDDLE
      INTEGER*4 I_OUT,N_FLOOR

      R_IN = R_ORIGIN + 0.5D0
      R_FLOOR = dfloat(IDINT(R_IN))
      IF (R_EVEN_ODD.lt.1.D-3) THEN
       I_OUT = IDINT(R_FLOOR)
      ELSE
       R_REST = R_IN - R_FLOOR
       R_MIDDLE = R_FLOOR + 0.5D0
       N_FLOOR = IDINT(R_FLOOR)
       IF (MOD(N_FLOOR,2).eq.0) THEN
c! /* even before modif. */
         R_HELP = R_MIDDLE + (R_REST - 0.5D0) * (1.D0 - R_EVEN_ODD)
       ELSE
c! /* odd before modification */
         R_HELP = R_MIDDLE + (R_REST - 0.5D0) * (1.D0 + R_EVEN_ODD)
       ENDIF
       I_OUT = IDINT(R_HELP)
      ENDIF
      RETURN
      END

c    /*=========================================================================*/

C    =========================================================================

      Function UMASS(Z,N,beta)

C     /* liquid-drop mass, Myers & Swiatecki, Lysekil, 1967  */
C     /* pure liquid drop, without pairing and shell effects */
C
C    /* On input:    Z     nuclear charge of nucleus        */
C     /*              N     number of neutrons in nucleus    */
C     /*              beta  deformation of nucleus           */
C     /* On output:   binding energy of nucleus              */

      real*8 Z,N,A,beta,umass
      real*8 alpha
      real*8 XCOM,XVS,XE
      real*8 pi
        Parameter (pi = 3.1416D0)

      A = N + Z
      alpha = dsqrt(5.D0/(4.D0*pi)) * beta
      XCOM = 1.D0 - 1.7826D0 * ((A - 2.D0*Z)/A)**2
C            /* factor for asymmetry dependence of surface and volume term */
      XVS = - XCOM * (15.4941D0*A
     &              - 17.9439D0*A**(2.D0/3.D0)*(1.D0+0.4D0*alpha**2))
C            /* sum of volume and surface energy */
      XE = Z**2 * (0.7053D0/A**(1.D0/3.D0)*(1.D0-0.2D0*alpha**2)
     &             - 1.1529D0/A)
      umass = XVS + XE
      RETURN
      END

c    /*=========================================================================*/

c     ECOUL: Procedure(Z1,N1,beta1,Z2,N2,beta2,d) RETURNS(DEC FLOAT(6));
      FUNCTION ECOUL(Z1,N1,beta1,Z2,N2,beta2,d)

c      /* Coulomb potential between two nuclei                    */
c      /* surfaces are in a distance of d                         */
c      /* in a tip to tip configuration                           */
c
c      /* approximate formulation                                 */
c      /* On input: Z1      nuclear charge of first nucleus       */
c      /*           N1      number of neutrons in first nucleus   */
c      /*           beta1   deformation of first nucleus          */
c      /*           Z2      nuclear charge of second nucleus      */
c      /*           N2      number of neutrons in second nucleus  */
c      /*           beta2   deformation of second nucleus         */
c      /*           d       distance of surfaces of the nuclei    */
c
      REAL*8 Z1,N1,beta1,Z2,N2,beta2,d,ecoul,dtot,E
      REAL r0
      PARAMETER (R0 = 1.16D0)

      dtot = r0 *( (Z1+N1)**0.3333333D0 * (1.D0+0.6666667D0*beta1)
     &        + (Z2+N2)**0.3333333D0 * (1.D0+0.6666667D0*beta2) )
     &        + d
      Ecoul = Z1 * Z2 * 1.44D0 / dtot

      RETURN
      END
C****************************************************************************
      Function UWASH(E,ECRIT,FREDUCTION,GAMMA)
        REAL*8 E ! /* excitation energy */
        REAL*8 Ecrit  !/* critical pairing energy */
        REAL*8 Freduction !/* reduction factor for shell washing in superfluid region */
        REAL*8 gamma
        REAL*8 R_wash,uwash
        If (E .LT. Ecrit) then
          R_wash = DEXP(-E * Freduction * gamma)
        Else
          R_wash = DEXP(- Ecrit * Freduction * gamma -(E-Ecrit) * gamma)
        Endif
        UWASH = R_WASH
      END

C***********************************************************************
      Function FRLDM(Z,N,beta)

C     Liquid-drop mass, Myers & Swiatecki, Lysekil, 1967
c     pure liquid drop, without pairing and shell effects
c
c     On input:    Z     nuclear charge of nucleus
c                  N     number of neutrons in nucleus
c                  beta  deformation of nucleus
c     On output:   binding energy of nucleus
c The idea is to use FRLDM model for beta=0 and using Lysekil
c model to get the deformation energy */

      REAL*8   Z, N, A, beta, frldm,EFLMAC_PROFI,umass
      A = N + Z
      frldm = EFLMAC_PROFI(A,Z) + umass(z,n,beta) - umass(z,n,0.D0)
      END


C**********************************************************************
C *
C * this function will calculate the liquid-drop nuclear mass for spheri
C * configuration according to the preprint NUCLEAR GROUND-STATE
C * MASSES and DEFORMATIONS by P. M"oller et al. from August 16, 1993 p.
C * All constants are taken from this publication for consistency.
C *
C * Parameters:
C *   a:    nuclear mass number
C *   z:    nuclear charge
C **********************************************************************/

      FUNCTION EFLMAC_PROFI(A,Z)

      REAL*8
     & Z,N,A,AV,AS,A0,C1,C4,B1,B3,F,CA,W,DP,DN,DPN,EFL,EFLMAC_PROFI,
     & RMAC,BS,H,R0,KF,KS,KV,RP,AY,ADEN,X0,Y0,MH,MN,ESQ,AEL,I

      REAL*8 pi
      PARAMETER (pi = 3.141593D0)

c /* fundamental constants */
c /* hydrogen-atom mass excess */
      MH  = 7.289034D0
c /* neutron mass excess       */
      MN  = 8.071431D0
c /* electronic charge squared */
      ESQ = 1.4399764D0

c /* constants from considerations other than nucl. masses */
c /* electronic binding        */
      AEL = 1.433D-5
c /* proton rms radius         */
      RP  = 0.8D0
c /* nuclear radius constant   */
      R0  = 1.16D0
c /* range of Yukawa-plus-expon. potential */
      AY  = 0.68D0
c /* range of Yukawa function used to generate
C     nuclear charge distribution */
      ADEN= 0.70D0
c /* constants from considering odd-even mass differences */
c /* average pairing gap       */
      RMAC= 4.80D0
c /* neutron-proton interaction*/
      H   = 6.6D0
c /* Wigner constant           */
      W   = 30.D0

c /* adjusted parameters */
c /* Volume energy              */
      AV  = 16.00126D0
c /* volume asymmetry           */
      KV  =  1.92240D0
c /* Surface energy             */
      AS  = 21.18466D0
c /* surface asymmetry          */
      KS  =  2.345D0
c /* A^0 constant               */
      A0  =  2.615D0
c /* charge asymmetry           */
      CA  =  0.10289D0

c /* we will account for deformation by using the microscopic
c   corrections tabulated from p. 68ff */
      BS=1.D0

      N   = A - Z
      DN  = RMAC*BS/N**(1.D0/3.D0)
      DP  = RMAC*BS/Z**(1.D0/3.D0)
      DPN = H/BS/A**(2.D0/3.D0)

      C1  = 3.D0/5.D0*ESQ/R0
      C4  = 5.D0/4.D0*(3.D0/(2.D0*PI))**(2.D0/3.D0) * C1


      KF  = (9.D0*PI*Z/(4.D0*A))**(1.D0/3.D0)/R0
      F   = -1.D0/8.D0*RP*RP*ESQ/R0**3 *
     &        (145.D0/48.D0 - 327.D0/2880.D0*KF**2 * RP**2 +
     &                    1527.D0/1209600.D0*KF**4 * RP**4)

      I   = (N-Z)/A

      X0  = R0 * A**(1.D0/3.D0) / AY
      Y0  = R0 * A**(1.D0/3.D0) / ADEN

      B1  = 1.D0 - 3.D0/(X0**2) +
     &      (1.D0 + X0) * (2.D0 + 3.D0/X0 + 3.D0/X0**2)
     &       * DEXP(-2.D0*X0)

      B3  = 1.D0 - 5.D0/Y0**2 * (1.D0 - 15.D0/(8.D0*Y0) +
     &        21.D0/(8.D0 * y0**3) -
     &      3.D0/4.D0 * (1.D0 + 9.D0/(2.D0*Y0) + 7.D0/Y0**2 +
     &      7.D0/(2.D0 * Y0**3)) * DEXP(-2.D0*Y0))

c/* NOW CALULATION OF TOTAL BINDING ENERGY A.J. 16.7.96*/


      EFL = - AV*(1.D0-KV*I*I)*A
     &       + AS*(1.D0-KS*I*I)*B1 * A**(2.D0/3.D0) + A0
     &       + C1*Z*Z*B3/A**(1.D0/3.D0)
     &       - C4*Z**(4.D0/3.D0)/A**(1.D0/3.D0)
     &       + F*Z**2/A -CA*(N-Z) - AEL * Z**(2.39D0)

        EFLMAC_PROFI = EFL + W * DABS(I)

      END

c  /*=========================================================================*/
