C********************************************************************
      SUBROUTINE ABRAINIT(path)
C
C     ON INPUT:  INPUT PARAMETERS FROM FILE
C---------------------------------------------------------------------
C     PROJECTILE AND TARGET PARAMETERS + CROSS SECTIONS
C     COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,
C                       R_0,R_P,R_T, IMAX,IRNDM,PI,
C                       BFPRO,SNPRO,SPPRO,SHELL
C
C     AP,ZP,AT,ZT   - PROJECTILE AND TARGET MASSES
C     EAP,BETA      - BEAM ENERGY PER NUCLEON, V/C
C     BMAXNUC       - MAX. IMPACT PARAMETER FOR NUCL. REAC.
C     CRTOT,CRNUC   - TOTAL AND NUCLEAR REACTION CROSS SECTION
C     R_0,R_P,R_T,  - RADIUS PARAMETER, PROJECTILE+ TARGET RADII
C     IMAX,IRNDM,PI - MAXIMUM NUMBER OF EVENTS, DUMMY, 3.141...
C     BFPRO         - FISSION BARRIER OF THE PROJECTILE
C     SNPRO         - NEUTRON SEPARATION ENERGY OF THE PROJECTILE
C     SPPRO         - PROTON    "           "   "    "   "
C     SHELL         - GROUND STATE SHELL CORRECTION
C---------------------------------------------------------------------
C
C     ENERGIES WIDTHS AND CROSS SECTIONS FOR EM EXCITATION
C     COMMON /EMDPAR/ EGDR,EGQR,FWHMGDR,FWHMGQR,CREMDE1,CREMDE2,
C                     AE1,BE1,CE1,AE2,BE2,CE2,SR1,SR2,XR
C
C     EGDR,EGQR       - MEAN ENERGY OF GDR AND GQR
C     FWHMGDR,FWHMGQR - FWHM OF GDR, GQR
C     CREMDE1,CREMDE2 - EM CROSS SECTION FOR E1 AND E2
C     AE1,BE1,CE1     - ARRAYS TO CALCULATE
C     AE2,BE2,CE2     - THE EXCITATION ENERGY AFTER E.M. EXC.
C     SR1,SR2,XR      - WITH MONTE CARLO
C---------------------------------------------------------------------
C
C     DEFORMATIONS AND G.S. SHELL EFFECTS
C     COMMON /ECLD/   ECGNZ,ECFNZ,VGSLD,ALPHA
C
C     ECGNZ - GROUND STATE SHELL CORR. FRLDM FOR A SPHERICAL G.S.
C     ECFNZ - SHELL CORRECTION FOR THE SADDLE POINT (NOW: == 0)
C     VGSLD - DIFFERENCE BETWEEN DEFORMED G.S. AND LDM VALUE
C     ALPHA - ALPHA GROUND STATE DEFORMATION (THIS IS NOT BETA2!)
C             BETA2 = SQRT(5/(4PI)) * ALPHA
C---------------------------------------------------------------------
C
C     ARRAYS FOR EXCITATION ENERGY BY STATISTICAL HOLE ENERY MODEL
C     COMMON /EENUC/  SHE, XHE
C
C     SHE, XHE - ARRAYS TO CALCULATE THE EXC. ENERGY AFTER
C                ABRASION BY THE STATISTICAL HOLE ENERGY MODEL
C---------------------------------------------------------------------
C
C     G.S. SHELL EFFECT
C     COMMON /EC2SUB/ ECNZ
C
C     ECNZ G.S. SHELL EFFECT FOR THE MASSES (IDENTICAL TO ECGNZ)
C---------------------------------------------------------------------
C
C     OPTIONS AND PARAMETERS FOR FISSION CHANNEL
C     COMMON /FISS/    AKAP,BET,HOMEGA,KOEFF,IFIS,
C                            OPTSHP,OPTXFIS,OPTCOL
C
C     AKAP   - HBAR**2/(2* MN * R_0**2) = 10 MEV
C     BETA_COEFF    - REDUCED NUCLEAR FRICTION COEFFICIENT IN (10**21 S**-1)
C     HOMEGA - CURVATURE OF THE FISSION BARRIER = 1 MEV
C     KOEFF  - COEFFICIENT FOR THE LD FISSION BARRIER == 1.0
C     IFIS   - 0/1 FISSION CHANNEL OFF/ON
C     OPTSHP - INTEGER SWITCH FOR SHELL CORRECTION IN MASSES/ENERGY
C            = 0 NO MICROSCOPIC CORRECTIONS IN MASSES AND ENERGY
C            = 1 SHELL ,  NO PAIRING
C            = 2 PAIRING, NO SHELL
C            = 3 SHELL AND PAIRING
C     OPTCOL - 0/1 COLLECTIVE ENHANCEMENT SWITCHED ON/OFF
C     OPTXFIS- 0,1,2 FOR MYERS & SWIATECKI, DAHLINGER, ANDREYEV
C              FISSILITY PARAMETER.
C     OPTCOL - 0/1 COLLECTIVE ENHANCEMENT OFF/ON
C---------------------------------------------------------------------
C
C     OPTIONS
C     COMMON /OPT/    OPTEMD,OPTCHA,EEFAC
C
C     OPTEMD - 0/1  NO EMD / INCL. EMD
C     OPTCHA - 0/1  0 GDR / 1 HYPERGEOMETRICAL PREFRAGMENT-CHARGE-DIST.
C              ***  RECOMMENDED IS OPTCHA = 1 ***
C     EEFAC  - EXCITATION ENERGY FACTOR, 2.0 RECOMMENDED
C---------------------------------------------------------------------
C
C     FISSION BARRIERS
C     COMMON /FB/     EFA
C     EFA    - ARRAY OF FISSION BARRIERS
C---------------------------------------------------------------------
C
C     LEVEL DENSITY PARAMETERS
C     COMMON /ALD/    AV,AS,AK,OPTAFAN
C     AV,AS,AK - VOLUME,SURFACE,CURVATURE DEPENDENCE OF THE
C                LEVEL DENSITY PARAMETER
C     OPTAFAN - 0/1  AF/AN >=1 OR AF/AN ==1
C               RECOMMENDED IS OPTAFAN = 0
C---------------------------------------------------------------------
C   ____________________________________________________________________
C  /
C  /  INITIALIZES PARAMETERS IN COMMON /ABRAMAIN/, /EMDPAR/, /ECLD/ ...
C  /  PROJECTILE PARAMETERS, EMD PARAMETERS, SHELL CORRECTION TABLES.
C  /  CALCULATES MAXIMUM IMPACT PARAMETER FOR NUCLEAR COLLISIONS AND
C  /  TOTAL GEOMETRICAL CROSS SECTION + EMD CROSS SECTIONS
C   ____________________________________________________________________
C
C
      IMPLICIT NONE
C
cdvd
c     Path to the tables
      character*1000 path,pathl
      integer long
cdvd
      INTEGER*4 IMAX,IRNDM,KINIT,MEXPIOP(0:153,0:12)
      INTEGER*4 NQUAD
            PARAMETER(NQUAD=1000)
      INTEGER*4 I,NNUC,NEE,L,I1,I2,ITAB,ISIGOPT
C      PARAMETER(NEE=2001,NNUC=50)
      INTEGER*4 Z,N,AP1,ZP1,AT1,ZT1,TSTAT,TSTATD
      INTEGER*4 OPTAFAN,OPTEXP,OPTSHPIMF
      INTEGER*4 OPTEMD,OPTCHA,IFIS,OPTSHP,J,K,OPTXFIS,IO,OPTCOL
      INTEGER*4 NZ,IZ,IN,IA
      INTEGER IREACT,IOPTCT
c      integer*4 INOW(1:2)

      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,R_P,R_T,PI,
     &       BFPRO,SNPRO,SPPRO,SHELL,T_freeze_out_in,EAP1
      REAL*8 massex,munc,binding,bunc,betafrdm,unc,atomic_mass,amunc
      REAL*8 EGDR,EGQR,FWHMGDR,FWHMGQR,CREMDE1,CREMDE2
      REAL*8 AE1(NQUAD+1),BE1(NQUAD+1),CE1(NQUAD+1),AE2(NQUAD+1),
     &       BE2(NQUAD+1),CE2(NQUAD+1),SRE1(NQUAD+1),SRE2(NQUAD+1),
     &       XRE1(NQUAD+1),XRE2(NQUAD+1),DS1,DS2
      REAL*8 DUQUAD,ELIM,EI,XR(NQUAD+1),SR1(NQUAD+1),SR2(NQUAD+1),
     &       U1,U2
      REAL*8 PR1(NQUAD+1),PR2(NQUAD+1)
      REAL*8 XEE(1:2001),EEC(50,1:2001),SEE(50,1:2001),YE
      REAL*8 SE1(1:2001), EPS, M
C XHE,SHE ARE COMMON TO BE USED IN REANUCL FOR THE LINEAR
C INTERPOLATION
      REAL*8 SHE(1:2001),XHE(50,1:2001)
      REAL*8 ECNZ(0:153,0:98),E0
      REAL*8 R13,DNDE1,DNDE2,BP,SPMIN,SIGE1,SIGE2
      REAL*8 Y,BS,BK,AF,AN,BIPOL,SPDEF,RDUMMY,BETA_COEFF
      REAL*8 AKAP,BET,HOMEGA,KOEFF,MAZ,MA1Z,MA1Z1,EEFAC
      REAL*8 ECGNZ(0:153,0:98),ECFNZ(0:153,0:98),
     &       VGSLD(0:153,0:98),ALPHA(0:153,0:98),BETA2(0:250,0:136),
     &       BETA4(0:250,0:136),AV,AK,AS,FISSILITY,DELTA_TOT
      REAL*8 ECGNZ1(0:153,0:98),ECFNZ1(0:153,0:98),
     &       VGSLD1(0:153,0:98),ECNZ1(0:153,0:98),
     &       BETA21(0:250,0:136),BETA41(0:250,0:136),
     &       RMS(0:153,0:98)
      REAL*8 BETA2P,BETA4P,PARA,PARZ,MN,MH,OMEGAP
      REAL*8 SEGS,SELMAX,SBFIS
      REAL*8 SEVAL,SIGMANUC,EFLMAC
      REAL*8 DP,DN,DPN
      REAL*4 HAZ
C
      CHARACTER cc,el,o,B
      CHARACTER*30 DUM(26)
      CHARACTER*50 NAME,FPATH,DUMPEDFILE
      CHARACTER*80 TUPLE,ECSERN,VGSTAB,DEFTAB,DUMPED
      CHARACTER*100 FINA1,FINA2

      COMMON /PROJ/ EAP1,AP1,ZP1
      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,
     &                 R_0,R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
      COMMON /EMDPAR/ EGDR,EGQR,FWHMGDR,FWHMGQR,CREMDE1,CREMDE2,
     &  AE1,BE1,CE1,AE2,BE2,CE2,SRE1,SRE2,XRE1,XRE2,DS1,DS2
      COMMON /EENUC/ SHE,XHE
      COMMON /EC2SUB/ ECNZ
      COMMON /EC2SUB1/ ECNZ1
      COMMON /FIS/ AKAP,BETA_COEFF,HOMEGA,KOEFF,IFIS,
     &        OPTSHP,OPTXFIS,OPTCOL
      COMMON /ECLD/ ECGNZ,ECFNZ,VGSLD,BETA2,BETA4
      COMMON /ECLD1/ ECGNZ1,ECFNZ1,VGSLD1,BETA21,BETA41
      COMMON /ALPHA2/ ALPHA
      COMMON /OPT/ OPTEMD,OPTCHA,EEFAC
      COMMON /ALD/AV,AS,AK,OPTAFAN
      COMMON /EXPANSION/ OPTEXP
      COMMON /BREAK_UP/T_freeze_out_in
      COMMON /CONSTTEMP/IOPTCT
C If IOPTCT=1 CT approach in DENSNIV is used, if = 0 only Fermi level-density
C description
C
C     *** Table of Bass barriers ***
      INTEGER*4 BASS_INDEX(4,95)  ! Must fit to the data in table BASSINDEX
c      REAL*4 BASS_TAB(205300)  ! Must fit to the data in table BASSTAB
      REAL*4 BASS_TAB(2505600)
      COMMON/BASS/ BASS_INDEX, BASS_TAB

C     *** Table of liquid-drop masses ***
      INTEGER*4 MASS_INDEX(4,95)  ! Must fit to the data in table MASSINDEX
      REAL*4 MASS_TAB(4105)  ! Must fit to the data in table MASSTAB
      COMMON/MASS/ MASS_INDEX, MASS_TAB

      REAL*8 MEXP(0:153,0:12),MFRLDM(0:153,0:98),BTEST(0:153,0:98),
     &       BIND(0:153,0:12)
      COMMON/EXPMASS/BIND,MEXPIOP
      COMMON/MFRLDM/MFRLDM
      COMMON/RMS/RMS
      COMMON/BTEST/BTEST
      COMMON/OPTIMF/OPTSHPIMF
      COMMON/ISIGOPT/ISIGOPT
      COMMON/MEXP/MEXP

      logical first
      data first /.true./
C IREACT = 0 -> full ABRABLA
C        = 1 -> own input + ABLA
      COMMON/IREACT/IREACT

      EAP=EAP1

cjcd
      PI = 3.141592653589793D0
      R13 = 1.D0/3.D0
C
C---------- SET INPUT VALUES
C
C *** INPUT FROM UNIT 10 IN THE FOLLOWING SEQUENCE !
C     AP1 =    INTEGER  !
C     ZP1 =    INTEGER  !
C     AT1 =    INTEGER  !
C     ZT1 =    INTEGER  !
C     EAP =    REAL     !
C     IMAX =   INTEGER  !
C     IFIS =   INTEGER SWITCH FOR FISSION
C     OPTSHP = INTEGER SWITCH FOR SHELL CORRECTION IN MASSES/ENERGY
C            =0 NO MICROSCOPIC CORRECTIONS IN MASSES AND ENERGY
C            =1 SHELL , NO PAIRING CORRECTION
C            =2 PAIRING, NO SHELL CORRECTION
C            =3 SHELL AND PAIRING CORRECTION IN MASSES AND ENERGY
C     OPTEMD =0,1  0 NO EMD, 1 INCL. EMD
C               ELECTROMAGNETIC DISSOZIATION IS CALCULATED AS WELL.
C     OPTCHA =0,1  0 GDR- , 1 HYPERGEOMETRICAL PREFRAGMENT-CHARGE-DIST.
C               RECOMMENDED IS OPTCHA=1
C     OPTCOL =0,1 COLLECTIVE ENHANCEMENT SWITCHED ON 1 OR OFF 0 IN DENSN
C     OPTAFAN=0,1 SWITCH FOR AF/AN = 1 IN DENSNIV 0 AF/AN>1 1 AF/AN=1
C     AKAP =  REAL    ALWAYS EQUALS 10
C     BET  =  REAL    REDUCED FRICTION COEFFICIENT / 10**(+21) S**(-1)
C     HOMEGA = REAL   CURVATURE / MEV RECOMMENDED = 1. MEV
C     KOEFF  = REAL   COEFFICIENT FOR FISSION BARRIER
C     OPTXFIS= INTEGER 0,1,2 FOR MYERS & SWIATECKI, DAHLINGER, ANDREYEV
C              FISSILITY PARAMETER.
C     EEFAC  = REAL EMPIRICAL FACTOR FOR THE EXCITATION ENERGY
C                   RECOMMENDED 2.D0, STATISTICAL ABRASION MODELL 1.D0
C     OPTBARR = SWITCH FOR CHARGED-PARTICLE EMISSION BARRIERS:
C               = 0 OLD PARAMETERISATION
c               = 1 BASS BARRIER (BASS, NPA 1974)
C               = 2 PROXIMITY POTENTIAL OF MYERS AND SWIATECKI (PRC 2000) (RECOMMENDED)
C     AV     = REAL KOEFFICIENTS FOR CALCULATION OF A(TILDE)
C     AS     = REAL LEVEL DENSITY PARAMETER
C     AK     = REAL
C

C  Last update:
C             28/10/13 - J.L.R.S. - Add file rms.tab from abrablav4 (AK)

cjcd - start (initialization of default parameters)
      IFIS            = 1
      OPTSHP          = 3
      OPTEMD          = 0
      OPTCHA          = 2
      BETA_COEFF      = 4.5d0
      OPTCOL          = 1
      OPTEXP          = 0
      KINIT           = -2
c      KINIT           = 1
      T_freeze_out_in = -6.5D0
      ISIGOPT         = 0
c
      zt1=zt
      at1=at
      OPTEMD = 0
c      IF(ZT1. GT. 1) OPTEMD = 1
      OPTCHA = 2
      IFIS = 1
      ISIGOPT = 0
C
      OPTXFIS = 1
      OPTAFAN = 0
      KOEFF = 1.D0
      OPTSHPIMF = 0
c   Collective enhancement taken into account only for cases where fission
C   plays role

      IF(ZP.LE.55.D0)THEN
        OPTCOL = 0
        OPTSHP = 3
      ELSE
       IF(OPTSHP.EQ.1 .OR. OPTSHP.EQ.3) OPTCOL = 1
      ENDIF


C Level-density parameters
      AV = 0.073D0
      AS = 0.095D0
      AK = 0.D0

C EXCITATION ENERGY FACTOR
      EEFAC = 2.D0

C CURVATURE OF THE FISSION BARRIER
      HOMEGA = 1.D0

C AKAP   - HBAR**2/(2* MN * R_0**2) = 10 MEV
      AKAP = 10.D0

      MN = 939.57287D0
      MH = 938.79044D0

      if(first) then

      pathl=path
      long=0
      i=0
      do while(long.eq.0)
      i = i+1
      if( pathl(i:i).eq.' ' ) long = i-1
      end do
      OPEN(UNIT = 31,STATUS='UNKNOWN',
     &             FILE = pathl(1:long) // 'massindex.tab')
      DO I = 1, 95
        READ(31,*,IOSTAT=IO) MASS_INDEX(1,I),MASS_INDEX(2,I),
     &                       MASS_INDEX(3,I),MASS_INDEX(4,I)
      ENDDO
      CLOSE(UNIT = 31)

        OPEN(UNIT = 32,STATUS='UNKNOWN',
     &             FILE=pathl(1:long) // 'masstab.tab')
         DO I = 1, 4105    ! Number of elements in MASSTAB.TAB
          READ(32,*,IOSTAT=IO) MASS_TAB(I)
         ENDDO
        CLOSE(UNIT = 32)

C----------- READ TABLE OF BASS BARRIERS

      OPEN(UNIT = 25,STATUS='UNKNOWN',
     &             FILE=pathl(1:long) // 'bassindex.tab')
      DO I = 1, 95
        READ(25,*,IOSTAT=IO) BASS_INDEX(1,I),BASS_INDEX(2,I),
     &                       BASS_INDEX(3,I),BASS_INDEX(4,I)
      ENDDO
      CLOSE(UNIT = 25)
C If IOPTCT=1 CT approach combined with Fermi model in DENSNIV is used,
C if = 0 only Fermi level-density description
      IOPTCT = 1

      OPEN(UNIT = 27,STATUS='UNKNOWN',
     &             FILE=pathl(1:long) // 'basstab.tab')
      DO I = 1, 2505600    ! Number of elements in BASSTAB.TAB
        READ(27,*,IOSTAT=IO) BASS_TAB(I)
      ENDDO
      CLOSE(UNIT = 27)

C----------- READ SHELL CORRECTION TABLES
C

      OPEN(UNIT = 88,STATUS='UNKNOWN',
     &     FILE=pathl(1:long) // 'rms.tab')

      OPEN(UNIT = 9,STATUS='UNKNOWN',
     &     FILE=pathl(1:long) // 'frldm.tab')
C
      OPEN(UNIT =11,STATUS='UNKNOWN',
     &     FILE=pathl(1:long) // 'vglsd.tab')
C
      OPEN(UNIT =13,STATUS='UNKNOWN',
     &     FILE=pathl(1:long) // 'defo.tab')
C
C----------- INITIALIZE RANDOM NUMBERS
C     CALL HAZ with -2 to initialize randomly
C     CALL HAZ with -1 to inititalze always with the same seed
#ifndef ABLA07_USE_USER_RNG
      RDUMMY = HAZ(KINIT)
#endif
C
C---------- INITIALIZE TABLES OF NUCLEAR SHELL EFFECTS.
C
        DO  Z = 0,98,1
        DO  N = 0,153,1
          READ(88,*)RMS(N,Z)
       ENDDO
      ENDDO
      CLOSE(UNIT = 88)

      DO 15  Z = 0,98,1
          READ(UNIT = 9,FMT=21)(ECNZ1(N,Z),N=0,153)
 21       FORMAT (7D11.3)
 15   CONTINUE
      CLOSE(UNIT = 9)
C
C For 2 < Z < 12 we take "experimental" shell corrections instead of calculated
C Read FRLDM tables
      DO 32  Z = 0,12,1
        DO 33  N = 0,153,1
         MEXPIOP(N,Z) = 0
C LD masses (even-odd effect is later considered according to Ignatyuk)
         if(n.eq.0.and.z.eq.0) then
          MFRLDM(N,Z) = 0.
         else
          MFRLDM(N,Z) = MH*DFLOAT(Z)+MN*DFLOAT(N)+EFLMAC(N+Z,Z,1,0)
         endif
         MEXP(N,Z) = MFRLDM(N,Z)
 33     CONTINUE
 32   CONTINUE

C----------- READ EXP MASSES FOR Z<12 ------------------------
C
       OPEN(UNIT=56,STATUS='UNKNOWN',
     &      FILE=pathl(1:long) // 'mass2003.tab')

 58    READ(56,111,END=57) cc,NZ,IN,IZ,IA,el,o,massex,munc,binding
       IF(IZ.GT.12) GOTO 58
       MEXP(IN,IZ) = 938.7829835D0*DFLOAT(IZ) + 939.5653301D0*DFLOAT(IN)
     &               - DFLOAT(IA) * BINDING / 1000.D0
       MEXPIOP(IN,IZ) = 1
       GOTO 58
111   format(a1,i3,i5,i5,i5,1x,a3,a4,1x,f13.5,f11.5,f11.3)
 57    CONTINUE
       CLOSE(UNIT = 56)

C
      DO Z = 0,98,1
       DO N = 0, 153, 1
         BTEST(N,Z) = ECNZ1(N,Z)
       ENDDO
      ENDDO

      DO 16  Z = 0,98,1
          READ(UNIT = 11,FMT=21)(VGSLD1(N,Z),N=0,153)
 16   CONTINUE
      CLOSE(UNIT = 11)
C
C M�ller & Nix, Atomic Data and Nuclear Data Tables, Vol 59 (1995) 185-383
C given are: Z,N,A,beta2,beta4
 2007 READ(13,*,END=2008)Z,N,BETA2P,BETA4P
       BETA21(N,Z) = BETA2P
       BETA41(N,Z) = BETA4P
      GOTO 2007
 2008 CONTINUE
      CLOSE(UNIT=13)
      DO Z = 0,98,1
       DO N = 0,153,1
         ECGNZ1(N,Z) = ECNZ1(N,Z)
       ENDDO
      ENDDO
C
      DO Z = 0,98,1
        DO  N = 0,153,1
         ALPHA(N,Z) = DSQRT(5.D0/(4.D0*3.141592653589793D0))*BETA21(N,Z)
         ECFNZ1(N,Z) = 0.D0
        ENDDO
      ENDDO


      DO Z = 0,136,1
       DO N = 0,153,1
         BETA2(N,Z) = BETA21(N,Z)
         BETA4(N,Z) = BETA41(N,Z)
       ENDDO
      ENDDO

      DO Z = 0,98,1
       DO N = 0,153,1
        VGSLD(N,Z) = VGSLD1(N,Z)
        ECNZ(N,Z) = ECNZ1(N,Z)
        ECGNZ(N,Z) = ECGNZ1(N,Z)
        ECFNZ(N,Z) = ECFNZ1(N,Z)
       ENDDO
      ENDDO

      IF(AP1 .GT.0)THEN
       DO Z = 1,12,1
        DO N = 1,153,1
           IF(MEXPIOP(N,Z).EQ.1)THEN
            IF(N.LT.3)THEN
              ECNZ(N,Z) = 0.D0
              ECGNZ(N,Z) = ECNZ(N,Z)
              BIND(N,Z) = MEXP(N,Z)
     &            - 938.7829835D0*DFLOAT(Z) - 939.5653301D0*DFLOAT(N)
              VGSLD(N,Z) = 0.D0
              E0 = 0.D0
              ELSE

C For these nuclei, we take "experimental" ground-state shell corrections
              IF((N+Z).GT.0) THEN

C Parametrization of CT model by Ignatyuk; note that E0 is shifted to correspond
C to pairing shift in Fermi-gas model (there, energy is shifted taking odd-odd nuclei
C  as bassis)
               CALL PARITE(DFLOAT(N+Z),PARA)
                IF (PARA.LT.0.D0) THEN
c e-o, o-e
                 E0 =  0.285D0+11.17D0*DFLOAT(N+Z)**(-0.464D0) -
     &               0.390D0-0.00058D0*DFLOAT(N+Z)
                ELSE
                  CALL PARITE(DFLOAT(Z),PARZ)
                 IF (PARZ.GT.0.D0) THEN
c e-e
                  E0 = 22.34D0*DFLOAT(N+Z)**(-0.464D0)-0.235D0
                 ELSE
c o-o
                 E0 = 0.D0
                 END IF
                END IF
              ELSE
C A la M�ller&Nix
      DN  = 4.80D0/(DFLOAT(N))**(1.D0/3.D0)
      DP  = 4.80D0/(DFLOAT(Z))**(1.D0/3.D0)
      DPN = 6.6D0/(DFLOAT(Z+N))**(2.D0/3.D0)

               IF(MOD(N,2).EQ.1.AND.MOD(Z,2).EQ.1)THEN
c o-o
               E0 = DPN - DN - DP
               ENDIF

               IF(MOD(N,2).EQ.1.AND.MOD(Z,2).EQ.0)THEN
               E0 = -1.D0*DN
               ENDIF

               IF(MOD(Z,2).EQ.1.AND.MOD(N,2).EQ.0)THEN
               E0 = -1.D0*DP
               ENDIF

             ENDIF

             IF((N.EQ.Z).AND.MOD(N,2).EQ.1.AND.MOD(Z,2).EQ.1)THEN
             E0 = E0 - 30.D0*(1.D0/DFLOAT(N+Z))
             END IF

            DELTA_TOT = ECNZ1(N,Z) - VGSLD(N,Z)
            ECNZ(N,Z) = MEXP(N,Z) - (MFRLDM(N,Z) - E0)

            VGSLD(N,Z) = MAX(0.D0,(ECNZ(N,Z) - DELTA_TOT))
            ECGNZ(N,Z) = ECNZ(N,Z)

            ENDIF
           ENDIF

        ENDDO
       ENDDO
      ENDIF

c      DO Z = 0,98,1
c       DO N = 0,153,1
c        VGSLD(N,Z) = VGSLD1(N,Z)
c        ECNZ(N,Z) = ECNZ1(N,Z)
c        ECGNZ(N,Z) = ECGNZ1(N,Z)
c        ECFNZ(N,Z) = ECFNZ1(N,Z)
c       ENDDO
c      ENDDO

      first=.false.

      end if !first
      return
C
C
C----------- SIMPLE GEOMETRICAL CROSS SECTION, RADIUS CONSTANT
C            (THIS WAS THE OLD CROSS SECTION, BUT IT IS NOT
C            RECOMMENDED, ESPECIALLY WITH EMD, REFOPT0=1 )
C     R_0= 1.16D0
C----------- EMPIRICAL GEOMETRICAL CROSS SECTION, RADIUS CONSTANT
C            (THIS IS THE NEW CROSS SECTION, IT IS RECOMMENDED
C            ESPECIALLY WITH EMD, REFOPT0=1 )
C            J.C. Benesh et al. Phys. Rev. C 40(1989) 1198
      R_0= 1.34D0*(AP**R13+AT**R13-0.75D0*(AP**(-R13)+AT**(-R13)))/
     +                                           (AP**R13+AT**R13)
C
C----------- NUCLEAR RADII AND MAXIMUM IMPACT PARAMETER
      R_P = R_0*AP**R13
      R_T = R_0*AT**R13
      BMAXNUC = R_P+R_T
C
C----------- NUCLEAR CROSS SECTION
      CRNUC = SIGMANUC(AP,ZP,AT,ZT,EAP)
C
C----------- CALCULATE ELECTROMAGNETIC CROSS SECTION
      IF (OPTEMD.NE.1) THEN
         CREMDE1 = 0.D0
         CREMDE2 = 0.D0
      END IF
C
C----------- SEPARATION ENERGIES AND FISSION BARRIER OF PROJECTILE
C
        K  = IDNINT(ZP)
        J  = IDNINT(AP - ZP)

        CALL MGLMS(AP,ZP,OPTSHP,MAZ)
        CALL MGLMS(AP-1.D0,ZP,OPTSHP,MA1Z)
        CALL MGLMS(AP-1.D0,ZP-1.D0,OPTSHP,MA1Z1)
        SNPRO = MA1Z - MAZ
        SPPRO = MA1Z1 - MAZ
C
C----------- BASS BARRIER
C
        CALL BARRS(0.D0,ZP-1.D0,AP-1.D0,1.D0,1.D0,BP,OMEGAP)
        SPMIN = SPPRO + BP
C PROJECTILE FISSION BARRIER FROM SIERK (BARFIT) FOR ZERO ANG. MOMENTUM
        CALL BARFIT(K,(K+J),0,SBFIS,SEGS,SELMAX)
        BFPRO = SBFIS * KOEFF
C ADD SHELL CORRECTION. PAIRING CANCELS.
C
           IF ((OPTSHP.EQ.1).OR.(OPTSHP.EQ.3)) THEN
             BFPRO = BFPRO -  ECGNZ(J,K)
           END IF
C AK - Jan 2011 - Corrected values of fission barriers
        IF(K.GE.94.AND.K.LE.98.AND.J.LE.152)THEN   ! Data in this range have been tested
C e-e
        IF(MOD(J,2).eq.0.and.MOD(K,2).eq.0)THEN
        IF(K.GE.94) BFPRO = BFPRO-(11.54108D0*(AP-ZP)/ZP-18.074D0)
        ENDIF
C O-O
        IF(MOD(J,2).eq.1.and.MOD(K,2).eq.1)THEN
        IF(K.GE.95)BFPRO = BFPRO-(14.567D0*(AP-ZP)/ZP-23.266D0)
        ENDIF
C Odd A
        IF(MOD(J,2).eq.0.and.MOD(K,2).eq.1)THEN
        IF(J.GE.144)BFPRO = BFPRO-(13.662D0*(AP-ZP)/ZP-
     &                                         21.656D0)
        ENDIF
        IF(MOD(J,2).eq.1.and.MOD(K,2).eq.0)THEN
        IF(J.GE.144)BFPRO = BFPRO-(13.662D0*(AP-ZP)/ZP-
     &                                         21.656D0)
        ENDIF
           ENDIF

C
C TO AVOID NEGATIVE VALUES FOR IMPOSSIBLE NUCLEI
C THE FISSION BARRIER IS SET TO ZERO IF SMALLER THAN ZERO.
C
           IF (BFPRO.LT.0.D0) BFPRO = 0.D0
C
C FOR OUTPUT OF SHELL CORRECTIONS
C
        SHELL = ECGNZ(J,K)
C
C OUTPUT OF PROJECTILE PARAMETERS
C
        WRITE (6,*)'PROJECTILE BINDING ENERGY ',MAZ
        WRITE (6,*)'PROJECTILE SN,SP,BF,SHC ',
     &              SNPRO,SPMIN,BFPRO,SHELL
        Y  = 1.D0 - FISSILITY(AP1,ZP1,OPTXFIS,0.D0,0.D0)
        IF (Y.LE.0.D0) Y = 0.D0
        IF (Y.GE.1.D0) Y = 1.D0
        IF (OPTAFAN.EQ.1) THEN
          BS = 1.D0
          BK = 1.D0
        ELSE
          BS = BIPOL(1,Y)
          BK = BIPOL(1,Y)
        END IF
        AF = AV * AP + AS * BS * AP**(2.D0/3.D0) +
     &       AK * BK * AP**(1.D0/3.D0)
        CALL BSBK(AP,ZP,BS,BK)
        AN = AV * AP + AS * BS * AP**(2.D0/3.D0) +
     &       AK * BK * AP**(1.D0/3.D0)
        WRITE(6,*)'PROJECTILE AF, AN: ',AF,AN
        WRITE(6,*)'PROJECTILE GROUND STATE QUADRUPOLE DEF:',
     &             BETA2(AP1-ZP1,ZP1)
        WRITE(6,*)'LIQUID DROP S.P. DEF. ',
     &             1.58533D0*SPDEF(AP1,ZP1,OPTXFIS,0.D0,0.D0)

C
C INITIALISATION OF ARRAYS FOR MC METHOD
C EXCITATION ENERGY AFTER EMD.
C
      IF (OPTEMD.EQ.1) THEN
C
      IF ((OPTSHP.EQ.0).OR.(OPTSHP.EQ.2)) THEN
         ELIM = DMIN1((BFPRO-SHELL),(SNPRO-SHELL),(SPMIN-SHELL))
      END IF
      IF ((OPTSHP.EQ.1).OR.(OPTSHP.EQ.3)) THEN
         ELIM = DMIN1(BFPRO,SNPRO,SPMIN)
      END IF
C
      DUQUAD = (50.D0 - ELIM) / NQUAD
      EI = ELIM
C
      DO 100 J=1,NQUAD+1,1
         XR(J) = EI
         PR1(J) = DNDE1(EI) * SIGE1(AP,ZP,ALPHA(AP1-ZP1,ZP1),EI)
C     Neu - AGre -
         PR2(J) = DNDE2(EI) * SIGE2(AP,ZP,EI)
         EI = EI + DUQUAD

 100  CONTINUE
C
C CUMULATIVE SUMS (INTEGRALS)
C
      SR1(1) = 0.D0
      SR2(1) = 0.D0
      DO 101 J=2,NQUAD+1,1
        SR1(J) = SR1(J-1) + (PR1(J) + PR1(J-1))/2.D0*DUQUAD
        SR2(J) = SR2(J-1) + (PR2(J) + PR2(J-1))/2.D0*DUQUAD
 101  CONTINUE
C
C TEST-OUTPUT OF EMD-CROSS SECTIONS
C
      CREMDE1 = SR1(NQUAD+1)
      CREMDE2 = SR2(NQUAD+1)
C
C NORMALIZATION
C
      DO 102 J=1,NQUAD+1,1
         SR1(J) = SR1(J)/SR1(NQUAD+1)
         SR2(J) = SR2(J)/SR2(NQUAD+1)
 102  CONTINUE
      CALL SPLINE(NQUAD+1,SR1,XR,AE1,BE1,CE1)
      CALL SPLINE(NQUAD+1,SR2,XR,AE2,BE2,CE2)
C
C CONVERSION TO EQUIDISTANT ARRAYS XRE1,2
C
      DS1 = (SR1(NQUAD+1) - SR1(1))/NQUAD
      DS2 = (SR2(NQUAD+1) - SR2(1))/NQUAD
      U1 = SR1(1)
      U2 = SR2(1)
      DO 103 J=1,NQUAD+1,1
         SRE1(J) = U1
         SRE2(J) = U2
         XRE1(J) = SEVAL(NQUAD+1,U1,SR1,XR,AE1,BE1,CE1)
         XRE2(J) = SEVAL(NQUAD+1,U2,SR2,XR,AE2,BE2,CE2)
         U1 = U1 + DS1
         U2 = U2 + DS2
 103  CONTINUE
C
C TO MAKE A SHORT SPLINE INTEPOLATION IN REANUCL
C WITHOUT HAVING TO SEARCH THE INDEX FOR A LONG TIME
C THE INDEX CAN BE CALCULATED IMMEDIATELY FOR EQUIDISTANT POINTS.
C
      CALL SPLINE(NQUAD+1,SRE1,XRE1,AE1,BE1,CE1)
      CALL SPLINE(NQUAD+1,SRE2,XRE2,AE2,BE2,CE2)
C
      END IF
C
C END OF INITIALISATION OF EMD EXCITATION ENERGIE ARRAYS
C
C
C----------- CALCULATE TOTAL CROSS SECTION
C
      CRTOT = CRNUC+CREMDE1+CREMDE2

C IF YOU USE YOUR OWN INPUT, HERE YOU SHOULD GIVE THE CROSS SECTIONS
      IF(IREACT.EQ.1)THEN
c      CRTOT = 1.D0
c      CRNUC = 1.D0
      CRTOT = CRTOT
      CRNUC = CRNUC
      CREMDE1 = 0.D0
      CREMDE2 = 0.D0
      ENDIF
C
C
C INITIALISATION OF ARRAYS FOR MC METHOD TO CALCULATE THE EXCITATION
C ENERGY AFTER ABRASION OF UP TO 50 NUCLEONS
C
C    MAXIMUM NUMBER OF ABRADED NUCLEONS IS N
C    MORE THAN 50 LEADS TO AN ENORMOUSLY LARGE ARRAY (MEMORY ?)
C
      NEE=40
      NNUC=50
      EPS = 1.D-14
C
C explicit zeroing of all arrays just to be shure.
C a.r.j. 26.05.98
C IT IS NECESSARY !
C
      DO J = 1,2001
        XEE(J) = 0.D0
        SE1(J) = 0.D0
        SHE(J) = 0.D0
      END DO
      DO K = 1,50,1
         DO J = 1,2001,1
            EEC(K,J) = 0.D0
            SEE(K,J) = 0.D0
            XHE(K,J) = 0.D0
         END DO
      END DO
C
C ENERGY-SCALE ARRAY
C
      XEE(1) = 0.D0
      DO J = 2,2000,1
         XEE(J) = XEE(J-1) + 1.D0
      END DO

C
C TRIANGLE FOR 1 ABRADED NUCLEON
C ENERGY UNITS ARE 1 MEV
C
      DO J = 1,40,1
         YE       = 0.05D0 - (DFLOAT(J-1)+0.5D0)*0.05D0/40.D0
         EEC(1,J) = YE
      END DO
C
C TWO OR MORE ABRADED NUCLEONS BY CONVOLUTION
C
      DO I = 2,NNUC,1
C
C "DIE DOPPELTE DO-LOOP" CONVOLUTION OF SINGLE HOLE EXCITATION ENERGY
C                        SPECTRUM
C
         DO K = 1,40,1
            DO J = K,(I*NEE),1
               EEC(I,J) = EEC(I,J) + EEC(1,K)*EEC(I-1,J-K+1)
            END DO
         END DO
C
      END DO
C
C CUMULATIVE SUMS  (NORMALISED TO UNITY)
C
      DO I = 1,NNUC,1
C         SEE(I,1) = 0.D0
          SEE(I,1) = EEC(I,1)
         DO J = 2,(I*NEE),1
            SEE(I,J) = SEE(I,J-1) + EEC(I,J)
         END DO
      END DO
C
C LINEAR INTERPOLATION BECAUSE SPLINE CAUSES PROBLEMS
C
C
C NEW EQUIDISTANT ARRAY IN SEE -> SHE
C
C      SHE(1) = 0.D0
C      DO J   = 2,2001,1
C         SHE(J) = SHE(J-1) + 0.0005D0
C      END DO
C
C LET'S TRY A LOGARITHMIC SCALE FROM 10**-66 UP TO 0.1
C
      SHE(1) = -66
      DO J = 2,1501,1
         SHE(J) = SHE(J-1) + 0.043333333333333D0
      END DO
C
C NOW LINEAR INTERVALS BUT IN LOGARITHMS AS WELL FROM 0.1 TO 1
C
      DO J = 1502,2001,1
         SHE(J) = DLOG10(0.1D0 + (DFLOAT(J-1501)*0.0018D0))
      END DO
C
      DO I = 1,NNUC,1
         DO J = 1,(I*NEE),1
            SE1(J) = SEE(I,J)
         END DO
         DO J = 1,2001,1
           CALL LOCATE(SE1,(I*NEE),(10.D0**SHE(J)),L)
           IF(L.EQ.0) THEN
             XHE(I,J) = 0.D0
           ELSE
              if(se1(l+1).eq.se1(l))then
              m = (xee(l+1)-xee(l))/1.d-30
              else
               M = (XEE(L+1) - XEE(L))/(SE1(L+1) - SE1(L))
              endif

C
C XHE ARRAY OF ENERGY VALUES FITTING TO SHE
C
             XHE(I,J) = XEE(L) + M * (10.D0**(SHE(J)) - SE1(L))

           END IF
         END DO
      END DO
C
C XHE,SHE ARE TO BE COMMON FOR CALULATION OF THE EXCITATION ENERGY
C THE SCALE OF SHE(J) IS SUBDIVIDED IN AN LOGARITHMIC AND A LINEAR PART
C SPACING  OF TWO POINT IN THE LOGARITHMIC PART FROM 10**-66 TO 0.1018
C IS 0.04333333333 (ON LOG-SCALE) AND THE DISTANCE IN THE LINEAR PART
C FROM 0.1018 TO 1.0 IS 0.0018 (LINEAR SCALE).
C
C  TEST OUTPUT
C
c      WRITE(6,*) '        '
c      WRITE(6,*)'H: X Y1(1),LTR0'
c      DO J = 1,2001,1
c         WRITE(6,*) XHE(1,J),10**(SHE(J))
c      END DO
c      WRITE(6,*)'H: X Y1(1),LTTR0'
c      DO J = 1,NEE,1
c         WRITE(6,*) XEE(J),SEE(1,J)
c      END DO
C
C      WRITE(6,*) '        '
C      WRITE(6,*)'H: X Y1(3),DT0'
C      DO J = 1,2001,1
C         WRITE(6,*) XHE(3,J),10**(SHE(J))
C      END DO
C      WRITE(6,*)'H: X Y1(3),DTT0'
C      DO J = 1,3*NEE,1
C         WRITE(6,*) XEE(J),SEE(3,J)
C      END DO
C
C      WRITE(6,*) '        '
C      WRITE(6,*)'H: X Y1(7),DBT0'
C      DO J = 1,2001,1
C         WRITE(6,*) XHE(7,J),10**(SHE(J))
C      END DO
C      WRITE(6,*)'H: X Y1(7),DBTT0'
C      DO J = 1,3*NEE,1
C         WRITE(6,*) XEE(J),SEE(7,J)
C      END DO
C
C      WRITE(6,*) '        '
C      WRITE(6,*)'H: X Y1(20),PTB0'
C      DO J = 1,2001,1
C         WRITE(6,*) XHE(20,J),10**(SHE(J))
C      END DO
C     WRITE(6,*)'H: X Y1(20),PTTB0'
C      DO J = 1,20*NEE,1
C         WRITE(6,*) XEE(J),SEE(20,J)
C      END DO
C
C      WRITE(6,*) '        '
C      WRITE(6,*)'H: X Y1(50),PTR0'
C      DO J = 1,2001,1
C         WRITE(6,*) XHE(50,J),10**(SHE(J))
C      END DO
C      WRITE(6,*)'H: X Y1(50),PTTR0'
C      DO J = 1,20*NEE,1
C         WRITE(6,*) XEE(J),SEE(50,J)
C      END DO
C       STOP
      RETURN
      END
C
C************************************************************

      DOUBLE PRECISION FUNCTION SIGE1(AP,ZP,ALPHA,E)
C
C  TWO HUMPED LORENTZIAN FOR VIRTUAL GAMMA ABSORPTION CROSS
C  SECTION.
C  OUTPUT : SIGE1 - CROSS SECTION FOR E1 PHOTONS
C  INPUT  : AP    - PROJECTILE MASS
C           ZP    - PROJECTILE CHARGE
C           ALPHA - PROJECTILE DEFORMATION (FROM TABLE)
C           E     - ENERGY VARIABLE
C
C
      REAL*8 AP,ZP,ALPHA,E1SUM,E,E0,E1,E2,G1,G2,LOR1,LOR2,PI
      PI = 3.141592653589793D0
C   * TRK - SUMMENREGEL
      E1SUM = 60.D0 * ZP*(AP-ZP)/AP
      E0    = 79.D0 * AP**(-1.D0/3.D0)
      E1    = E0/(1.D0 + ALPHA)
      E2    = E0/(1.D0 - ALPHA/2.D0)
      G1    = 0.026D0*E1*E1
      G2    = 0.026D0*E2*E2
      LOR1  = ((E*G1)**2)/((E*E - E1*E1)**2 + (G1*E)**2)
      LOR2  = ((E*G2)**2)/((E*E - E2*E2)**2 + (G2*E)**2)
      SIGE1 =2.D0*E1SUM/PI*(LOR1/(3.D0*G1)+2.D0*LOR2/(3.D0*G2))
      RETURN
      END
C
C ******************************************************************
C
       DOUBLE PRECISION FUNCTION SIGE2(A,Z,E)
C
C Two single humped curve for both isoscalar and isovectorial
C virtual gamma absorption cross section.
C Shape of curve is according to R. Ligensa and W. Greiner
C                                Phys.Rev.Lett 16 (1966) 364
C Parameter ACCORDING TO THE PUBLICATION (Zitate da)
C output : SIGE2 - cross section for e2 photon absorption
C input  : A     - nuclear mass number
C          Z     - nuclear charge number
C          E     - energy of photon
C
C
       REAL*8 A,Z,E,PI
       REAL*8 EGQRS,EGQRV,GAMMAS,GAMMAV,E2SUM,E2SUMS,E2SUMV

      PI = 3.141592653589793D0

C   E2- GIANT RESONANCE
C   POSITION OF  IS RESONANCE
      EGQRS = 64.7D0 * A**(-1.D0/3.D0)
C   POSITION OF  IV RESONANCE
      EGQRV = 130.D0 * A**(-1.D0/3.D0)
C   WIDTH IS
      GAMMAS = 17.5D0 * A**(-1.D0/3.D0)
C   WIDTH IV
      GAMMAV = 10.48D0-0.0733D0*A**(2.D0/3.D0)-0.00174*A**(4.D0/3.D0)
C   SUM RULE
       E2SUM  = 2.2D-4 * Z * A**(2.D0/3.D0)
C   WEIGHTS FOR IS/IV FROM [War69] T. Aumann
       E2SUMS = E2SUM*Z/A
       E2SUMV = E2SUM-E2SUMS

       SIGE2 = 2.D0/PI*E**2*
     &   (E2SUMS/GAMMAS/(((E*E-EGQRS*EGQRS)/(E*GAMMAS))**2.D0+1.D0)
     &   +E2SUMV/GAMMAV/(((E*E-EGQRV*EGQRV)/(E*GAMMAV))**2.D0+1.D0))
       RETURN
       END
C

