#include "G4INCLAblaInterface.hh"
#include "G4INCLParticleTable.hh"
#include "G4INCLGlobals.hh"
#include <iostream>

G4INCLAblaInterface::G4INCLAblaInterface(G4INCL::Config *config) :
  IDeExcitation(config),
  theConfig(config),
  ablaResult(new G4VarNtp),
  volant(new G4Volant),
  theABLAModel(new G4Abla(theConfig, volant, ablaResult))
{
  theABLAModel->initEvapora();
}

G4INCLAblaInterface::~G4INCLAblaInterface() {
  delete volant;
  delete ablaResult;
  delete theABLAModel;
}

void G4INCLAblaInterface::deExciteRemnant(G4INCL::EventInfo *eventInfo, const int i) {
  int particleIndex = eventInfo->nParticles;

  volant->clear();
  ablaResult->clear();

  double nuclearMass = G4INCL::ParticleTable::getTableMass(eventInfo->ARem[i],
                                                           eventInfo->ZRem[i]);
  theABLAModel->breakItUp(eventInfo->ARem[i], eventInfo->ZRem[i],
                          nuclearMass,
                          eventInfo->EStarRem[i],
                          eventInfo->JRem[i],
                          eventInfo->EKinRem[i],
                          eventInfo->pxRem[i],
                          eventInfo->pyRem[i],
                          eventInfo->pzRem[i],
                          eventInfo->eventNumber);

  for(int j = 0; j < ablaResult->ntrack; ++j) { // Copy ABLA result to the EventInfo
    eventInfo->A[particleIndex] = ablaResult->avv[j];
    eventInfo->Z[particleIndex] = ablaResult->zvv[j];
    eventInfo->emissionTime[particleIndex] = -1.0;
    eventInfo->EKin[particleIndex] = ablaResult->enerj[j];
    eventInfo->px[particleIndex] = ablaResult->plab[j]*std::sin(ablaResult->tetlab[j]*G4INCL::Math::pi/180.0)*std::cos(ablaResult->philab[j]*G4INCL::Math::pi/180.0);
    eventInfo->py[particleIndex] = ablaResult->plab[j]*std::sin(ablaResult->tetlab[j]*G4INCL::Math::pi/180.0)*std::sin(ablaResult->philab[j]*G4INCL::Math::pi/180.0);
    eventInfo->pz[particleIndex] = ablaResult->plab[j]*std::cos(ablaResult->tetlab[j]*G4INCL::Math::pi/180.0);
    eventInfo->theta[particleIndex] = ablaResult->tetlab[j];
    eventInfo->phi[particleIndex] = ablaResult->philab[j];
    eventInfo->origin[particleIndex] = i; // Origin: De-excitation
    eventInfo->history.push_back(""); // No history tracking for ABLAv3p
    particleIndex++;
  }
  eventInfo->nParticles = particleIndex;
}

