#include "G4AblaDataFile.hh"
//#include "G4HadronicException.hh"
#ifdef ABLAXX_IN_GEANT4_MODE
#include "globals.hh"
#else
#include "G4INCLGeant4Compat.hh"
#endif
#include <fstream>
#include <cmath>
#include <iostream>
#include <cstdlib>

#ifdef ABLAXX_IN_GEANT4_MODE
G4AblaDataFile::G4AblaDataFile() {
#else
G4AblaDataFile::G4AblaDataFile(G4INCL::Config *config)
  : G4AblaVirtualData(config) {
  theConfig = config;
#endif
  verboseLevel = 0;
}

G4AblaDataFile::~G4AblaDataFile()
{
}

/**
 * Read all data from files.
 */
bool G4AblaDataFile::readData()
{
#ifdef ABLAXX_IN_GEANT4_MODE
  if(!getenv("G4ABLADATA")) {
    //    throw G4HadronicException(__FILE__, __LINE__, "ERROR: Data
    //    missing. Set environment variable G4ABLA3.0 to point to the
    //    directory containing data files needed by INCL and ABLA
    //    models.");
    // G4String errorMessage1 = "ERROR: Data missing. Set environment variable G4ABLADATA\n";
    // G4String errorMessage2 = "\t to point to the directory containing data files needed\n";
    // G4String errorMessage3 = "\t by INCL and ABLA models.\n";
    // G4String errorMessage = errorMessage1 + errorMessage2 + errorMessage3;
    // G4Exception(errorMessage);
    G4ExceptionDescription ed;
    ed << " Data missing: set environment variable G4ABLADATA\n"
       << " to point to the directory containing data files needed\n"
       << " by the ABLA model" << G4endl;
    G4Exception("G4AblaDataFile::readData()","ABLA_001",
                FatalException, ed);
  }
  
  G4String dataPath(getenv("G4ABLADATA"));
#else
  G4String dataPath(theConfig->getABLAv3pCxxDataFilePath().c_str());
#endif
  G4String flAlphaFile(dataPath + "/flalpha.dat");
  G4String frldmFile(  dataPath + "/frldm.dat");
  G4String vgsldFile(  dataPath + "/vgsld.dat");
  G4String pace2File(  dataPath + "/pace2.dat");

  if(verboseLevel > 1) {
    // G4cout <<"Data path   = " << dataPath    << G4endl;
    // G4cout <<"FlAlphaFile = " << flAlphaFile << G4endl;
    // G4cout <<"FrldmFile   = " << frldmFile   << G4endl;
    // G4cout <<"VgsldFile   = " << vgsldFile   << G4endl;
    // G4cout <<"Pace2File   = " << pace2File   << G4endl;
  }
  
  std::ifstream flalphain(flAlphaFile.c_str());
  std::ifstream frldmin(frldmFile.c_str());  
  std::ifstream vgsldin(vgsldFile.c_str());  
  std::ifstream pace2in(pace2File.c_str());

  std::filebuf *buf1 = flalphain.rdbuf();
  std::filebuf *buf2 = frldmin.rdbuf();
  std::filebuf *buf3 = vgsldin.rdbuf();
  std::filebuf *buf4 = pace2in.rdbuf();  
  if (!((buf1->is_open()) && (buf2->is_open()) && (buf3->is_open()) && (buf4->is_open()))) {
#ifdef ABLAXX_IN_GEANT4_MODE
    G4ExceptionDescription ed;
    ed << " Data missing: could not find ABLA data file in " << dataPath
       << "defined by environment variable G4ABLADATA" << G4endl;
    G4Exception("G4AblaDataFile::readData()", "ABLA002", FatalException, ed);
#else
    std::cerr << "Error opening file." << std::endl;
#endif
  }
  
  G4double fflalpha, ffrldm, fvgsld, fpace2;
  const G4int rows = 99;
  const G4int cols = 154;
  const G4int massnumbers = 263;
  for(int i = 0; i < rows; i++) {
    for(int j = 0; j < cols; j++) {
      setAlpha(j, i, 0.0);
      setEcnz( j, i, 0.0);
      setVgsld(j, i, 0.0);
    }
  }
  
  for(int i = 0; i < rows; i++) {
    for(int j = 0; j < cols; j++) {
      flalphain >> fflalpha;
      frldmin >> ffrldm;
      vgsldin >> fvgsld;      
      setAlpha(j, i, fflalpha);
      setEcnz( j, i, ffrldm);
      setVgsld(j, i, fvgsld);
    }
  }
  flalphain.close();
  frldmin.close();  
  vgsldin.close();

  G4String str1, str2, str3;
  for(int i = 0; i < 500; i++) {
    for(int j = 0; j < 500; j++) {
      setPace2(i, j, 0.0);
    }
  }
  
  int A = 0, Zbegin = 0, Zend = 0;
  for(int i = 0; i < massnumbers; i++) {
    pace2in >> str1 >> A >> str2 >> Zbegin >> str3 >> Zend;
    if(Zbegin >= 0 && Zbegin < getPaceCols() &&
       A >= 0 && A < getPaceRows()) {
      for(int j = Zbegin; j <= Zend; j++) {
	pace2in >> fpace2;
	setPace2(A, j, fpace2);
      }
    }
  }
  pace2in.close();
  if(std::abs(getPace2(A, Zend) - 114516.10) > 1e-6) {
    std::cerr << "ERROR: Problem in parsing datafile " + pace2File << std::endl;
    return false;
  }

  return true;
}

