#include "G4INCLFermiBreakUpInterface.hh"
#include "G4INCLParticleTable.hh"
#include "G4INCLCluster.hh"
#include "G4INCLParticleVector.hh"
#include "G4INCLFermiBreakUp.hh"
#include <iostream>

G4INCLFermiBreakUpInterface::G4INCLFermiBreakUpInterface()
{
  theFermiBreakUpModel = new G4INCLFermiBreakUp();
}

G4INCLFermiBreakUpInterface::~G4INCLFermiBreakUpInterface() {
  delete theFermiBreakUpModel;
}

void G4INCLFermiBreakUpInterface::deExciteRemnant(G4INCL::EventInfo *eventInfo, const int i) {
  int particleIndex = eventInfo->nParticles;

  G4INCL::Cluster aCluster(eventInfo->ZRem[i], eventInfo->ARem[i],false);
  const G4INCL::ThreeVector momentum(eventInfo->pxRem[i], eventInfo->pyRem[i], eventInfo->pzRem[i]);
  aCluster.setExcitationEnergy(eventInfo->EStarRem[i]);
  aCluster.setMass(aCluster.getRealMass()+eventInfo->EStarRem[i]);
  aCluster.setMomentum(momentum);
  aCluster.adjustEnergyFromMomentum();
  G4INCL::ParticleVector *theResult = theFermiBreakUpModel->BreakItUp(aCluster);

  for(unsigned int j = 0; j < theResult->size(); ++j) { // Copy FermiBreakUp result to the EventInfo
    eventInfo->A[particleIndex] = (*theResult)[j]->getA();
    eventInfo->Z[particleIndex] = (*theResult)[j]->getZ();
    eventInfo->emissionTime[particleIndex] = -1.0;
    eventInfo->EKin[particleIndex] = (*theResult)[j]->getKineticEnergy();
    eventInfo->px[particleIndex] = (*theResult)[j]->getMomentum().getX();
    eventInfo->py[particleIndex] = (*theResult)[j]->getMomentum().getY();
    eventInfo->pz[particleIndex] = (*theResult)[j]->getMomentum().getZ();
    eventInfo->theta[particleIndex] = (*theResult)[j]->getMomentum().theta() * 180. / G4INCL::Math::pi;
    eventInfo->phi[particleIndex] = (*theResult)[j]->getMomentum().phi() * 180. / G4INCL::Math::pi;
    eventInfo->origin[particleIndex] = i; // Origin: De-excitation
    eventInfo->history.push_back("u"); // use "u" for Fermi break-up
    particleIndex++;
    delete (*theResult)[j];
  }
  delete theResult;
  eventInfo->nParticles = particleIndex;
}

