#include "G4INCLSMMInterface.hh"
#include "G4INCLLogger.hh"
#include "G4INCLParticleTable.hh"
#include "G4INCLGlobals.hh"
#include "G4INCLRandom.hh"
#include <algorithm>

namespace SMMCXX {

  SMMInterface::SMMInterface(G4INCL::Config *conf) :
    IDeExcitation(conf)
  {
#ifndef SMM_USE_USER_RNG
    int i=0;
    rndm1(&i);
#endif
  }

  SMMInterface::~SMMInterface() {
  }

  void SMMInterface::deExciteRemnant(G4INCL::EventInfo *eventInfo, const int i) {
    int particleIndex = eventInfo->nParticles;

    int ARem = eventInfo->ARem[i];
    int ZRem = eventInfo->ZRem[i];
    float EStarRem = eventInfo->EStarRem[i]/(1000.*ARem);
    float pxRem = eventInfo->pxRem[i];
    float pyRem = eventInfo->pyRem[i];
    float pzRem = eventInfo->pzRem[i];
    float EKinRem = eventInfo->EKinRem[i];
    int nSMMParticles = 0;
    int kfis;

    // Call the de-excitation subroutine
    smm_botvina_(&ARem, &ZRem, &EStarRem, &pxRem, &pyRem, &pzRem, &nSMMParticles, &EKinRem, &kfis);

    for(int j = 0; j < nSMMParticles; ++j) { // Copy SMM result to the EventInfo
      const int Apart = volant_.acv[j];
      const int Zpart = volant_.zpcv[j];
      eventInfo->A[particleIndex] = Apart;
      eventInfo->Z[particleIndex] = Zpart;
      eventInfo->emissionTime[particleIndex] = -1.0;

      const double cx = std::min(std::max((double)volant_.xcv[j], -1.),1.);
      const double cy = std::min(std::max((double)volant_.ycv[j], -1.),1.);
      const double cz = std::min(std::max((double)volant_.zcv[j], -1.),1.);
      const double plab = volant_.pcv[j];
      eventInfo->px[particleIndex] = plab*cx;
      eventInfo->py[particleIndex] = plab*cy;
      eventInfo->pz[particleIndex] = plab*cz;

      eventInfo->theta[particleIndex] = G4INCL::Math::toDegrees(G4INCL::Math::arcCos(cz));
      eventInfo->phi[particleIndex] = G4INCL::Math::toDegrees(std::atan2(cy,cx));

      double mass;
      if(Apart>0) {
        mass = G4INCL::ParticleTable::getTableMass(Apart, Zpart);
      } else
        mass = 0.;

      eventInfo->EKin[particleIndex] = std::sqrt(plab*plab + mass*mass) - mass;
      eventInfo->origin[particleIndex] = i; // Origin: De-excitation
      eventInfo->history.push_back(""); // No history tracking for SMM
      particleIndex++;
    }
    eventInfo->nParticles = particleIndex;
  }
}

float smmuserrng_() {
  return ((float)G4INCL::Random::shoot());
}
