#include "G4INCLIDeExcitation.hh"

namespace G4INCL {

  IDeExcitation::IDeExcitation(G4INCL::Config *conf) :
    theConfig(conf)
#ifdef INCL_DEEXCITATION_FERMI_BREAKUP
    , theFermiBreakUp(new G4INCLFermiBreakUpInterface)
#endif
  {}

  IDeExcitation::~IDeExcitation() {
#ifdef INCL_DEEXCITATION_FERMI_BREAKUP
    delete theFermiBreakUp;
#endif
  }

  void IDeExcitation::deExcite(G4INCL::EventInfo *eventInfo) {
    int particleIndex = eventInfo->nCascadeParticles;

    for(int i = 0; i < eventInfo->nRemnants; ++i) {

      if(eventInfo->EStarRem[i]>1E-10) { // remnant with some excitation energy
#ifdef INCL_DEEXCITATION_FERMI_BREAKUP
        if(theFermiBreakUp
           && eventInfo->ARem[i] <= theConfig->getMaxMassFermiBreakUp()
           && eventInfo->ZRem[i] <= theConfig->getMaxChargeFermiBreakUp())
          theFermiBreakUp->deExciteRemnant(eventInfo, i);
        else
          deExciteRemnant(eventInfo, i);
#else
        deExciteRemnant(eventInfo, i);
#endif
      } else { // remnant without excitation energy -- copy it directly to the output
        eventInfo->remnantToParticle(i);
        particleIndex++;
      }
    }
  }

}

