#include "G4INCLAvatarDumpAction.hh"
#include <sstream>
#include <string>

namespace G4INCL {

  AvatarDumpAction::AvatarDumpAction() :
    oFile(0),
    eventCounter(0)
  {
  }

  AvatarDumpAction::~AvatarDumpAction() {}

  void AvatarDumpAction::beforeCascadeUserAction(IPropagationModel * /*pm*/) {
    std::stringstream ss;
    ss << "avatar-dump-" << eventCounter << ".dat";
    oFile = new std::ofstream(ss.str().c_str());
  }

  void AvatarDumpAction::afterAvatarUserAction(IAvatar *avatar, Nucleus *nucleus, FinalState *finalState) {
    ParticleList particles = nucleus->getStore()->getParticles();
    ParticleList highlight;
    if(finalState) {
      ParticleList const &modified = finalState->getModifiedParticles();
      highlight.insert(highlight.end(), modified.begin(), modified.end());
      ParticleList const &outgoing = finalState->getOutgoingParticles();
      highlight.insert(highlight.end(), outgoing.begin(), outgoing.end());
      ParticleList const &destroyed = finalState->getDestroyedParticles();
      highlight.insert(highlight.end(), destroyed.begin(), destroyed.end());
      ParticleList const &created = finalState->getCreatedParticles();
      highlight.insert(highlight.end(), created.begin(), created.end());
      ParticleList const &entering = finalState->getEnteringParticles();
      highlight.insert(highlight.end(), entering.begin(), entering.end());
      particles.insert(particles.end(), created.begin(), created.end());
      particles.insert(particles.end(), entering.begin(), entering.end());
    }

    (*oFile) << avatar->getTime() << '\t' << avatar->getType() << '\t' << particles.size() << '\n';
    for(ParticleIter p=particles.begin(), e=particles.end(); p!=e; ++p) {
      ThreeVector const &pos = (*p)->getPosition();
      ThreeVector const &vel = (*p)->getPropagationVelocity();
      int highlightIt = highlight.contains(*p);
      (*oFile)
        << (*p)->getID() << '\t'
        << (*p)->getParticipantType() << '\t'
        << (*p)->getType() << '\t'
        << pos.getX() << '\t'
        << pos.getY() << '\t'
        << pos.getZ() << '\t'
        << vel.getX() << '\t'
        << vel.getY() << '\t'
        << vel.getZ() << '\t'
        << (*p)->getKineticEnergy() << '\t'
        << (*p)->getPotentialEnergy() << '\t'
        << highlightIt << '\n';
    }
  }

  void AvatarDumpAction::afterCascadeUserAction(Nucleus * /*nucleus*/) {
    oFile->close();
    delete oFile;
    ++eventCounter;
  }

}
