#include "G4INCLCDPP.hh"
#include <functional>
#include <algorithm>

namespace G4INCL {

  CDPP::CDPP() :
    Sk(0.0),
    TbelowTf(0.0),
    thePotential(NULL)
  {}

  CDPP::~CDPP() {}

  bool CDPP::isBlocked(ParticleList const &created, Nucleus const * const nucleus) {
    double S = nucleus->computeSeparationEnergyBalance();

    thePotential = nucleus->getPotential();

    ParticleList const &remnantParticles = nucleus->getStore()->getParticles();

    Sk = 0.0;
    TbelowTf = 0.0;

    std::for_each(remnantParticles.begin(), remnantParticles.end(), std::bind1st(std::mem_fun(&G4INCL::CDPP::processOneParticle), this));
    std::for_each(created.begin(), created.end(), std::bind1st(std::mem_fun(&G4INCL::CDPP::processOneParticle), this));

    const double Tinitial = nucleus->getInitialInternalEnergy();
    const double Eblock = TbelowTf - Tinitial - Sk - S;

    return (Eblock < 0.0);
  }
}
