#include "G4INCLCascadeAction.hh"
#include "G4INCLLogger.hh"
#include "G4INCLRandom.hh"

namespace G4INCL {

  CascadeAction::CascadeAction() :
    stepCounter(0)
  {}

  CascadeAction::~CascadeAction()
  {}

  void CascadeAction::beforeRunAction(Config const *config) {
    beforeRunDefaultAction(config);
    beforeRunUserAction(config);
  }

  void CascadeAction::beforeCascadeAction(IPropagationModel *pm) {
    beforeCascadeDefaultAction(pm);
    beforeCascadeUserAction(pm);
  }

  void CascadeAction::beforePropagationAction(IPropagationModel *pm) {
    beforePropagationDefaultAction(pm);
    beforePropagationUserAction(pm);
  }

  void CascadeAction::beforeAvatarAction(IAvatar *a, Nucleus *n) {
    beforeAvatarDefaultAction(a, n);
    beforeAvatarUserAction(a, n);
  }

  void CascadeAction::afterAvatarAction(IAvatar *a, Nucleus *n, FinalState *fs) {
    afterAvatarDefaultAction(a, n, fs);
    afterAvatarUserAction(a, n, fs);
  }

  void CascadeAction::afterPropagationAction(IPropagationModel *pm, IAvatar *avatar) {
    afterPropagationDefaultAction(pm, avatar);
    afterPropagationUserAction(pm, avatar);
  }

  void CascadeAction::afterCascadeAction(Nucleus *n) {
    afterCascadeDefaultAction(n);
    afterCascadeUserAction(n);
  }

  void CascadeAction::afterRunAction() {
    afterRunDefaultAction();
    afterRunUserAction();
  }



  void CascadeAction::beforeRunDefaultAction(Config const * /*pm*/) {}

  void CascadeAction::beforeCascadeDefaultAction(IPropagationModel * /*pm*/) {}

  void CascadeAction::beforePropagationDefaultAction(IPropagationModel * /*pm*/) {
    // assert(pm->getNucleus()->getStore()->getBook().getCascading() == pm->getNucleus()->getStore()->countCascading());
  }

  void CascadeAction::beforeAvatarDefaultAction(IAvatar *a, Nucleus *n) {
    n->getStore()->getBook().incrementAvatars(a->getType());
    INCL_DEBUG("Random seeds before avatar " << a->getID() << ": "
          << G4INCL::Random::getSeeds() << '\n');
    INCL_DEBUG("Next avatar:" << '\n' << a->dump() << '\n');
  }

  void CascadeAction::afterAvatarDefaultAction(IAvatar *a, Nucleus * /*n*/, FinalState *fs) {

    if(!fs) // do nothing if there is no final state
      return;

    INCL_DEBUG("Random seeds after avatar " << a->getID() << ": "
          << G4INCL::Random::getSeeds() << '\n');

    ParticleList const &modified = fs->getModifiedParticles();
    for(ParticleIter p=modified.begin(), e=modified.end(); p!=e; ++p )
      if(a->isACollision())
        (*p)->incrementNumberOfCollisions();
      else if(a->isADecay())
        (*p)->incrementNumberOfDecays();

    ParticleList const &created = fs->getCreatedParticles();
    for(ParticleIter p=created.begin(), e=created.end(); p!=e; ++p )
      if(a->isACollision())
        (*p)->incrementNumberOfCollisions();
      else if(a->isADecay())
        (*p)->incrementNumberOfDecays();

  }

  void CascadeAction::afterPropagationDefaultAction(IPropagationModel * /* pm */,
                                                IAvatar * /*avatar */) {
    ++stepCounter; // Increment the step counter

#ifdef INCL_DEBUG_LOG
    //   INCL_DATABLOCK(pm->getNucleus()->getStore()->printParticleConfiguration());
#endif
  }

  void CascadeAction::afterCascadeDefaultAction(Nucleus * /*pm*/) {}

  void CascadeAction::afterRunDefaultAction() {}

}
