/** \file G4INCLClustering.cc
 * \brief Static class for cluster formation
 *
 * \date 13th July 2011
 * \author Davide Mancusi
 */

#include "G4INCLIClusteringModel.hh"
#include "G4INCLClustering.hh"
#include "G4INCLClusteringModelIntercomparison.hh"
#include "G4INCLClusteringModelNone.hh"

namespace G4INCL {

  namespace Clustering {

    namespace {
      G4ThreadLocal IClusteringModel *theClusteringModel = NULL;
    }

    Cluster* getCluster(Nucleus *n, Particle *p) {
#if !defined(NDEBUG) && !defined(INCLXX_IN_GEANT4_MODE)
      Cluster * const c=theClusteringModel->getCluster(n,p);
      assert(!c || c->getA()<=n->getA()/2);
      return c;
#else
      return theClusteringModel->getCluster(n,p);
#endif
    }

    bool clusterCanEscape(Nucleus const * const n, Cluster const * const c) {
      return theClusteringModel->clusterCanEscape(n, c);
    }

    IClusteringModel *getClusteringModel() { return theClusteringModel; }

    void setClusteringModel(IClusteringModel * const model) {
      theClusteringModel = model;
    }

    void deleteClusteringModel() {
      delete theClusteringModel;
      theClusteringModel = NULL;
    }

    void initialize(Config const * const theConfig) {
      ClusterAlgorithmType clusterAlgorithm = theConfig->getClusterAlgorithm();
      if(clusterAlgorithm == IntercomparisonClusterAlgorithm)
        setClusteringModel(new ClusteringModelIntercomparison(theConfig));
      else // if(clusterAlgorithm == NoClusterAlgorithm)
        setClusteringModel(new ClusteringModelNone);
    }

  }
}
