/** \file G4INCLCoulombDistortion.cc
 * \brief Static class for selecting Coulomb distortion.
 *
 * \date 14 February 2011
 * \author Davide Mancusi
 */

#include "G4INCLCoulombDistortion.hh"
#include "G4INCLCoulombNone.hh"
#include "G4INCLCoulombNonRelativistic.hh"

namespace G4INCL {

  namespace CoulombDistortion {

    namespace {
      G4ThreadLocal ICoulomb* theCoulomb = 0;
    }

    ParticleEntryAvatar *bringToSurface(Particle *p, Nucleus * const n) {
      return theCoulomb->bringToSurface(p, n);
    }

    IAvatarList bringToSurface(Cluster * const c, Nucleus * const n) {
      return theCoulomb->bringToSurface(c, n);
    }

    void distortOut(ParticleList const &pL, Nucleus const * const n) {
      theCoulomb->distortOut(pL, n);
    }

    double maxImpactParameter(ParticleSpecies const &p, const double kinE, Nucleus const * const n) {
      return theCoulomb->maxImpactParameter(p, kinE, n);
    }

    double maxImpactParameter(Particle const * const p, Nucleus const * const n) {
      return maxImpactParameter(p->getSpecies(), p->getKineticEnergy(), n);
    }

    void setCoulomb(ICoulomb * const coulomb) { theCoulomb = coulomb; }

    void deleteCoulomb() {
      delete theCoulomb;
      theCoulomb = 0;
    }

    void initialize(Config const * const theConfig) {
      CoulombType coulombType = theConfig->getCoulombType();
      if(coulombType == NonRelativisticCoulomb)
        setCoulomb(new CoulombNonRelativistic);
      else if(coulombType == NoCoulomb)
        setCoulomb(new CoulombNone);
      else
        setCoulomb(NULL);
    }
  }

}
