/** \file G4INCLCrossSectionsTruncatedMultiPions.cc
 * \brief Truncated multipion cross sections
 *
 * \date 10th December 2014
 * \author Davide Mancusi
 */

#include "G4INCLCrossSectionsTruncatedMultiPions.hh"
#include <cassert>

namespace G4INCL {

  CrossSectionsTruncatedMultiPions::CrossSectionsTruncatedMultiPions(const int nPi) :
    nMaxPi(nPi)
  {}

  double CrossSectionsTruncatedMultiPions::elastic(Particle const * const p1, Particle const * const p2) {
    if(!p1->isPion() && !p2->isPion()) {
      return NNElastic(p1, p2);
    } else if (p1->isNucleon() || p2->isNucleon()) {
      double pielas = piNTot(p1,p2) - piNIne(p1,p2) - CrossSectionsMultiPions::piNToDelta(p1,p2);
      if (pielas < 0.)
        pielas = 0.;
      return pielas;
    } else
      return 0.0;
  }

  double CrossSectionsTruncatedMultiPions::piNToDelta(Particle const * const particle1, Particle const * const particle2) {
    double sum = CrossSectionsMultiPions::piNToDelta(particle1, particle2);
    if(nMaxPi<=1) {
      for(int i=nMaxPi+1; i<=nMaxPiPiN; ++i)
        sum += CrossSectionsMultiPions::piNToxPiN(i, particle1, particle2);
    }
    return sum;
  }

  double CrossSectionsTruncatedMultiPions::NNToxPiNN(const int xpi, Particle const * const particle1, Particle const * const particle2) {
    if(xpi<nMaxPi)
      return CrossSectionsMultiPions::NNToxPiNN(xpi, particle1, particle2);
    else if(xpi==nMaxPi) {
      double sum = 0.;
      for(int i=xpi; i<=nMaxPiNN; ++i)
        sum += CrossSectionsMultiPions::NNToxPiNN(i, particle1, particle2);
      return sum;
    } else
      return 0.;
  }

  double CrossSectionsTruncatedMultiPions::piNToxPiN(const int xpi, Particle const * const particle1, Particle const * const particle2) {
    if(xpi<nMaxPi)
      return CrossSectionsMultiPions::piNToxPiN(xpi, particle1, particle2);
    else if(xpi==nMaxPi) {
      double sum = 0.;
      for(int i=xpi; i<=nMaxPiPiN; ++i)
        sum += CrossSectionsMultiPions::piNToxPiN(i, particle1, particle2);
      return sum;
    } else
      return 0.;
  }

} // namespace G4INCL

