/** \file G4INCLDeJongSpin.cc
 * \brief Simple class implementing De Jong's spin model for nucleus-nucleus
 *        collisions
 *
 * Reference: De Jong, Ignatyuk and Schmidt, Nucl. Phys. A613 (1997) 435-444.
 *
 * \date 2 April 2012
 * \author Davide Mancusi
 */

#include "G4INCLGlobals.hh"
#include "G4INCLRandom.hh"
#include "G4INCLDeJongSpin.hh"

namespace G4INCL {

  namespace DeJongSpin {

    namespace {

      const double jzFactor = PhysicalConstants::hcSquared * 0.16;

      double getSpinCutoffParameter(const int Ap, const int Af) {
        const double jz2 = jzFactor * Math::pow23((double) Ap); // No deformation assumed
        const double sigma = jz2 * Af*(Ap-Af)/((double)(Ap-1));
        return std::sqrt(sigma);
      }

    }

    ThreeVector shoot(const int Ap, const int Af) {
      return Random::gaussVector(getSpinCutoffParameter(Ap, Af));
    }
  }
}
