/** \file G4INCLNuclearPotentialEnergyIsospin.cc
 * \brief Isospin- and energy-dependent nuclear potential.
 *
 * Provides an isospin- and energy-dependent nuclear potential.
 *
 * \date 21 March 2011
 * \author Davide Mancusi
 */

#include "G4INCLNuclearPotentialEnergyIsospin.hh"
#include "G4INCLParticleTable.hh"
#include "G4INCLGlobals.hh"

namespace G4INCL {

  namespace NuclearPotential {

    const double NuclearPotentialEnergyIsospin::alpha= 0.223;

    // Constructors
    NuclearPotentialEnergyIsospin::NuclearPotentialEnergyIsospin(const int A, const int Z, const bool aPionPotential)
      : NuclearPotentialIsospin(A,Z,aPionPotential)
    {}

    // Destructor
    NuclearPotentialEnergyIsospin::~NuclearPotentialEnergyIsospin() {}

    double NuclearPotentialEnergyIsospin::computePotentialEnergy(const Particle *particle) const {

      const double v0 = NuclearPotentialIsospin::computePotentialEnergy(particle);

      if(particle->isNucleon()) {
        const double t = particle->getKineticEnergy();
        const double tf = getFermiEnergy(particle);
        // Constant potential for T<Tf
        if(t < tf)
          return v0;

        // Linear function for T>Tf
        const double v = v0 - alpha*(t-tf)/(1-alpha);
        return (v>0.0) ? v : 0.0; // return 0.0 if v is negative
      } else
        return v0;
    }

  }
}

