#include "G4INCLRandom.hh"
#include "G4INCLPauliBlocking.hh"
#include "G4INCLGlobals.hh"

#include "G4INCLPauliStrict.hh"
#include "G4INCLPauliStandard.hh"
#include "G4INCLPauliStrictStandard.hh"
#include "G4INCLPauliGlobal.hh"
#include "G4INCLCDPP.hh"

namespace G4INCL {

  namespace Pauli {

    namespace {
      G4ThreadLocal IPauli * thePauliBlocker = NULL;
      G4ThreadLocal IPauli * theCDPP = NULL;
    }

    IPauli * getBlocker() { return thePauliBlocker; }

    IPauli * getCDPP() { return theCDPP; }

    void setBlocker(IPauli * const pauliBlocker) {
      thePauliBlocker = pauliBlocker;
    }

    void setCDPP(IPauli * const cdpp) {
      theCDPP = cdpp;
    }

    bool isBlocked(ParticleList const &modifiedAndCreated, Nucleus const * const nucleus) {
      bool isPauliBlocked = false;
      if(thePauliBlocker != 0) {
        isPauliBlocked = thePauliBlocker->isBlocked(modifiedAndCreated, nucleus);
      }

      return isPauliBlocked;
    }

    bool isCDPPBlocked(ParticleList const &created, Nucleus const * const nucleus) {
      bool isCDPPBlocked = false;
      if(theCDPP != 0) {
        isCDPPBlocked = theCDPP->isBlocked(created, nucleus);
      }

      return isCDPPBlocked;
    }

    void deleteBlockers() {
      delete thePauliBlocker;
      thePauliBlocker=NULL;
      delete theCDPP;
      theCDPP=NULL;
    }

    void initialize(Config const * const aConfig) {
      // Select the Pauli blocking algorithm:
      PauliType pauli = aConfig->getPauliType();
      if(pauli == StrictStatisticalPauli)
        setBlocker(new PauliStrictStandard);
      else if(pauli == StatisticalPauli)
        setBlocker(new PauliStandard);
      else if(pauli == StrictPauli)
        setBlocker(new PauliStrict);
      else if(pauli == GlobalPauli)
        setBlocker(new PauliGlobal);
      else if(pauli == NoPauli)
        setBlocker(NULL);

      if(aConfig->getCDPP())
        setCDPP(new CDPP);
      else
        setCDPP(NULL);

    }
  }
}
