#include "G4INCLPauliGlobal.hh"
#include "G4INCLRandom.hh"

namespace G4INCL {

  PauliGlobal::PauliGlobal() {}
  PauliGlobal::~PauliGlobal() {}

  bool PauliGlobal::isBlocked(ParticleList const &pL, Nucleus const * const n) {
    for(ParticleIter p=pL.begin(), e=pL.end(); p!=e; ++p) {
      // Pauli blocking only applies to nucleons
      if(!(*p)->isNucleon()) continue;

      // If the particle is above T_F, it is never blocked
      const ParticleType t = (*p)->getType();
      const double pFermi = n->getPotential()->getFermiMomentum(t);
      const double pFermiSquared = pFermi*pFermi;
      if((*p)->getMomentum().mag2() > pFermiSquared) continue;

      // Count particles of the same type as p below the Fermi sea
      ParticleList const &particles = n->getStore()->getParticles();
      int nSea = 0;
      for(ParticleIter i=particles.begin(), end=particles.end(); i!=end; ++i) {
        if((*i)->getType() != t) continue;
        const double pmod2 = (*i)->getMomentum().mag2();
        if(pmod2<pFermiSquared) nSea++;
      }

      // Compute the blocking probability
      double probBlocking;
      if(t==Proton)
        probBlocking = ((double) nSea)/((double) n->getInitialZ());
      else
        probBlocking = ((double) nSea)/((double) (n->getInitialA() - n->getInitialZ()));

      // The avatar is blocked if any particle is blocked
      if(Random::shoot() < probBlocking) return true;

    }

    // Not blocked
    return false;

  }
}
