#include "G4INCLPiNElasticChannel.hh"
#include "G4INCLKinematicsUtils.hh"
#include "G4INCLBinaryCollisionAvatar.hh"
#include "G4INCLRandom.hh"
#include "G4INCLGlobals.hh"
#include "G4INCLLogger.hh"

namespace G4INCL {

    PiNElasticChannel::PiNElasticChannel(Particle *p1, Particle *p2)
    : particle1(p1), particle2(p2)
    {

    }

    PiNElasticChannel::~PiNElasticChannel(){

    }

    void PiNElasticChannel::fillFinalState(FinalState *fs) {
        Particle * nucleon;
        Particle * pion;
        if(particle1->isNucleon()) {
            nucleon = particle1;
            pion = particle2;
        } else {
            nucleon = particle2;
            pion = particle1;
        }

        /*    ParticleType deltaType = DeltaZero;
         if(ParticleConfig::isPair(particle1, particle2, Proton, PiPlus)) {
         deltaType = DeltaPlusPlus;
         } else if(ParticleConfig::isPair(particle1, particle2, Neutron, PiPlus)) {
         deltaType = DeltaPlus;
         } else if(ParticleConfig::isPair(particle1, particle2, Proton, PiZero)) {
         deltaType = DeltaPlus;
         } else if(ParticleConfig::isPair(particle1, particle2, Neutron, PiZero)) {
         deltaType = DeltaZero;
         } else if(ParticleConfig::isPair(particle1, particle2, Proton, PiMinus)) {
         deltaType = DeltaZero;
         } else if(ParticleConfig::isPair(particle1, particle2, Neutron, PiMinus)) {
         deltaType = DeltaMinus;
         } else {
         INCL_ERROR("Unknown particle pair in Pi-N collision." << '\n');
         }

         double deltaEnergy = nucleon->getEnergy()+ pion->getEnergy();

         nucleon->setType(deltaType); // nucleon becomes the delta
         nucleon->setEnergy(deltaEnergy); // set the energy of the delta

         ThreeVector deltaMomentum = nucleon->getMomentum() + pion->getMomentum();
         nucleon->setMomentum(deltaMomentum);

         const double deltaMass = std::sqrt(deltaEnergy*deltaEnergy - deltaMomentum.mag2());
         nucleon->setMass(deltaMass);
         nucleon->setHelicity(1.0);

         fs->addModifiedParticle(nucleon); // nucleon became a delta
         fs->addDestroyedParticle(pion);  // pion was removed
         fs->setDeltaFixed(true);
         */
        double bpn=8e-6;  // B-parameter of exp(Bt) - (MeV/c)^-2
        double px_nucleon=nucleon->getMomentum().getX();
        double py_nucleon=nucleon->getMomentum().getY();
        double pz_nucleon=nucleon->getMomentum().getZ();
        double cnorm1=px_nucleon*px_nucleon+py_nucleon*py_nucleon;
        double cnorm2=std::sqrt(cnorm1);
        double tnorm=cnorm1+pz_nucleon*pz_nucleon;
        double cnorm=std::sqrt(tnorm);
        double btm=std::exp(-4.*tnorm*bpn);
        double rndm=Random::shoot();
        double yrn=1.-rndm*(1.-btm);
        double tt=std::log(yrn)/bpn;
        double ctheta=1.+0.5*tt/tnorm;
        double stheta=std::sqrt(1.-ctheta*ctheta);
        double t7=1.-2.*Random::shoot();
        double t8=std::sqrt(1-t7*t7);
        double t1=-py_nucleon/cnorm2;
        double t2=px_nucleon/cnorm2;
        double t4=t2*pz_nucleon/cnorm;
        double t5=-t1*pz_nucleon/cnorm;
        double t6=-cnorm2/cnorm;
        ThreeVector mom_nucleon(
                                px_nucleon*ctheta+cnorm*stheta*(t1*t7+t4*t8),
                                py_nucleon*ctheta+cnorm*stheta*(t2*t7+t5*t8),
                                pz_nucleon*ctheta+cnorm*stheta*t6*t8
                                );
        nucleon->setMomentum(mom_nucleon);
        pion->setMomentum(-mom_nucleon);

        int iso=ParticleTable::getIsospin(nucleon->getType())+ParticleTable::getIsospin(pion->getType());
        if (iso == 1 || iso == -1) {
            rndm=3*Random::shoot();
            if (rndm < 1.) {
                ParticleType nucleonType=ParticleTable::getNucleonType(-iso);
                nucleon->setType(nucleonType);
                ParticleType pionType=ParticleTable::getPionType(2*iso);
                pion->setType(pionType);
            }
            else {
                ParticleType nucleonType=ParticleTable::getNucleonType(iso);
                nucleon->setType(nucleonType);
                pion->setType(PiZero);
            }
        }
        else {
            ParticleType nucleonType=ParticleTable::getNucleonType(iso/3);
            nucleon->setType(nucleonType);
            ParticleType pionType=ParticleTable::getPionType(2*iso/3);
            pion->setType(pionType);
        }

        fs->addModifiedParticle(nucleon);
        fs->addModifiedParticle(pion);
    }

}
