#include "G4INCLPiNToDeltaChannel.hh"
#include "G4INCLKinematicsUtils.hh"
#include "G4INCLBinaryCollisionAvatar.hh"
#include "G4INCLRandom.hh"
#include "G4INCLGlobals.hh"
#include "G4INCLLogger.hh"

namespace G4INCL {

  PiNToDeltaChannel::PiNToDeltaChannel(Particle *p1, Particle *p2)
    : particle1(p1), particle2(p2)
  {

  }

  PiNToDeltaChannel::~PiNToDeltaChannel(){

  }

  void PiNToDeltaChannel::fillFinalState(FinalState *fs) {
    Particle * nucleon;
    Particle * pion;
    if(particle1->isNucleon()) {
      nucleon = particle1;
      pion = particle2;
    } else {
      nucleon = particle2;
      pion = particle1;
    }

    ParticleType deltaType = DeltaZero;
    if(ParticleConfig::isPair(particle1, particle2, Proton, PiPlus)) {
      deltaType = DeltaPlusPlus;
    } else if(ParticleConfig::isPair(particle1, particle2, Neutron, PiPlus)) {
      deltaType = DeltaPlus;
    } else if(ParticleConfig::isPair(particle1, particle2, Proton, PiZero)) {
      deltaType = DeltaPlus;
    } else if(ParticleConfig::isPair(particle1, particle2, Neutron, PiZero)) {
      deltaType = DeltaZero;
    } else if(ParticleConfig::isPair(particle1, particle2, Proton, PiMinus)) {
      deltaType = DeltaZero;
    } else if(ParticleConfig::isPair(particle1, particle2, Neutron, PiMinus)) {
      deltaType = DeltaMinus;
    } else {
      INCL_ERROR("Unknown particle pair in Pi-N collision." << '\n');
    }

    double deltaEnergy = nucleon->getEnergy()+ pion->getEnergy();

    nucleon->setType(deltaType); // nucleon becomes the delta
    nucleon->setEnergy(deltaEnergy); // set the energy of the delta

    ThreeVector deltaMomentum = nucleon->getMomentum() + pion->getMomentum();
    nucleon->setMomentum(deltaMomentum);

    const double deltaMass = std::sqrt(deltaEnergy*deltaEnergy - deltaMomentum.mag2());
    nucleon->setMass(deltaMass);

    fs->addModifiedParticle(nucleon); // nucleon became a delta
    fs->addDestroyedParticle(pion);  // pion was removed
  }

}
