#include "HDF5Writer.hh"

#ifdef INCL_HDF5_USE
namespace G4INCL {
  namespace IO {

    HDF5Writer::HDF5Writer()
      :outputStreamOpen(false)
    {}

    HDF5Writer::~HDF5Writer() {
      if(outputStreamOpen) closeFile();
    }

    bool HDF5Writer::openFile(std::string filename) {
      //      std::ofstream output(filename.c_str());
      fileID = H5Fcreate ("test.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
      //      herr_t status = H5LTmake_dataset(fileID,

      /*
       * Create dataspace.  Setting maximum size to NULL sets the maximum
       * size to be the current size.
       */
      dims[0] = 6;
      dims[1] = 1;

      space = H5Screate_simple (2, dims, NULL);

      /*
       * Create the dataset.  We will use all default properties for this
       * example.
       */
#define DATASET "INCLPARTICLES"
      dset = H5Dcreate (fileID, DATASET, H5T_STD_I32LE, space, H5P_DEFAULT);

      outputStreamOpen = true;
      return true;
    }

    bool HDF5Writer::writeParticle(int A, int Z, double E, double P, double theta, double phi) {
      int         data[6]={(double) A, (double) Z, E,P,theta,phi};
      herr_t      status;
      double write_buffer[6][8];
      //      status = H5LTmake_dataset(fileID,"/dset",2,dims,H5T_NATIVE_DOUBLE,data);
      H5Dwrite (dset, H5T_NATIVE_DOUBLE, H5S_ALL, space, H5P_DEFAULT,
                data);
      return true;
    }

    bool HDF5Writer::closeFile() {
      H5Dclose(dset);
      H5Sclose(space);
      H5Fclose(fileID);
      outputStreamOpen = false;
      return true;
    }
  }
}

#endif
