#include "ProtobufWriter.hh"
#include "G4INCLLogger.hh"

#ifdef INCL_PROTOBUF_USE

namespace G4INCL {
  namespace IO {

    ProtobufWriter::ProtobufWriter()
      :outputStreamOpen(false)
    {
      GOOGLE_PROTOBUF_VERIFY_VERSION;
    }

    ProtobufWriter::~ProtobufWriter() {
      google::protobuf::ShutdownProtobufLibrary();
      if(outputStreamOpen) closeFile();
      // gzip_stream->close();
      // delete gzip_steam;
      // file_stream->close();
      // delete file_stream;

    }

    bool ProtobufWriter::openFile(std::string filename) {
      fd = open(filename.c_str(), O_WRONLY | O_CREAT | O_TRUNC,
                S_IREAD | S_IWRITE);
      if (fd == -1) {
	throw "open failed on output file";
      }
      file_stream = new google::protobuf::io::FileOutputStream(fd);
      //google::protobuf::io::FileOutputStream file_stream(fd);
      google::protobuf::io::GzipOutputStream::Options options;
      options.format = google::protobuf::io::GzipOutputStream::GZIP;
      unsigned int compressionLevel = 9;
      options.compression_level = compressionLevel;
      //      google::protobuf::io::GzipOutputStream gzip_stream(&file_stream, options);
      gzip_stream = new google::protobuf::io::GzipOutputStream(file_stream, options);
      //      outputStream = new std::ofstream(filename.c_str(),
      //				       std::ios::out | std::ios::trunc | std::ios::binary);
      outputStreamOpen = true;
      return true;
    }

    bool ProtobufWriter::writeEventInfo(G4INCL::EventInfo const &eventInfo) {
      eventio::Event event;
      eventio::EventHeader *event_header = event.mutable_header();
      eventio::Target *target = event_header->mutable_target();
      target->set_z(eventInfo.At);
      target->set_a(eventInfo.Zt);
      eventio::Particle *projectile = event_header->mutable_projectile();
      projectile->set_a(eventInfo.Ap);
      projectile->set_z(eventInfo.Zp);
      projectile->set_energy(eventInfo.Ep);
      projectile->set_theta(0.0);
      projectile->set_phi(0.0);

      event_header->set_impactparameter(eventInfo.impactParameter);
      event_header->set_effectiveimpactparameter(eventInfo.effectiveImpactParameter);
      event_header->set_stoppingtime(eventInfo.stoppingTime);
      event_header->set_forceddeltasinside(eventInfo.forcedDeltasInside);
      event_header->set_forceddeltasoutside(eventInfo.forcedDeltasOutside);
      event_header->set_deltasinside(eventInfo.deltasInside);
      event_header->set_clusterdecay(eventInfo.clusterDecay);
      event_header->set_energybalance(eventInfo.EBalance);
      event_header->set_plongbalance(eventInfo.pLongBalance);
      event_header->set_firstcollisiontime(eventInfo.firstCollisionTime);
      event_header->set_firstcollisionxsec(eventInfo.firstCollisionXSec);
      event_header->set_firstcollisionspectatorposition(eventInfo.firstCollisionSpectatorPosition);
      event_header->set_firstcollisionspectatormomentum(eventInfo.firstCollisionSpectatorMomentum);
      event_header->set_firstcollisioniselastic(eventInfo.firstCollisionIsElastic);
      event_header->set_ptransbalance(eventInfo.pTransBalance);
      event_header->set_ncollisions(eventInfo.nCollisions);
      event_header->set_nblockedcollisions(eventInfo.nBlockedCollisions);
      event_header->set_ndecays(eventInfo.nDecays);
      event_header->set_nblockeddecays(eventInfo.nBlockedDecays);
      event_header->set_ncascadeparticles(eventInfo.nCascadeParticles);
      event_header->set_nunmergedspectators(eventInfo.nUnmergedSpectators);
      event_header->set_nremnants(eventInfo.nRemnants);
      event_header->set_nparticles(eventInfo.nParticles);

      for( int i=0; i<eventInfo.nParticles; i++ ) {
	eventio::Particle *particle = event.add_particle();
	particle->set_a(eventInfo.A[i]);
	particle->set_z(eventInfo.Z[i]);
	particle->set_energy(eventInfo.EKin[i]);
	particle->set_theta(eventInfo.theta[i]);
	particle->set_phi(eventInfo.phi[i]);
	particle->set_px(eventInfo.px[i]);
	particle->set_py(eventInfo.py[i]);
	particle->set_pz(eventInfo.pz[i]);
	particle->set_emissiontime(eventInfo.emissionTime[i]);
	if(eventInfo.origin[i] == -1) {
	  particle->set_origin(eventio::CASCADEORIGIN);
	} else if(eventInfo.origin[i] >= 1) {
	  particle->set_origin(eventio::DEEXCITATIONORIGIN);
	} else {
	  particle->set_origin(eventio::UNKNOWNORIGIN);
	}
      }

      for( int i=0; i<eventInfo.nRemnants; i++ ) {
	eventio::Fragment *remnant = event.add_fragment();
	remnant->set_a(eventInfo.ARem[i]);
	remnant->set_z(eventInfo.ZRem[i]);
	remnant->set_excitationenergy(eventInfo.EStarRem[i]);
	remnant->set_angularmomentum(eventInfo.JRem[i]);
	remnant->set_kineticenergy(eventInfo.EKinRem[i]);
	remnant->set_px(eventInfo.pxRem[i]);
	remnant->set_py(eventInfo.pyRem[i]);
	remnant->set_pz(eventInfo.pzRem[i]);
	remnant->set_theta(eventInfo.thetaRem[i]);
	remnant->set_phi(eventInfo.phiRem[i]);
      }

      if (!event.SerializeToZeroCopyStream(gzip_stream)) {
	INCL_ERROR("Failed to serialize event!");
	return false;
      }

      return true;
    }

    bool ProtobufWriter::writeGlobalInfo(G4INCL::GlobalInfo const &globalInfo) {
      // (*outputStream) << "g " << globalInfo.nShots << '\t' <<
      //   globalInfo.nTransparents << '\t' << globalInfo.nNucleonAbsorptions << '\t' << globalInfo.nPionAbsorptions << '\t' << globalInfo.geometricCrossSection <<
      //   '\t' << globalInfo.reactionCrossSection << '\t' <<
      //   globalInfo.errorReactionCrossSection << '\t' << globalInfo.nucleonAbsorptionCrossSection << '\t' << globalInfo.pionAbsorptionCrossSection << '\t' << globalInfo.Ap << '\t'
      //   << globalInfo.Zp << '\t' << globalInfo.Ep << '\t' << globalInfo.At <<
      //   '\t' << globalInfo.Zt << '\t' << globalInfo.cascadeModel << '\t' <<
      //   globalInfo.deexcitationModel << '\n';

      return true;
    }

    bool ProtobufWriter::closeFile() {
      //      outputStream->close();
      gzip_stream->Close();
      file_stream->Close();
      delete gzip_stream;
      delete file_stream;
      close(fd);
      outputStreamOpen = false;
      return true;
    }

    void ProtobufWriter::flush() {
      gzip_stream->Flush();
      file_stream->Flush();
  }
}

#endif
