#include "G4INCLParticleType.hh"
#include "G4INCLConfig.hh"
#include "G4INCLParticleSpecies.hh"
#include "G4INCLParticleTable.hh"

namespace G4INCL {

  Config::Config() {
    init();
  }

  Config::~Config() {}

  void Config::init() {
    verbosity = 1;
    logFileName = "-";
    inputFileName = "";
    title = "INCL default run title";
    nShots = 1000;
    naturalTarget = false;
    projectileString = "proton";
    projectileSpecies = G4INCL::Proton;
    projectileKineticEnergy = 1000.0;
    verboseEvent = -1;
    randomSeeds = "";
    randomSeedVector.push_back(666);
    randomSeedVector.push_back(777);
    randomSeedVector.push_back(1234);
    pauliString = "strict-statistical";
    pauliType = StrictStatisticalPauli;
    CDPP = true;
    coulombString = "non-relativistic";
    coulombType = NonRelativisticCoulomb;
    potentialString = "isospin-energy";
    potentialType = IsospinEnergyPotential;
    pionPotential = true;
    localEnergyBBString = "first-collision";
    localEnergyBBType = FirstCollisionLocalEnergy;
    localEnergyPiString = "first-collision";
    localEnergyPiType = FirstCollisionLocalEnergy;
    deExcitationString = "none";
    deExcitationType = DeExcitationNone;
    clusterAlgorithmString = "intercomparison";
    clusterAlgorithmType = IntercomparisonClusterAlgorithm;
    clusterMaxMass = 8;
    backToSpectator = true;
    useRealMasses = true;
    impactParameter = -1.;
    separationEnergyString = "INCL";
    separationEnergyType = INCLSeparationEnergy;
    fermiMomentumString = "constant";
    fermiMomentumType = ConstantFermiMomentum;
    fermiMomentum = -1.;
    cutNN = 1910.;
#ifdef INCL_DEEXCITATION_FERMI_BREAKUP
    maxMassFermiBreakUp = 16;
    maxChargeFermiBreakUp = 8;
#endif
    rpCorrelationCoefficient = 1.;
    rpCorrelationCoefficientProton = 1.;
    rpCorrelationCoefficientNeutron = 1.;
    neutronSkin = 0.;
    neutronHalo = 0.;
    refraction=false;
    phaseSpaceGenerator = "Raubold-Lynch";
    phaseSpaceGeneratorType = RauboldLynchType;
    cascadeAction = "default";
    cascadeActionType = DefaultActionType;
    randomNumberGenerator = "Ranecu";
    rngType = RanecuType;
    autosaveFrequency = 10000;
    maxNumberMultipions = -1;
    crossSectionsString = "multipions";
    crossSectionsType = MultiPionsCrossSections;
    hadronizationTime = 0.;
#ifdef INCL_ROOT_USE
    conciseROOTTree = false;
#endif
    inverseKinematics = false;
  }

  std::string Config::summary() {
    std::stringstream message;
    message << "INCL++ version " << getVersionString() << '\n';
    if(projectileSpecies.theType != Composite)
      message << "Projectile: " << ParticleTable::getName(projectileSpecies) << '\n';
    else
      message << "Projectile: composite, A=" << projectileSpecies.theA << ", Z=" << projectileSpecies.theZ << '\n';
    message << "  energy = " << projectileKineticEnergy << '\n';
    if(targetSpecies.theA>0)
      message << "Target: A = " << targetSpecies.theA << " Z = " << targetSpecies.theZ << '\n';
    else
      message << "Target: natural isotopic composition, Z = " << targetSpecies.theZ << '\n';
    message << "Number of requested shots = " << nShots << '\n';
    return message.str();
  }

}
