#include "G4INCLFinalState.hh"

namespace G4INCL {

  FinalState::FinalState() {
    reset();
  }

  FinalState::~FinalState()
  {
  }

  void FinalState::reset() {
    totalEnergyBeforeInteraction = 0.0;
    validity = ValidFS;
    outgoing.clear();
    created.clear();
    destroyed.clear();
    modified.clear();
    entering.clear();
  }

  void FinalState::addModifiedParticle(Particle *p)
  {
    modified.push_back(p);
  }

  void FinalState::addOutgoingParticle(Particle *p)
  {
    outgoing.push_back(p);
  }

  void FinalState::addDestroyedParticle(Particle *p)
  {
    destroyed.push_back(p);
  }

  void FinalState::addCreatedParticle(Particle *p)
  {
    created.push_back(p);
  }

  void FinalState::addEnteringParticle(Particle *p)
  {
    entering.push_back(p);
  }

  ParticleList const &FinalState::getModifiedParticles() const
  {
    return modified;
  }

  ParticleList const &FinalState::getOutgoingParticles() const
  {
    return outgoing;
  }

  ParticleList const &FinalState::getDestroyedParticles() const
  {
    return destroyed;
  }

  ParticleList const &FinalState::getCreatedParticles() const
  {
    return created;
  }

  ParticleList const &FinalState::getEnteringParticles() const
  {
    return entering;
  }

  std::string FinalState::print() const {
    std::stringstream ss;
    ss << "Modified particles:" << '\n';
    for(ParticleIter iter=modified.begin(), e=modified.end(); iter!=e; ++iter)
      ss << (*iter)->print();
    ss << "Outgoing particles:" << '\n';
    for(ParticleIter iter=outgoing.begin(), e=outgoing.end(); iter!=e; ++iter)
      ss << (*iter)->print();
    ss << "Destroyed particles:" << '\n';
    for(ParticleIter iter=destroyed.begin(), e=destroyed.end(); iter!=e; ++iter)
      ss << (*iter)->print();
    ss << "Created particles:" << '\n';
    for(ParticleIter iter=created.begin(), e=created.end(); iter!=e; ++iter)
      ss << (*iter)->print();
    ss << "Entering particles:" << '\n';
    for(ParticleIter iter=entering.begin(), e=entering.end(); iter!=e; ++iter)
      ss << (*iter)->print();
    return ss.str();
  }

}
