/*
 * IAvatar.cc
 *
 *  \date 4 juin 2009
 * \author Pekka Kaitaniemi
 */

#include "G4INCLIAvatar.hh"
#include "G4INCLRandom.hh"
#include <sstream>

namespace G4INCL {

  G4ThreadLocal long IAvatar::nextID = 1;

  IAvatar::IAvatar() :
    type(UnknownAvatarType),
    theTime(0.)
  {
    ID = nextID;
    nextID++;
  }

  IAvatar::IAvatar(double time) :
    type(UnknownAvatarType),
    theTime(time)
  {
    ID = nextID;
    nextID++;
  }

  IAvatar::~IAvatar() {
    INCL_DEBUG("destroying avatar " << this << std::endl);
  }

  std::string IAvatar::toString() {
    std::stringstream entry;
    std::stringstream particleString;
    ParticleList const &pl = getParticles();
    int numberOfParticles = 0;
    for(ParticleIter i=pl.begin(), e=pl.end(); i!=e; ++i) {
      numberOfParticles++;
      particleString << (*i)->getID() << " ";
    }
    if(numberOfParticles == 1) particleString << "-1";
    entry << getID() << " "
	  << getType() << " "
	  << getTime() << " "
	  << particleString.str();
    return entry.str();
  }

  FinalState *IAvatar::getFinalState() {
    FinalState *fs = new FinalState;
    fillFinalState(fs);
    return fs;
  }

  void IAvatar::fillFinalState(FinalState *fs) {
    INCL_DEBUG("Random seeds before preInteraction: " << Random::getSeeds() << '\n');
    preInteraction();
    INCL_DEBUG("Random seeds before getChannel: " << Random::getSeeds() << '\n');
    IChannel *c = getChannel();
    if( !c )
      return;
    INCL_DEBUG("Random seeds before getFinalState: " << Random::getSeeds() << '\n');
    c->fillFinalState(fs);
    INCL_DEBUG("Random seeds before postInteraction: " << Random::getSeeds() << '\n');
    postInteraction(fs);
    delete c;
  }

}
