/** \file G4INCLInvFInterpolationTable.cc
 * \brief Simple interpolation table for the inverse of a IFunction1D functor
 *
 * \date 17 July 2012
 * \author Davide Mancusi
 */

#include <cassert>
#include <algorithm>
#include <functional>
#include "G4INCLInvFInterpolationTable.hh"

namespace G4INCL {

  InvFInterpolationTable::InvFInterpolationTable(IFunction1D const &f, const unsigned int nNodes) {
    assert(nNodes>2);

    const double x0 = f.getXMinimum();
    const double x1 = f.getXMaximum();

    // Build the nodes
    double last = f(x0);
    InterpolationNode firstNode(last, x0, 0.);
    nodes.push_back(firstNode);
    int skippedNodes = 0;
    for(unsigned i = 1; i < nNodes; i++) {
      const double xi = x0 + i*(x1-x0)/((double)(nNodes-1));
      // Make sure that the x vector is sorted (corresponding to a monotonous
      // function)
      const double value = f(xi);
      if(value <= last) {
        ++skippedNodes;
        continue;
      }
      InterpolationNode node(value, xi, 0.);
      nodes.push_back(node);
      last = value;
    }

    assert(nNodes==nodes.size()+skippedNodes);

    // Initialise the "derivative" values
    initDerivatives();
  }

}
