#include "G4INCLLogger.hh"
#include "G4INCLGlobals.hh"

#ifdef INCLXX_IN_GEANT4_MODE
#include <cstdlib>
#endif

namespace G4INCL {
#if defined(INCL_DEBUG_LOG) && !defined(INCLXX_IN_GEANT4_MODE)
  std::string typeToString(const MessageType t) {
    if(t == ErrorMsg)
      return std::string("Error");
    else if(t == FatalMsg)
      return std::string ("Fatal");
    else if(t == WarningMsg)
      return std::string("Warning");
    else if(t == DebugMsg)
      return std::string("Debug");
    else if(t == InfoMsg)
      return std::string("");
    else if(t == DataBlockMsg)
      return std::string("DataBlock");
    else
      return std::string("Unknown");
  }

  void LoggerSlave::logMessage(const MessageType type, const std::string &fileName, const int lineNumber, std::string const &s, const bool prefixHash) const {
    std::string cont = (prefixHash ? "\n# " : "\n");
    std::string header = (prefixHash ? "# " : "");
    if(type!=InfoMsg) {
      std::stringstream headerss;
      headerss << typeToString(type) << " [";
      cont += headerss.str();
      headerss <<
        fileName.substr(fileName.find_last_of("/")+1) <<
        ":" << lineNumber << "] ";
      header += headerss.str();
      cont.append(header.size() - cont.size() - 1, '.');
      cont += "] ";
    }

    std::string message(s);
    String::replaceAll(message, "\n", cont, s.size()-2);
    (*logStream) << header << message;

    return;
  }

  void LoggerSlave::logDataBlock(const std::string &block, const std::string &fileName, const int lineNumber) const {
    (*logStream) << typeToString(DataBlockMsg) << " [" <<
      fileName.substr(fileName.find_last_of("/")+1) <<
      ":" << lineNumber << "] " << '\n'
      << "BEGINDATA"
      << block
      << "ENDDATA" << '\n';
  }

  namespace Logger {

    namespace {
      G4ThreadLocal LoggerSlave *theLoggerSlave = NULL;
    }

    void logMessage(const MessageType type, std::string const &fileName, const int lineNumber, std::string const &s, const bool prefixHash) {
      if(theLoggerSlave)
        theLoggerSlave->logMessage(type, fileName, lineNumber, s, prefixHash);
    }

    void flush() {
      if(theLoggerSlave)
        theLoggerSlave->flush();
    }

    void dataBlock(const std::string &block, const std::string &fileName, const int lineNumber) {
      theLoggerSlave->logDataBlock(block, fileName, lineNumber);
    }

    void setLoggerSlave(LoggerSlave * const logger) { theLoggerSlave = logger; }

    void setVerbosityLevel(int lvl) {
      if(theLoggerSlave)
        theLoggerSlave->setVerbosityLevel(lvl);
    }

    int getVerbosityLevel() {
      if(theLoggerSlave)
        return theLoggerSlave->getVerbosityLevel();
      else
        return 0;
    }

    void deleteLoggerSlave() {
      delete theLoggerSlave;
      theLoggerSlave=NULL;
    }

    void initialize(Config const * const theConfig) {
      setLoggerSlave(new LoggerSlave(theConfig->getLogFileName(), theConfig->getVerbosity()));
    }

  }

#else // defined(INCL_DEBUG_LOG) && !defined(INCLXX_IN_GEANT4_MODE)

  namespace Logger {

    namespace {
      G4ThreadLocal int verbosityLevel = 0;
    }

    void initVerbosityLevelFromEnvvar() {
      const char * const envVar = getenv("G4INCL_DEBUG_VERBOSITY");
      if(envVar) {
        std::stringstream verbss(envVar);
        verbss >> verbosityLevel;
      } else {
        verbosityLevel = 0;
      }
    }

    int getVerbosityLevel() {
      return verbosityLevel;
    }

  }

#endif // defined(INCL_DEBUG_LOG) && !defined(INCLXX_IN_GEANT4_MODE)
}
