C***********************************************************************
C
C
C            MODIFIED MONTE-CARLO VERSION OF ABRABLA
C
C  TO DESCRIBE PERIPHERAL COLLISIONS OF RELATIVISTIC HEAVY IONS
C  ON THE  BASIS OF THE ABRASION-ABLATION MODEL.
C
C
C  BASED ON: GAIMARD, SCHMIDT, NUCL. PHYS. A 531 (1991) 709
C            SCHMIDT ET AL., PHYS. LETT. B 300 (1993) 313
C            DE JONG ET AL., NUCL. PHYS. A 628 (1998) 479
C            JUNGHANS ET AL., NUCL. PHYS. A 629 (1998) 635
C
C  FISSION MODEL:
C            BENLLIURE ET AL., NUCL. PHYS. A 628 (1998) 458
C
C  INPUT PARAMETERS FROM INPUT FILE (SEE SUBROUTINE ABRAINIT)
C  SEVERAL PROJECTILE & TARGET COMBINATIONS CAN BE CALCULATED
C
C  OUTPUT: 1. LOG FILE (UNIT=6)
C          2. FILES FOR ISOTOPIC PRODUCTION CROSS SECTIONS
C             FOR EACH ELEMENT
C          3. TABLE FOR PRODUCTION CROSS SEC. ON A CHART OF NUCLEI
C          4. ASCII NTUPLE FOR LISTMODE ANALYSIS (ONLY ONE SINGLE FILE)
C          5. PARTIAL FISSION CROSS SECTIONS
C          6. INTERMEDIATE RESULTS OF CROSS SECTIONS SAVED
C
C  LINK LIBRARIES NEEDED:   (CERN-LIBRARY not needed any more)
C                           BESSEL FUNCTIONS, RANDOM NUMBERS ...
C
C
C***********************************************************************
c
c  August 2002: fission width from the analytical solution of FP equation
c               (see PhD from B.Jurado);
c               subroutine AMOMENT is included to calculate J.

c  3. 12. 2000 New version of BURST model (F. Vives, KHS)
c              Parameters roughly adjusted to p+Pb and d+Pb, 1AGeV
C
C  Mean angular momentum  modified according to
C  M.de Jong, Ignatyuk, Schmidt Nuc.Phys A 613, pg 442, 7th line
C  20-jul-1999 J.B. and M.V.R.
C
C  RN104 copied from RN103 on 26.2.00 (KHS)
C  for further development. BURST included.
C
C  RN103 same as RN102 but evaporation from ABLA_V2.FOR (inlcuding angular
C  momentum in the fission barrier  J.B. 22.2.99
C
C
C  RN102 modified output from RN101 ==> should be used without collective
C  enhancement J.B. 5.2.99
C
C  NEWLY COMMENTED. A.R.J. 19.01.99
C
C  ...
C
C  GRAND UNIFIED PROGRAM INCLUDES NOW ALSO FISSION CHANNEL IN
C  EVAPORATION FROM RN56.F (VERSION RN60, 5/25/93 AUTHORS:
C  G. KUDYAEV, A. R. JUNGHANS)
C
C  CALCULATES THE LONGITUDINAL AND TRANSVERSAL MOMENTUM DISTRIBUTIONS
C  OF PROJECTILE FRAGMENTS (VERSION RN43 , 10/07/92 , AUTHOR: E.HANELT)
C
C  ELECTROMAGNETIC DISSOCIATION HAS BEEN INCLUDED AND FOR CONSISTENCY
C  THE INTERACTION CROSS SECTION IS TAKEN FROM AN EMPIRICAL FORMULA
C  (VERSION RN42 , 08/05/92 , AUTHOR: T. BROHM)
C
C  ORIGINAL MONTE-CARLO-PROGRAM ABRABLA:
C  REACTION OF NUCLEAR FRAGMENTATION FROM THE STATISTICAL ABRASION-
C  ABLATION MODEL (VERION RN41 , 02/10/91 , AUTHOR: J.J.GAIMARD)
C
C
C***********************************************************************
cjcd - 0start      SUBROUTINE MAIN(TSTAT,TSTATEV,IREACT)
      SUBROUTINE Rn07(inum,AABRA,ZABRA,EABRA,PXPRFP,PYPRFP,PZPRFP,JPRF)
cjcd - stop

C
C   ____________________________________________________________________
C  /
C  /  MAIN LOOP: CALCULATION AND OUTPUT OF FRAGMENT PARAMETERS
C  /  THE REACTION (BOTH ELECTROMAGNETIC OR NUCLEAR INTERACTION)
C  /  IS CALCULATED IN SUBROUTINE REANUCL WHILE IN SUBROUTINE
C  /  ABRAINIT VARIABLES FOR THE CALCULATION ARE INITIALIZED.
C  /____________________________________________________________________
C
      INTEGER*4 APRFP,ZPRFP,I,L,IOUT,INTTYPE,TSTATD,IOUT_SAVE
      INTEGER*4 AABRA,ZABRA,AFPIMF,ZFPIMF,ACN,ZCN
      INTEGER*4 AFP,ZFP,AFP1,ZFP1,AFP2,ZFP2,IA,KMAX,ZFIMF,AFIMF
      INTEGER*4 NBNABR,NBPABR,NBNEVA,NBPEVA,NBDEVA,NBTEVA,
     &          NBHEEVA,NBAEVA,NB6HEEVA,NBNIMFP,NBNIMF,NBN_PRESADD,
     &          NBN_PRESCIS,NBP_PRESADD,NBP_PRESCIS,NBA_PRESADD,
     &          NBA_PRESCIS
      INTEGER*4 NBHEEVATOT,NBAEVATOT,NB6HEEVATOT,ISECDEC,IMULTBU
      INTEGER*4 NBNEVAFRA,NBPEVAFRA,NBDEVAFRA,NBTEVAFRA
      INTEGER*4 NBHEEVAFRA,NBAEVAFRA,NB6HEEVAFRA,FSUMS1,FSUMS2
      INTEGER*4 NNTOT,NPTOT,imode,IMULTIFR,IMULT,IFIREBALL,IMULTBUZGT1
      INTEGER*4 IMAX,IRNDM,FSUM,TSTAT,TSTATEV,ZPROF(0:100),COUNT,I1,I2
      INTEGER*4 FF,IPLPRFP,IPTPRFP,IPLEVA,IPTEVA,J,K,FTYPE,FTYPE1
      INTEGER*4 FSEMD,IEMD,ZCHA,JM,KM,IIA,ZPROF_EMD(0:100)
      INTEGER*4 POSTAL1,POSTAL2,POSTN1,POSTN2,POSTP1,POSTP2
      INTEGER*4 IFIS,OPTSHP,OPTXFIS,OPTCOL,OPTAFAN,IWRITE,TSAVE
      INTEGER IBURST,IREACT
      INTEGER NQUAD
      PARAMETER(NQUAD=1000)
C
      REAL*8 ARMAX
      REAL*8 PLEVA,PTEVA,B,ISOTON,ISOTOP,ISOBAR,WQH
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,R_P,R_T,PI,
     &       BFPRO,SNPRO,SPPRO,SHELL
      REAL*8 WQN,WQP,WQD,WQT,WQHE,WQA,WQNTOT,WQPTOT
      REAL*8 WQNFRA,WQPFRA,WQDFRA,WQTFRA,WQHEFRA,WQAFRA,WQ6HEFRA
      REAL*8 EGDR,EGQR,FWHMGDR,FWHMGQR,CREMDE1,CREMDE2
      REAL*8 AE1(NQUAD+1),BE1(NQUAD+1),CE1(NQUAD+1),AE2(NQUAD+1),
     &  BE2(NQUAD+1),CE2(NQUAD+1),SRE1(NQUAD+1),SRE2(NQUAD+1),
     &  XRE1(NQUAD+1),XRE2(NQUAD+1),DS1,DS2
      REAL*8 WQ2(0:165,0:100),WQ(0:165,0:100), SIGA
      REAL*8 WQ2FRA(0:165,0:100),WQ2FIS(0:165,0:100),WQ2IMF(0:165,0:100)
      REAL*8 WQ2BU(0:165,0:100),WQEMD(0:165,0:100),WQ2LP(0:8,0:2),
     &       WQ2FISS1(0:165,0:100),WQ2FISS2(0:165,0:100)
      REAL*8 WQ2BU_FRA(0:165,0:100),WQ2BU_FIS(0:165,0:100),
     &       WQ2BU_IMF(0:165,0:100),WQ2BU_MF(0:165,0:100)
      REAL*8 EPRFP,FEE,ZSUM_frag(0:100),ZSUM_fiss(0:100),ZSUM(0:100),
     &       TOTFIS,NUFISZ,FISZ,ZSUM_IMF(0:100),ZSUM_BU(0:100),
     &       ZSUM_LP(0:100)
      real*8 PLPRFP,PTPRFP,EABRA,VZ_PREF,VX_ABRA,VY_ABRA,VZ_ABRA
      REAL*8 AKAP,BETA_COEFF,HOMEGA,KOEFF
      REAL*8 AV,AS,AK
      REAL*8 JPRF,JPRFFIN,PXABRA,PYABRA,PZABRA
      REAL*8 QPRE,QPOST,TKE_PRE,TKE_POST,EFF1,EFF2
C
      REAL*8 T_FREEZE_OUT_IN,BU_TAB(200,11),EV_TAB(200,5)
      integer hev_tab(200),hbu_tab(200)
      REAL*8 VFP1_CM(3),VFP2_CM(3),VIMF_CM(3)
cjcd - start
      real*8 PXPRFP,PYPRFP,PZPRFP
      common /ablasmop/ isecdec,imultbu,kproc
cjcd - stop

      CHARACTER*100 FILE79NAME

      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
      COMMON /EMDPAR/ EGDR,EGQR,FWHMGDR,FWHMGQR,CREMDE1,CREMDE2,
     &  AE1,BE1,CE1,AE2,BE2,CE2,SRE1,SRE2,XRE1,XRE2,DS1,DS2
      COMMON /FIS/ AKAP,BETA_COEFF,HOMEGA,KOEFF,IFIS,OPTSHP,
     &       OPTXFIS,OPTCOL
      COMMON /ALD/AV,AS,AK,OPTAFAN
      COMMON/NEVENT/I
      COMMON /BREAK_UP/T_freeze_out_IN
      COMMON /EVAPORATION/ EV_TAB,hev_tab
      COMMON/MULTIFRAG/BU_TAB,hbu_tab
      COMMON /VELOCITY/ VFP1_CM,VFP2_CM,VIMF_CM,VZ_PREF
cjcd - start
      real*8 APRFinc,ZPRFinc,EEinc,JPRFjcd,Pxinc,Pyinc,Pzinc
      common /frominc/ APRFinc,ZPRFinc,EEinc,JPRFjcd,Pxinc,Pyinc,Pzinc
      APRFinc=dfloat(AABRA)
      ZPRFinc=dfloat(ZABRA)
      EEinc=EABRA
      JPRFjcd=JPRF
      Pxinc=PXPRFP
      Pyinc=PYPRFP
      Pzinc=PZPRFP
cjcd - stop

C
C-----------------------------------------------------------------------
C LIST OF MOST IMPORTANT VARIABLES:
C
C     AP                MASS NUMBER OF THE PROJECTILE
C     ZP                NUCLEAR CHARGE OF THE PROJECTILE
C     AT                MASS NUMBER OF THE TARGET
C     ZT                PROTON NUMBER OF THE TARGET
C     EAP               ENERGY PER NUCLEON OF PROJECTILE
C     AFP1              MASS NUMBER OF THE FINAL FRAGMENT
C     ZFP1              NUCLEAR CHARGE OF THE FINAL FRAGMENT
C     AFP2              IN CASE OF FISSION: SECOND FISSION FRAGMENT
C     ZFP2
C     FEE               EXCITATION ENERGY OF THE FISSIONING NUCLEUS
C     APRFP             MASS NUMBER OF THE PREFRAGMENT
C     ZPRFP             NUCLEAR CHARGE OF THE PREFRAGMENT
C     EPRFP             EXCITATION ENERGY OF THE PREFRAGMENT
C     NBNABR            NUMBER OF ABRADED NEUTRONS
C     NBPABR            NUMBER OF ABRADED PROTONS
C     NBNEVA            NUMBER OF EVAPORATED NEUTRONS
C     NBPEVA            NUMBER OF EVAPORATED PROTONS
C     NBAEVA            NUMBER OF EVAPORATED ALPHA PARTICLES
C     PLFP              LONGITUDINAL MOMENTUM OF PROJECTILE FRAGMENTS
C     PTFP              TRANSVERSAL MOMENTUM OF PROJECTILE FRAGMENTS
C     PLPRFP            LONGITUDINAL MOMENTUM OF PREFRAGMENTS
C     PTPRFP            TRANSVERSAL MOMENTUM OF PREFRAGMENTS
C     PLEVA             LONGITUDINAL RECOIL MOMENTUM OF EVAPORATION
C     PTEVA             TRANSVERSAL RECOIL MOMENTUM OF EVAPORATION
C     INTTYPE           REACTION TYPE (0 = PROJECTILE FRAGMENTATION,
C                                      1 = ELECTROMAGNETIC DISSOZIATION,
C                                      2 = EMD WITH FISSION,
C                                      3 = NUCLEAR REAKTION WITH FISS.)
C     B                 IMPACT PARAMETER OF THE REACTION
C     BMAXNUC           MAXIMUM IMPACT PARAMETER OF NUCLEAR REACTION
C     CRTOT             TOTAL REACTION CROSS SECTION
C     I         LOCAL   LOOP VARIABLE
C     IOUT      LOCAL   NUMBER OF REACTIONS FOR CONTROL OUTPUT
C     IMAX              TOTAL NUMBER OF REACTIONS
C     PI                PI
C     EGDR,EGQR         MEAN ENERGIES OF D IPOLE AND Q UADRUPOLE RESONAN
C     FWHMGDR, FWHMGQR  FWHM OF D, Q RESONANCES
C     CREMDE1,CREMDE2   CROSS SECTIONS FOR EXCITATION OF D (E1) Q (E2)
C                       RESONANCES
C     IREACT = 0 -> Full ABRABLA (i.E: high-energy collisions)
C             = 1 -> Low-energy collisions (CN is given as input)
C
C-----------------------------------------------------------------------
C

9998  continue
c9998  WRITE(6,*)'VERSION: ABRABLA07'

      IWRITE = 0

c DM: initialize the variables for the INCL-based run
      ireact=1
      EPRFP=eabra
      APRFP=aabra
      ZPRFP=zabra
      pxabra=pxprfp
      pyabra=pyprfp
      pzabra=pzprfp
c DM end

C
C COUNTERS ARE RESET
C
      NNTOT = 0
      NPTOT = 0
      NBNEVAFRA = 0
      NBPEVAFRA = 0
      NBDEVAFRA = 0
      NBTEVAFRA = 0
      NBHEEVAFRA = 0
      NBAEVAFRA = 0
      NB6HEEVAFRA = 0
       DO J = 0, 100
        ZSUM_FRAG(J)  = 0.D0
        ZSUM_FISS(J)  = 0.D0
        ZSUM_IMF(J)   = 0.D0
        ZSUM_BU(J)    = 0.D0
        ZSUM(J)       = 0.D0
        ZSUM_LP(J)    = 0.D0
        DO K = 0,165
         WQ(K,J) = 0.D0
         WQ2FRA(K,J) = 0.D0
         WQ2FIS(K,J) = 0.D0
         WQ2FISS1(K,J) = 0.D0
         WQ2FISS2(K,J) = 0.D0
         WQ2IMF(K,J) = 0.D0
         WQ2BU(K,J) = 0.D0
         WQ2BU_MF(K,J) = 0.D0
         WQ2BU_FRA(K,J) = 0.D0
         WQ2BU_FIS(K,J) = 0.D0
         WQ2BU_IMF(K,J) = 0.D0
         WQEMD(K,J) = 0.D0
        ENDDO
      ENDDO
      DO J = 0, 2
       DO K = 0, 4
        WQ2LP(K,J) = 0.D0
       ENDDO
      ENDDO
      COUNT = 0
      ZCHA  = 0
      IRNDM = 0
      IEMD  = 0
      FSUM  = 0
      FSUMS1 = 0
      FSUMS2 = 0
      DO 2 L = 0,100,1
        ZPROF(L) = 0
        ZPROF_EMD(L) = 0
 2    CONTINUE
      FSEMD = 0
C
C---------- INTIALISATION OF TABLES AND VARIABLES
C
c      CALL ABRAINIT
C
C---------- MAIN LOOP OVER THE NUMBER OF REACTIONS ASKED FOR (IMAX)
C           WITH CONTROL OUTPUT EVERY IOUT EVENTS
C
C      IOUT = MAX(1,IMAX/100)
C      IOUT_SAVE = MAX(1,IMAX/10)
CC  Preliminary estimate of ARMAX for model decision
CC  either abrasiom model or burst model
C        IBURST = 0
C        ARMAX = 10.3D-3 * AP * AT / (AP + AT) * EAP
C        IF((ARMAX .LT. AP/4.D0).AND.(ZT.LT.3.D0)) THEN
C         IBURST = 1
C        WRITE(*,*) 'BURST model applied!'
C        ENDIF
CC
C      DO 100  I = 1,IMAX,1
C
C---------  MAIN REACTION ROUTINE
C
      CALL REANUCL(IREACT,I,IBURST,AFP,ZFP,AFP1,ZFP1,AFP2,ZFP2,AFPIMF,
     + ZFPIMF,ACN,ZCN,APRFP,ZPRFP,EPRFP,PXABRA,PYABRA,PZABRA,
     + NBNABR,NBPABR,NBNEVA,NBPEVA,NBDEVA,NBTEVA,NBHEEVA,NBAEVA,
     + NB6HEEVA,FEE,
     + FTYPE,B,INTTYPE,JPRF,AABRA,ZABRA,EABRA,VX_ABRA,VY_ABRA,
     + VZ_ABRA,EFF1,EFF2,imode,IMULTBU,
     + IFIREBALL,IMULTIFR,ISECDEC,JPRFFIN,FTYPE1,NBN_PRESADD,
     + NBN_PRESCIS,NBP_PRESADD,NBP_PRESCIS,NBA_PRESADD,
     + NBA_PRESCIS,POSTN1,POSTN2)
cjcd - on saute les sorties d'abla
      goto 999
cjcd - stop

C
C COUNTING EVENTS FOR THE CHARGE CHANGING CROSS SECTION
C
        IF ( ZFP1.LT.IDNINT(ZP) ) ZCHA = ZCHA + 1

C IMPORTANT: CLASSIFICATION OF REACTION TYPES
C     FTYPE = 0 Only evaporation ->
C            AFP1, ZFP1, VFP1_CM(I)
C
C     FTYPE = 1 Fission ->
C            AFP1, ZFP1, VFP1_CM(I), AFP2, ZFP2, VFP2_CM(I)
C
C     FTYPE = 2 IMF emission ->
C            Heavy partner: AFP1, ZFP1, VFP1_CM(I)
C            IMF: AFPIMF, ZFPIMF, VIMF_CM(I)
C
C     FTYPE = 21 Fission after IMF emission ->
C            IMF: AFPIMF, ZFPIMF, VIMF_CM(I)
C            Fission fragments: AFP1, ZFP1, VFP1_CM(I), AFP2, ZFP2, VFP2_CM(I)
C
C     FTYPE = 22 IMF following IMF emission->
C            First IMF: AFPIMF, ZFPIMF, VIMF_CM(I)
C            Heavy residue: AFP1, ZFP1, VFP1_CM(I)
C            Second IMF: AFP2, ZFP2, VFP2_CM(I)
C
C     FTYPE = 3 Break up (multifragment production)
C
C     FTYPE = -1 Central collisions, with no heavy fragment surviving

C
C    IMULTIFR = 0 -> No multifragmentation took place
C    IMULTIFR = 1 -> Multifragmentation occured
C
C
C       INTTYPE = 0  NUCLEAR REACTION W/O FISSION
C       INTTYPE = 1  ELECTROMAGNETIC DISSOZIATION W/O FISSION
C       INTTYPE  = 2  EMD WITH FISSION
C       INTTYPE = 3  NUCLEAR REACTION WITH FISSION
C
c     IMODE = 1 means asymmetric fission, mode 1
c     IMODE = 2 means asymmetric fission, mode 2
c     IMODE = 3 means symmetric fission
c     IMODE = 0 means no fission

        IF ((FTYPE.EQ.1.OR.FTYPE.EQ.21).AND.(INTTYPE.EQ.1)) THEN
           INTTYPE = 2
           FSEMD = FSEMD + 1
           IEMD = IEMD + 1
        END IF
       IF ((FTYPE.EQ.1.OR.FTYPE.EQ.21).AND.(INTTYPE.EQ.0)) INTTYPE = 3

      IF(TSTATEV.EQ.1)THEN
      DO K = 1,ISECDEC,1
C EV_TAB(K,I), I=1-5: Z,A,Vx,Vy,Vz
      WRITE(1100,1101)I,IMULTIFR,FTYPE,AFP1,ZFP1,EPRFP,
     &   IDINT(EV_TAB(K,2)),
     &   IDINT(EV_TAB(K,1)),EV_TAB(K,3),EV_TAB(K,4),EV_TAB(K,5)
      ENDDO
      IF (IFIREBALL.EQ.0) THEN
      IF(ZFP1.LT.3.AND.ZFP1.GT.0) WRITE(1100,1101)I,IMULTIFR,FTYPE,AFP1,
     &   ZFP1,EPRFP,AFP1,ZFP1,VFP1_CM(1),VFP1_CM(2),VFP1_CM(3)
      IF(ZFP2.LT.3.AND.ZFP2.GT.0) WRITE(1100,1101)I,IMULTIFR,FTYPE,AFP1,
     &   ZFP1,EPRFP,AFP2,ZFP2,VFP2_CM(1),VFP2_CM(2),VFP2_CM(3)
      IF(ZFPIMF.LT.3.AND.ZFPIMF.GT.0) WRITE(1100,1101)I,IMULTIFR,FTYPE,
     &   AFP1,ZFP1,EPRFP,AFPIMF,ZFPIMF,VIMF_CM(1),VIMF_CM(2),VIMF_CM(3)
      ENDIF
      ENDIF


 1101 FORMAT(i12,x,i2,x,i2,x,i3,x,i3,x,f12.6,x,2i2,x,3f11.6)
C
C PARTIAL FISSION CROSS SECTIONS
C
       DO 96 L=0,IDNINT(ZP),1
         IF ((INTTYPE.EQ.3).AND.(ZCN.EQ.DNINT(ZP-L))) THEN
           ZPROF(L) = ZPROF(L)+1
         ELSEIF((INTTYPE.EQ.2).AND.(ZCN.EQ.DNINT(ZP-L)))THEN
           ZPROF_EMD(L) = ZPROF_EMD(L)+1
         ENDIF
 96    CONTINUE
C
       IF((FTYPE.NE.1.AND.FTYPE.NE.21).AND.(INTTYPE.EQ.1)) IEMD =IEMD+1
C
      IF(INTTYPE.EQ.1.OR.INTTYPE.EQ.2) B = BMAXNUC + 1.D0

C CALCULATION OF THE CROSS SECTIONS
      IF (IMULTIFR .EQ. 0) THEN  ! Events where no multifragmentation took place
       IF (FTYPE.EQ.1.OR.FTYPE.EQ.21)THEN   ! Fission occured

         FSUM = FSUM + 1
         IF(FTYPE1.EQ.1) FSUMS1 = FSUMS1 + 1
         IF(FTYPE1.EQ.2) FSUMS2 = FSUMS2 + 1

         WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
         WQ(AFP2-ZFP2,ZFP2) = WQ(AFP2-ZFP2,ZFP2) + 1.D0
         WQ2FIS(AFP1-ZFP1,ZFP1) = WQ2FIS(AFP1-ZFP1,ZFP1) + 1.D0
         WQ2FIS(AFP2-ZFP2,ZFP2) = WQ2FIS(AFP2-ZFP2,ZFP2) + 1.D0

         IF(INTTYPE.EQ.2)THEN
          WQEMD(AFP1-ZFP1,ZFP1) = WQEMD(AFP1-ZFP1,ZFP1) + 1.D0
          WQEMD(AFP2-ZFP2,ZFP2) = WQEMD(AFP2-ZFP2,ZFP2) + 1.D0
         ENDIF

         IF(FTYPE1.EQ.1) THEN
          WQ2FISS1(AFP1-ZFP1,ZFP1) = WQ2FISS1(AFP1-ZFP1,ZFP1) + 1.D0
          WQ2FISS1(AFP2-ZFP2,ZFP2) = WQ2FISS1(AFP2-ZFP2,ZFP2) + 1.D0
         ENDIF

         IF(FTYPE1.EQ.2) THEN
          WQ2FISS2(AFP1-ZFP1,ZFP1) = WQ2FISS2(AFP1-ZFP1,ZFP1) + 1.D0
          WQ2FISS2(AFP2-ZFP2,ZFP2) = WQ2FISS2(AFP2-ZFP2,ZFP2) + 1.D0
         ENDIF

         IF(FTYPE.EQ.21)THEN     ! IMF emitted, too
          WQ(AFPIMF-ZFPIMF,ZFPIMF) = WQ(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
           IF(ZFPIMF.GT.2)THEN
            WQ2IMF(AFPIMF-ZFPIMF,ZFPIMF) = WQ2IMF(AFPIMF-ZFPIMF,ZFPIMF)
     &                                   + 1.D0
             ELSE
            WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) = WQ2LP(AFPIMF-ZFPIMF,ZFPIMF)
     &                                   + 1.D0
            ENDIF
            IF(INTTYPE.EQ.2)THEN
           WQEMD(AFPIMF-ZFPIMF,ZFPIMF) = WQEMD(AFPIMF-ZFPIMF,ZFPIMF)
     &                                   + 1.D0
          ENDIF
         ENDIF

C IFIREBALL = 1 corresponds to central collisions where no heavy prefragment is formed; these
C               events we do not write
         IF (TSTAT.EQ.1.AND.IFIREBALL.EQ.0) THEN
           WRITE(UNIT=21,FMT=45)I,B,FTYPE,INTTYPE,IMULTBU,IMULTIFR,
     +      AABRA,ZABRA,EABRA,VX_ABRA,VY_ABRA,VZ_ABRA,
     +      APRFP,ZPRFP,EPRFP,IDNINT(JPRF),VZ_PREF,
     +      ACN,ZCN,FEE,IDNINT(JPRFFIN),
     +      AFP1,ZFP1,AFP2,ZFP2,AFPIMF,ZFPIMF,NBAEVA,NBHEEVA,
     +      VFP1_CM(1),VFP1_CM(2),VFP1_CM(3),
     +      VFP2_CM(1),VFP2_CM(2),VFP2_CM(3),
     +      VIMF_CM(1),VIMF_CM(2),VIMF_CM(3),
     +      imode,
     +      NBN_PRESADD,NBN_PRESCIS,NBP_PRESADD,NBP_PRESCIS
         END IF

       DO K = 1,ISECDEC,1
       WQ(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1))) =
     + WQ(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1))) +
     + 1.D0
       WQ2LP(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1)))
     + = WQ2LP(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1)))
     + + 1.D0
        ENDDO
C
C
         ELSEIF(FTYPE.EQ.0) THEN   ! Only evaporation on neutrons and LCPs
C
           WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
           IF(ZFP1.GT.2)THEN
           WQ2FRA(AFP1-ZFP1,ZFP1) = WQ2FRA(AFP1-ZFP1,ZFP1) + 1.D0
           ELSE
           WQ2LP(AFP1-ZFP1,ZFP1) = WQ2LP(AFP1-ZFP1,ZFP1) + 1.D0
           ENDIF

           IF(INTTYPE.EQ.1)THEN
            WQEMD(AFP1-ZFP1,ZFP1) = WQEMD(AFP1-ZFP1,ZFP1) + 1.D0
           ENDIF
C
         IF (TSTAT.EQ.1.AND.IFIREBALL.EQ.0) THEN
           WRITE(UNIT=21,FMT=45)I,B,FTYPE,INTTYPE,IMULTBU,IMULTIFR,
     +      AABRA,ZABRA,EABRA,VX_ABRA,VY_ABRA,VZ_ABRA,
     +      APRFP,ZPRFP,EPRFP,IDNINT(JPRF),VZ_PREF,
     +      ACN,ZCN,FEE,IDNINT(JPRFFIN),
     +      AFP1,ZFP1,AFP2,ZFP2,AFPIMF,ZFPIMF,NBAEVA,NBHEEVA,
     +      VFP1_CM(1),VFP1_CM(2),VFP1_CM(3),
     +      VFP2_CM(1),VFP2_CM(2),VFP2_CM(3),
     +      VIMF_CM(1),VIMF_CM(2),VIMF_CM(3),
     +      imode,
     +      0,0,0,0
         END IF

       DO K = 1,ISECDEC,1
       WQ(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1))) =
     + WQ(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1))) +
     + 1.D0
       WQ2LP(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1)))
     + = WQ2LP(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1)))
     + + 1.D0
        ENDDO

         ELSEIF(FTYPE.EQ.2.OR.FTYPE.EQ.22)THEN   ! IMF emission

          WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
          WQ(AFPIMF-ZFPIMF,ZFPIMF) = WQ(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
           IF(INTTYPE.EQ.1)THEN
            WQEMD(AFP1-ZFP1,ZFP1) = WQEMD(AFP1-ZFP1,ZFP1) + 1.D0
            WQEMD(AFPIMF-ZFPIMF,ZFPIMF) = WQEMD(AFPIMF-ZFPIMF,ZFPIMF)
     &                                  + 1.D0
           ENDIF
           IF(ZFP1.GT.2)THEN
            WQ2FRA(AFP1-ZFP1,ZFP1) = WQ2FRA(AFP1-ZFP1,ZFP1) + 1.D0
             ELSE
            WQ2LP(AFP1-ZFP1,ZFP1) = WQ2LP(AFP1-ZFP1,ZFP1) + 1.D0
           ENDIF
           IF(ZFPIMF.GT.2)THEN
            WQ2IMF(AFPIMF-ZFPIMF,ZFPIMF) =
     +           WQ2IMF(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
             ELSE
            WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) =
     +           WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
           ENDIF
           IF(FTYPE.EQ.22)THEN
            WQ(AFP2-ZFP2,ZFP2) = WQ(AFP2-ZFP2,ZFP2) + 1.D0
             IF(ZFP2.GT.2)THEN
              WQ2IMF(AFP2-ZFP2,ZFP2) = WQ2IMF(AFP2-ZFP2,ZFP2) + 1.D0
               ELSE
              WQ2LP(AFP2-ZFP2,ZFP2) = WQ2LP(AFP2-ZFP2,ZFP2) + 1.D0
             ENDIF
            IF(INTTYPE.EQ.1)THEN
             WQEMD(AFP2-ZFP2,ZFP2) = WQEMD(AFP2-ZFP2,ZFP2) + 1.D0
            ENDIF
           ENDIF

         IF (TSTAT.EQ.1.AND.IFIREBALL.EQ.0) THEN
           WRITE(UNIT=21,FMT=45) I,B,FTYPE,INTTYPE,IMULTBU,IMULTIFR,
     +      AABRA,ZABRA,EABRA,VX_ABRA,VY_ABRA,VZ_ABRA,
     +      APRFP,ZPRFP,EPRFP,IDNINT(JPRF),VZ_PREF,
     +      ACN,ZCN,FEE,IDNINT(JPRFFIN),
     +      AFP1,ZFP1,AFP2,ZFP2,AFPIMF,ZFPIMF,NBAEVA,NBHEEVA,
     +      VFP1_CM(1),VFP1_CM(2),VFP1_CM(3),
     +      VFP2_CM(1),VFP2_CM(2),VFP2_CM(3),
     +      VIMF_CM(1),VIMF_CM(2),VIMF_CM(3),
     +      imode,
     +      0,0,0,0
         END IF

       DO K = 1,ISECDEC,1
       WQ(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1))) =
     + WQ(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1))) +
     + 1.D0
       WQ2LP(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1)))
     + = WQ2LP(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1)))
     + + 1.D0
        ENDDO

       END IF

      END IF

      IF(IMULTIFR .GT. 0) THEN   ! Events where break-up took place
C WQ2BU_i, i=fra,fis,imf -> Production cross sections of "heavy" fragment residues in multifragmentation;
C WQ2BU_MF -> Production cross sections of light multifragment residues
C WQ2BU -> Total, LCPs are not inlcuded.


C In the output file are written only those MF products with Z>1; IMULTBU
C includes also those events with Z<=1; in the lines below is calculated
C how many events with Z>1 are present in each event, and this number is
C written instead of IMULTBU (in case of FTYPE=3).
        IMULTBUZGT1 = 0
        DO K = 1,IMULTBU,1
         IF(IDINT(BU_TAB(K,8)).GT.1) IMULTBUZGT1=IMULTBUZGT1+1
        ENDDO

        IF(FTYPE.EQ.0) THEN
         WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
         IF(ZFP1.GT.2)THEN
         WQ2BU(AFP1-ZFP1,ZFP1) = WQ2BU(AFP1-ZFP1,ZFP1) + 1.D0
         WQ2BU_FRA(AFP1-ZFP1,ZFP1) = WQ2BU_FRA(AFP1-ZFP1,ZFP1) + 1.D0
         ELSE
         WQ2LP(AFP1-ZFP1,ZFP1) = WQ2LP(AFP1-ZFP1,ZFP1) + 1.D0
         ENDIF
        ELSEIF(FTYPE.EQ.1) THEN
         WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
         WQ(AFP2-ZFP2,ZFP2) = WQ(AFP2-ZFP2,ZFP2) + 1.D0
         WQ2BU(AFP1-ZFP1,ZFP1) = WQ2BU(AFP1-ZFP1,ZFP1) + 1.D0
         WQ2BU(AFP2-ZFP2,ZFP2) = WQ2BU(AFP2-ZFP2,ZFP2) + 1.D0
         WQ2BU_FIS(AFP1-ZFP1,ZFP1) = WQ2BU_FIS(AFP1-ZFP1,ZFP1) + 1.D0
         WQ2BU_FIS(AFP2-ZFP2,ZFP2) = WQ2BU_FIS(AFP2-ZFP2,ZFP2) + 1.D0
        ELSEIF(FTYPE.EQ.2) THEN
         WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
         WQ(AFPIMF-ZFPIMF,ZFPIMF) = WQ(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         IF(ZFP1.GT.2)THEN
         WQ2BU(AFP1-ZFP1,ZFP1) = WQ2BU(AFP1-ZFP1,ZFP1) + 1.D0
         WQ2BU_FRA(AFP1-ZFP1,ZFP1) = WQ2BU_FRA(AFP1-ZFP1,ZFP1) + 1.D0
         ELSE
         WQ2LP(AFP1-ZFP1,ZFP1) = WQ2LP(AFP1-ZFP1,ZFP1) + 1.D0
         ENDIF
         IF(ZFPIMF.GT.2)THEN
         WQ2BU(AFPIMF-ZFPIMF,ZFPIMF) = WQ2BU(AFPIMF-ZFPIMF,ZFPIMF) +1.D0
         WQ2BU_IMF(AFPIMF-ZFPIMF,ZFPIMF) =
     +           WQ2BU_IMF(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         ELSE
         WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) =
     +           WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         ENDIF
        ELSEIF(FTYPE.EQ.21) THEN
         WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
         WQ(AFP2-ZFP2,ZFP2) = WQ(AFP2-ZFP2,ZFP2) + 1.D0
         WQ(AFPIMF-ZFPIMF,ZFPIMF) = WQ(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         WQ2BU(AFP1-ZFP1,ZFP1) = WQ2BU(AFP1-ZFP1,ZFP1) + 1.D0
         WQ2BU(AFP2-ZFP2,ZFP2) = WQ2BU(AFP2-ZFP2,ZFP2) + 1.D0
         IF(ZFPIMF.GT.2)THEN
         WQ2BU(AFPIMF-ZFPIMF,ZFPIMF) = WQ2BU(AFPIMF-ZFPIMF,ZFPIMF) +1.D0
         WQ2BU_IMF(AFPIMF-ZFPIMF,ZFPIMF) =
     &                  WQ2BU_IMF(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         ELSE
         WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) =
     +           WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         ENDIF
         WQ2BU_FIS(AFP1-ZFP1,ZFP1) = WQ2BU_FIS(AFP1-ZFP1,ZFP1) + 1.D0
         WQ2BU_FIS(AFP2-ZFP2,ZFP2) = WQ2BU_FIS(AFP2-ZFP2,ZFP2) + 1.D0
        ELSEIF(FTYPE.EQ.22)THEN
         WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
         WQ(AFP2-ZFP2,ZFP2) = WQ(AFP2-ZFP2,ZFP2) + 1.D0
         WQ(AFPIMF-ZFPIMF,ZFPIMF) = WQ(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         IF(ZFP1.GT.2)THEN
         WQ2BU(AFP1-ZFP1,ZFP1) = WQ2BU(AFP1-ZFP1,ZFP1) + 1.D0
         WQ2BU_FRA(AFP1-ZFP1,ZFP1) = WQ2BU_FRA(AFP1-ZFP1,ZFP1) + 1.D0
         ELSE
         WQ2LP(AFP1-ZFP1,ZFP1) = WQ2LP(AFP1-ZFP1,ZFP1) + 1.D0
         ENDIF
         IF(ZFPIMF.GT.2)THEN
         WQ2BU(AFPIMF-ZFPIMF,ZFPIMF) = WQ2BU(AFPIMF-ZFPIMF,ZFPIMF) +1.D0
         WQ2BU_IMF(AFPIMF-ZFPIMF,ZFPIMF) =
     +           WQ2BU_IMF(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         ELSE
         WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) =
     +           WQ2LP(AFPIMF-ZFPIMF,ZFPIMF) + 1.D0
         ENDIF
         IF(ZFP2.GT.2)THEN
         WQ2BU(AFP2-ZFP2,ZFP2) = WQ2BU(AFP2-ZFP2,ZFP2) + 1.D0
         WQ2BU_IMF(AFP2-ZFP2,ZFP2) = WQ2BU_IMF(AFP2-ZFP2,ZFP2) + 1.D0
         ELSE
         WQ2LP(AFP2-ZFP2,ZFP2) = WQ2LP(AFP2-ZFP2,ZFP2) + 1.D0
         ENDIF
        ENDIF
         IF (TSTAT.EQ.1.AND.IFIREBALL.EQ.0) THEN
           WRITE(UNIT=21,FMT=45)I,B,FTYPE,INTTYPE,IMULTBU,IMULTIFR,
     +      AABRA,ZABRA,EABRA,VX_ABRA,VY_ABRA,VZ_ABRA,
     +      APRFP,ZPRFP,EPRFP,IDNINT(JPRF),VZ_PREF,
     +      ACN,ZCN,FEE,IDNINT(JPRFFIN),
     +      AFP1,ZFP1,AFP2,ZFP2,AFPIMF,ZFPIMF,NBAEVA,NBHEEVA,
     +      VFP1_CM(1),VFP1_CM(2),VFP1_CM(3),
     +      VFP2_CM(1),VFP2_CM(2),VFP2_CM(3),
     +      VIMF_CM(1),VIMF_CM(2),VIMF_CM(3),
     +      imode,
     +      NBN_PRESADD,NBN_PRESCIS,NBP_PRESADD,NBP_PRESCIS
         END IF
       DO K = 1,ISECDEC,1
       WQ(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1))) =
     + WQ(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1))) +
     + 1.D0
       WQ2LP(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1)))
     + = WQ2LP(IDINT(EV_TAB(K,2))-IDINT(EV_TAB(K,1)),IDINT(EV_TAB(K,1)))
     + + 1.D0
        ENDDO
       FTYPE = 3
       DO K = 1,IMULTBU,1
c         IF (BU_TAB(K,8).GT.0.D0) THEN
           ZFP1 = IDINT(BU_TAB(K,8))
           AFP1 = IDINT(BU_TAB(K,9))
           WQ(AFP1-ZFP1,ZFP1) = WQ(AFP1-ZFP1,ZFP1) + 1.D0
           IF(ZFP1.GT.2)THEN
           WQ2BU(AFP1-ZFP1,ZFP1) = WQ2BU(AFP1-ZFP1,ZFP1) + 1.D0
           WQ2BU_MF(AFP1-ZFP1,ZFP1) = WQ2BU_MF(AFP1-ZFP1,ZFP1) + 1.D0
           ELSE
           WQ2LP(AFP1-ZFP1,ZFP1) = WQ2LP(AFP1-ZFP1,ZFP1) + 1.D0
           ENDIF
           IF (TSTAT.EQ.1.AND.IFIREBALL.EQ.0.AND.ZFP1.GT.1) THEN
C Neutrons and protons are not written in the listmode file, otherwise it is too big;
C Therefore, here is the variable IMULTBUZGT1 given and not IMULTBU (to facilitate the
C analysis of data)
            WRITE(UNIT=21,FMT=45)I,B,FTYPE,INTTYPE,IMULTBUZGT1,IMULTIFR,
     +      AABRA,ZABRA,EABRA,VX_ABRA,VY_ABRA,VZ_ABRA,
     +      IDNINT(BU_TAB(K,2)),IDNINT(BU_TAB(K,1)),BU_TAB(K,3),
     +      IDNINT(BU_TAB(K,4)), BU_TAB(K,11),
     +      0,0,0.D0,IDNINT(BU_TAB(K,10)),
     +      AFP1,ZFP1,0,0,0,0,0,0,
     +      BU_TAB(K,5),BU_TAB(K,6),BU_TAB(K,7),
     +      0.D0,0.D0,0.D0,
     +      0.D0,0.D0,0.D0,
     +      imode,
     +      0,0,0,0
           ENDIF
c         ENDIF
       ENDDO
      ENDIF

  45  FORMAT(i12,x,f7.4,x,i2,x,i2,x,i3,x,i2,x,
     +       i3,x,i3,x,f11.3,x,3f11.6,x,
     +       i3,x,i3,x,f11.3,x,i5,x,f11.6,x,
     +       i3,x,i3,x,f11.3,x,i5,x,
     +       i3,x,i3,x,i3,x,i3,x,i3,x,i3,x,i4,x,i4,x,
     +       9f11.6,x,
     +       i2,x,
     +       4i3)
C
       IF (MOD(I,IOUT).EQ.0) THEN
         WRITE(6,*)'I=',I
       END IF

C
cjcd - start
cjcd - on ne fait pas la boucle 100
cjcd - stop
cjcd - 0start 100   CONTINUE
cjcd - stop
C
C---------- END OF THE MAIN LOOP
C
C---------- PRINT TOTAL NUMBER OF EVENTS, MAXIMUM IMPACT PARAMETER,
C           TOTAL CROSS SECTION
C
      IF(IEMD.EQ.0)THEN
      TOTFIS= DFLOAT(FSUM)/DFLOAT(IMAX)*CRNUC
      ELSE
      TOTFIS= DFLOAT(FSEMD)/DFLOAT(IEMD)*(CRTOT-CRNUC) +
     & DFLOAT(FSUM-FSEMD)/DFLOAT(IMAX-IEMD)*CRNUC
      ENDIF

      WRITE(55,*)'    '
      WRITE(55,*)'NUMBER OF EVENTS, IMAX=',IMAX
      WRITE(55,*)'C: AP,ZP   ',IDNINT(AP),IDNINT(ZP)
      WRITE(55,*)'C: AT,ZT   ',IDNINT(AT),IDNINT(ZT)
      WRITE(55,*)'C: E (AMeV)',EAP
      WRITE(55,*)'C: BETA   ',BETA_COEFF
      WRITE(55,*)'C: OPTSHP ',OPTSHP
      WRITE(55,*)'C: OPTCOL ',OPTCOL
      WRITE(55,*)'    '
      WRITE(55,*)'MAXIMUM IMPACT PARAMETER=',BMAXNUC,'FM'
      WRITE(55,*)'TOTAL CROSS SECTION=',CRTOT,'MBARN'
      WRITE(55,*)'NUCLEAR CROSS SECTION=',CRNUC,'MBARN'
      WRITE(55,*)'ELECTROMAGNETIC CROSS SECTION=',CRTOT-CRNUC,'MBARN'
      WRITE(55,*)'TOTAL CHARGE CHANGING CROSS SECTION= ',
     &           DFLOAT(ZCHA)/DFLOAT(IMAX)*CRTOT,' MBARN'
      WRITE(55,*)'TOTAL FISSION CROSS SECTION ',TOTFIS,' MBARN'
      IF(IEMD.EQ.0)THEN
      WRITE(55,*)'ELECTROMAGNETIC FISSION CROSS SECTION= 0 MBARN'
      ELSE
      WRITE(55,*)'ELECTROMAGNETIC FISSION CROSS SECTION= ',
     &           DFLOAT(FSEMD)/DFLOAT(IEMD)*(CRTOT-CRNUC),' MBARN'
      ENDIF
      WRITE(55,*)'NUCLEAR FISSION CROSS SECTION= ',
     &           DFLOAT(FSUM-FSEMD)/DFLOAT(IMAX-IEMD)*CRNUC,' MBARN'

C
      WRITE(18,*)'C: VERSION RN07'
      WRITE(18,*)'C: AP,ZP   ',IDNINT(AP),IDNINT(ZP)
      WRITE(18,*)'C: AT,ZT   ',IDNINT(AT),IDNINT(ZT)
      WRITE(18,*)'C: E (AMeV)',EAP
      WRITE(18,*)'C: BETA   ',BETA_COEFF
      WRITE(18,*)'C: OPTSHP ',OPTSHP
      WRITE(18,*)'C: OPTCOL ',OPTCOL
      WRITE(18,*)'C: NUMBER OF EVENTS ',IMAX
      WRITE(18,*)' '
      WRITE(18,*)'C:PARTIAL NUCLEAR FISSION CROSS SECTION: Z, MBARN '
      WRITE(18,*)'S:ANALYZER(PART_NUCL_FISS)'
      WRITE(18,*)'X: Z of fissioning nucleus'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: X Y,ltb11b'
      DO 97 L = 0,IDNINT(ZP),1
        NUFISZ = DFLOAT(ZPROF(L))/DFLOAT(IMAX-IEMD)*CRNUC
      IF((ZP-L).GE.0)THEN
      WRITE(18,*) (ZP-L),NUFISZ
      ENDIF
 97   CONTINUE
C
      WRITE(55,*)'TOTAL NUMBER OF FISSION EVENTS ',FSUM
      WRITE(55,*)'TOTAL NUMBER OF CHARGE CHANGING EVENTS ',ZCHA
      WRITE(55,*)'NUMBER OF EMD-EVENTS ',IEMD
      WRITE(55,*)'NUMBER OF EMD - FISSIONING EVENTS ',FSEMD
      WRITE(55,*)'PERCENTAGE OF FISSION ',
     &           DFLOAT(FSUM)/DFLOAT(IMAX)*100.D0

C
C---------- OUTPUT OF ASCII ARRAY OF FRAGMENT CROSS SECTIONS
C
      DO J=0,IDNINT(ZP)
        DO K=0,160
        IF (WQ(K,J).GT.0.D0) THEN
         WQH  = WQ(K,J) * CRTOT/DFLOAT(IMAX)
         ZSUM(J) = ZSUM(J) + WQH
         ZSUM_FRAG(J) = ZSUM_FRAG(J) + WQ2FRA(K,J)*CRTOT/DFLOAT(IMAX)
         ZSUM_FRAG(J) = ZSUM_FRAG(J) + WQ2BU_FRA(K,J)*CRTOT/DFLOAT(IMAX)
         ZSUM_FISS(J) = ZSUM_FISS(J) + WQ2FIS(K,J)*CRTOT/DFLOAT(IMAX)
         ZSUM_FISS(J) = ZSUM_FISS(J) + WQ2BU_FIS(K,J)*CRTOT/DFLOAT(IMAX)
         ZSUM_IMF(J)  = ZSUM_IMF(J)  + WQ2IMF(K,J)*CRTOT/DFLOAT(IMAX)
         ZSUM_IMF(J)  = ZSUM_IMF(J)  + WQ2BU_IMF(K,J)*CRTOT/DFLOAT(IMAX)
         ZSUM_BU(J)   = ZSUM_BU(J)   + WQ2BU(K,J)*CRTOT/DFLOAT(IMAX)
         IF(K.LT.9.AND.J.LT.3) THEN
         ZSUM_LP(J)   = ZSUM_LP(J)   + WQ2LP(K,J)*CRTOT/DFLOAT(IMAX)
         ENDIF
        END IF
      ENDDO
      ENDDO

      WRITE(18,*)' '
      WRITE(18,*)'C:CUMULATIVE CROSS SECTIONS: A SIGMA(A)'
      WRITE(18,*)'S:ANALYZER(A)'
      WRITE(18,*)'X: A'
      WRITE(18,*)'Y: si^(mb)'
      WRITE(18,*)'A: X Y,ltb12b'

      DO IIA=1,IDNINT(AP),1
         SIGA = 0.D0
         DO J = 0,IIA,1
            K = IIA - J
            IF ((J.LE.164).AND.(K.LE.100)) THEN
               IF (WQ(J,K).GT.0.D0) THEN
                  SIGA = SIGA + WQ(J,K)*CRTOT/DFLOAT(IMAX)
               END IF
            END IF
         END DO
          WRITE(18,*) IIA,SIGA
      END DO
      WRITE(18,*)' '
      WRITE(18,*)'C:CUMULATIVE CROSS SECTIONS: Z SIGMA(Z)'
      WRITE(18,*)'S:ANALYZER(Z)'
      WRITE(18,*)'X: Z'
      WRITE(18,*)'Y: si^(mb)'
      WRITE(18,*)'A: X Y,ltg12g'
      DO J = 0,IDNINT(ZP),1
        WRITE(18,*) J, ZSUM(J)
      END DO
      WRITE(18,*)' '
      WRITE(18,*)'C:CUMULATIVE FRAGMENTATION CROSS SECTIONS: Z SIGMA(Z)'
      WRITE(18,*)'S:ANALYZER(Z_frag)'
      WRITE(18,*)'X: Z'
      WRITE(18,*)'Y: si^(mb)'
      WRITE(18,*)'A: X Y,ltg12g'
      DO J = 1,IDNINT(ZP),1
        WRITE(18,*) J, ZSUM_frag(J)
      END DO
      WRITE(18,*)' '
      WRITE(18,*)'C:CUMULATIVE FISSION CROSS SECTIONS: Z SIGMA(Z)'
      WRITE(18,*)'S:ANALYZER(Z_fiss)'
      WRITE(18,*)'X: Z'
      WRITE(18,*)'Y: si^(mb)'
      WRITE(18,*)'A: X Y,ltg12g'
      DO J = 1,IDNINT(ZP),1
        WRITE(18,*) J, ZSUM_fiss(J)
      END DO
      WRITE(18,*)' '
      WRITE(18,*)'C:CUMULATIVE IMF CROSS SECTIONS: Z SIGMA(Z)'
      WRITE(18,*)'S:ANALYZER(Z_IMF)'
      WRITE(18,*)'X: Z'
      WRITE(18,*)'Y: si^(mb)'
      WRITE(18,*)'A: X Y,ltg12g'
      DO J = 1,IDNINT(ZP),1
        WRITE(18,*) J, ZSUM_IMF(J)
      END DO
      WRITE(18,*)' '
      WRITE(18,*)'C:CUMULATIVE LP CROSS SECTIONS: Z SIGMA(Z)'
      WRITE(18,*)'S:ANALYZER(Z_LP)'
      WRITE(18,*)'X: Z'
      WRITE(18,*)'Y: si^(mb)'
      WRITE(18,*)'A: X Y,ltg12g'
      DO J = 0,IDNINT(ZP),1
        WRITE(18,*) J, ZSUM_LP(J)
      END DO

      DO 130 J=1,IDNINT(ZP)

c       IF (ZSUM(J).GT.0.D0) THEN

      WRITE(18,*) ' '
      WRITE(18,1972) J
      WRITE(18,*)'C:Proton number, Neutron number, Fission x-sect'
 1972 FORMAT('S:ANALYZER(nfiss(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltr0'
       DO K = 1, 160
       if( WQ2FIS(K,J).gt.0.d0)
     &        WRITE(UNIT=18,FMT=150) J,K, WQ2FIS(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO
      WRITE(18,*) ' '
      WRITE(18,1982) J
      WRITE(18,*)'C:Proton number, Neutron number, Fission x-sect'
 1982 FORMAT('S:ANALYZER(nfiss_bu(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltr0'
       DO K = 1, 160
       if( WQ2BU_FIS(K,J).gt.0.d0)
     &   WRITE(UNIT=18,FMT=150) J,K, WQ2BU_FIS(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO
      WRITE(18,*) ' '
      WRITE(18,1973) J
      WRITE(18,*)'C:Proton number, Neutron number, Fragmentation x-sect'
 1973 FORMAT('S:ANALYZER(nfrag(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltb0'
       DO K = 1, 160
        if(WQ2FRA(K,J).gt.0.d0)
     &        WRITE(UNIT=18,FMT=150) J,K, WQ2FRA(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO
      WRITE(18,*) ' '
      WRITE(18,1983) J
      WRITE(18,*)'C:Proton number, Neutron number, Fragmentation x-sect'
 1983 FORMAT('S:ANALYZER(nfrag_bu(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltb0'
       DO K = 1, 160
        if(WQ2BU_FRA(K,J).gt.0.d0)
     &     WRITE(UNIT=18,FMT=150) J,K, WQ2BU_FRA(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO
      WRITE(18,*) ' '
      WRITE(18,1974) J
      WRITE(18,*)'C:Proton number, Neutron number, IMF x-sect'
 1974 FORMAT('S:ANALYZER(nimf(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltb0'
       DO K = 1, 160
        if(WQ2IMF(K,J).gt.0.d0)
     &        WRITE(UNIT=18,FMT=150) J,K, WQ2IMF(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO
      WRITE(18,*) ' '
      WRITE(18,1984) J
      WRITE(18,*)'C:Proton number, Neutron number, IMF x-sect'
 1984 FORMAT('S:ANALYZER(nimf_bu(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltb0'
       DO K = 1, 160
        if(WQ2BU_IMF(K,J).gt.0.d0)
     &   WRITE(UNIT=18,FMT=150) J,K, WQ2BU_IMF(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO
      WRITE(18,*) ' '
      WRITE(18,1976) J
      WRITE(18,*)'C:Proton number, Neutron number, BU x-sect'
 1976 FORMAT('S:ANALYZER(nmf_bu(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltb0'
       DO K = 1, 160
        if(WQ2BU_MF(K,J).gt.0.d0)
     &      WRITE(UNIT=18,FMT=150) J,K, WQ2BU_MF(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO

      WRITE(18,*) ' '
      WRITE(18,1985) J
      WRITE(18,*)'C:Proton number, Neutron number, BU x-sect'
 1985 FORMAT('S:ANALYZER(nbu(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltb0'
       DO K = 1, 160
        if(WQ2BU(K,J).gt.0.d0)
     &      WRITE(UNIT=18,FMT=150) J,K, WQ2BU(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO

      WRITE(18,*) ' '
      WRITE(18,1977) J
      WRITE(18,*)'C:Proton number, Neutron number, LP x-sect'
 1977 FORMAT('S:ANALYZER(nlp(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltb0'
       IF(J.LT.3)THEN
        DO K = 0, 8
        if(WQ2LP(K,J).gt.0.d0)
     &      WRITE(UNIT=18,FMT=150) J,K, WQ2LP(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO
      ENDIF

      WRITE(18,*) ' '
      WRITE(18,1975) J
      WRITE(18,*)'C:Proton number, Neutron number, total x-sect'
 1975 FORMAT('S:ANALYZER(n(',i2,'))')
      WRITE(18,*)'X:Neutron number'
      WRITE(18,*)'Y:si^ (mb)'
      WRITE(18,*)'A: N X Y,ltb0'
       DO K = 0, 160
        if(WQ(K,J).gt.0.d0)
     &        WRITE(UNIT=18,FMT=150) J,K, WQ(K,J)*CRTOT/DFLOAT(IMAX)
      ENDDO

c         ENDIF
 130  CONTINUE
C
C------- OUTPUT OF ISOTONIC CROSS SECTIONS TO LOG FILE
C
      WRITE(6,*)'    '
      IF(IWRITE.EQ.1)
     &        WRITE(6,*)'MAXIMUM ISOTONIC CROSS SECTIONS: N, SIGMA() '
      DO 131 K=1,160
        ISOTON = 0.D0
        DO 132 J=1,100
        IF (WQ(K,J).GT.0.D0) THEN
           WQH  = WQ(K,J) * CRTOT/DFLOAT(IMAX)
           IF (WQH.GT.ISOTON) THEN
            JM = J
            KM = K
            ISOTON = WQH
           END IF
        END IF
 132    CONTINUE
      IF(IWRITE.EQ.1) THEN
      IF (ISOTON.GT.0.D0) WRITE(6,*) JM,KM,ISOTON
      ENDIF
 131  CONTINUE
C
C------- OUTPUT OF ISOTOPIC CROSS SECTIONS TO LOG FILE
C
      WRITE(6,*)'     '
      IF(IWRITE.EQ.1)
     &        WRITE(6,*)'MAXIMUM ISOTOPIC CROSS SECTIONS: Z, SIGMA() '
      DO 133 J=1,100
        ISOTOP = 0.D0
        DO 134 K=1,160
        IF (WQ(K,J).GT.0.D0) THEN
           WQH  = WQ(K,J) * CRTOT/DFLOAT(IMAX)
           IF (WQH.GT.ISOTOP) THEN
            JM = J
            KM = K
            ISOTOP = WQH
           END IF
        END IF
 134    CONTINUE
      IF(IWRITE.EQ.1) THEN
       IF (ISOTOP.GT.0.D0) WRITE(6,*) JM,KM,ISOTOP
      ENDIF
 133  CONTINUE
C
C--------- OUTPUT OF ISOBARIC CROSS SECTION TO LOG FILE
C
      WRITE(6,*)'    '
      IF(IWRITE.EQ.1) WRITE(6,*)'MAXIMUM ISOBARIC CROSS SECTIONS:'
      K = 0
      DO 135 IA = 1,240,1
        ISOBAR = 0.D0
        KMAX = MIN(IA,160)
       DO 136 K = 1,KMAX,1
         J =  IA - K
         IF(J.LE.100) THEN
         IF (WQ(K,J).GT.0.D0) THEN
           WQH  = WQ(K,J) * CRTOT/DFLOAT(IMAX)
           IF (WQH.GT.ISOBAR) THEN
            JM = J
            KM = K
            ISOBAR = WQH
           END IF
         END IF
       ENDIF
 136   CONTINUE
      IF(IWRITE.EQ.1) THEN
       IF (ISOBAR.GT.0.D0) WRITE(6,*) IA,JM,KM,ISOBAR
      ENDIF
 135   CONTINUE
C
C--------  OUTPUT OF NUCLIDE CHART DATA SET

      WRITE (78,*)'C: VERSION RN07'
      WRITE (78,*)'C: AP,ZP   ',IDNINT(AP),IDNINT(ZP)
      WRITE (78,*)'C: AT,ZT   ',IDNINT(AT),IDNINT(ZT)
      WRITE (78,*)'C: E (AMeV)',EAP
      WRITE (78,*)'C: BETA   ',BETA_COEFF
      WRITE (78,*)'C: OPTSHP ',OPTSHP
      WRITE (78,*)'C: OPTCOL ',OPTCOL
      WRITE (78,*)'C: NUMBER OF EVENTS ',IMAX

C
      WRITE(6,*)'*** NUCLIDE CHARTS BEING WRITTEN'
      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZ)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZfra)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2FRA(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FRA(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FRA(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FRA(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FRA(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZimf)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,160,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2IMF(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2IMF(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2IMF(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2IMF(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2IMF(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZfis)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,160,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2FIS(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FIS(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FIS(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FIS(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FIS(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO


      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZfisS1)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2FISS1(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FISS1(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FISS1(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FISS1(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FISS1(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZfisS2)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2FISS2(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FISS2(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FISS2(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FISS2(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2FISS2(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZlp)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,2
        DO K=0,8,1
           WRITE(UNIT=78,FMT='(1F15.6)')
     &           WQ2LP(K,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO
C For neutrons and LCPs are given multiplicites and not cross sections
      DO J=0,2
       DO K = 0,8
       IF(WQ2LP(K,J).GT.0.D0) WRITE(222,*) IDNINT(ZP),J,J+K,
     &                        WQ2LP(K,J)/DFLOAT(IMAX)
       ENDDO
      ENDDO


C   Break-up products
      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZbu)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2bu(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

C   Break-up products
      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZbu_FIS)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2bu_FIS(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_FIS(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_FIS(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_FIS(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_FIS(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

C   Break-up products
      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZbu_FRA)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,160,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2bu_FRA(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_FRA(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_FRA(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_FRA(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_FRA(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

C   Break-up products
      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZbu_IMF)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2bu_IMF(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_IMF(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_IMF(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_IMF(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_IMF(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

C   Break-up products
      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZbu_MF)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQ2bu_MF(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_MF(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_MF(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_MF(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQ2bu_MF(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

      WRITE(UNIT=78,FMT=*) '    '
      WRITE(UNIT=78,FMT=*)
     &      'S: ANALYZER(NZemd)'
      WRITE(UNIT=78,FMT=*)
     &      'A2: Y = 0 TO 100 BY 1; X = 0 TO 164 BY 1;'
      DO J=0,100
        DO K=0,164,5
           WRITE(UNIT=78,FMT='(5F15.6)')
     &           WQEMD(K,J)*CRTOT/DFLOAT(IMAX),
     &           WQEMD(K+1,J)*CRTOT/DFLOAT(IMAX),
     &           WQEMD(K+2,J)*CRTOT/DFLOAT(IMAX),
     &           WQEMD(K+3,J)*CRTOT/DFLOAT(IMAX),
     &           WQEMD(K+4,J)*CRTOT/DFLOAT(IMAX)
        ENDDO
      ENDDO

C
 150  FORMAT(2I9,1X,F13.6)

      COUNT = COUNT+1
C
cjcd - 0start      RETURN
cjcd - stop
cjcd - start
cjcd - on a saute les sorties d'abla
  999 continue
cjcd - stop
      END
C
C **********************************************************************
C
      SUBROUTINE REANUCL(IREACT,INUM,IBURST,AFP,ZFP,AFP1,ZFP1,AFP2,ZFP2,
     + AFPIMF,ZFPIMF,ACN,ZCN,APRFP,ZPRFP,EPRFP,PXABRA,PYABRA,PZABRA,
     + NBNABR,NBPABR,NBNEVA,NBPEVA,NBDEVA,NBTEVA,NBHEEVA,NBAEVA,
     + NB6HEEVA,FEE,FTYPE,B,INTTYPE,JPRF,AABRA,ZABRA,EABRA,VX_ABRA,
     + VY_ABRA,VZ_ABRA,EFF1,EFF2,imode,IMULTBU,
     + IFIREBALL,IMULTIFR,IEV_TAB,JPRFFIN,FTYPE1,NBN_PRESADD,
     + NBN_PRESCIS,NBP_PRESADD,NBP_PRESCIS,NBA_PRESADD,
     + NBA_PRESCIS,POSTN1,POSTN2)
C
C     INPUT FROM COMMON BLOCKS:
C---------------------------------------------------------------------
C     PROJECTILE AND TARGET PARAMETERS + CROSS SECTIONS
C     COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,
C                       R_0,R_P,R_T, IMAX,IRNDM,PI,
C                       BFPRO,SNPRO,SPPRO,SHELL
C
C     AP,ZP,AT,ZT   - PROJECTILE AND TARGET MASSES
C     EAP,BETA      - BEAM ENERGY PER NUCLEON, V/C
C     BMAXNUC       - MAX. IMPACT PARAMETER FOR NUCL. REAC.
C     CRTOT,CRNUC   - TOTAL AND NUCLEAR REACTION CROSS SECTION
C     R_0,R_P,R_T,  - RADIUS PARAMETER, PROJECTILE+ TARGET RADII
C     IMAX,IRNDM,PI - MAXIMUM NUMBER OF EVENTS, DUMMY, 3.141...
C     BFPRO         - FISSION BARRIER OF THE PROJECTILE
C     SNPRO         - NEUTRON SEPARATION ENERGY OF THE PROJECTILE
C     SPPRO         - PROTON    "           "   "    "   "
C     SHELL         - GROUND STATE SHELL CORRECTION
C---------------------------------------------------------------------
C
C     ENERGIES WIDTHS AND CROSS SECTIONS FOR EM EXCITATION
C     COMMON /EMDPAR/ EGDR,EGQR,FWHMGDR,FWHMGQR,CREMDE1,CREMDE2,
C                     AE1,BE1,CE1,AE2,BE2,CE2,SR1,SR2,XR
C
C     EGDR,EGQR       - MEAN ENERGY OF GDR AND GQR
C     FWHMGDR,FWHMGQR - FWHM OF GDR, GQR
C     CREMDE1,CREMDE2 - EM CROSS SECTION FOR E1 AND E2
C     AE1,BE1,CE1     - ARRAYS TO CALCULATE
C     AE2,BE2,CE2     - THE EXCITATION ENERGY AFTER E.M. EXC.
C     SR1,SR2,XR      - WITH MONTE CARLO
C---------------------------------------------------------------------
C
C     OPTIONS
C     COMMON /OPT/    OPTEMD,OPTCHA,EEFAC
C
C     OPTEMD - 0/1  NO EMD / INCL. EMD
C     OPTCHA - 0/1  0 GDR / 1 HYPERGEOMETRICAL PREFRAGMENT-CHARGE-DIST.
C              ***  RECOMMENDED IS OPTCHA = 1 ***
C     EEFAC  - EXCITATION ENERGY FACTOR, 2.0 RECOMMENDED
C---------------------------------------------------------------------
C
C     OPTIONS AND PARAMETERS FOR FISSION CHANNEL
C     COMMON /FIS/    AKAP,BET,HOMEGA,KOEFF,IFIS,
C                            OPTSHP,OPTXFIS,OPTCOL
C
C     AKAP   - HBAR**2/(2* MN * R_0**2) = 10 MEV
C     BETA_COEFF    - REDUCED NUCLEAR FRICTION COEFFICIENT IN (10**21 S**-1)
C     HOMEGA - CURVATURE OF THE FISSION BARRIER = 1 MEV
C     KOEFF  - COEFFICIENT FOR THE LD FISSION BARRIER == 1.0
C     IFIS   - 0/1 FISSION CHANNEL OFF/ON
C     OPTSHP - INTEGER SWITCH FOR SHELL CORRECTION IN MASSES/ENERGY
C            = 0 NO MICROSCOPIC CORRECTIONS IN MASSES AND ENERGY
C            = 1 SHELL ,  NO PAIRING
C            = 2 PAIRING, NO SHELL
C            = 3 SHELL AND PAIRING
C     OPTCOL - 0/1 COLLECTIVE ENHANCEMENT SWITCHED ON/OFF
C     OPTXFIS- 0,1,2 FOR MYERS & SWIATECKI, DAHLINGER, ANDREYEV
C              FISSILITY PARAMETER.
C     OPTCOL - 0/1 COLLECTIVE ENHANCEMENT OFF/ON
C---------------------------------------------------------------------
C
C     ARRAYS FOR EXCITATION ENERGY BY STATISTICAL HOLE ENERY MODEL
C     COMMON /EENUC/  SHE, XHE
C
C     SHE, XHE - ARRAYS TO CALCULATE THE EXC. ENERGY AFTER
C                ABRASION BY THE STATISTICAL HOLE ENERGY MODEL
C---------------------------------------------------------------------
C
C     OUTPUT IN VARIABLES:
C
C     FISSION CAN OCCUR AFTER PROJECTILE FRAGMENTATION
C
C     AFP,ZFP        - MASS AND CHARGE OF THE PROJECTILE FRAGMENT
C     AFP1,ZFP1      - MASS AND CHARGE OF THE FIRST FISSION FRAGMENTS
C     AFP2,ZFP2      -  "    "    "    "   "  SECOND  "        "
C     APRFP,ZPRFP,   - PREFRAGMENT MASS CHARGE
C     EPRFP          - EXCITATION ENERGIE
C     NBNABR,NBPABR  - NUMBER OF ABRADED NEUTRONS AND PROTONS
C     NBNEVA,NBPEVA  - NUMBER OF EVAPORATED NEUTRON, PROTONS
C     NBAEVA         - ALPHA PARTICLES
C
C THE CALCULATION OF THE MOMENTUM PARAMETERS IS NOT UP TO DATE !
C
C     PLFP,PTFP      - LONGITUDINAL AND TRANSVERSAL MOMENTA OF
C                      THE FINAL FRAGMENTS
C     PLPRFP,PTPRFP  - LONG. AND TRANS. MOMENTA OF THE PREFRAGMENTS
C                      (GOLDHABER)
C     PLEVA,PTEVA    - INFLUENCE OF EVAPORATION
C
C     FEE            - EXCITATION ENERGY OF THE FISSIONING NUCLEUS
C     FF             - FLAG FOR FISSION EVENT
C     B,INTTYPE      - IMPACT PARAMETER, TYPE OF EVENT 0/1/2/3
C
C CHECK THAT THE CALLS FOR THE EVAPORATION ROUTINES IN REANUCL
C ARE NOT SKIPPED.
C
C     POSTAL1,POSTAL2- EVAPORATED ALPHAS FROM FISSION FRAGMENTS
C     POSTN1,POSTN2  - EVAPORATED NEUTRONS FROM FISSION FRAGMENTS
C     POSTP1,POSTP2  - EVAPORATED PROTONS FROM FISSION FRAGMENTS
C
C   ____________________________________________________________________
C  /
C  /  PROCEDURE PRINCIPALE DU CALCUL DES SECTIONS EFFICACES DE
C  /  FRAGMENTATION NUCLEAIRE
C  /
C  /  CENTRAL PROCEDURE FOR CALCULATION OF NUCLEAR-FRAGMENTATION
C  /  CROSS SECTIONS
C  /
C  /____________________________________________________________________
C     IREACT = 0 -> Full ABRABLA (i.e: high-energy collisions)
C             = 1 -> Low-energy collisions (CN is given as input)
C
      SAVE

      INTEGER*4 IMAX,IRNDM,INUM,IFIREBALL,AFIREBALL,ZFIREBALL,FTYPE1
      INTEGER*4 AP1,ZP1,NP1,APRFP,ZPRFP,V,AFP,ZFP,INTTYPE,IEMDTYPE
      INTEGER*4 NBNABR,NBPABR,NBNEVA,NBPEVA,NBDEVA,NBTEVA,IMULTIFR,
     &          NBHEEVA,NBAEVA,L,NBNBRE,NBPBRE,NBNBU,NBPBU,NBDBU,NBTBU,
     &          NBHEBU,NBABU,NBN_PRESADD,NBN_PRESCIS,NBP_PRESADD,
     &          NBP_PRESCIS,IEV_TAB,IEV_TAB_TEMP,IEV_TAB_FIS,IK,
     &          NB6HEEVA,NB6HEBU,NBA_PRESADD,
     &          NBA_PRESCIS
      INTEGER*4 EVOPT,ISWITCH, IPROBA
      INTEGER*4 POSTAL1,POSTAL2,POSTN1,POSTN2,POSTP1,POSTP2,POSTD1,
     &          POSTD2,POSTT1,POSTT2,POSTHE1,POSTHE2,POST6HE1,POST6HE2
      integer*4 n_err_n, n_err_p,ILOOP,INEW,INMIN,INMAX
      INTEGER*4 IBURST,AABRA, ZABRA,ACN,ZCN,ABREAKUP_SUM,ZBREAKUP_SUM,
     &          ABU_EVA_SUM,ZBU_EVA_SUM,NBU_DIFF_MEM
      INTEGER*4 OPTEMD,OPTCHA,LMI,LMA,FF,REFOPT1
      INTEGER*4 AFP1,AFP2,ZFP1,ZFP2,FF1,FF2,FIMF1,FIMF2,FIMF,FF11,
     &          FF22,FIMF11,FIMF22,FTYPE,AFPIMF,ZFPIMF,IMULTBU,FFBU1,
     &          FIMFBU1,FFBU2,FIMFBU2,IMEM_BU(101)
c	INTEGER*4 AT_REST,ALOSS_ACC,ZLOSS_ACC
      INTEGER*4 IFIS,OPTSHP,OPTXFIS,OPTCOL
      INTEGER*4 IA,HI
      INTEGER*4 OPTAFAN,imode
      integer*4 k,i,IMULT,IMEM,IIS,ILESS
      INTEGER*4 ISGAMMA30,ISGAMMA50,IX1
      INTEGER*4 NQUAD
      PARAMETER(NQUAD=1000)
      INTEGER*4 NEVA,PEVA,DEVA,TEVA,HEEVA,AEVA,HE6EVA,AFPNEW,ZFPNEW
      INTEGER*4 ILOOPBU,IOUNSTABLE
      INTEGER   IREACT,IS
C
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,R_P,R_T,PI,
     &       BFPRO,SNPRO,SPPRO,SHELL
      REAL*8 X,XB,XBMIN,XPROB,CMIN,CMAX,B,XCROSS,XEMDTYPE
      REAL*8 ARMAX,FESTRETCH
C	REAL*8 B_0,B_1,B_2,A_1,A_2,U_ALPHA,BMAX_D,DELTA
      REAL*8 MTOTN,MTOTP,MTOTA,INC,ZPRF,DTCTL,IN,VVTOTL,VV(0:240)
      REAL*8 B1,B1S,B2,B2S,BF1,BF2,BF2S,A1,A1S,A2,A2S,DT1,DT2,DT1S
      REAL*8 DT2S,YBF1,YBF1S,YBF2,YBF2S,AAA(1:3),BBB(1:3),CCC(1:3),
     &       DDD(1:3),AR,B0
      REAL*8 GOLDHA,PTEVA,PLEVA,JPRF,SIG2
      REAL*8 EGDR,EGQR,FWHMGDR,FWHMGQR,CREMDE1,CREMDE2
      REAL*8 AE1(NQUAD+1),BE1(NQUAD+1),CE1(NQUAD+1),AE2(NQUAD+1),
     &       BE2(NQUAD+1),CE2(NQUAD+1),SRE1(NQUAD+1),SRE2(NQUAD+1),
     &       XRE1(NQUAD+1),XRE2(NQUAD+1),DS1,DS2
      REAL*8 XEE,PTPRFP,PLPRFP
      REAL   RNDM
      REAL*8 AFF1,AFF2,ZFF1,ZFF2,EFF1,EFF2,beta_in
      REAL*8 EABRA,AAABRA,ZAABRA
      REAL*8 AF1,AF2,ZF1,ZF2,FFPLEVA1,FFPLEVA2,FFPTEVA1,FFPTEVA2
C Number of particles emitted from evaporation residue or in case of
C fission before saddle
      REAL*8 NTOTNCN,NTOTPCN,NTOTDCN,NTOTTCN,NTOTHECN,NTOTACN,NTOT6HECN
      REAL*8 NTOTNBU,NTOTPBU,NTOTDBU,NTOTTBU,NTOTHEBU,NTOTABU,NTOT6HEBU
      REAL*8 NTOTNIMP,NTOTPIMP,NTOTDIMP,NTOTTIMP,NTOTHEIMP,NTOTAIMP,
     &       NTOT6HEIMP
      REAL*8 NTOTNIMF,NTOTPIMF,NTOTDIMF,NTOTTIMF,NTOTHEIMF,NTOTAIMF,
     &       NTOT6HEIMF
      REAL*8 NTOTNIMP1,NTOTPIMP1,NTOTDIMP1,NTOTTIMP1,NTOTHEIMP1,
     +       NTOTAIMP1,NTOT6HEIMP1
      REAL*8 NTOTNIMF1,NTOTPIMF1,NTOTDIMF1,NTOTTIMF1,NTOTHEIMF1,
     +       NTOTAIMF1,NTOT6HEIMF1
      REAL*8 NTOTNIMPBU,NTOTPIMPBU,NTOTDIMPBU,NTOTTIMPBU,NTOTHEIMPBU,
     +       NTOTAIMPBU,NTOT6HEIMPBU
      REAL*8 NTOTNIMFBU,NTOTPIMFBU,NTOTDIMFBU,NTOTTIMFBU,NTOTHEIMFBU,
     +       NTOTAIMFBU,NTOT6HEIMFBU
      REAL*8 NTOTN_POST,NTOTP_POST,NTOTD_POST,NTOTT_POST,
     &       NTOTHE_POST,NTOTA_POST,NTOT6HE_POST
      REAL*8 TSUMBU,TAUCBU,TKEIMFBU,AFFBU,ZFFBU
C Number of particles emitted between saddle and scission
      REAL*8 NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,N6HE_POST
C Number of particles emitted from each fission fragment
      REAL*8 NTOTN1,NTOTP1,NTOTD1,NTOTT1,NTOTHE1,NTOTA1,NTOT6HE1
      REAL*8 NTOTN2,NTOTP2,NTOTD2,NTOTT2,NTOTHE2,NTOTA2,NTOT6HE2
      REAL*8 DELTAB,A_B(1:3),BMED,BTAB(1:3),EPRFP
      REAL*8 SIGRND, EEFAC,SPLINT
      REAL*8 ZF,AF,APRF,EE,FEE,EEFACTOR,EEI
c	REAL*8 EE_ACC
      REAL*8 AKAP,BETA_COEFF,HOMEGA,KOEFF
      REAL*8 M,M1,M2,FA,SIGLOG,EE1,EE2
      REAL*8 SHE(1:2001),XHE(50,1:2001)
      REAL*8  Ecm,Eexcmean
c      real*8  Ekinmax,Ecapture,Ekinmean,Pfus
c      real*8 MAPREF,MAPROJ,QFUSION
      REAL*8 ECGNZ(0:153,0:98),ECFNZ(0:153,0:98),
     &       VGSLD(0:153,0:98),ALPHA(0:153,0:98),
     &       BETA2(0:203,0:136),BETA4(0:203,0:136)
      REAL*8 EGAMMA30(50),EGAMMA50(50)
      REAL*8 a_tilda,T_init,T_diff,T_Freeze_out,EE_diff,A_diff,
     &       E_FINAL,A_FINAL,Z_FINAL,diffpath,T_freeze_out_in
      REAL*4 poissonHaz, haz, gausshaz,FVMAXHAZ_NEUT
      REAL*8 AV,AS,AK
      REAL*8 VX_EVA,VY_EVA,VZ_EVA,VX_PREF,VY_PREF,VZ_PREF,ETOT_PRF,C
C Velocity vectors of evaporation and fission products
      REAL*8 VX1_EVA,VY1_EVA,VZ1_EVA,VX2_EVA,VY2_EVA,VZ2_EVA
      REAL*8 VFP1_CM(3),VFP2_CM(3),VIMF_CM(3),V_CM(3)
      REAL*8 VX1_FISSION,VY1_FISSION,VZ1_FISSION
      REAL*8 VX2_FISSION,VY2_FISSION,VZ2_FISSION
      REAL*8 VX1_IMF,VY1_IMF,VZ1_IMF,VX2_IMF,VY2_IMF,VZ2_IMF
      REAL*8 EFACTOR,Z_FINAL_MEAN
      REAL*8 PXPRFP,PYPRFP,PZPRFP,PPRFP,JPRFX,JPRFY,JPRFZ,JPRFNEW
      REAL*8 VPERP1,ALPHA1
      REAL*8 TSUM,TAUC,TSUM1,TSUM2,TAUC1,TAUC2
      REAL*8 QPRE,QPOST,TKE_PRE,TKE_POST,BIND_CN,BIND_FF1_PRE,
     &       BIND_FF1_POST,BIND_FF2_PRE,BIND_FF2_POST,MASS_FF1_PRE,
     &       MASS_FF1_POST,MASS_FF2_PRE,MASS_FF2_POST,MASS_CN
      REAL*8 TKEIMF,AIMF,ZIMF,TKEIMF1,AIMFF1,ZIMFF1,TKEIMF2,AIMFF2,
     &       ZIMFF2,AFF,ZFF,AFFIMF,ZFFIMF,EEIMF,EEIMFP,tkdummy,adummy,
     &       zdummy,tkdummy1,adummy1,zdummy1
      REAL*8 EkinR1,EkinR2,V1,V2,ALPHA1VPERP1,
     &       VX1EV_IMF,VY1EV_IMF,VZ1EV_IMF,
     &       VX2EV_IMF,VY2EV_IMF,VZ2EV_IMF,IINERTTOT,JPRFHEAVY,JPRFLIGHT
      REAL*8 WEIGHT1, WEIGHT2,EEBU,G_SYMM
      REAL*8 PMORR,UPMORR
      REAL*8 ZFFS,AFFS,ZFFIMFS,AFFIMFS,
     &       VX1EV_IMFS,VY1EV_IMFS,VZ1EV_IMFS,
     &       VX2EV_IMFS,VY2EV_IMFS,VZ2EV_IMFS,
     &       VX1_IMFS,VY1_IMFS,VZ1_IMFS,
     &       VX2_IMFS,VY2_IMFS,VZ2_IMFS,
     &       VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC
      REAL*8 JPRFFIN,JPRF0,JPRF1,JPRF2,JPRF3,JPRF4
C  For the formation of multifragments
      REAL*8 BU_TAB(200,11),TKER,VX1_BU,VY1_BU,VZ1_BU,VX2_BU,VY2_BU,
     &       VZ2_BU,ASLOPE1,ASLOPE2,AMU
      REAL*8 VX_EVABU,VY_EVABU,VZ_EVABU,AFBU,ZFBU,AIMFBU,ZIMFBU
      REAL*8 A_ACC,A_Breakup,Z_Breakup,a_tilda_BU,E_Breakup,
     &       CZ,Sigma_Z, AMEM,ZMEM,EMEM,JMEM,ABU_SLOPE,ACORR,
     &       ETOT_BU,PX_BU,PY_BU,PZ_BU,P_BU,Z_Breakup_mean,RHAZ,ZTEMP,
     &       Z_Breakup_sum,N_Breakup,Ekin_thermal,ABU_SUM,ZBU_SUM,ATEMP
      REAL*8 PXABRA,PYABRA,PZABRA,VX_ABRA,VY_ABRA,VZ_ABRA,ETOT_ABRA,
     &       PABRA,RNDT,CTET1,STET1,RNDP,PHI1,PX_BU_SUM,PY_BU_SUM,
     &       PZ_BU_SUM,VX_BU_SUM,VY_BU_SUM,VZ_BU_SUM,VBU2,GAMMA_REL,
     &       ETOT_SUM,P_BU_SUM,Eexc_BU_SUM,E_tot_BU,Qvalue,Bvalue_BU,
     &       EFLMAC,EKIN_BU,VXOUT,VYOUT,VZOUT,vx2out,vy2out,vz2out
      REAL*8 VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z,BU_TAB_TEMP(200,5),
     &       BU_TAB_TEMP1(200,5),EV_TAB_TEMP(200,5),EV_TAB(200,5),
     &       EV_TEMP(200,5),EEBURST
      integer hev_tab(200),hbu_tab(200)
      integer hev_temp(200),hist_fis1,hist_fis2
      integer hist0,hist1,hist2,histimf
      integer step_ev, step_er, step_bu, step_imfl, step_imfh,
     &   step_fisl, step_fish, step_s2s, step_unst
      parameter( step_ev = 1 )
      parameter( step_er = 2 )
      parameter( step_bu = 3 )
      parameter( step_imfl = 4 )
      parameter( step_imfh = 5 )
      parameter( step_fisl = 6 )
      parameter( step_fish = 7 )
      parameter( step_s2s = 8 )
      parameter( step_unst = 9 )
      INTEGER*4 IPOWERHAZ,IPOWERLIMHAZ,I_Breakup,I1,I2,INEWLOOP
      INTEGER*4 TEST_DIFF,ABU_DIFF,ZBU_DIFF,NBU_DIFF,IEL,IJ

      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     &                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
      COMMON /EMDPAR/ EGDR,EGQR,FWHMGDR,FWHMGQR,CREMDE1,CREMDE2,
     &       AE1,BE1,CE1,AE2,BE2,CE2,SRE1,SRE2,XRE1,XRE2,DS1,DS2
      COMMON /OPT/ OPTEMD,OPTCHA,EEFAC
      COMMON /FIS/ AKAP,BETA_COEFF,HOMEGA,KOEFF,IFIS,OPTSHP,
     &       OPTXFIS,OPTCOL
      COMMON /EENUC/ SHE,XHE
      COMMON /ECLD/ ECGNZ,ECFNZ,VGSLD,BETA2,BETA4
      COMMON /ALD/AV,AS,AK,OPTAFAN
      COMMON /BREAK_UP/T_freeze_out_in
      COMMON /TFO/T_freeze_out
      COMMON /EVAPORATION/ EV_TAB,hev_tab
      COMMON/MULTIFRAG/BU_TAB,hbu_tab
      COMMON /VELOCITY/ VFP1_CM,VFP2_CM,VIMF_CM,VZ_PREF

C   Treatment of photospectrum (from J. Benlliure)
C       30 MeV electrons
      DATA EGAMMA30/ 41.D0, 82.D0, 96.D0,118.D0,172.D0,201.D0,251.D0,
     &              370.D0,600.D0,874.D0,916.D0, 78.D0,674.D0,560.D0,
     &              425.D0,282.D0,176.D0,103.D0, 69.D0, 45.D0, 56.D0,
     &               25.D0, 36.D0, 13.D0, 11.D0,  9.D0,  5.D0,  6.D0,
     &                2.D0,  0.D0,  0.D0,  0.D0,  0.D0,  0.D0,  0.D0,
     &                0.D0,  0.D0,  0.D0,  0.D0,  0.D0,  0.D0,  0.D0,
     &                0.D0,  0.D0,  0.D0,  0.D0,  0.D0,  0.D0,  0.D0,
     &                0.D0/
C       50 MeV electrons
      DATA EGAMMA50/  73.D0, 136.D0, 151.D0, 210.D0, 289.D0, 331.D0,
     &               438.D0, 689.D0,1146.D0,1700.D0,1802.D0,1300.D0,
     &              1197.D0,1084.D0, 798.D0, 547.D0, 375.D0, 248.D0,
     &               143.D0, 142.D0, 100.D0,  79.D0,  43.D0,  56.D0,
     &                35.D0,  29.D0,  20.D0,  22.D0,  24.D0,  10.D0,
     &                10.D0,   6.D0,   3.D0,   6.D0,   3.D0,   4.D0,
     &                 7.D0,   2.D0,   2.D0,   2.D0,   0.D0,   1.D0,
     &                 0.D0,   2.D0,   0.D0,   0.D0,   0.D0,   0.D0,
     &                 0.D0,  0.D0/

c     Common block to extract data about the fissioning nucleus
      parameter (nbiv=300)
      integer*4 na07,ww_a07,nna_a07,nnz_a07,izfa07,iafa07
      real*8 ppx_a07,ppy_a07,ppz_a07,esfa07,eekin_a07
      common /zablaout/ppx_a07(nbiv),ppy_a07(nbiv),ppz_a07(nbiv),
     &     eekin_a07(nbiv),
     &     ww_a07(nbiv),nna_a07(nbiv),nnz_a07(nbiv),na07,kproca07,
     &     izfa07,iafa07,esfa07
C
C
C
C-----------------------------------------------------------------------
C     IRNDM             DUMMY ARGUMENT FOR RANDOM-NUMBER FUNCTION
C     INTTYPE           0=NUCLEAR, 1=ELECTROMAGNETIC INTERACTION
C     AP1               MASS NUMBER OF THE PROJECTILE
C     ZP1               NUCLEAR CHARGE OF THE PROJECTILE
C     APRFP             MASS NUMBER OF THE PREFRAGMENT
C     ZPRFP             NUCLEAR CHARGE OF THE PREFRAGMENT
C     V                 LOOP VARIABLE
C     REFOPT1           OPTION (SEE ABOVE)
C     REFOPT0           OPTION (SEE ABOVE)
C     AFP               MASS NUMBER OF THE FINAL FRAGMENT
C     ZFP               NUCLEAR CHARGE OF THE FINAL FRAGMENT
C     NBNABR            NUMBER OF ABRABED NEUTRONS
C     NBPABR            NUMBER OF ABRADED PROTONS
C     NBNEVA            NUMBER OF EVAPORATED NEUTRONS
C     NBPEVA            NUMBER OF EVAPORATED PROTONS
C     NBAEVA            NUMBER OF EVAPORATED ALPHA PARTICLES
C     R_P               RADIUS OF THE PROJECTILE NUCLEUS
C     R_T               RADIUS OF THE TARGET NUCLEUS
C     APRF              MASS NUMBER OF THE PREFRAGMENT (REAL VARIABLE)
C     EE                EXCITATION ENERGY OF THE PREFRAGMENT
C     EEFACTOR          ARBITRARY FACTOR FOR THE EXCITATION ENERGY
C     AP                MASS NUMBER OF THE PROJECTILE (REAL VARIABLE OF
C                        INTEGER NUMBER FOR COMPUTATIONAL REASON)
C     ZP                NUCLEAR CHARGE OF THE PROJECTILE (REAL VARIABLE
C                        OF INTEGER NUMBER FOR COMPUTATIONAL REASON)
C     EAP               ENERGY PER NUCLEON OF PROJECTILE
C     AT                MASS NUMBER OF THE TARGET
C     ZT                PROTON NUMBER OF THE TARGET
C     CRNUC             NUCLEAR CROSS SECTION
C     CRTOT             TOTAL REACTION CROSS SECTION
C     IEMDTYPE          TYPE OF ELECTROMAGNETIC INTERACTION (E1,E2)
C     CREMDE1   LOCAL   ELECTROMAGNETIC CROSS SECTION DIPOLE
C     CREMDE2   LOCAL   ELECTROMAGNETIC CROSS SECTION QUADRUPOLE
C     EEI               MEAN EXCITATION ENERGY OF THE PREFRAGMENT
C     X         LOCAL   UNIFORM RANDOM NUMBER FOR THE PREFRAGMENT CHARGE
C     XB,XPROB  LOCAL   UNIFORM RANDOM NUMBERS FOR THE IMPACT PARAMETER
C     XBMIN     LOCAL   LOWER LIMIT OF XB FOR THE IMPACT PARAMETER
C     XCROSS    LOCAL   UNIFORM RANDOM NUMBER FOR THE INTERACTION TYPE
C     XEMD      LOCAL   LORENTZ RANDOM NUMBER FOR EMD-EXCITATION-ENERGY
C     XEMDTYPE  LOCAL   UNIFORM RANDOM NUMBER FOR THE EMD-TYPE
C     CMIN,CMAX LOCAL   LOWER AND UPPER LIMIT OF INTERVAL IN INVERTED
C                       PROBABILITY DISTRIBUTION
C     B                 IMPACT PARAMETER OF THE REACTION
C     BMAXNUC  LOCAL    MAXIMUM IMPACT PARAMETER OF NUCLEAR REACTION
C     PI                PI VALUE
C     PLFP              LONGITUDINAL MOMENTUM OF PROJECTILE FRAGMENTS
C     PTFP              TRANSVERSAL MOMENTUM OF PROJECTILE FRAGMENTS
C     PLPRFP            MOMENTUM OF PREFRAGMENTS IN BEAM DIRECTION
C     PTPRFP            TRANSVERSAL MOMENTUM OF PREFRAGMENTS
C                       ALL MOMENTA ARE GIVEN IN THE PROJECTILE SYSTEM
C     PLEVA             LONGITUDINAL RECOIL MOMENTUM OF EVAPORATION
C     PTEVA             TRANSVERSAL RECOIL MOMENTUM OF EVAPORATION
C     GOLDHA            VARIANCE OF MOMENTUM DISTRIBUTION AFTER ABRASION
C     MTOTN             TOTAL MASS OF THE EVAPORATED NEUTRONS
C     MTOTP             TOTAL MASS OF THE EVAPORATED PROTONS
C     MTOTA             TOTAL MASS OF THE EVAPORATED ALPHA PARTICLES
C     INC      LOCAL    INCREMENT OF LOOP OVER IMPACT PARAMETER
C     ZPRF              NUCLEAR CHARGE (PRE- AND FINAL FRAGMENT)
C     DTCTL             SURFACE-EXCESS ENERGY (USED TO AVOID NEGATIVE
C                        VALUES)
C     IN       LOCAL    HELP VARIABLE FOR INTERVAL OF CORRECTION
C     VV( )             TABLE OF PROBABILITIES OF CHARGE DISTRIBUTION
C                        OF THE PREFRAGMENT (NOT NORMALIZED)
C     VVTOTL            NORMALIZATION CONSTANT FOR VV( )
C     B0                IMPACT PARAMETER (LOOP VARIABLE)
C     B1,B2,B1S,B2S     IMPACT-PARAMETER LIMITS FOR CORRECTING
C                        MASS AND EXCITATION ENERGY OF PREFRAGMENTS
C     BF1,BF2,BF1S,BF2S  DITTO
C     A1,A2,A1S,A2S     PREFRAGMENT MASS VALUES CORRESPONDING TO B1,...
C     DT1,DT2,DT1S,DT2S SURFACE-EXCESS ENERGIES CORRESSPONDING TO B1,...
C     YBF1,YBF2,YBF1S,YBF2S   FRICTION ENERGIES CORRESPONDING TO BF1,...
C     AAA( ),BBB( )     COEFFICIENTS OF CORRECTIVE POLYNOMIALS
C     CCC( ),DDD( )      DITTO
C     XE,XPE   LOCAL    UNIFORM RANDOM NUMBERS FOR THE EXCITATION ENERGY
C     XEE               RANDOM NUMBER OF A GAUSSIAN
C     AR       LOCAL    NUMBER OF ABRADED NUCLEONS
C     FM       LOCAL    MAXIMUM VALUE OF EXCITATION-ENERGY PROBABILITIES
C     PROBEE            PROBABILITY FOR A GIVEN EXCITATION ENERGY OF THE
C                        PREFRAGMENT
C     ARMAX      LOCAL  ESTIMATION OF MAX. MASS REMOVED IN SPALLATION
C     FESTRETCH  LOCAL  FUTCH FACTOR FOR E* FOR SMALL TARGET NUCLEI
C-----------------------------------------------------------------------
C
cjcd - start
      real*8 APRFinc,ZPRFinc,EEinc,JPRFjcd,Pxinc,Pyinc,Pzinc
      common /frominc/ APRFinc,ZPRFinc,EEinc,JPRFjcd,Pxinc,Pyinc,Pzinc
      common /ablasmop/ isecdec1,imultbu1,kproc
      JPRFjcd=JPRF
cjcd - stop

      C = 29.9792458D0
      AMU = 931.494D0
C
C----------- SET LOWER LIMIT OF IMPACT PARAMETER
C          (XBMIN IS DEFINED AS FRACTION OF THE MAXIMUM IMPACT PARAMETER
C          FOR NUCLEAR REACTIONS:   0.D0 <= XBMIN <= 1.D0 ) XE0.5,AU0.7
      XBMIN = 0.0D0
      DELTAB = 1.D-03
C
C----------- SET FACTOR FOR THE EXCITATION ENERGY OF THE PREFRAGMENTS
C            (STATISTICAL ABRASION MODEL EEFACTOR = 1.D0)
C            EEFAC IS IN COMMON /OPT/ AND READ IN FROM INPUT FILE
C            A.J. 8/06/93
      EEFACTOR=EEFAC
C
C----------- SET OPTION TO RECOMMENDED VALUE
C          (FOR ANY OPTION OF REFOPT1 UNEQUAL 2 THE TREATMENT OF THE
C          WIDTH OF THE PREFRAGMENT EXCITATION-ENERGY DISTRIBUTION
C          HAS TO BE REMOVED)
      REFOPT1 = 2
C
C----------- DECIDE FOR NUCLEAR OR ELECTROMAGNETIC INTERACTION
C
      IF (OPTEMD.EQ.1) THEN
        XCROSS = CRTOT*dble(HAZ(1))
        IF (XCROSS.GT.CRNUC) THEN
          INTTYPE = 1
        ELSE
          INTTYPE = 0
        END IF
      ELSE
        INTTYPE = 0
      END IF
C
C----------- INITIALIZE START VALUES

 10   CONTINUE
      IS = 0

      NBN_PRESADD = 0
      NBN_PRESCIS = 0
      NBP_PRESADD = 0
      NBP_PRESCIS = 0
      NBA_PRESADD = 0
      NBA_PRESCIS = 0
      FTYPE1 = 0
      NTOTNCN = 0.D0
      NTOTPCN = 0.D0
      NTOTDCN = 0.D0
      NTOTTCN = 0.D0
      NTOTHECN = 0.D0
      NTOTACN = 0.D0
      NTOT6HECN = 0.D0
      NTOTNBU = 0.D0
      NTOTPBU = 0.D0
      NTOTDBU = 0.D0
      NTOTTBU = 0.D0
      NTOTHEBU = 0.D0
      NTOTABU = 0.D0
      NTOT6HEBU = 0.D0
      NTOTNIMP = 0.D0
      NTOTPIMP = 0.D0
      NTOTDIMP = 0.D0
      NTOTTIMP = 0.D0
      NTOTHEIMP = 0.D0
      NTOTAIMP = 0.D0
      NTOT6HEIMP = 0.D0
      NTOTNIMF = 0.D0
      NTOTPIMF = 0.D0
      NTOTDIMF = 0.D0
      NTOTTIMF = 0.D0
      NTOTHEIMF = 0.D0
      NTOTAIMF = 0.D0
      NTOT6HEIMF = 0.D0
      NTOTNIMPBU = 0.D0
      NTOTPIMPBU = 0.D0
      NTOTDIMPBU = 0.D0
      NTOTTIMPBU = 0.D0
      NTOTHEIMPBU = 0.D0
      NTOTAIMPBU = 0.D0
      NTOT6HEIMPBU = 0.D0
      NTOTNIMFBU = 0.D0
      NTOTPIMFBU = 0.D0
      NTOTDIMFBU = 0.D0
      NTOTTIMFBU = 0.D0
      NTOTHEIMFBU = 0.D0
      NTOTAIMFBU = 0.D0
      NTOT6HEIMFBU = 0.D0
      NTOTNIMP1 = 0.D0
      NTOTPIMP1 = 0.D0
      NTOTDIMP1 = 0.D0
      NTOTTIMP1 = 0.D0
      NTOTHEIMP1 = 0.D0
      NTOTAIMP1 = 0.D0
      NTOT6HEIMP1 = 0.D0
      NTOTNIMF1 = 0.D0
      NTOTPIMF1 = 0.D0
      NTOTDIMF1 = 0.D0
      NTOTTIMF1 = 0.D0
      NTOTHEIMF1 = 0.D0
      NTOTAIMF1 = 0.D0
      NTOT6HEIMF1 = 0.D0
      NTOTN_POST = 0.D0
      NTOTP_POST = 0.D0
      NTOTD_POST = 0.D0
      NTOTT_POST = 0.D0
      NTOTHE_POST = 0.D0
      NTOTA_POST = 0.D0
      NTOT6HE_POST = 0.D0
      NTOTN1 = 0.D0
      NTOTP1 = 0.D0
      NTOTD1 = 0.D0
      NTOTT1 = 0.D0
      NTOTHE1 = 0.D0
      NTOTA1 = 0.D0
      NTOT6HE1 = 0.D0
      NTOTN2 = 0.D0
      NTOTP2 = 0.D0
      NTOTD2 = 0.D0
      NTOTT2 = 0.D0
      NTOTHE2 = 0.D0
      NTOTA2 = 0.D0
      NTOT6HE2 = 0.D0
      NBNBU = 0
      NBPBU = 0
      NBDBU = 0
      NBTBU = 0
      NBHEBU = 0
      NBABU = 0
      NB6HEBU = 0
      AP1 = IDNINT(AP)
      ZP1 = IDNINT(ZP)
      NP1 = AP1 - ZP1
      ABREAKUP_SUM = 0
      ZBREAKUP_SUM = 0
      ABU_SUM = 0.D0
      ZBU_SUM = 0.D0
      IRNDM = 1
      FEE = 0.D0
      ACN = 0
      ZCN = 0
      AAABRA = 0.D0
      ZAABRA = 0.D0
      AFIREBALL = 0
      ZFIREBALL = 0
C     Reset velocity values
      DO I = 1, 3
       V_CM(I) = 0.D0
       VIMF_CM(I) = 0.D0
       VFP1_CM(I) = 0.D0
       VFP2_CM(I) = 0.D0
      ENDDO
      VX_ABRA = 0.D0
      VY_ABRA = 0.D0
      VZ_ABRA = 0.D0
      VX_PREF = 0.D0
      VY_PREF = 0.D0
      VZ_PREF = 0.D0
      VX1_BU = 0.D0
      VY1_BU = 0.D0
      VZ1_BU = 0.D0
      VX2_BU = 0.D0
      VY2_BU = 0.D0
      VZ2_BU = 0.D0
      VX_EVABU = 0.D0
      VY_EVABU = 0.D0
      VZ_EVABU = 0.D0
      VX_EVA = 0.D0
      VY_EVA = 0.D0
      VZ_EVA = 0.D0
      VX_EVA_SC = 0.D0
      VY_EVA_SC = 0.D0
      VZ_EVA_SC = 0.D0
      DO I1 = 1,200,1
       DO I2 = 1,11,1
        BU_TAB(I1,I2) = 0.D0
       ENDDO
       DO I2 = 1, 5, 1
        BU_TAB_TEMP(I1,I2) = 0.D0
        BU_TAB_TEMP1(I1,I2) = 0.D0
        EV_TAB_TEMP(I1,I2) = 0.D0
        EV_TAB(I1,I2) = 0.D0
        EV_TEMP(I1,I2) = 0.D0
       ENDDO
       hev_tab(I1) = 0
       hbu_tab(I1) = 0
      ENDDO
c     IMODE = 1 means asymmetric fission, mode 1
c     IMODE = 2 means asymmetric fission, mode 2
c     IMODE = 3 means symmetric fission
      IMODE = 0

      IFIREBALL = 0
      AFIREBALL = 0
      ZFIREBALL = 0
      EKIN_BU = 0.D0

      kproc = 0
C
      IF(IREACT.EQ.1) GOTO 9753
C-------  START NUCLEAR COLLISION
C
      IF (INTTYPE.EQ.0) THEN

C------ Estimation of maximum mass removed in spallation
C       Value of the abrasion model:
        FESTRETCH = 1.D0

        Ecm = EAP * AP * AT / (AP + AT)

         IF(IBURST .EQ. 0) GOTO 50

         IF(IBURST .EQ. 1) THEN
           CALL BURST(APRF,EEFACTOR,FESTRETCH,EE)
           GOTO 85
c           CALL BURST(APRF,EEFACTOR,EE)
c           APRFP = IDNINT(APRF)
c           GOTO 999
         ENDIF

C
C----------- TAKE RANDOM VALUE OF IMPACT PARAMETER (B)
 50     CONTINUE
        XB = DBLE(HAZ(1))
C       LOWER LIMIT OF IMPACT PARAMETER
C       IF (XB.LT.XBMIN) XB=1.D0-XB*(1.D0-XBMIN)/XBMIN
        XPROB = DBLE(HAZ(1))
        IS = IS +1
        IF(IS.EQ.100)THEN
         PRINT*,'WARNING: HAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING B IN REANUCL. A NEW EVENT WILL BE DICED.'
         GOTO 10
        ENDIF

        IF ((XPROB-XB).GT.0.00000D0) GO TO 50

        B = XB*BMAXNUC
C       WRITE(6,*) I,XB,B
C
C ********* CALCULATION OF PREFRAGMENTMASSES FOR 3 IMPACT PARAMETERS
C           TO OBTAIN 2 DIFFERENCE-QUOTIENTS DA/DB NEEDED FOR CHARGE
C           DISTRIBUTION ACCORDING TO THE GDR-MODEL MORRISSEY ET AL
C           1978
C ********* INITIALIZING LOOP VARIABLES FOR HYPERGEOMETRIC MODEL
C           NEEDS  ONLY 1 IMPACT PARAMETER, 1 MASS PER EVENT
C
        LMI = 1
        LMA = 1
        BMED= B
        BTAB(1) = B
        A_B(1) = 0.D0
        A_B(2) = 0.D0
        A_B(3) = 0.D0
C
C ********  IF GDR-MODEL SELECTED, CHANGES IN LOOP VARIABLES ****
C           TO 3 LOOP-RUNS IN ORDER TO CALCULATE 3 MASSES FOR
C           3 IMPACT PARAMETERS PER EVENT
C
        IF (OPTCHA.EQ.0) THEN
          BTAB(1) = B - DELTAB
          BTAB(2) = B
          BTAB(3) = B + DELTAB
          LMI = 1
          LMA = 3
          IF (BTAB(3).GT.BMAXNUC) THEN
              LMI = 1
              LMA = 2
              BTAB(1) = B - DELTAB
              BTAB(2) = B
              BTAB(3) = B
          END IF
          IF  (BTAB(1).LT.0.D0) THEN
              BTAB(1) = B
              BTAB(2) = B
              BTAB(3) = B + DELTAB
              LMI  = 2
              LMA  = 3
          END IF
        END IF
C
C START OF IMPACT PARAMETER LOOP
C ONLY IF THE GDR MODEL IS USED FOR THE CHARGE DISTRIBUTION OF
C THE PREFRAGMENT (NOT RECOMMENDED) A.R.J. 19.01.1999
C
        DO 60  L=LMI,LMA
C
        B = BTAB(L)
C
C---------- DETECTION AND TREATMENT OF CRITICAL REGIONS FOR CALCULATING
C           MASS AND EXCITATION ENERGY OF THE PREFRAGMENTS AS A FUNCTION
C           OF IMPACT PARAMETER
C
        INC = BMAXNUC/10.D0/AP
        IF (R_P.GT.R_T) THEN
          IN = R_0
          IN = DMIN1(R_P-R_T,IN)
          B1 = R_P-R_T-IN
          B1S = B1+0.0001D0
          B2 = R_P-R_T+2.0D0*IN/3.0D0
          IF (B1.EQ.B2) GO TO 78
          B2S = B2+0.0001D0
          IF (REFOPT1.GT.0) THEN
            BF1 = 0.D0
            BF2 = DMAX1(R_T,R_P-R_T)
            BF2S = BF2+0.0001D0
            YBF1 =(26.0D0+8.68D0*(2.0D0*R_T-1.5D0))*DSQRT(R_P**2-R_T**2)
            YBF1S = YBF1
            YBF2 = (13.0D0+4.34D0*(R_P+R_T-BF2-1.5D0)) *
     +                         DSQRT(R_P**2-(R_T-BF2)**2)
            YBF2S = (13.0D0+4.34D0*(R_P+R_T-BF2S-1.5D0)) *
     +                         DSQRT(R_P**2 - (R_T-BF2S)**2)
            CALL COURBE(BF1,BF2,YBF1,YBF1S,YBF2,YBF2S,AAA(3),BBB(3),
     +                                                 CCC(3),DDD(3))
          END IF
          CALL FBPB(B1,R_P,R_T,AP,A1,DT1)
          CALL FBPB(B1S,R_P,R_T,AP,A1S,DT1S)
          CALL FBPB(B2,R_P,R_T,AP,A2,DT2)
          CALL FBPB(B2S,R_P,R_T,AP,A2S,DT2S)
          CALL COURBE(B1,B2,A1,A1S,A2,A2S,AAA(1),BBB(1),CCC(1),DDD(1))
          CALL COURBE(B1,B2,DT1,DT1S,DT2,DT2S,
     +                  AAA(2),BBB(2),CCC(2),DDD(2))
        ELSE
            B1 = R_T-R_P
            ibsteps = idnint((r0-inc)/inc)
c            DO 75  B0 = B1+INC,B1+R_0,INC
            do 75  ib0 = 1,ibsteps
              b0 = b1 + ib0*inc
              CALL FBPB(B0,R_P,R_T,AP,A1,DTCTL)
              IF (DTCTL.GE.0.D0) THEN
                IN = B0-B1
                IF (IN.EQ.INC) THEN
                  IN = 0.D0
                END IF
                GO TO 76
              END IF
 75         CONTINUE
            WRITE(6,*)'ERREUR DANS REANUCL, IN > R_0'
 76         CONTINUE
            B2 = B1+2.0D0*IN
            IF ((B2+0.1D0).GE.(R_P+R_T)) THEN
              WRITE(6,*)'ERREUR DANS REANUCL, B-SUP. > R_P+R_T'
            END IF
            IF (B1.EQ.B2) GO TO 78
            B2S = B2+0.0001D0
            DT1 = 0.D0
            DT1S = 0.D0
            CALL FBPB(B2,R_P,R_T,AP,A2,DT2)
            CALL FBPB(B2S,R_P,R_T,AP,A2S,DT2S)
          CALL COURBE(B1,B2,DT1,DT1S,DT2,DT2S,
     +                     AAA(2),BBB(2),CCC(2),DDD(2))
        END IF
 78     CONTINUE
C
C---------- CALCULATE MASS AND MEAN EXCITATION ENERGY OF THE PREFRAGMENT
C
        CALL MASSEEX(B,R_P,R_T,AP,REFOPT1,B1,B2,BF1,BF2,
     +                                     AAA,BBB,CCC,DDD,APRF,EEI)
c	  write(6,*) EEI
        IF (APRF.LT.5.D0) THEN
          APRFP = 0
          ZPRFP = 0
          AFP = 0
          ZFP = 0
          AFP1 = 0
          ZFP1 = 0
          AFP2 = 0
          ZFP2 = 0
          AFPIMF = 0
          ZFPIMF = 0
          ACN = 0
          ZCN = 0
          EPRFP = 0.D0
          AABRA = 0
          ZABRA = 0
          EABRA = 0
          IEV_TAB = 0
          FTYPE = -1
          IFIREBALL = 1
          AFIREBALL = AP1
          ZFIREBALL = ZP1
c          GO TO 300
          RETURN
        END IF
C
C ******* AUSGABE VON DEN  STOSSPARAMETERN UND MASSEN *******
c       WRITE(6,*)'L = ',L,'APRFP = ',APRF,' BTAB = ',BTAB(L)
        A_B(L) = APRF
C
C END OF IMPACT PARAMETER LOOP (ONLY NEEDED FOR GDR MODEL)
C
 60     CONTINUE
C
        IF (OPTCHA.EQ.0) THEN
          APRF = A_B(2)
          B    = BMED
        END IF
C ******* KONTROLLE OB DIE EVENT WERTE WIEDERHERGESTELLT SIND ****
C       WRITE(6,*)'*********'
C       WRITE(6,*)B,APRF
C       WRITE(6,*)'*********'
 85      APRFP = IDNINT(APRF)
C
C----------- TAKE RANDOM VALUE OF EXCITATION ENERGY OF THE PREFRAGMENT
C            OUT OF THE EXCITATION-ENERGY DISTRIBUTION
C
C UP TO 50 ABRADED NUCLEONS TAKE THE CONVOLUTION
C
 90     continue

 501    AR = AP-DNINT(APRF)
        IF (AR.LE.49.0D0) THEN
           IA = IDINT(AR)
           IF (AR.LT.1.D0) IA = 1
C
C UNIFORM  RANDOM NUMBER (0,1)
C
           SIGRND = DBLE(HAZ(1))
           SIGLOG = DLOG10(SIGRND)
C
C LINEAR PART OF THE TOTAL RANGE OF SHE: 0.1018 - 1.0
C
c           IF (SIGRND.GE.0.1018D0) THEN
           IF (IDNINT(SIGRND*100000.D0).GE.10180) THEN
              HI = IDNINT(1502.D0 + (SIGRND-0.1018D0)/0.0018D0)
              IF (HI.GE.2001) HI = 2000
              M1  = (XHE(IA,HI+1)-XHE(IA,HI))/0.0018D0
              EE1 = XHE(IA,HI) + M1 * (SIGRND - 10.D0**(SHE(HI)))
              M2  = (XHE(IA+1,HI+1)-XHE(IA+1,HI))/0.0018D0
              EE2 = XHE(IA+1,HI) + M2 * (SIGRND - 10.D0**(SHE(HI)))
           ELSE
C
C LOG PART OF THE RANGE OF SHE: 10**-66 - 0.1018
C
              HI = IDNINT(1501.D0 + (SIGLOG+1.D0)/0.04333333333333D0)
              M1  = (XHE(IA,HI+1)-XHE(IA,HI))/
     &              (10.D0**(SHE(HI+1))-10.D0**(SHE(HI)))
              EE1 = XHE(IA,HI) + M1 * (SIGRND - 10.D0**(SHE(HI)))
              M2  = (XHE(IA+1,HI+1)-XHE(IA+1,HI))/
     &              (10.D0**(SHE(HI+1))- 10.D0**(SHE(HI)))
              EE2 = XHE(IA+1,HI) + M2 * (SIGRND - 10.D0**(SHE(HI)))
           END IF
C
C INTERPOLATION IF MORE THAN ONE ABRADED NUCLEON
C
           IF (AR.GT.1.D0) THEN
               FA = AR - DFLOAT(IDINT(AR))
               M = (EE2 - EE1)
               EE = M * FA + EE1
           ELSE
              EE = AR * EE1
           END IF
C
        ELSE
C
C FOR MORE THAN 50 NUCLEONS USE GAUSSIAN APPROXIMATION
C
c          CALL NORRAN(XEE)
           XEE = DBLE(GAUSSHAZ(1,0.E0,1.E0))
           if (AR.lt.0.D0) write(6,*) 'AR < 0'
           EE = EEI+XEE*DSQRT(2.D0*AR*91.D0)
        END IF
C
C CHECK FOR SENSIBLE RANGE
C
        IF ((EE.LT.0.D0).OR.(EE.GT.40.D0*AR)) GO TO 90
C         Negative values and values larger than AR times depth of nuclear potential
C         are not physical

C
c         WRITE(6,*) 'AR,EE,SIGRND',AR,EE,SIGRND
 94     CONTINUE
C
C----------- MODIFY PREFRAGMENT-EXCITATION ENERGY
C
        EE = EE * EEFACTOR * FESTRETCH
C

C----------- CHECK FOR LIMIT DUE TO INITIAL ENERGY
C    limitation by Ecm and energy deposited in the participant zone:
 95     IF (EE.GT.(ECM - 8.D0*AR)) GO TO 90

C
C----------- CALCULATE NUCLEAR-CHARGE DISTRIBUTION OF THE PREFRAGMENT
C
 999    CONTINUE

        VVTOTL = 0.D0
        DO 96  V = 0,240,1
          VV(V) = 0.D0
 96     CONTINUE
C
        IF (OPTCHA.EQ.0) THEN
            CALL DCHAGDR(AP1,ZP1,APRFP,A_B,DELTAB,VVTOTL,VV,LMI,LMA)
        ELSE
            CALL DCHAHYP(AP1,ZP1,APRFP,VVTOTL,VV)
        ENDIF
C
C----------- TAKE RANDOM VALUE OF NUCLEAR CHARGE OF THE PREFRAGMENT
C
        CMIN = 0.D0
        CMAX = 0.D0
        X = DBLE(HAZ(1))*VVTOTL
        DO 98  V = 0,240,1
          CMIN = CMAX
          CMAX = CMAX+VV(V)
          IF ((X.GT.CMIN).AND.(X.LE.CMAX)) THEN
            ZPRFP = V
            GO TO 99
          END IF
 98     CONTINUE
        WRITE(6,*)'ERROR IN REANUCL, FOR Z OF PREFRAG.',ZPRFP
        WRITE(6,*)'CMAX < VVTOTL'
 99     CONTINUE

C Added by KHS, Feb 2006: In this option, the fluctuations in Z reduce
c exponentially with increasing number of abraded nucleons. The (vague)
c idea behind is that the statistics of protons and neutrons is not indepent
c any more, if a larger part of the projectile is abraded. I assume rather a
c transition to an abraded piece of liquid with the same N/Z as the projectile
c fixed. For the system 136Xe + Pb this reduces the neutron-rich tail in the
c calculation in lighter masses, which exceeds the experimental data strongly.
c The parameter to be adjusted is the factor 0.03 in the equation for Weight 1.
        IF(OPTCHA.EQ.2) THEN
         ! Width of hypergeometrical model reduced with increasing mass loss
C           ZPRFP=INT((FLOAT(APRFP)/FLOAT(AP1) * FLOAT(ZPRFP)
C     &       + ( FLOAT(AP1-APRFP))/FLOAT(AP1)
C     &          * FLOAT(ZP1)*FLOAT(APRFP)/FLOAT(AP1)) + 0.5E0)
           Weight1 = DEXP(-DFLOAT(AP1-APRFP)*0.03D0)   ! Adjusted to 136Xe + Pb
           Weight2 = 1.D0 - Weight1
           ZPRFP=DINT(DFLOAT(ZPRFP) * Weight1
     &            + DFLOAT(APRFP)*DFLOAT(ZP1)/DFLOAT(AP1)*Weight2+0.5D0)
        END IF

        IF(ZPRFP.GT.APRFP)THEN
         WRITE(6,*)'ERROR IN REANUCL, ZPRF > APRF',ZPRFP,APRFP
         GOTO 999
        ENDIF


        APRF = DFLOAT(APRFP)
        ZPRF = DFLOAT(ZPRFP)

        IF ( ZPRFP .LT. 3 .OR. EE .GT. 7.99D0*APRF) THEN
          APRFP = 0
          ZPRFP = 0
          AFP = 0
          ZFP = 0
          AFP1 = 0
          ZFP1 = 0
          AFP2 = 0
          ZFP2 = 0
          AFPIMF = 0
          ZFPIMF = 0
          ACN = 0
          ZCN = 0
          EPRFP = 0.D0
          AABRA = 0
          ZABRA = 0
          EABRA = 0
          IEV_TAB = 0
          FTYPE = -1
          IFIREBALL = 1
          AFIREBALL = AP1
          ZFIREBALL = ZP1
c          GO TO 300
          RETURN
        END IF
C

c        IF(EE.EQ.0.D0) GOTO 10

C
C Angular momentum of prefragment:
       beta_in = BETA2(NP1,ZP1)
       CALL AMOMENT_deJong(AT,AP,APRF,EE,IRNDM,beta_in,SIG2,JPRF)

C       WRITE(6,*) 'AR,EE,APRFP,ZPRFP',AR,EE,APRFP,ZPRFP
C
C--------- NUCLEAR COLLISION FINISHED
C
      ELSE
C
C--------- START EM-DISSOCIATION
C
        APRF = AP
        ZPRF = ZP
        APRFP = IDNINT(AP)
        ZPRFP = IDNINT(ZP)
        B = R_P + R_T + 1.D0
C
C--------- DECIDE FOR DIPOLE OR QUADRUPOLE EXCITATION
C
C NEW MC METHOD FOR EXCITATION ENERGY G.K. 13.2.95
C
        XEMDTYPE=DBLE(HAZ(1))*(CREMDE1+CREMDE2)
        IF (XEMDTYPE.LT.CREMDE1) THEN
           IEMDTYPE=1
           SIGRND = DBLE(HAZ(1))
           EE = SPLINT(DS1,NQUAD+1,SIGRND,SRE1,XRE1,AE1,BE1,CE1)
        ELSE
           IEMDTYPE=2
           SIGRND = DBLE(HAZ(1))
           EE = SPLINT(DS2,NQUAD+1,SIGRND,SRE2,XRE2,AE2,BE2,CE2)
        ENDIF
C
C--------- RECOIL OF "VIRTUAL" PHOTONS
        PLPRFP=0.D0
        PTPRFP=0.D0
        JPRF = 0.D0
        JPRFX = 0.D0
        JPRFY = 0.D0
        JPRFZ = 0.D0
        PXPRFP = 0.D0
        PYPRFP = 0.D0
        PZPRFP = 0.D0
        VX_PREF = 0.D0
        VY_PREF = 0.D0
        VZ_PREF = 0.D0

C
C--------- EM-DISSOCIATION FINISHED
C
C ENERGY SHIFT ONLY IF SHELL CORRECTIONS ARE DISREGARDED
C IN THE ABLATION PHASE  A.R.J. 23.03.98
C SEE THESIS A.HEINZ, TU DARMSTADT 1998
C
        IF ((OPTSHP.EQ.0).OR.(OPTSHP.EQ.2)) THEN
          EE = EE + SHELL
        ENDIF

      END IF
C
        EPRFP = EE
C
C
C****************************************************************************
C   Treatment of photospectrum (from J. Benlliure)
C
C The calculations were done with the following target thicknesses:
C
C W = 0.2 cm    aprox 1 radiation length
C U = 3 cm      aprox 10 radiation lengths
C
C these results were induced by 30 and 50 MeV electron beams. You can generate
C them from the corresponding photoabsortion spectra:
C
C photoabs_spect_30MeV.dat -> EGAMMA30(*) :  6926 fission / 1.E6 electrons
C photoabs_spect_50MeV.dat -> EGAMMA50(*) : 13265 fission / 1.E6 electrons
C
C The numbers in those spectra correspond to the energy distribution of the
C photons absorved after the interaction of 1000000 incoming electrons

      ISGAMMA30 = 0
      ISGAMMA50 = 0
798   IF (ISGAMMA30.eq.1) THEN
        IX1 = INT(HAZ(1)*29.E0)
c        write(6,*) 'IX1, EGAMMA30(IX1) ',IX1,EGAMMA30(IX1)
        IF (dble(HAZ(1))*916.D0 .gt. EGAMMA30(IX1)) GOTO 798
        EE = dfloat(IX1)
        APRF = AP
        ZPRF = ZP
        APRFP = IDNINT(AP)
        ZPRFP = IDNINT(ZP)
        CRTOT = 6926.D0/1.D6/1.6D-16
               ! to obtain fission products per mA electron beam
      ENDIF
c      WRITE(6,*)'EE', EE
799   IF (ISGAMMA50.eq.1) THEN
        IX1 = INT(HAZ(1)*40.E0)
        IF (dble(HAZ(1))*1802.D0 .gt. EGAMMA50(IX1)) GOTO 799
        EE = DFLOAT(IX1)
        APRF = AP
        ZPRF = ZP
        APRFP = IDNINT(AP)
        ZPRFP = IDNINT(ZP)
        CRTOT = 13265.D0/1.D6/1.6D-16
               ! to obtain fission products per mA electron beam
      ENDIF


      IF (APRFP .LE. 0 .OR. ZPRFP .LE. 0) GO TO 10
      IF (APRFP .EQ. ZPRFP .AND. ZPRFP .GT. 1)THEN
c        PRINT*,'ABRASION -> ZPRFP = APRFP'
        GOTO 10
      ENDIF
C     *** Find the limits that fragment is bound :
        CALL ISOSTAB_LIM(ZPRFP,INMIN,INMAX)
C        INMIN = MAX(1,INMIN-2)
        IF(ZPRFP.GT.2.D0)THEN
          IF((APRFP-ZPRFP).LE.INMIN
     &     .OR.(APRFP-ZPRFP).GE.(INMAX+5)) THEN
C             PRINT*,'ABRASION -> FRAGMENT UNBOUND:',APRFP,ZPRFP
            GOTO 10
          ENDIF
        ENDIF

cjcd - start
cjcd - on a saute abra
 9999 continue
cjcd - stop
 9753 CONTINUE

         IF(IREACT.EQ.1) THEN   ! Own input
         EE = EPRFP
         APRF = DFLOAT(APRFP)
         ZPRF = DFLOAT(ZPRFP)
         ENDIF


         AAABRA = APRF
         ZAABRA = ZPRF
         AABRA = IDNINT(AAABRA)
         ZABRA = IDNINT(ZAABRA)
         EABRA = EE

         APRF = DFLOAT(APRFP)
         ZPRF = DFLOAT(ZPRFP)

         AFIREBALL = AP1 - AABRA
         ZFIREBALL = ZP1 - ZABRA


C Velocity after the first stage of reaction (abrasion or INCL)
C For coupling with INCL, comment the lines below, and use output
C of INCL as PXABRA, PYABRA,PZABRA, AAABRA, ZAABRA, EABRA
C
C These are the momenta in the rest frame of the projectile
      IF(IREACT.EQ.0)THEN
C *****************************************
      CALL AMOMENT_ABRA(AAABRA,EE,0,PXABRA,PYABRA,PZABRA)
C *******************************************************
      ENDIF
      PABRA = DSQRT(PXABRA*PXABRA + PYABRA*PYABRA + PZABRA*PZABRA)
C PPRFP is in MeV/c
      ETOT_ABRA = DSQRT(PABRA*PABRA + (AAABRA * AMU)*(AAABRA * AMU))
      VX_ABRA = C * PXABRA / ETOT_ABRA
      VY_ABRA = C * PYABRA / ETOT_ABRA
      VZ_ABRA = C * PZABRA / ETOT_ABRA

        IEV_TAB = 0
        IMULTIFR = 0
C Multiplicity in the break-up event
        IMULT = 1
        ILESS = 0
        IMULTBU = 0

C*************************************************************************

C     Set maximum temperature for sequential decay (evaporation)
C     Remove additional energy by simultaneous break up
C                          (vaporisation or multi-fragmentation)

C     Idea: If the temperature of the projectile spectator exceeds
c           the limiting temperature T_freeze_out, the additional
C           energy which is present in the spectator is used for
C           a stage of simultaneous break up. It is either the
C           simultaneous emission of a gaseous phase or the simultaneous
C           emission of several intermediate-mass fragments. Only one
C           piece of the projectile spectator (assumed to be the largest
C           one) is kept track.

C        MVR, KHS, October 2001
C        KHS, AK 2007 - Masses from the power low; slope parameter dependent on
C                      energy  per nucleon; symmtery-energy coeff. dependent on
C                      energy per nucleon.

c       Clear BU_TAB (array of multifragmentation products)

        IF(T_FREEZE_OUT_IN .GE. 0.D0)THEN
          T_FREEZE_OUT = T_FREEZE_OUT_IN
        ELSE
         T_freeze_out = DMAX1(9.33D0*DEXP(-0.00282D0*AAABRA),5.5D0)
c         ! See: J. Natowitz et al, PRC65 (2002) 034618
c        T_freeze_out=DMAX1(9.0D0*DEXP(-0.001D0*AAABRA),
c     &                     5.5D0)

        ENDIF

        a_tilda = AV*APRF + AS*APRF**(2.D0/3.D0) + AK*APRF**(1.D0/3.D0)

        T_init = dsqrt(EABRA/a_tilda)

        T_diff = T_init - T_freeze_out


c        IF(EE.GT.8.D0*APRF)print*,'**',IDNINT(APRF),IDNINT(ZPRF),EE,B,
c     &  T_FREEZE_OUT



         IF (T_diff.GT.0.1D0) THEN
C T_Diff is set to be larger than 0.1 MeV in order to avoid strange cases for which
C T_Diff is of the order of 1.e-3 and less.

            kproc = kproc + 2

          DO I = 1,5,1
            EE_diff = EABRA - a_tilda * T_freeze_out**2
C            Energy removed 10*5/T_init per nucleon removed in simultaneous breakup
C            adjusted to frag. xsections 238U (1AGeV) + Pb data, KHS Dec. 2005
C This should maybe be re-checked, in a meanwhile several things in break-up description
c have changed (AK).
            A_diff = DNINT(EE_diff / (8.D0 * 5.D0 / T_freeze_out))

            IF (A_diff.GT.AAABRA) A_diff = AAABRA

            A_FINAL = AAABRA - A_diff

            a_tilda = AV*A_FINAL + AS*A_FINAL**(2.D0/3.D0) +
     &                             AK*A_FINAL**(1.D0/3.D0)
            E_FINAL = a_tilda * T_freeze_out**2

            IF (A_FINAL.lt.4.D0) THEN  ! To avoid numerical problems
              EE_DIFF = EABRA - E_FINAL
              A_FINAL = 1.D0
              Z_FINAL = 1.D0
              E_FINAL = 0.D0
              GOTO 4325
            ENDIF

          ENDDO

 4325     CONTINUE

C The idea is similar to Z determination of multifragment - Z of "heavy" partner is not
C fixed by the A/Z of the prefragment, but randomly picked from Gaussian
          Z_FINAL_MEAN = DNINT(ZPRF * A_FINAL / (APRF))

          Z_FINAL = DNINT(ZPRF * A_FINAL / (APRF))

          IF (E_FINAL.LT.0.D0) E_FINAL = 0.D0

          APRF = A_FINAL
          ZPRF = Z_FINAL
          EE = E_FINAL

          A_DIFF = AAABRA - APRF

C Creation of multifragmentation products by breakup
          IF (A_DIFF.LE.1.D0) THEN
           APRF = AAABRA
           ZPRF = ZAABRA
           EE = EABRA
           IMULTIFR = 0
           GOTO 7777
          ELSEIF(A_DIFF.GT.1.D0)THEN

c         print*,idnint(AAABRA),EABRA,IDNINT(A_FINAL),E_FINAL

          A_ACC = 0.D0
C Energy-dependence of the slope parameter, acc. to A. Botvina, fits also to exp. data (see
C e.g. Sfienti et al, NPA 2007)
          ASLOPE1 = -2.40D0  ! e*/a=7   -2.4
          ASLOPE2 = -1.20D0  ! e*/a=3   -1.2


            a_tilda = AV*AAABRA + AS*AAABRA**(2.D0/3.D0) +
     &                             AK*AAABRA**(1.D0/3.D0)
            E_FINAL = a_tilda * T_freeze_out**2

          ABU_SLOPE = (ASLOPE1-ASLOPE2)/4.D0*(E_FINAL/AAABRA)+
     &                ASLOPE1-(ASLOPE1-ASLOPE2)*7.D0/4.D0

C Botvina et al, PRC 74 (2006) 044609, fig. 5 for B0=18 MeV
c          ABU_SLOPE = 5.57489D0-2.08149D0*(E_FINAL/AAABRA)+
c     &    0.3552D0*(E_FINAL/AAABRA)**2-0.024927D0*(E_FINAL/AAABRA)**3+
c     &    7.268D-4*(E_FINAL/AAABRA)**4
cc They fit with A**(-tau) and here is done A**(tau)
c          ABU_SLOPE = ABU_SLOPE*(-1.D0)

c           ABU_SLOPE = -2.60D0
c          print*,ABU_SLOPE,(E_FINAL/AAABRA)

          IF(ABU_SLOPE .GT. -1.01D0) ABU_SLOPE = -1.01D0

          I_Breakup = 0
          Z_Breakup_sum = Z_FINAL
          ABU_SUM = 0.D0
          ZBU_SUM = 0.D0

c          print*,'IN',IDNINT(Z_FINAL),IDNINT(A_FINAL),ZABRA,AABRA
          DO I = 1,100,1
             ISWITCH = 0
            IS = 0
 4326       A_Breakup = DNINT(
     &        DBLE(IPOWERLIMHAZ(1,SNGL(ABU_SLOPE),1,IDNINT(A_DIFF))))
                  ! Power law with exponent ABU_SLOPE
             IS = IS +1
             IF(IS.GT.100)THEN
         PRINT*,'WARNING: IPOWERLIMHAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING A_BREAKUP IN Rn07.FOR. NEW EVENT WILL BE DICED.'
            GOTO 10
            ENDIF

            IF (A_Breakup.GT.AAABRA) goto 4326

              ISWITCH = 0

             IF(A_BREAKUP.LE.0.D0)THEN
              PRINT*,'A_BREAKUP <= 0 '
              GOTO 10
             ENDIF

            A_ACC = A_ACC + A_Breakup

            IF (A_ACC.LE.A_DIFF) THEN

              Z_Breakup_Mean = DNINT(A_Breakup * ZAABRA / AAABRA)

              Z_Breakup_sum = Z_breakup_sum + Z_breakup_Mean

c         if(Z_breakup_sum.GT.ZAABRA) print*,'111',ZABRA,
c     &    idnint(Z_Breakup_mean),idnint(A_Breakup),
c     &    idnint(Z_Breakup_sum)

 4339         CONTINUE

c See G.A. Souliotis et al, PRC 75 (2007) 011601R (Fig. 2)
              G_SYMM = 34.2281D0 - 5.14037D0 * E_FINAL/AAABRA
              IF(E_FINAL/AAABRA .LT. 2.D0) G_SYMM = 25.D0
              IF(E_FINAL/AAABRA .GT. 4.D0) G_SYMM = 15.D0

c             G_SYMM = 23.6D0

              G_SYMM = 25.D0      !25
              CZ = 2.D0 * G_SYMM * 4.D0 / A_Breakup
               ! 2*CZ=d^2(Esym)/dZ^2, Esym=Gamma*(A-2Z)**2/A
               ! gamma = 23.6D0 is the symmetry-energy coefficient
              IIS = 0
              Sigma_Z = DSQRT(T_freeze_out/CZ)

              IS = 0
 4333         Z_Breakup =  DNINT(
     &          DBLE(GAUSSHAZ(1,SNGL(Z_Breakup_Mean),SNGL(Sigma_Z))))
             IS = IS +1
             IF(IS.GT.100)THEN
         PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING Z_BREAKUP IN Rn07.FOR. NEW EVENT WILL BE DICED.'
            GOTO 10
            ENDIF

             if (Z_Breakup.lt.0.D0 ) goto 4333
             if ((A_Breakup-Z_Breakup).lt.0.D0) goto 4333
             if ((A_Breakup-Z_Breakup).eq.0.D0.and.Z_Breakup.ne.1.D0)
     &        goto 4333

             if(Z_Breakup.GE.ZAABRA)then
               IIS = IIS + 1
                 IF(IIS .GT. 10) THEN
                   PRINT*,'Z_BREAKUP RESAMPLED MORE THAN 10 TIMES; EVENT
     & WILL BE RESAMPLED AGAIN'
                   GOTO 10
                 ENDIF
               goto 4333
             endif

C     *** Find the limits that fragment is bound :
        CALL ISOSTAB_LIM(IDNINT(Z_breakup),INMIN,INMAX)
C        INMIN = MAX(1,INMIN-2)
        IF(Z_Breakup.GT.2.D0)THEN
          IF(IDNINT(A_Breakup-Z_Breakup).LT.INMIN
     &     .OR.IDNINT(A_Breakup-Z_Breakup).GT.(INMAX+5)) THEN
c             PRINT*,'N_Breakup >< NMAX',
c     &      IDNINT(Z_Breakup),IDNINT(A_Breakup-Z_Breakup),INMIN,INMAX
            GOTO 4333
          ENDIF
        ENDIF

 4343   CONTINUE

C We consider all products, also nucleons created in the break-up
               I_Breakup = I_Breakup + 1

               N_Breakup = A_Breakup - Z_Breakup
               BU_TAB(I_Breakup,1) = DNINT(Z_Breakup)    ! Mass of break-up product
               BU_TAB(I_Breakup,2) = DNINT(A_Breakup)    ! Z of break-up product
               ABU_SUM = ABU_SUM + BU_TAB(I,2)
               ZBU_SUM = ZBU_SUM + BU_TAB(I,1)

C Break-up products are given zero angular momentum (simplification)
               BU_TAB(I_Breakup,4) = 0.D0
               IMULTBU = IMULTBU + 1
           ELSE
C     There are A_DIFF - A_ACC nucleons lost by breakup, but they do not end up in multifragmentation products.
C     This is a deficiency of the Monte-Carlo method applied above to determine the sizes of the fragments
C     according to the power law.
c            print*,'Deficiency',IDNINT(A_DIFF-A_ACC)
             ISWITCH = 1

              GOTO 4327
           ENDIF
c          ENDDO
          ENDDO
 4327     CONTINUE
          ENDIF
         IMULTIFR = 1

C For central collisions:
c         ZMEM = ZPRF
c         DO I = 1, IMULTBU, 1
c         IF(BU_TAB(I,1) .GT. ZMEM)THEN
c           ZMEM = BU_TAB(I,1)
c         ENDIF
c        ENDDO
c        IF(ZMEM.LT.3.D0)THEN
c         GOTO 10
c        ENDIF

C "Missing" A and Z picked from the power law:
        ABU_DIFF = IDNINT(ABU_SUM+APRF-AAABRA)
        ZBU_DIFF = IDNINT(ZBU_SUM+ZPRF-ZAABRA)
        NBU_DIFF = IDNINT((ABU_SUM-ZBU_SUM)+(APRF-ZPRF)-(AAABRA-ZAABRA))

c      if(aprf.gt.1.d0)then
c      print*,'111',abu_diff,zbu_diff,nbu_diff
c      print*,'222',idnint(aprf),idnint(zprf),idnint(aaabra),
c     &             idnint(zaabra)
c      endif

        IF(IMULTBU .GT. 200)
     &       PRINT*,'WARNING - MORE THAN 200 BU',IMULTBU

        IF(IMULTBU .LT. 1)
     &       PRINT*,'WARNING - LESS THAN 1 BU',IMULTBU,AABRA,ZABRA,
     &       IDNINT(APRF),IDNINT(ZPRF),ABU_DIFF,ZBU_DIFF

      IPROBA = 0
      DO I = 1, IMULTBU+1
        IMEM_BU(I) = 0
      ENDDO


        DO WHILE(NBU_DIFF .NE. 0 .AND. ZBU_DIFF .NE. 0 )
C (APRF,ZPRF) is also inlcuded in this game, as from time to time the program
C is entering into endless loop, as it can not find proper nucleus for adapting A and Z.
         IS = 0
 5555    RHAZ = DBLE(HAZ(1))*DFLOAT(IMULTBU+1)
         IPROBA = IPROBA + 1
         IS = IS + 1
         IF(IS.GT.100)THEN
         PRINT*,'WARNING: HAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING N_BREAKUP IN Rn07.FOR. NEW EVENT WILL BE DICED.'
         GOTO 10
          ENDIF
          IEL = INT(SNGL(RHAZ))+1
          IF(IMEM_BU(IEL).EQ.1) GOTO 5555
           IF(IEL.GT.200)PRINT*,'5555:',IEL,RHAZ,IMULTBU
           IF(IEL.LT.1)PRINT*,'5555:',IEL,RHAZ,IMULTBU
           IF(IEL.LE.IMULTBU) THEN
            N_Breakup = DNINT(BU_TAB(IEL,2)-BU_TAB(IEL,1) -
     &               DSIGN(1.D0,DFLOAT(NBU_DIFF)))
            ELSEIF(IEL.GT.IMULTBU)THEN
            N_Breakup = DNINT(APRF - ZPRF -
     &               DSIGN(1.D0,DFLOAT(NBU_DIFF)))
            ENDIF
            IF(N_Breakup.LT.0.D0) THEN
             IMEM_BU(IEL) = 1
             GOTO 5555
            ENDIF
             IF(IEL.LE.IMULTBU)THEN
             ZTEMP = DNINT(BU_TAB(IEL,1) - DSIGN(1.D0,DFLOAT(ZBU_DIFF)))
             ELSEIF(IEL.GT.IMULTBU)THEN
             ZTEMP = DNINT(ZPRF - DSIGN(1.D0,DFLOAT(ZBU_DIFF)))
             ENDIF
              IF(ZTEMP.LT.0.D0) THEN
               IMEM_BU(IEL) = 1
               GOTO 5555
              ENDIF
             IF(ZTEMP.LT.1.D0 .AND. N_Breakup.LT.1.D0) THEN
              IMEM_BU(IEL) = 1
              GOTO 5555
             ENDIF
C Nuclei with A=Z and Z>1 are allowed in this stage, as otherwise,
C for more central collisions there is not enough mass which can be
C shufeled in order to conserve A and Z. These are mostly nuclei with
C Z=2 and in less extent 3, 4 or 5.
c             IF(ZTEMP.GT.1.D0 .AND. N_Breakup.EQ.0.D0) THEN
c              GOTO 5555
c             ENDIF
            IF(IEL.LE.IMULTBU)THEN
            BU_TAB(IEL,1) = DNINT(ZTEMP)
            BU_TAB(IEL,2) = DNINT(ZTEMP + N_breakup)
            ELSEIF(IEL.GT.IMULTBU)THEN
            ZPRF = DNINT(ZTEMP)
            APRF = DNINT(ZTEMP + N_breakup)
            ENDIF
           NBU_DIFF = NBU_DIFF - ISIGN(1,NBU_DIFF)
          ZBU_DIFF = ZBU_DIFF - ISIGN(1,ZBU_DIFF)
         ENDDO

      IPROBA = 0
      DO I = 1, IMULTBU+1
        IMEM_BU(I) = 0
      ENDDO

         IF(NBU_DIFF .NE. 0 .AND. ZBU_DIFF .EQ. 0) THEN
          DO WHILE( NBU_DIFF .GT. 0 .OR. NBU_DIFF .LT. 0 )
          IS = 0
 5556      RHAZ = DBLE(HAZ(1))*DFLOAT(IMULTBU+1)
         IS = IS + 1
         IF(IS.GT.100)THEN
         PRINT*,'WARNING: HAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING N_BREAKUP IN Rn07.FOR. NEW EVENT WILL BE DICED.'
         GOTO 10
          ENDIF
          IEL = INT(SNGL(RHAZ))+1
         IF(IMEM_BU(IEL) .EQ. 1) GOTO 5556
         IPROBA = IPROBA + 1
        IF(IPROBA.GT.IMULTBU+1.AND.NBU_DIFF.GT.0) THEN
         PRINT*,'###',IPROBA,IMULTBU,NBU_DIFF,ZBU_DIFF,T_freeze_out
           IF(IEL.LE.IMULTBU)THEN
            BU_TAB(IEL,2) = DNINT(BU_TAB(IEL,2)-DFLOAT(NBU_DIFF))
            ELSEIF(IEL.GT.IMULTBU)THEN
            APRF = DNINT(APRF - DFLOAT(NBU_DIFF))
            ENDIF
         GOTO 5432
        ENDIF
           IF(IEL.GT.200)PRINT*,'5556:',IEL,RHAZ,IMULTBU
           IF(IEL.LT.1)PRINT*,'5556:',IEL,RHAZ,IMULTBU
           IF(IEL.LE.IMULTBU)THEN
            N_Breakup = DNINT(BU_TAB(IEL,2)-BU_TAB(IEL,1) -
     &               DSIGN(1.D0,DFLOAT(NBU_DIFF)))
            ELSEIF(IEL.GT.IMULTBU)THEN
            N_Breakup = DNINT(APRF - ZPRF -
     &                  DSIGN(1.D0,DFLOAT(NBU_DIFF)))
            ENDIF
             IF(N_Breakup.LT.0.D0) THEN
              IMEM_BU(IEL) = 1
              GOTO 5556
             ENDIF
             IF(IEL.LE.IMULTBU)THEN
              ATEMP = DNINT(BU_TAB(IEL,1) + N_Breakup)
              ELSEIF(IEL.GT.IMULTBU)THEN
              ATEMP = DNINT(ZPRF + N_Breakup)
             ENDIF
             IF((ATEMP - N_Breakup).LT.1.D0 .AND.
     &        N_Breakup.LT.1.D0) THEN
              IMEM_BU(IEL) = 1
              GOTO 5556
             ENDIF
c             IF((ATEMP - N_Breakup).GT.1.D0 .AND.
c     &        N_Breakup.EQ.0.D0) THEN
c              IMEM_BU(IEL) = 1
c              GOTO 5556
c             ENDIF
             IF(IEL.LE.IMULTBU)THEN
             BU_TAB(IEL,2) = DNINT(BU_TAB(IEL,1) + N_breakup)
             ELSEIF(IEL.GT.IMULTBU)THEN
             APRF = DNINT(ZPRF + N_Breakup)
             ENDIF
           NBU_DIFF = NBU_DIFF - ISIGN(1,NBU_DIFF)
          ENDDO

      IPROBA = 0
      DO I = 1, IMULTBU+1
        IMEM_BU(I) = 0
      ENDDO
         ELSEIF(ZBU_DIFF .NE. 0 .AND. NBU_DIFF .EQ. 0) THEN
          DO WHILE( ZBU_DIFF .GT. 0 .OR. ZBU_DIFF .LT. 0 )
           IS = 0
 5557      RHAZ = DBLE(HAZ(1))*DFLOAT(IMULTBU+1)
         IS = IS + 1
         IF(IS.GT.100)THEN
         PRINT*,'WARNING: HAZ CALLED MORE THAN 100 TIMES WHEN
     &   CALCULATING N_BREAKUP IN Rn07.FOR. NEW EVENT WILL BE DICED.'
         GOTO 10
          ENDIF
           IEL = INT(SNGL(RHAZ))+1
        IF(IMEM_BU(IEL) .EQ. 1) GOTO 5557
        IPROBA = IPROBA + 1
        IF(IPROBA.GT.IMULTBU+1.AND.ZBU_DIFF.GT.0) THEN
         PRINT*,'***',IPROBA,IMULTBU,NBU_DIFF,ZBU_DIFF,T_freeze_out
           IF(IEL.LE.IMULTBU)THEN
            N_Breakup = DNINT(BU_TAB(IEL,2)-BU_TAB(IEL,1))
            BU_TAB(IEL,1) = DNINT(BU_TAB(IEL,1)-DFLOAT(ZBU_DIFF))
            BU_TAB(IEL,2) = DNINT(BU_TAB(IEL,1) + N_breakup)
            ELSEIF(IEL.GT.IMULTBU)THEN
            N_Breakup = APRF - ZPRF
            ZPRF = DNINT(ZPRF - DFLOAT(ZBU_DIFF))
            APRF = DNINT(ZPRF + N_Breakup)
            ENDIF
         GOTO 5432
        ENDIF
           IF(IEL.GT.200)PRINT*,'5557:',IEL,RHAZ,IMULTBU
           IF(IEL.LT.1)PRINT*,'5557:',IEL,RHAZ,IMULTBU
            IF(IEL.LE.IMULTBU)THEN
              N_Breakup = DNINT(BU_TAB(IEL,2)-BU_TAB(IEL,1))
               ZTEMP = DNINT(BU_TAB(IEL,1) -
     &                  DSIGN(1.D0,DFLOAT(ZBU_DIFF)))
               ELSEIF(IEL.GT.IMULTBU)THEN
               N_Breakup = DNINT(APRF - ZPRF)
              ZTEMP = DNINT(ZPRF - DSIGN(1.D0,DFLOAT(ZBU_DIFF)))
             ENDIF
                ATEMP = DNINT(ZTEMP + N_breakup)
               IF(ZTEMP.LT.0.D0) THEN
               IMEM_BU(IEL) = 1
                GOTO 5557
               ENDIF
               IF((ATEMP - ZTEMP).LT.0.D0) THEN
               IMEM_BU(IEL) = 1
                GOTO 5557
               ENDIF
              IF((ATEMP - ZTEMP).LT.1.D0 .AND. ZTEMP.LT.1.D0) THEN
               IMEM_BU(IEL) = 1
               GOTO 5557
              ENDIF
c              IF((ATEMP - ZTEMP).EQ.0.D0 .AND. ZTEMP.GT.1.D0) THEN
c               IMEM_BU(IEL) = 1
c               GOTO 5557
c              ENDIF
             IF(IEL.LE.IMULTBU)THEN
             BU_TAB(IEL,1) = DNINT(ZTEMP)
            BU_TAB(IEL,2) = DNINT(ZTEMP + N_breakup)
            ELSEIF(IEL.GT.IMULTBU)THEN
            ZPRF = DNINT(ZTEMP)
            APRF = DNINT(ZTEMP + N_breakup)
            ENDIF
           ZBU_DIFF = ZBU_DIFF - ISIGN(1,ZBU_DIFF)
          ENDDO
         ENDIF
 5432 CONTINUE


C Looking for the heaviest fragment among all multifragmentation events, and
C "giving" excitation energy to fragments
         ZMEM = 0.D0

         DO I = 1, IMULTBU, 1
C For particles with Z>2 we calculate excitation energy from freeze-out temperature.
C For particels with Z<3 we assume that they form a gas, and that temperature results
C in kinetic energy (which is sampled from Maxwell distribution with T=Tfreeze-out)
C and not excitation energy.
         IF(BU_TAB(I,1).GT.2.D0)THEN
           a_tilda_BU = AV*BU_TAB(I,2) + AS*BU_TAB(I,2)**(2.D0/3.D0) +
     &                 AK*BU_TAB(I,2)**(1.D0/3.D0)
            BU_TAB(I,3) = a_tilda_BU * T_freeze_out**2          ! E* of break-up product
              ELSE
               BU_TAB(I,3) = 0.D0
           ENDIF
c          print*,I,idnint(bu_tab(i,2)),idnint(bu_tab(i,1)),BU_TAB(I,3)
          IF(BU_TAB(I,1) .GT. ZMEM)THEN
           IMEM = I
            ZMEM = BU_TAB(I,1)
             AMEM = BU_TAB(I,2)
            EMEM = BU_TAB(I,3)
           JMEM = BU_TAB(I,4)
         ENDIF
        ENDDO

        IF(ZPRF .LT. ZMEM)THEN
         BU_TAB(IMEM,1) = ZPRF
          BU_TAB(IMEM,2) = APRF
           BU_TAB(IMEM,3) = EE
            BU_TAB(IMEM,4) = JPRF
            ZPRF =  ZMEM
            APRF =  AMEM
           APRFP = IDNINT(APRF)
           ZPRFP = IDNINT(ZPRF)
          EE   =  EMEM
         JPRF =  JMEM
        ENDIF

C     Just for checking:
        ABU_SUM = APRF
        ZBU_SUM = ZPRF
        DO I = 1, IMULTBU, 1
         ABU_SUM = ABU_SUM + BU_TAB(I,2)
         ZBU_SUM = ZBU_SUM + BU_TAB(I,1)
        ENDDO
         ABU_DIFF = IDNINT(ABU_SUM-AAABRA)
         ZBU_DIFF = IDNINT(ZBU_SUM-ZAABRA)
         IF(ABU_DIFF.NE.0 .OR. ZBU_DIFF.NE.0)
     &   PRINT*,'Problem of mass in BU',ABU_DIFF,ZBU_DIFF

        PX_BU_SUM = 0.D0
        PY_BU_SUM = 0.D0
        PZ_BU_SUM = 0.D0
C Momenta of break-up products are calculated. They are all given in the rest frame
C of the primary prefragment (i.e. after abrasion or incl):
C Goldhaber model ****************************************
C "Heavy" residue
        CALL AMOMENT(AAABRA,APRF,EE,1,PXPRFP,PYPRFP,PZPRFP)
        PPRFP = DSQRT(PXPRFP*PXPRFP + PYPRFP*PYPRFP + PZPRFP*PZPRFP)
C ********************************************************
C PPRFP is in MeV/c
        ETOT_PRF = DSQRT(PPRFP*PPRFP + (APRF * AMU)*(APRF * AMU))
        VX_PREF = C * PXPRFP / ETOT_PRF
        VY_PREF = C * PYPRFP / ETOT_PRF
        VZ_PREF = C * PZPRFP / ETOT_PRF

C Contribution from Coulomb repulsion ********************
        CALL TKE_BU(ZPRF,APRF,ZAABRA,AAABRA,
     &              VX1_BU,VY1_BU,VZ1_BU)

c Lorentz kinematics - DM 17/5/2010
c        VX_PREF = VX_PREF + VX1_BU
c        VY_PREF = VY_PREF + VY1_BU
c        VZ_PREF = VZ_PREF + VZ1_BU
      CALL LORENTZ_BOOST(VX1_BU,VY1_BU,VZ1_BU,VX_PREF,VY_PREF,
     &                   VZ_PREF,VXOUT,VYOUT,VZOUT)

       VX_PREF = VXOUT
       VY_PREF = VYOUT
       VZ_PREF = VZOUT

C Total momentum: Goldhaber + Coulomb
        VBU2 = VX_PREF*VX_PREF + VY_PREF*VY_PREF + VZ_PREF*VZ_PREF
        GAMMA_REL = DSQRT(1.000D0 - VBU2 / (C*C))
        ETOT_PRF = APRF * AMU / GAMMA_REL
        PXPRFP = ETOT_PRF * VX_PREF / C
        PYPRFP = ETOT_PRF * VY_PREF / C
        PZPRFP = ETOT_PRF * VZ_PREF / C

C ********************************************************
C       Momentum: Total width of abrasion and breakup assumed to be given by Fermi momenta of nucleons
C *****************************************

        PX_BU_SUM = PXPRFP
        PY_BU_SUM = PYPRFP
        PZ_BU_SUM = PZPRFP

        Eexc_BU_SUM = EE
        Bvalue_BU = EFLMAC(IDNINT(APRF),IDNINT(ZPRF),1,0)

        DO I_Breakup = 1, IMULTBU, 1
C For bu products:
         Bvalue_BU = Bvalue_BU +
     &   EFLMAC(IDNINT(BU_TAB(I_Breakup,2)),
     &          IDNINT(BU_TAB(I_Breakup,1)),1,0)
          Eexc_BU_SUM = Eexc_BU_SUM + BU_TAB(I_Breakup,3)

          CALL AMOMENT(AAABRA,BU_TAB(I_Breakup,2),BU_TAB(I_Breakup,3),1,
     &               PX_BU,PY_BU,PZ_BU)
          P_BU = DSQRT(PX_BU*PX_BU + PY_BU*PY_BU + PZ_BU*PZ_BU)
C *******************************************************
C PPRFP is in MeV/c
          ETOT_BU = DSQRT(P_BU*P_BU +
     &             (BU_TAB(I_Breakup,2)*AMU)*(BU_TAB(I_Breakup,2)*AMU))
          BU_TAB(I_Breakup,5) = C * PX_BU / ETOT_BU     ! Velocity in x
          BU_TAB(I_Breakup,6) = C * PY_BU / ETOT_BU     ! Velocity in y
          BU_TAB(I_Breakup,7) = C * PZ_BU / ETOT_BU     ! Velocity in z
C Contribution from Coulomb repulsion:
          CALL TKE_BU(BU_TAB(I_Breakup,1),BU_TAB(I_Breakup,2),
     &              ZAABRA,AAABRA,VX2_BU,VY2_BU,VZ2_BU)
c Lorentz kinematics - DM 17/5/2010
c          BU_TAB(I_Breakup,5) = BU_TAB(I_Breakup,5) + VX2_BU ! velocity change by Coulomb repulsion
c          BU_TAB(I_Breakup,6) = BU_TAB(I_Breakup,6) + VY2_BU
c          BU_TAB(I_Breakup,7) = BU_TAB(I_Breakup,7) + VZ2_BU
      CALL LORENTZ_BOOST(VX2_BU,VY2_BU,VZ2_BU,
     &      bu_tab(i_breakup,5),bu_tab(i_breakup,6),bu_tab(i_breakup,7),
     &      VXOUT,VYOUT,VZOUT)

       bu_tab(i_breakup,5) = VXOUT
       bu_tab(i_breakup,6) = VYOUT
       bu_tab(i_breakup,7) = VZOUT

C Total momentum: Goldhaber + Coulomb
          VBU2 = BU_TAB(I_Breakup,5)*BU_TAB(I_Breakup,5) +
     &           BU_TAB(I_Breakup,6)*BU_TAB(I_Breakup,6) +
     &           BU_TAB(I_Breakup,7)*BU_TAB(I_Breakup,7)
          GAMMA_REL = DSQRT(1.000D0 - VBU2 / (C*C))
          ETOT_BU = BU_TAB(I_Breakup,2) * AMU /
     &              GAMMA_REL
          PX_BU = ETOT_BU * BU_TAB(I_Breakup,5) / C
          PY_BU = ETOT_BU * BU_TAB(I_Breakup,6) / C
          PZ_BU = ETOT_BU * BU_TAB(I_Breakup,7) / C

          PX_BU_SUM = PX_BU_SUM + PX_BU
          PY_BU_SUM = PY_BU_SUM + PY_BU
          PZ_BU_SUM = PZ_BU_SUM + PZ_BU

        ENDDO

c      print*,'BEFORE:',PX_BU_SUM,PY_BU_SUM,PZ_BU_SUM

C   In the frame of source (i.e. prefragment after abrasion or INCL)
        P_BU_SUM = DSQRT(PX_BU_SUM*PX_BU_SUM + PY_BU_SUM*PY_BU_SUM +
     &             PZ_BU_SUM*PZ_BU_SUM)
C ********************************************************
C PPRFP is in MeV/c
        ETOT_SUM = DSQRT(P_BU_SUM*P_BU_SUM +
     &             (AAABRA * AMU)*(AAABRA * AMU))

        VX_BU_SUM = C * PX_BU_SUM / ETOT_SUM
        VY_BU_SUM = C * PY_BU_SUM / ETOT_SUM
        VZ_BU_SUM = C * PZ_BU_SUM / ETOT_SUM

c Lorentz kinematics - DM 17/5/2010
c        VX_PREF = VX_PREF - VX_BU_SUM
c        VY_PREF = VY_PREF - VY_BU_SUM
c        VZ_PREF = VZ_PREF - VZ_BU_SUM
        CALL LORENTZ_BOOST(-VX_BU_SUM,-VY_BU_SUM,-VZ_BU_SUM,
     &    VX_PREF,VY_PREF,VZ_PREF,VXOUT,VYOUT,VZOUT)

        VX_PREF = VXOUT
        VY_PREF = VYOUT
        VZ_PREF = VZOUT

        VBU2 = VX_PREF*VX_PREF + VY_PREF*VY_PREF + VZ_PREF*VZ_PREF
        GAMMA_REL = DSQRT(1.000D0 - VBU2 / (C*C))
        ETOT_PRF = APRF * AMU / GAMMA_REL
        PXPRFP = ETOT_PRF * VX_PREF / C
        PYPRFP = ETOT_PRF * VY_PREF / C
        PZPRFP = ETOT_PRF * VZ_PREF / C

        PX_BU_SUM = 0.D0
        PY_BU_SUM = 0.D0
        PZ_BU_SUM = 0.D0

        PX_BU_SUM = PXPRFP
        PY_BU_SUM = PYPRFP
        PZ_BU_SUM = PZPRFP
        E_tot_BU = ETOT_PRF

        EKIN_BU = APRF * AMU / GAMMA_REL - APRF * AMU

        DO I_Breakup = 1, IMULTBU, 1
c Lorentz kinematics - DM 17/5/2010
c         BU_TAB(I_Breakup,5) = BU_TAB(I_Breakup,5) - VX_BU_SUM
c         BU_TAB(I_Breakup,6) = BU_TAB(I_Breakup,6) - VY_BU_SUM
c         BU_TAB(I_Breakup,7) = BU_TAB(I_Breakup,7) - VZ_BU_SUM
         CALL LORENTZ_BOOST(-VX_BU_SUM,-VY_BU_SUM,-VZ_BU_SUM,
     &     bu_tab(i_breakup,5),bu_tab(i_breakup,6),bu_tab(i_breakup,7),
     &     VXOUT,VYOUT,VZOUT)

         bu_tab(i_breakup,5) = VXOUT
         bu_tab(i_breakup,6) = VYOUT
         bu_tab(i_breakup,7) = VZOUT

         VBU2 = BU_TAB(I_Breakup,5)*BU_TAB(I_Breakup,5) +
     &          BU_TAB(I_Breakup,6)*BU_TAB(I_Breakup,6) +
     &          BU_TAB(I_Breakup,7)*BU_TAB(I_Breakup,7)
         GAMMA_REL = DSQRT(1.000D0 - VBU2 / (C*C))

         ETOT_BU = BU_TAB(I_Breakup,2) * AMU /
     &             GAMMA_REL

         EKIN_BU = EKIN_BU + BU_TAB(I_Breakup,2) * AMU /
     &             GAMMA_REL - BU_TAB(I_Breakup,2) * AMU

          PX_BU = ETOT_BU * BU_TAB(I_Breakup,5) / C
          PY_BU = ETOT_BU * BU_TAB(I_Breakup,6) / C
          PZ_BU = ETOT_BU * BU_TAB(I_Breakup,7) / C
          E_tot_BU = E_tot_BU + ETOT_BU

          PX_BU_SUM = PX_BU_SUM + PX_BU
          PY_BU_SUM = PY_BU_SUM + PY_BU
          PZ_BU_SUM = PZ_BU_SUM + PZ_BU
        ENDDO

c      IF(DABS(PX_BU_SUM).GT.10.d0.OR. DABS(PY_BU_SUM).GT.10.d0 .OR.
c     &   DABS(PZ_BU_SUM).GT.10.d0) print*,'1st time P not conserved:',
c     &   PX_BU_SUM,PY_BU_SUM,PZ_BU_SUM


      IF(DABS(PX_BU_SUM).GT.10.d0.OR. DABS(PY_BU_SUM).GT.10.d0 .OR.
     &   DABS(PZ_BU_SUM).GT.10.d0)THEN

C   In the frame of source (i.e. prefragment after abrasion or INCL)
        P_BU_SUM = DSQRT(PX_BU_SUM*PX_BU_SUM + PY_BU_SUM*PY_BU_SUM +
     &             PZ_BU_SUM*PZ_BU_SUM)
C ********************************************************
C PPRFP is in MeV/c
        ETOT_SUM = DSQRT(P_BU_SUM*P_BU_SUM +
     &             (AAABRA * AMU)*(AAABRA * AMU))

        VX_BU_SUM = C * PX_BU_SUM / ETOT_SUM
        VY_BU_SUM = C * PY_BU_SUM / ETOT_SUM
        VZ_BU_SUM = C * PZ_BU_SUM / ETOT_SUM

c Lorentz kinematics - DM 17/5/2010
c        VX_PREF = VX_PREF - VX_BU_SUM
c        VY_PREF = VY_PREF - VY_BU_SUM
c        VZ_PREF = VZ_PREF - VZ_BU_SUM
        CALL LORENTZ_BOOST(-VX_BU_SUM,-VY_BU_SUM,-VZ_BU_SUM,
     &    VX_PREF,VY_PREF,VZ_PREF,VXOUT,VYOUT,VZOUT)

        VX_PREF = VXOUT
        VY_PREF = VYOUT
        VZ_PREF = VZOUT

        VBU2 = VX_PREF*VX_PREF + VY_PREF*VY_PREF + VZ_PREF*VZ_PREF
        GAMMA_REL = DSQRT(1.000D0 - VBU2 / (C*C))
        ETOT_PRF = APRF * AMU / GAMMA_REL
        PXPRFP = ETOT_PRF * VX_PREF / C
        PYPRFP = ETOT_PRF * VY_PREF / C
        PZPRFP = ETOT_PRF * VZ_PREF / C

        PX_BU_SUM = 0.D0
        PY_BU_SUM = 0.D0
        PZ_BU_SUM = 0.D0

        PX_BU_SUM = PXPRFP
        PY_BU_SUM = PYPRFP
        PZ_BU_SUM = PZPRFP
        E_tot_BU = ETOT_PRF

        EKIN_BU = APRF * AMU / GAMMA_REL - APRF * AMU

        DO I_Breakup = 1, IMULTBU, 1
c Lorentz kinematics - DM 17/5/2010
c         BU_TAB(I_Breakup,5) = BU_TAB(I_Breakup,5) - VX_BU_SUM
c         BU_TAB(I_Breakup,6) = BU_TAB(I_Breakup,6) - VY_BU_SUM
c         BU_TAB(I_Breakup,7) = BU_TAB(I_Breakup,7) - VZ_BU_SUM
         CALL LORENTZ_BOOST(-VX_BU_SUM,-VY_BU_SUM,-VZ_BU_SUM,
     &     bu_tab(i_breakup,5),bu_tab(i_breakup,6),bu_tab(i_breakup,7),
     &     VXOUT,VYOUT,VZOUT)

         bu_tab(i_breakup,5) = VXOUT
         bu_tab(i_breakup,6) = VYOUT
         bu_tab(i_breakup,7) = VZOUT

         VBU2 = BU_TAB(I_Breakup,5)*BU_TAB(I_Breakup,5) +
     &          BU_TAB(I_Breakup,6)*BU_TAB(I_Breakup,6) +
     &          BU_TAB(I_Breakup,7)*BU_TAB(I_Breakup,7)
         GAMMA_REL = DSQRT(1.000D0 - VBU2 / (C*C))
         ETOT_BU = BU_TAB(I_Breakup,2) * AMU /
     &             GAMMA_REL

         EKIN_BU = EKIN_BU + BU_TAB(I_Breakup,2) * AMU /
     &             GAMMA_REL - BU_TAB(I_Breakup,2) * AMU
          PX_BU = ETOT_BU * BU_TAB(I_Breakup,5) / C
          PY_BU = ETOT_BU * BU_TAB(I_Breakup,6) / C
          PZ_BU = ETOT_BU * BU_TAB(I_Breakup,7) / C
          E_tot_BU = E_tot_BU + ETOT_BU

          PX_BU_SUM = PX_BU_SUM + PX_BU
          PY_BU_SUM = PY_BU_SUM + PY_BU
          PZ_BU_SUM = PZ_BU_SUM + PZ_BU
        ENDDO
      ENDIF

c      IF(DABS(PX_BU_SUM).GT.3.d0.OR. DABS(PY_BU_SUM).GT.3.d0 .OR.
c     &   DABS(PZ_BU_SUM).GT.3.d0) print*,'P not conserved:',
c     &   PX_BU_SUM,PY_BU_SUM,PZ_BU_SUM

C Checking the energy conservation:
      Qvalue = EFLMAC(AABRA,ZABRA,1,0) - Bvalue_BU
c      print*,'Energy',EABRA+Qvalue-(Eexc_BU_SUM+EKIN_BU)
c      write(9999,*)aabra,EABRA+Qvalue-(Eexc_BU_SUM+E_tot_BU-
c     &             (AAABRA-ZAABRA)*939.57287D0-ZAABRA*938.79044D0-
c     &              Bvalue_BU)
c      write(9999,9998)aabra,EABRA,Qvalue,Eexc_BU_SUM,E_tot_BU,
c     &             (AAABRA-ZAABRA)*939.57287D0,ZAABRA*938.79044D0,
c     &              Bvalue_BU,EKIN_BU
c 9998 format(i3,x,8f18.8)

C     *** Find the limits that fragment is bound - only done for neutrons and LCPs and for
C        nuclei with A=Z, for other nuclei it will be done after decay:


         INEWLOOP = 0
         DO I = 1, IMULTBU, 1
           hbu_tab(i)=step_bu
          IF(BU_TAB(I,1).LT.3.D0.OR.BU_TAB(I,1).EQ.BU_TAB(I,2)) THEN
             CALL UNSTABLE_NUCLEI_BU(IDNINT(BU_TAB(I,2)),
     &        IDNINT(BU_TAB(I,1)),NEVA,PEVA,DEVA,TEVA,HEEVA,AEVA,HE6EVA,
     &        AFPNEW,
     &        ZFPNEW,IOUNSTABLE,BU_TAB(I,5),BU_TAB(I,6),BU_TAB(I,7),
     &            VP1X,VP1Y,VP1Z,BU_TAB_TEMP,ILOOP)

          IF(IOUNSTABLE.GT.0)THEN
C Properties of "heavy fragment":
             BU_TAB(I,2) = DFLOAT(AFPNEW)
             BU_TAB(I,1) = DFLOAT(ZFPNEW)
             BU_TAB(I,5) = VP1X
             BU_TAB(I,6) = VP1Y
             BU_TAB(I,7) = VP1Z
             call addtohist(step_unst, hbu_tab(i))

c Properties of "light" fragments:
            DO IJ = 1,ILOOP
             BU_TAB(IMULTBU+INEWLOOP+IJ,1) = BU_TAB_TEMP(IJ,1)
             BU_TAB(IMULTBU+INEWLOOP+IJ,2) = BU_TAB_TEMP(IJ,2)
             BU_TAB(IMULTBU+INEWLOOP+IJ,5) = BU_TAB_TEMP(IJ,3)
             BU_TAB(IMULTBU+INEWLOOP+IJ,6) = BU_TAB_TEMP(IJ,4)
             BU_TAB(IMULTBU+INEWLOOP+IJ,7) = BU_TAB_TEMP(IJ,5)
             BU_TAB(IMULTBU+INEWLOOP+IJ,3) = 0.D0
             BU_TAB(IMULTBU+INEWLOOP+IJ,4) = 0.D0
             hbu_tab(imultbu+inewloop+ij) = hbu_tab(i)
c             call addtohist(step_unst, hbu_tab(imultbu+inewloop+ij))

            ENDDO

             INEWLOOP = INEWLOOP + ILOOP

          ENDIF
         ENDIF
        ENDDO

C Increased array of BU_TAB
        IMULTBU = IMULTBU + INEWLOOP
C Evaporation from multifragmentation products
        FF = 0
        FIMF = 1

        NBNBU = 0
        NBPBU = 0
        NBDBU = 0
        NBTBU = 0
        NBHEBU = 0
        NB6HEBU = 0
        NBABU = 0
        ILOOPBU = 0

        DO I=1,IMULTBU,1
         EEBU = BU_TAB(I,3)
          BU_TAB(I,11) = BU_TAB(I,7)
          IF (BU_TAB(I,1).GT.2.D0) THEN
           CALL EVAPORA(BU_TAB(I,1),BU_TAB(I,2),EEBU,
     &      0.D0,ZFBU,AFBU,NTOTNBU,NTOTPBU,NTOTDBU,
     &        NTOTTBU,NTOTHEBU,NTOTABU,NTOT6HEBU,FF,FIMF,
     &         INTTYPE,INUM,TSUM,TAUC,vx_evabu,vy_evabu,
     &          vz_evabu,TKEIMFBU,AIMFBU,ZIMFBU,BU_TAB(I,10),0,
     &             EV_TEMP,IEV_TAB_TEMP)

C Velocities of evaporated particles (in the frame of the primary prefragment)
                DO IJ = 1, IEV_TAB_TEMP, 1
                EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
                EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 17/5/2010
c                 DO IK = 3, 5, 1
c                 EV_TAB(IJ+IEV_TAB,IK) = EV_TEMP(IJ,IK) + BU_TAB(I,IK+2)
c                 ENDDO
               CALL LORENTZ_BOOST(bu_tab(i,5),bu_tab(i,6),bu_tab(i,7),
     &              EV_TEMP(IJ,3),EV_TEMP(IJ,4),EV_TEMP(IJ,5),
     &              VXOUT,VYOUT,VZOUT)
               EV_TAB(IJ+IEV_TAB,3) = VXOUT
               EV_TAB(IJ+IEV_TAB,4) = VYOUT
               EV_TAB(IJ+IEV_TAB,5) = VZOUT
                  hev_tab(ij+iev_tab) = hbu_tab(i)
                  call addtohist(step_ev, hev_tab(ij+iev_tab))
                ENDDO
                IEV_TAB = IEV_TAB + IEV_TAB_TEMP

                NBNBU = NBNBU + IDNINT(NTOTNBU)
                 NBPBU = NBPBU + IDNINT(NTOTPBU)
                  NBDBU = NBDBU + IDNINT(NTOTDBU)
                  NBTBU = NBTBU + IDNINT(NTOTTBU)
                 NBHEBU = NBHEBU + IDNINT(NTOTHEBU)
                NBABU = NBABU + IDNINT(NTOTABU)
                NB6HEBU = NB6HEBU + IDNINT(NTOT6HEBU)

C All velocities in the frame of the "primary" prefragment (after abrasion or INC)
c Lorentz kinematics - DM 17/5/2010
c                BU_TAB(I,5) = BU_TAB(I,5) + VX_EVABU  ! velocity change by evaporation
c                BU_TAB(I,6) = BU_TAB(I,6) + VY_EVABU
c                BU_TAB(I,7) = BU_TAB(I,7) + VZ_EVABU
               CALL LORENTZ_BOOST(vx_evabu,vy_evabu,vz_evabu,
     &              bu_tab(i,5),bu_tab(i,6),bu_tab(i,7),
     &              VXOUT,VYOUT,VZOUT)
               bu_tab(i,5) = VXOUT
               bu_tab(i,6) = VYOUT
               bu_tab(i,7) = VZOUT
               if(iev_tab_temp.gt.0) call addtohist(step_er,hbu_tab(i))

                TEST_DIFF = IDNINT(BU_TAB(I,2) - AFBU - AIMFBU) -
     &           (NTOTNBU+NTOTPBU+2*NTOTDBU+3*NTOTTBU+3*NTOTHEBU+
     &            4*NTOTABU+6*NTOT6HEBU)
                  IF(TEST_DIFF.NE.0) PRINT*,'TESTING-BUEVA',TEST_DIFF,
     &           BU_TAB(I,2),AF,(NTOTNBU+NTOTPBU+2*NTOTDBU+3*NTOTTBU+3*
     &          NTOTHEBU+4*NTOTABU+6*NTOT6HEBU)


                IF(FIMF.EQ.0) THEN
                 BU_TAB(I,8) = DNINT(ZFBU)
                 BU_TAB(I,9) = DNINT(AFBU)
                ENDIF

         IF(FIMF.EQ.1)THEN
C            PRINT*,'IMF EMISSION FROM BU PRODUCTS'
C IMF emission: Heavy partner is not allowed to fission or to emitt IMF.
             FEE = EEBU
              FFBU1 = 0
               FIMFBU1 = 0
C Velocities of IMF and partner: 1 denotes partner, 2 denotes IMF
                EkinR1 = TKEIMFBU * AIMFBU / (AFBU+AIMFBU)
                 EkinR2 = TKEIMFBU * AFBU / (AFBU+AIMFBU)
                  V1 = DSQRT(EkinR1/AFBU) * 1.3887D0
                   V2 = DSQRT(EkinR2/AIMFBU) * 1.3887D0
                   VZ1_IMF = (2.D0 * dble(HAZ(1)) - 1.D0) * V1
                   VPERP1 = DSQRT(V1**2 - VZ1_IMF**2)
                   ALPHA1 = dble(HAZ(1)) * 2.D0 * 3.142D0
                  VX1_IMF = VPERP1 * DSIN(ALPHA1)
                 VY1_IMF = VPERP1 * DCOS(ALPHA1)
                VX2_IMF = - VX1_IMF / V1 * V2
               VY2_IMF = - VY1_IMF / V1 * V2
             VZ2_IMF = - VZ1_IMF / V1 * V2

            EEIMFP = EEBU * AFBU /(AFBU + AIMFBU)
           EEIMF = EEBU * AIMFBU /(AFBU + AIMFBU)

C Decay of heavy partner
           IINERTTOT = 0.4D0 * 931.49D0 * 1.16D0**2 *
     &      (AIMFBU**(5.D0/3.D0) + AFBU**(5.D0/3.D0)) +
     &       931.49D0 * 1.16D0**2 * AIMFBU * AFBU / (AIMFBU + AFBU) *
     &        (AIMFBU**(1.D0/3.D0) + AFBU**(1.D0/3.D0))**2
               JPRFHEAVY = BU_TAB(I,10) * 0.4D0 * 931.49D0 * 1.16D0**2 *
     &        AFBU**(5.D0/3.D0) / IINERTTOT
            JPRFLIGHT = BU_TAB(I,10) * 0.4D0 * 931.49D0 * 1.16D0**2 *
     &     AIMFBU**(5.D0/3.D0) / IINERTTOT

c Lorentz kinematics - DM 17/5/2010
c           BU_TAB(I,5) = BU_TAB(I,5) + VX1_IMF
c           BU_TAB(I,6) = BU_TAB(I,6) + VY1_IMF
c           BU_TAB(I,7) = BU_TAB(I,7) + VZ1_IMF
           CALL LORENTZ_BOOST(vx1_imf,vy1_imf,vz1_imf,
     &          bu_tab(i,5),bu_tab(i,6),bu_tab(i,7),
     &          VXOUT,VYOUT,VZOUT)
           bu_tab(i,5) = VXOUT
           bu_tab(i,6) = VYOUT
           bu_tab(i,7) = VZOUT

           call addtohist(step_imfh,hbu_tab(i))

C Decay of IMF's partner:
           CALL EVAPORA(ZFBU,AFBU,EEIMFP,JPRFHEAVY,ZFFBU,AFFBU,
     &             NTOTNIMPBU,NTOTPIMPBU,NTOTDIMPBU,NTOTTIMPBU,
     &             NTOTHEIMPBU,NTOTAIMPBU,NTOT6HEIMPBU,FFBU1,FIMFBU1,
     &             INTTYPE,INUM,
     &             TSUMBU,TAUCBU,vx1ev_imf,vy1ev_imf,vz1ev_imf,tkdummy,
     &             adummy,zdummy,JPRF1,0,
     &             EV_TEMP,IEV_TAB_TEMP)


               DO IJ = 1, IEV_TAB_TEMP, 1
               EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
               EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 17/5/2010
c                DO IK = 3, 5, 1
c                 EV_TAB(IJ+IEV_TAB,IK) = EV_TEMP(IJ,IK) + BU_TAB(I,IK+2)
c                ENDDO
               CALL LORENTZ_BOOST(bu_tab(i,5),bu_tab(i,6),bu_tab(i,7),
     &              EV_TEMP(IJ,3),EV_TEMP(IJ,4),EV_TEMP(IJ,5),
     &              VXOUT,VYOUT,VZOUT)
               EV_TAB(IJ+IEV_TAB,3) = VXOUT
               EV_TAB(IJ+IEV_TAB,4) = VYOUT
               EV_TAB(IJ+IEV_TAB,5) = VZOUT
               hev_tab(ij+iev_tab)=hbu_tab(i)
               call addtohist(step_ev,hev_tab(ij+iev_tab))
               ENDDO
               IEV_TAB = IEV_TAB + IEV_TAB_TEMP

           NBNBU = NBNBU + IDNINT(NTOTNIMPBU)
            NBPBU = NBPBU + IDNINT(NTOTPIMPBU)
             NBDBU = NBDBU + IDNINT(NTOTDIMPBU)
             NBTBU = NBTBU + IDNINT(NTOTTIMPBU)
            NBHEBU = NBHEBU + IDNINT(NTOTHEIMPBU)
           NBABU = NBABU + IDNINT(NTOTAIMPBU)
            NB6HEBU = NB6HEBU + IDNINT(NTOT6HEIMPBU)

           BU_TAB(I,8) = DNINT(ZFFBU)
            BU_TAB(I,9) = DNINT(AFFBU)
c Lorentz kinematics - DM 17/5/2010
c           BU_TAB(I,5) = BU_TAB(I,5) + vx1ev_imf
c           BU_TAB(I,6) = BU_TAB(I,6) + vy1ev_imf
c           BU_TAB(I,7) = BU_TAB(I,7) + vz1ev_imf
           CALL LORENTZ_BOOST(vx1ev_imf,vy1ev_imf,vz1ev_imf,
     &          bu_tab(i,5),bu_tab(i,6),bu_tab(i,7),
     &          VXOUT,VYOUT,VZOUT)
           bu_tab(i,5) = VXOUT
           bu_tab(i,6) = VYOUT
           bu_tab(i,7) = VZOUT
           if(iev_tab_temp.gt.0) call addtohist(step_er,hbu_tab(i))

C For IMF - fission and IMF emission are not allowed
          FFBU2 = 0
          FIMFBU2 = 0

C Decay of IMF
          CALL EVAPORA(ZIMFBU,AIMFBU,EEIMF,JPRFLIGHT,ZFFIMF,AFFIMF,
     &             NTOTNIMFBU,NTOTPIMFBU,NTOTDIMFBU,NTOTTIMFBU,
     &             NTOTHEIMFBU,NTOTAIMFBU,NTOT6HEIMFBU,FFBU2,FIMFBU2,
     &             INTTYPE,INUM,
     &             TSUMBU,TAUCBU,vx2ev_imf,vy2ev_imf,vz2ev_imf,tkdummy1,
     &             adummy1,zdummy1,JPRF2,0,
     &             EV_TEMP,IEV_TAB_TEMP)


           DO IJ = 1, IEV_TAB_TEMP, 1
            EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
            EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 17/5/2010
c            EV_TAB(IJ+IEV_TAB,3) = EV_TEMP(IJ,3) + BU_TAB(I,5) +VX2_IMF
c            EV_TAB(IJ+IEV_TAB,4) = EV_TEMP(IJ,4) + BU_TAB(I,6) +VY2_IMF
c            EV_TAB(IJ+IEV_TAB,5) = EV_TEMP(IJ,5) + BU_TAB(I,7) +VZ2_IMF
            CALL LORENTZ_BOOST(bu_tab(i,5),bu_tab(i,6),bu_tab(i,7),
     &           EV_TEMP(IJ,3),EV_TEMP(IJ,4),EV_TEMP(IJ,5),
     &           VXOUT,VYOUT,VZOUT)
            CALL LORENTZ_BOOST(vx2_imf,vy2_imf,vz2_imf,
     &           vxout,vyout,vzout,
     &           VX2OUT,VY2OUT,VZ2OUT)
            EV_TAB(IJ+IEV_TAB,3) = VX2OUT
            EV_TAB(IJ+IEV_TAB,4) = VY2OUT
            EV_TAB(IJ+IEV_TAB,5) = VZ2OUT
            hev_tab(ij+iev_tab) = hbu_tab(i)
            call addtohist(step_imfl, hev_tab(ij+iev_tab))
            call addtohist(step_ev, hev_tab(ij+iev_tab))
           ENDDO
           IEV_TAB = IEV_TAB + IEV_TAB_TEMP

           NBNBU = NBNBU + IDNINT(NTOTNIMFBU)
            NBPBU = NBPBU + IDNINT(NTOTPIMFBU)
             NBDBU = NBDBU + IDNINT(NTOTDIMFBU)
             NBTBU = NBTBU + IDNINT(NTOTTIMFBU)
            NBHEBU = NBHEBU + IDNINT(NTOTHEIMFBU)
           NBABU = NBABU + IDNINT(NTOTAIMFBU)
            NB6HEBU = NB6HEBU + IDNINT(NTOT6HEIMFBU)

           ILOOPBU = ILOOPBU + 1
            BU_TAB(IMULTBU+ILOOPBU,1) = BU_TAB(I,1)
             BU_TAB(IMULTBU+ILOOPBU,2) = BU_TAB(I,2)
              BU_TAB(IMULTBU+ILOOPBU,3) = BU_TAB(I,3)
               BU_TAB(IMULTBU+ILOOPBU,4) = BU_TAB(I,4)
               BU_TAB(IMULTBU+ILOOPBU,8) = DNINT(ZFFIMF)
              BU_TAB(IMULTBU+ILOOPBU,9) = DNINT(AFFIMF)
c            BU_TAB(IMULTBU+ILOOPBU,5) = BU_TAB(I,5)+VX2_IMF+vx2ev_imf
c            BU_TAB(IMULTBU+ILOOPBU,6) = BU_TAB(I,6)+VY2_IMF+vy2ev_imf
c            BU_TAB(IMULTBU+ILOOPBU,7) = BU_TAB(I,7)+VZ2_IMF+vz2ev_imf
            CALL LORENTZ_BOOST(vx2_imf,vy2_imf,vz2_imf,
     &           bu_tab(i,5),bu_tab(i,6),bu_tab(i,7),
     &           VXOUT,VYOUT,VZOUT)
            CALL LORENTZ_BOOST(vx2ev_imf,vy2ev_imf,vz2ev_imf,
     &           vxout,vyout,vzout,
     &           VX2OUT,VY2OUT,VZ2OUT)
            bu_tab(imultbu+iloopbu,5)=vx2out
            bu_tab(imultbu+iloopbu,6)=vy2out
            bu_tab(imultbu+iloopbu,7)=vz2out
            hbu_tab(imultbu+iloopbu)=hbu_tab(i)
            call addtohist(step_imfl,hbu_tab(imultbu+iloopbu))
            if(iev_tab_temp.gt.0)
     +         call addtohist(step_er,hbu_tab(imultbu+iloopbu))
          ENDIF      ! For FIMF.EQ.1

        ELSE        ! If BU_TAB(I,1).le.2.D0
          BU_TAB(I,1) = BU_TAB(I,1)
           BU_TAB(I,2) = BU_TAB(I,2)
            BU_TAB(I,3) = BU_TAB(I,3)
             BU_TAB(I,4) = BU_TAB(I,4)
             BU_TAB(I,8) = BU_TAB(I,1)
            BU_TAB(I,9) = BU_TAB(I,2)
           BU_TAB(I,5) = BU_TAB(I,5)
          BU_TAB(I,6) = BU_TAB(I,6)
         BU_TAB(I,7) = BU_TAB(I,7)
        ENDIF

      ENDDO      ! I=1,IMULTBU,1

      IMULTBU = IMULTBU + ILOOPBU
c     PRINT*,'TS',IMULTBU, ILOOPBU

C
C RESOLVE UNSTABLE NUCLEI
C
      INEWLOOP = 0
      ABU_SUM = 0.D0
      ZBU_SUM = 0.D0

      DO I = 1, IMULTBU, 1
       ABU_SUM = ABU_SUM + BU_TAB(I,9)
        ZBU_SUM = ZBU_SUM + BU_TAB(I,8)
c        IF (BU_TAB(I,8).GT.1.D0) THEN
            CALL UNSTABLE_NUCLEI(IDNINT(BU_TAB(I,9)),
     &        IDNINT(BU_TAB(I,8)),NEVA,PEVA,DEVA,TEVA,HEEVA,AEVA,HE6EVA,
     &        AFPNEW,
     &        ZFPNEW,IOUNSTABLE,BU_TAB(I,5),BU_TAB(I,6),BU_TAB(I,7),
     &            VP1X,VP1Y,VP1Z,BU_TAB_TEMP1,ILOOP)

C From now on, all neutrons and LCP created in above subroutine are part of the
C BU_TAB array (see below - Properties of "light" fragments). Therefore,
C NEVA, PEVA ... are not needed any more in the break-up stage.

          IF(IOUNSTABLE.GT.0)THEN
C Properties of "heavy fragment":
            ABU_SUM = ABU_SUM + DFLOAT(AFPNEW) - BU_TAB(I,9)
            ZBU_SUM = ZBU_SUM + DFLOAT(ZFPNEW) - BU_TAB(I,8)

c          print*,'333',IDNINT(BU_TAB(I,8)),
c     &        IDNINT(BU_TAB(I,9)),zfpnew,afpnew

             BU_TAB(I,9) = DFLOAT(AFPNEW)
             BU_TAB(I,8) = DFLOAT(ZFPNEW)
             BU_TAB(I,5) = VP1X
             BU_TAB(I,6) = VP1Y
             BU_TAB(I,7) = VP1Z
             call addtohist(step_unst, hbu_tab(i))

c Properties of "light" fragments:
            DO IJ = 1,ILOOP
             BU_TAB(IMULTBU+INEWLOOP+IJ,8) = BU_TAB_TEMP1(IJ,1)
             BU_TAB(IMULTBU+INEWLOOP+IJ,9) = BU_TAB_TEMP1(IJ,2)
             BU_TAB(IMULTBU+INEWLOOP+IJ,5) = BU_TAB_TEMP1(IJ,3)
             BU_TAB(IMULTBU+INEWLOOP+IJ,6) = BU_TAB_TEMP1(IJ,4)
             BU_TAB(IMULTBU+INEWLOOP+IJ,7) = BU_TAB_TEMP1(IJ,5)
             BU_TAB(IMULTBU+INEWLOOP+IJ,3) = 0.D0
             BU_TAB(IMULTBU+INEWLOOP+IJ,4) = 0.D0
             BU_TAB(IMULTBU+INEWLOOP+IJ,1) = BU_TAB(I,1)
             BU_TAB(IMULTBU+INEWLOOP+IJ,2) = BU_TAB(I,2)
             hbu_tab(imultbu+inewloop+ij) = hbu_tab(i)
c             call addtohist(step_unst, hbu_tab(imultbu+inewloop+ij))

            ABU_SUM = ABU_SUM + BU_TAB(IMULTBU+INEWLOOP+IJ,9)
            ZBU_SUM = ZBU_SUM + BU_TAB(IMULTBU+INEWLOOP+IJ,8)
            ENDDO

             INEWLOOP = INEWLOOP + ILOOP

          ENDIF
c         ENDIF
       ENDDO

C Increased array of BU_TAB
        IMULTBU = IMULTBU + INEWLOOP

 1133  continue
C Transform all velocities into the rest frame of the projectile
      CALL LORENTZ_BOOST(VX_ABRA,VY_ABRA,VZ_ABRA,VX_PREF,VY_PREF,
     &                   VZ_PREF,VXOUT,VYOUT,VZOUT)

       VX_PREF = VXOUT
       VY_PREF = VYOUT
       VZ_PREF = VZOUT

      DO I = 1, IMULTBU, 1
       CALL LORENTZ_BOOST(VX_ABRA,VY_ABRA,VZ_ABRA,
     &                    BU_TAB(I,5),BU_TAB(I,6),BU_TAB(I,7),
     &                    VXOUT,VYOUT,VZOUT)
       BU_TAB(I,5) = VXOUT
       BU_TAB(I,6) = VYOUT
       BU_TAB(I,7) = VZOUT
      ENDDO

      DO I = 1, IEV_TAB, 1
       CALL LORENTZ_BOOST(VX_ABRA,VY_ABRA,VZ_ABRA,
     &                    EV_TAB(I,3),EV_TAB(I,4),EV_TAB(I,5),
     &                    VXOUT,VYOUT,VZOUT)
       EV_TAB(I,3) = VXOUT
       EV_TAB(I,4) = VYOUT
       EV_TAB(I,5) = VZOUT
      ENDDO

      TEST_DIFF = (AABRA-ZABRA) - IDNINT(ABU_SUM-ZBU_SUM+APRF-ZPRF)-
     &  (NBNBU+NBDBU+2*NBTBU+1*NBHEBU+2*NBABU+4*NB6HEBU)
      IF(TEST_DIFF.NE.0) then
       print*,'N not conserved in BU',TEST_DIFF,AABRA,
     &  ZABRA,IDNINT(ABU_SUM),IDNINT(ZBU_SUM),IDNINT(APRF),IDNINT(ZPRF),
     &  IMULTBU,NBNBU,NBDBU,NBTBU,NBHEBU,NBABU,NB6HEBU
       print*,'BU products:'
       do i=1,imultbu
        print*,BU_TAB(i,9),BU_TAB(i,8),BU_TAB(i,2),BU_TAB(i,1)
       enddo
       endif
      TEST_DIFF = ZABRA - IDNINT(ZBU_SUM+ZPRF)-
     &  (NBPBU+NBDBU+NBTBU+2*NBHEBU+2*NBABU+2*NB6HEBU)
      IF(TEST_DIFF.NE.0) then
       print*,'Z not conserved in BU',TEST_DIFF,IDNINT(ZBU_SUM),
     & NBPBU+NBDBU+NBTBU+2*NBHEBU+2*NBABU+2*NB6HEBU,IMULTBU
      endif


      IF(IMULTBU.GT.200) PRINT*,'IMULTBU>200',IMULTBU,AABRA,ZABRA,EABRA,
     & T_freeze_out


      ENDIF  ! For T_diff.gt.0.1D0

C***************************************************

C**** DECAY OF HEAVY PREFRAGMENT ********************

 7777   CONTINUE

        APRFP = IDNINT(APRF)
        ZPRFP = IDNINT(ZPRF)
        EPRFP = EE


       hist0 = step_bu
      IF(IMULTIFR.EQ.0) THEN
C These momenta are in the frame of the projectile (or target in case of direct kinematics)
       PXPRFP = PXABRA
       PYPRFP = PYABRA
       PZPRFP = PZABRA
       PPRFP = PABRA
       ETOT_PRF = ETOT_ABRA
       VX_PREF = VX_ABRA
       VY_PREF = VY_ABRA
       VZ_PREF = VZ_ABRA
       hist0 = 0
      ENDIF
C New JPRF model*****************************************
c      CALL AMOMENT(AAABRA,APRF,EE,IMULTIFR,PXPRFP,PYPRFP,PZPRFP)
c      PPRFP = DSQRT(PXPRFP**2.d0 + PYPRFP**2.D0 + PZPRFP**2.D0)
C *******************************************************
C PPRFP is in MeV/c
c      ETOT_PRF = DSQRT(PPRFP**2.D0 + (APRF * 931.494D0)**2.D0)
c      VX_PREF = C * PXPRFP / ETOT_PRF
c      VY_PREF = C * PYPRFP / ETOT_PRF
c      VZ_PREF = C * PZPRFP / ETOT_PRF


C CALL THE EVAPORATION SUBROUTINE
C
      FF   = 1 ! fission is allowed
      FIMF = 1  ! IMF is allowed

C To spare computing time; these events in any case cannot decay
      IF(ZPRFP.LE.2.AND.ZPRFP.LT.APRFP)THEN
       ZF = ZPRF
       AF = APRF
       EE = 0.D0
       FF = 0
       FIMF = 0
       FTYPE = 0
       AIMF = 0.D0
       ZIMF = 0.D0
       TKEIMF = 0.D0
       NTOTNCN = 0.D0
       NTOTPCN = 0.D0
       NTOTDCN = 0.D0
       NTOTTCN = 0.D0
       NTOTHECN = 0.D0
       NTOTACN = 0.D0
       NTOT6HECN = 0.D0
       vx_eva = 0.D0
       vy_eva = 0.D0
       vz_eva = 0.D0
       JPRF0 = JPRF
       GOTO 1972
      ENDIF

      IF(ZPRFP.LE.2.AND.ZPRFP.EQ.APRFP)THEN
       CALL UNSTABLE_NUCLEI(APRFP,ZPRFP,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &                      VX_PREF, VY_PREF, VZ_PREF,
     &                     VP1X,VP1Y,VP1Z,EV_TAB_TEMP,ILOOP)
         AF = DFLOAT(AFPNEW)
         ZF = DFLOAT(ZFPNEW)
         VX_PREF = VP1X
         VY_PREF = VP1Y
         VZ_PREF = VP1Z
         DO I = 1, ILOOP
          DO IJ = 1, 5
           EV_TAB(I+IEV_TAB,IJ) = EV_TAB_TEMP(I,IJ)
          ENDDO
          hev_tab(i+iev_tab) = hist0
          call addtohist(step_unst, hev_tab(i+iev_tab))
         ENDDO
        IEV_TAB = IEV_TAB + ILOOP
        EE = 0.D0
        FF = 0
        FIMF = 0
        FTYPE = 0
        AIMF = 0.D0
        ZIMF = 0.D0
        TKEIMF = 0.D0
        NTOTNCN = DFLOAT(NEVA)
        NTOTPCN = DFLOAT(PEVA)
        NTOTDCN = DFLOAT(DEVA)
        NTOTTCN = DFLOAT(TEVA)
        NTOTHECN = DFLOAT(HEEVA)
        NTOTACN = DFLOAT(AEVA)
        NTOT6HECN = DFLOAT(HE6EVA)
        vx_eva = 0.D0
        vy_eva = 0.D0
        vz_eva = 0.D0
        JPRF0 = JPRF
        if(iloop.gt.0) call addtohist(step_unst, hist0)
       GOTO 1972
      ENDIF

      IF(ZPRFP.EQ.APRFP)THEN
      CALL UNSTABLE_NUCLEI(APRFP,ZPRFP,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &                      VX_PREF, VY_PREF, VZ_PREF,
     &                     VP1X,VP1Y,VP1Z,EV_TAB_TEMP,ILOOP)
         AF = DFLOAT(AFPNEW)
         ZF = DFLOAT(ZFPNEW)
         VX_PREF = VP1X
         VY_PREF = VP1Y
         VZ_PREF = VP1Z
         DO I = 1, ILOOP
          DO IJ = 1, 5
           EV_TAB(I+IEV_TAB,IJ) = EV_TAB_TEMP(I,IJ)
          ENDDO
          hev_tab(i+iev_tab) = hist0
          call addtohist(step_unst, hev_tab(i+iev_tab))
         ENDDO
        IEV_TAB = IEV_TAB + ILOOP
        EE = 0.D0
        FF = 0
        FIMF = 0
        FTYPE = 0
        AIMF = 0.D0
        ZIMF = 0.D0
        TKEIMF = 0.D0
        NTOTNCN = DFLOAT(NEVA)
        NTOTPCN = DFLOAT(PEVA)
        NTOTDCN = DFLOAT(DEVA)
        NTOTTCN = DFLOAT(TEVA)
        NTOTHECN = DFLOAT(HEEVA)
        NTOTACN = DFLOAT(AEVA)
        NTOT6HECN = DFLOAT(HE6EVA)
        vx_eva = 0.D0
        vy_eva = 0.D0
        vz_eva = 0.D0
        JPRF0 = JPRF
        if(iloop.gt.0) call addtohist(step_unst, hist0)
       GOTO 1972
      ENDIF

       CALL EVAPORA(ZPRF,APRF,EE,JPRF,ZF,AF,NTOTNCN,NTOTPCN,NTOTDCN,
     &             NTOTTCN,NTOTHECN,NTOTACN,NTOT6HECN,FF,FIMF,
     &             INTTYPE,INUM,TSUM,TAUC,vx_eva,vy_eva,
     &             vz_eva,TKEIMF,AIMF,ZIMF,JPRF0,0,
     &             EV_TEMP,IEV_TAB_TEMP)


               DO IJ = 1, IEV_TAB_TEMP, 1
               EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
               EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c               EV_TAB(IJ+IEV_TAB,3) = EV_TEMP(IJ,3) + VX_PREF
c               EV_TAB(IJ+IEV_TAB,4) = EV_TEMP(IJ,4) + VY_PREF
c               EV_TAB(IJ+IEV_TAB,5) = EV_TEMP(IJ,5) + VZ_PREF
C Lorentz transformation
               CALL LORENTZ_BOOST(VX_PREF,VY_PREF,VZ_PREF,
     &              EV_TEMP(IJ,3),EV_TEMP(IJ,4),EV_TEMP(IJ,5),
     &              VXOUT,VYOUT,VZOUT)
               EV_TAB(IJ+IEV_TAB,3) = VXOUT
               EV_TAB(IJ+IEV_TAB,4) = VYOUT
               EV_TAB(IJ+IEV_TAB,5) = VZOUT
               hev_tab(ij+iev_tab) = hist0
               call addtohist(step_ev,hev_tab(ij+iev_tab))
               ENDDO
               IEV_TAB = IEV_TAB + IEV_TAB_TEMP

               if(iev_tab_temp.gt.0) call addtohist(step_er,hist0)

      TEST_DIFF = APRFP - IDNINT(AF+AIMF)-
     & IDNINT(NTOTNCN+NTOTPCN+2.D0*NTOTDCN+
     &        3.D0*NTOTTCN+3.D0*NTOTHECN+4.D0*NTOTACN+6.D0*NTOT6HECN)
      IF(TEST_DIFF.NE.0)THEN
       PRINT*,'TESTING-1:',test_diff,aprfp,af,aimf,idnint(af+aimf),
     &             NTOTNCN,NTOTPCN,
     &             NTOTDCN,NTOTTCN,NTOTHECN,NTOTACN
       print*,' '
      ENDIF

 1972 CONTINUE

c vi_pref - velocity of the prefragment; vi_eva - recoil due to evaporation
      CALL LORENTZ_BOOST(VX_PREF,VY_PREF,VZ_PREF,
     &     VX_EVA,VY_EVA,VZ_EVA,
     &     VXOUT,VYOUT,VZOUT)
      V_CM(1) = VXOUT
      V_CM(2) = VYOUT
      V_CM(3) = VZOUT

      IF(FF.EQ.0 .AND. FIMF .EQ. 0) THEN
C Evaporation of neutrons and LCP; no IMF, no fission
      FTYPE = 0
      ZFP1 = IDNINT(ZF)
      AFP1 = IDNINT(AF)
      hist1 = hist0
      AFPIMF = 0
      ZFPIMF = 0
      ZFP2 = 0
      AFP2 = 0
      JPRFFIN = JPRF
      VFP1_CM(1) = V_CM(1)
      VFP1_CM(2) = V_CM(2)
      VFP1_CM(3) = V_CM(3)
       DO I = 1, 3
        VIMF_CM(I) = 0.D0
        VFP2_CM(I) = 0.D0
       ENDDO
      ENDIF

      IF(FF.EQ.1 .AND. FIMF .EQ. 0) FTYPE = 1        ! fission
      IF(FF.EQ.0 .AND. FIMF .EQ. 1) FTYPE = 2        ! IMF emission

C
C AFP,ZFP IS THE FINAL FRAGMENT IF NO FISSION OR IMF EMISSION OCCURS
C IN CASE OF FISSION IT IS THE NUCLEUS THAT UNDERGOES FISSION OR IMF
C

      ZFP = IDNINT(ZF)
      AFP = IDNINT(AF)
C
C***************** FISSION ***************************************
C
      IF (FTYPE.EQ.1) THEN

         kproc = kproc + 1

          FTYPE1 = 0
C
C FEE: ENERGY OF FISSIONING NUCLEUS ABOVE THE FISSION BARRIER.
C ACN, ZCN: A and Z OF FISSIONING NUCLEUS AT THE FISSION BARRIER
        FEE = EE
        ACN = IDNINT(AF)
        ZCN = IDNINT(ZF)

        izfa07=nint(zf)
        iafa07=nint(af)
        esfa07=ee

      CALL FISSION(AF,ZF,EE,JPRF0,INUM,
     &    VX1_FISSION,VY1_FISSION,VZ1_FISSION,
     &    VX2_FISSION,VY2_FISSION,VZ2_FISSION,
     &    NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,N6HE_POST,
     &    ZFP1,AFP1,POSTN1,POSTP1,POSTD1,POSTT1,POSTHE1,POSTAL1,
     &    POST6HE1,FF1,FIMF1,TSUM1,TAUC1,TKEIMF1,AIMFF1,ZIMFF1,
     &    ZFP2,AFP2,POSTN2,POSTP2,POSTD2,POSTT2,POSTHE2,POSTAL2,
     &    POST6HE2,FF2,FIMF2,TSUM2,TAUC2,TKEIMF2,AIMFF2,ZIMFF2,imode,
     &    VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC,EV_TEMP,IEV_TAB_FIS,
     &    hev_temp,hist_fis1,hist_fis2)

               DO IJ = 1, IEV_TAB_FIS, 1
               EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
               EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 18/5/2010
c               EV_TAB(IJ+IEV_TAB,3) = EV_TEMP(IJ,3) + V_CM(1)
c               EV_TAB(IJ+IEV_TAB,4) = EV_TEMP(IJ,4) + V_CM(2)
c               EV_TAB(IJ+IEV_TAB,5) = EV_TEMP(IJ,5) + V_CM(3)
               CALL LORENTZ_BOOST(v_cm(1),v_cm(2),v_cm(3),
     &             ev_temp(ij,3),ev_temp(ij,4),ev_temp(ij,5),
     &             VXOUT,VYOUT,VZOUT)
               ev_tab(ij+iev_tab,3) = VXOUT
               ev_tab(ij+iev_tab,4) = VYOUT
               ev_tab(ij+iev_tab,5) = VZOUT
               hev_tab(ij+iev_tab) = hist0
               call addtohist(hev_temp(ij),hev_tab(ij+iev_tab))
               ENDDO
               IEV_TAB = IEV_TAB + IEV_TAB_FIS

       IF(IMODE.EQ.1) FTYPE1 = 1    ! S1 mode
       IF(IMODE.EQ.2) FTYPE1 = 2    ! S2 mode
C
C NN_POST .... NA_POST -> number of particles emitted between saddle and scission
C POSTN1, POSTN2 ... POSTAL1,POSTAL2 -> number of particles emitted from two fragments

          AFPIMF = 0
          ZFPIMF = 0

        TEST_DIFF = IDNINT(AF) - (AFP1+AFP2) -
     &  IDNINT(NN_POST+NP_POST+2.d0*ND_POST+3.d0*NT_POST+3.d0*NHE_POST+
     &   4.d0*NA_POST+6.d0*N6HE_POST) -
     & (POSTN1+POSTP1+2*POSTD1+3*POSTT1+3*POSTHE1+4*POSTAL1+6*POST6HE1)-
     &  (POSTN2+POSTP2+2*POSTD2+3*POSTT2+3*POSTHE2+4*POSTAL2+6*POST6HE2)
        IF(TEST_DIFF.NE.0)THEN
        PRINT*,'TESTING_F1-F2:',TEST_DIFF,IDNINT(AF),AFP1,AFP2,
     &  NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,N6HE_POST,
     &  POSTN1,POSTP1,POSTD1,POSTT1,POSTHE1,POSTAL1,POST6HE1,
     &  POSTN2,POSTP2,POSTD2,POSTT2,POSTHE2,POSTAL2,POST6HE2
        ENDIF

C VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC - recoil due to particle emisison
C between saddle and scission
c Lorentz kinematics - DM 18/5/2010
c        VFP1_CM(1) = V_CM(1) + VX1_FISSION + VX_EVA_SC ! Velocity of FF1 in x
c        VFP1_CM(2) = V_CM(2) + VY1_FISSION + VY_EVA_SC ! Velocity of FF1 in y
c        VFP1_CM(3) = V_CM(3) + VZ1_FISSION + VZ_EVA_SC ! Velocity of FF1 in x
        CALL LORENTZ_BOOST(vx1_fission,vy1_fission,vz1_fission,
     &       v_cm(1),v_cm(2),v_cm(3),
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx_eva_sc,vy_eva_sc,vz_eva_sc,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        vfp1_cm(1) = VX2OUT
        vfp1_cm(2) = VY2OUT
        vfp1_cm(3) = VZ2OUT
        hist1 = hist0
        call addtohist(hist_fis1,hist1)

c Lorentz kinematics - DM 18/5/2010
c        VFP2_CM(1) = V_CM(1) + VX2_FISSION + VX_EVA_SC ! Velocity of FF2 in x
c        VFP2_CM(2) = V_CM(2) + VY2_FISSION + VY_EVA_SC ! Velocity of FF2 in y
c        VFP2_CM(3) = V_CM(3) + VZ2_FISSION + VZ_EVA_SC ! Velocity of FF2 in x
        CALL LORENTZ_BOOST(vx2_fission,vy2_fission,vz2_fission,
     &       v_cm(1),v_cm(2),v_cm(3),
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx_eva_sc,vy_eva_sc,vz_eva_sc,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        vfp2_cm(1) = VX2OUT
        vfp2_cm(2) = VY2OUT
        vfp2_cm(3) = VZ2OUT
        hist2 = hist0
        call addtohist(hist_fis2,hist2)
        DO I = 1, 3
         VIMF_CM(I) = 0.D0
        ENDDO
       JPRFFIN = 0.D0
C
C************** IMF EMISSION ************************************************
C
      ELSEIF ( FTYPE.EQ.2) THEN
C IMF emission: Heavy partner is allowed to fission and to emitt IMF, but ONLY once.
      FEE = EE
      FF11 = 1
      FIMF11 = 1

C Velocities of IMF and partner: 1 denotes partner, 2 denotes IMF
        EkinR1 = TKEIMF * AIMF / (AF+AIMF)
        EkinR2 = TKEIMF * AF / (AF+AIMF)
        V1 = DSQRT(EkinR1/AF) * 1.3887D0
        V2 = DSQRT(EkinR2/AIMF) * 1.3887D0
        VZ1_IMF = (2.D0 * dble(HAZ(1)) - 1.D0) * V1
        VPERP1 = DSQRT(V1**2 - VZ1_IMF**2)
        ALPHA1 = dble(HAZ(1)) * 2.D0 * 3.142D0
        VX1_IMF = VPERP1 * DSIN(ALPHA1)
        VY1_IMF = VPERP1 * DCOS(ALPHA1)
        VX2_IMF = - VX1_IMF / V1 * V2
        VY2_IMF = - VY1_IMF / V1 * V2
        VZ2_IMF = - VZ1_IMF / V1 * V2

      EEIMFP = EE * AF /(AF + AIMF)
      EEIMF = EE * AIMF /(AF + AIMF)

C Decay of heavy partner
      IINERTTOT = 0.4D0 * 931.49D0 * 1.16D0**2 *
     &          (AIMF**(5.D0/3.D0) + AF**(5.D0/3.D0)) +
     &          931.49D0 * 1.16D0**2 * AIMF * AF / (AIMF + AF) *
     &          (AIMF**(1.D0/3.D0) + AF**(1.D0/3.D0))**2
      JPRFHEAVY = JPRF0 * 0.4D0 * 931.49D0 * 1.16D0**2 *
     &            AF**(5.D0/3.D0) / IINERTTOT
      JPRFLIGHT = JPRF0 * 0.4D0 * 931.49D0 * 1.16D0**2 *
     &            AIMF**(5.D0/3.D0) / IINERTTOT

      IF(AF.LT.2.D0) PRINT*,'RN117-4',AF,ZF,EE,JPRFheavy

      CALL EVAPORA(ZF,AF,EEIMFP,JPRFHEAVY,ZFF,AFF,NTOTNIMP,NTOTPIMP,
     &             NTOTDIMP,NTOTTIMP,NTOTHEIMP,NTOTAIMP,NTOT6HEIMP,
     &             FF11,FIMF11,
     &             INTTYPE,INUM,TSUM,TAUC,vx1ev_imf,vy1ev_imf,
     &             vz1ev_imf,tkdummy,adummy,zdummy,JPRF1,0,
     &             EV_TEMP,IEV_TAB_TEMP)


               DO IJ = 1, IEV_TAB_TEMP, 1
               EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
               EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 18/5/2010
c               EV_TAB(IJ+IEV_TAB,3) = EV_TEMP(IJ,3) + V_CM(1) + VX1_IMF
c               EV_TAB(IJ+IEV_TAB,4) = EV_TEMP(IJ,4) + V_CM(2) + VY1_IMF
c               EV_TAB(IJ+IEV_TAB,5) = EV_TEMP(IJ,5) + V_CM(3) + VZ1_IMF
               CALL LORENTZ_BOOST(v_cm(1),v_cm(2),v_cm(3),
     &              ev_temp(ij,3),ev_temp(ij,4),ev_temp(ij,5),
     &              VXOUT,VYOUT,VZOUT)
               CALL LORENTZ_BOOST(vx1_imf,vy1_imf,vz1_imf,
     &              vxout,vyout,vzout,
     &              VX2OUT,VY2OUT,VZ2OUT)
               ev_tab(ij+iev_tab,3) = VX2OUT
               ev_tab(ij+iev_tab,4) = VY2OUT
               ev_tab(ij+iev_tab,5) = VZ2OUT
               hev_tab(ij+iev_tab) = hist0
               call addtohist(step_imfh,hev_tab(ij+iev_tab))
               call addtohist(step_ev,hev_tab(ij+iev_tab))
               ENDDO
               IEV_TAB = IEV_TAB + IEV_TAB_TEMP
               hist1 = hist0
               call addtohist(step_imfh,hist1)
               if(iev_tab_temp.gt.0)
     +            call addtohist(step_er,hist1)

      TEST_DIFF = IDNINT(AF - AFF - ADUMMY) -
     & IDNINT(NTOTNIMP+NTOTPIMP+2.D0*NTOTDIMP+
     &       3.D0*NTOTTIMP+3.D0*NTOTHEIMP+4.D0*NTOTAIMP+6.D0*NTOT6HEIMP)
      IF(TEST_DIFF.NE.0)THEN
       PRINT*,'TESTING-2:',test_diff,AF,AFF,ADUMMY,
     &             NTOTNIMP,NTOTPIMP,
     &             NTOTDIMP,NTOTTIMP,NTOTHEIMP,NTOTAIMP,NTOT6HEIMP
       print*,' '
      ENDIF

C For IMF - fission and IMF emission are not allowed
      FF22 = 0
      FIMF22 = 0

C Decay of IMF

      CALL EVAPORA(ZIMF,AIMF,EEIMF,JPRFLIGHT,ZFFIMF,AFFIMF,NTOTNIMF,
     &             NTOTPIMF,NTOTDIMF,NTOTTIMF,NTOTHEIMF,NTOTAIMF,
     &             NTOT6HEIMF,FF22,
     &             FIMF22,INTTYPE,INUM,TSUM,TAUC,vx2ev_imf,vy2ev_imf,
     &             vz2ev_imf,tkdummy1,adummy1,zdummy1,JPRF2,0,
     &             EV_TEMP,IEV_TAB_TEMP)


               DO IJ = 1, IEV_TAB_TEMP, 1
               EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
               EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 18/5/2010
c               EV_TAB(IJ+IEV_TAB,3) = EV_TEMP(IJ,3) + V_CM(1) + VX2_IMF
c               EV_TAB(IJ+IEV_TAB,4) = EV_TEMP(IJ,4) + V_CM(2) + VY2_IMF
c               EV_TAB(IJ+IEV_TAB,5) = EV_TEMP(IJ,5) + V_CM(3) + VZ2_IMF
               CALL LORENTZ_BOOST(v_cm(1),v_cm(2),v_cm(3),
     &              ev_temp(ij,3),ev_temp(ij,4),ev_temp(ij,5),
     &              VXOUT,VYOUT,VZOUT)
               CALL LORENTZ_BOOST(vx2_imf,vy2_imf,vz2_imf,
     &              vxout,vyout,vzout,
     &              VX2OUT,VY2OUT,VZ2OUT)
               ev_tab(ij+iev_tab,3) = VX2OUT
               ev_tab(ij+iev_tab,4) = VY2OUT
               ev_tab(ij+iev_tab,5) = VZ2OUT
               hev_tab(ij+iev_tab) = hist0
               call addtohist(step_imfl,hev_tab(ij+iev_tab))
               call addtohist(step_ev,hev_tab(ij+iev_tab))
               ENDDO
               IEV_TAB = IEV_TAB + IEV_TAB_TEMP
               histimf=hist0
               call addtohist(step_imfl,histimf)
               if(iev_tab_temp.gt.0)
     +            call addtohist(step_er,histimf)

C As IMF is not allowed to emit IMF, adummy1=zdummy1=0

      AFPIMF = IDNINT(AFFIMF)
      ZFPIMF = IDNINT(ZFFIMF)

      TEST_DIFF = IDNINT(AIMF - AFFIMF - ADUMMY1) -
     & IDNINT(NTOTNIMF+NTOTPIMF+2.D0*NTOTDIMF+
     &       3.D0*NTOTTIMF+3.D0*NTOTHEIMF+4.D0*NTOTAIMF+6.D0*NTOT6HEIMF)
      IF(TEST_DIFF.NE.0)THEN
       PRINT*,'TESTING-3:',test_diff,AIMF,AFFIMF,ADUMMY1,
     &             NTOTNIMF,NTOTPIMF,
     &             NTOTDIMF,NTOTTIMF,NTOTHEIMF,NTOTAIMF,NTOT6HEIMF
       print*,' '
      ENDIF

c vi1_imf, vi2_imf - velocities of imf and partner from TKE;
c vi1ev_imf, vi2_imf - recoil of partner and imf due to evaporation
c Lorentz kinematics - DM 18/5/2010
c        VIMF_CM(1) = V_CM(1) + VX2_IMF + VX2EV_IMF
c        VIMF_CM(2) = V_CM(2) + VY2_IMF + VY2EV_IMF
c        VIMF_CM(3) = V_CM(3) + VZ2_IMF + VZ2EV_IMF
        CALL LORENTZ_BOOST(vx2_imf,vy2_imf,vz2_imf,
     &       v_cm(1),v_cm(2),v_cm(3),
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx2ev_imf,vy2ev_imf,vz2ev_imf,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        vimf_cm(1) = VX2OUT
        vimf_cm(2) = VY2OUT
        vimf_cm(3) = VZ2OUT
c Lorentz kinematics - DM 18/5/2010
c       VFP1_CM(1) = V_CM(1) + VX1_IMF + VX1EV_IMF
c       VFP1_CM(2) = V_CM(2) + VY1_IMF + VY1EV_IMF
c       VFP1_CM(3) = V_CM(3) + VZ1_IMF + VZ1EV_IMF
        CALL LORENTZ_BOOST(vx1_imf,vy1_imf,vz1_imf,
     &       v_cm(1),v_cm(2),v_cm(3),
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx1ev_imf,vy1ev_imf,vz1ev_imf,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        vfp1_cm(1) = VX2OUT
        vfp1_cm(2) = VY2OUT
        vfp1_cm(3) = VZ2OUT

      IF(FF11.EQ.0.AND.FIMF11.EQ.0)THEN
c heavy partner deexcites by emission of light particles
      AFP1 = IDNINT(AFF)
      ZFP1 = IDNINT(ZFF)
      ZFP2 = 0
      AFP2 = 0
      FTYPE = 2
      JPRFFIN = JPRFheavy
      AFPIMF = IDNINT(AFFIMF)
      ZFPIMF = IDNINT(ZFFIMF)
        DO I = 1, 3
         VFP2_CM(I) = 0.D0
        ENDDO

      ENDIF

      IF(FF11.EQ.1.AND.FIMF11.EQ.0) THEN
C Heavy partner fissions
      ZF = ZFF
      AF = AFF
      EE = EEIMFP
      FF11 = 0
      FIMF11 = 0
      FTYPE = 21
      FTYPE1 = 0
      ACN = IDNINT(AF)
      ZCN = IDNINT(ZF)
      FEE = EE

      CALL FISSION(AF,ZF,EE,JPRF1,INUM,
     &    VX1_FISSION,VY1_FISSION,VZ1_FISSION,
     &    VX2_FISSION,VY2_FISSION,VZ2_FISSION,
     &    NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,N6HE_POST,
     &   ZFP1,AFP1,POSTN1,POSTP1,POSTD1,POSTT1,POSTHE1,POSTAL1,POST6HE1,
     &    FF1,FIMF1,TSUM1,TAUC1,TKEIMF1,AIMFF1,ZIMFF1,
     &   ZFP2,AFP2,POSTN2,POSTP2,POSTD2,POSTT2,POSTHE2,POSTAL2,POST6HE2,
     &    FF2,FIMF2,TSUM2,TAUC2,TKEIMF2,AIMFF2,ZIMFF2,imode,
     &    VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC,EV_TEMP,IEV_TAB_FIS,
     &    hev_temp,hist_fis1,hist_fis2)

               DO IJ = 1, IEV_TAB_FIS, 1
               EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
               EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 18/5/2010
c               EV_TAB(IJ+IEV_TAB,3) = EV_TEMP(IJ,3) + VIMF_CM(1)
c               EV_TAB(IJ+IEV_TAB,4) = EV_TEMP(IJ,4) + VIMF_CM(2)
c               EV_TAB(IJ+IEV_TAB,5) = EV_TEMP(IJ,5) + VIMF_CM(3)
c               CALL LORENTZ_BOOST(vimf_cm(1),vimf_cm(2),vimf_cm(3),
c     &             ev_temp(ij,3),ev_temp(ij,4),ev_temp(ij,5),
c     &             VXOUT,VYOUT,VZOUT)
c I think we should use vfp1_cm, not vimf_cm, DM 21/5/2010
               CALL LORENTZ_BOOST(vfp1_cm(1),vfp1_cm(2),vfp1_cm(3),
     &             ev_temp(ij,3),ev_temp(ij,4),ev_temp(ij,5),
     &             VXOUT,VYOUT,VZOUT)
               ev_tab(ij+iev_tab,3) = VXOUT
               ev_tab(ij+iev_tab,4) = VYOUT
               ev_tab(ij+iev_tab,5) = VZOUT
               hev_tab(ij+iev_tab) = hist1
               call addtohist(hev_temp(ij),hev_tab(ij+iev_tab))
               ENDDO
               IEV_TAB = IEV_TAB + IEV_TAB_FIS

       IF(IMODE.EQ.1) FTYPE1 = 1    ! S1 mode
       IF(IMODE.EQ.2) FTYPE1 = 2    ! S2 mode

c Lorentz kinematics - DM 18/5/2010
c        VFP1_CM(1) = V_CM(1) + VX1_IMF + VX1EV_IMF + VX1_FISSION +
c     &               VX_EVA_SC ! Velocity of FF1 in x
c        VFP1_CM(2) = V_CM(2) + VY1_IMF + VY1EV_IMF + VY1_FISSION +
c     &               VY_EVA_SC ! Velocity of FF1 in y
c        VFP1_CM(3) = V_CM(3) + VZ1_IMF + VZ1EV_IMF + VZ1_FISSION +
c     &               VZ_EVA_SC ! Velocity of FF1 in x
        CALL LORENTZ_BOOST(vx1_imf,vy1_imf,vz1_imf,
     &       v_cm(1),v_cm(2),v_cm(3),
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx1ev_imf,vy1ev_imf,vz1ev_imf,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        CALL LORENTZ_BOOST(vx1_fission,vy1_fission,vz1_fission,
     &       vx2out,vy2out,vz2out,
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx_eva_sc,vy_eva_sc,vz_eva_sc,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        vfp1_cm(1) = VX2OUT
        vfp1_cm(2) = VY2OUT
        vfp1_cm(3) = VZ2OUT
        hist2 = hist1
        call addtohist(hist_fis1,hist1)

c Lorentz kinematics - DM 18/5/2010
c        VFP2_CM(1) = V_CM(1) + VX1_IMF + VX1EV_IMF + VX2_FISSION +
c     &               VX_EVA_SC ! Velocity of FF2 in x
c        VFP2_CM(2) = V_CM(2) + VY1_IMF + VY1EV_IMF + VY2_FISSION +
c     &               VY_EVA_SC ! Velocity of FF2 in y
c        VFP2_CM(3) = V_CM(3) + VZ1_IMF + VZ1EV_IMF + VZ2_FISSION +
c     &               VZ_EVA_SC ! Velocity of FF2 in x
        CALL LORENTZ_BOOST(vx1_imf,vy1_imf,vz1_imf,
     &       v_cm(1),v_cm(2),v_cm(3),
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx1ev_imf,vy1ev_imf,vz1ev_imf,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        CALL LORENTZ_BOOST(vx2_fission,vy2_fission,vz2_fission,
     &       vx2out,vy2out,vz2out,
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx_eva_sc,vy_eva_sc,vz_eva_sc,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        vfp2_cm(1) = VX2OUT
        vfp2_cm(2) = VY2OUT
        vfp2_cm(3) = VZ2OUT
        call addtohist(hist_fis2,hist2)

        JPRFFIN = 0.D0
C
      ELSEIF(FIMF11.EQ.1.AND.FF11.EQ.0)THEN
C
c heavy partner emits imf, consequtive imf emission or fission is not allowed
      ZF = ZFF
      AF = AFF
      EE = EEIMFP
      AIMF = ADUMMY
      ZIMF = ZDUMMY
      TKEIMF = TKDUMMY
      FF11 = 0
      FIMF11 = 0
      FTYPE = 22
C Velocities of IMF and partner: 1 denotes partner, 2 denotes IMF
        EkinR1 = TKEIMF * AIMF / (AF+AIMF)
        EkinR2 = TKEIMF * AF / (AF+AIMF)
        V1 = DSQRT(EkinR1/AF) * 1.3887D0
        V2 = DSQRT(EkinR2/AIMF) * 1.3887D0
        VZ1_IMFS = (2.D0 * dble(HAZ(1)) - 1.D0) * V1
        VPERP1 = DSQRT(V1**2 - VZ1_IMFS**2)
        ALPHA1 = dble(HAZ(1)) * 2.D0 * 3.142D0
        VX1_IMFS = VPERP1 * DSIN(ALPHA1)
        VY1_IMFS = VPERP1 * DCOS(ALPHA1)
        VX2_IMFS = - VX1_IMFS / V1 * V2
        VY2_IMFS = - VY1_IMFS / V1 * V2
        VZ2_IMFS = - VZ1_IMFS / V1 * V2

      EEIMFP = EE * AF /(AF + AIMF)
      EEIMF = EE * AIMF /(AF + AIMF)

C Decay of heavy partner
      IINERTTOT = 0.4D0 * 931.49D0 * 1.16D0**2 *
     &          (AIMF**(5.D0/3.D0) + AF**(5.D0/3.D0)) +
     &          931.49D0 * 1.16D0**2 * AIMF * AF / (AIMF + AF) *
     &          (AIMF**(1.D0/3.D0) + AF**(1.D0/3.D0))**2
      JPRFHEAVY = JPRF1 * 0.4D0 * 931.49D0 * 1.16D0**2 *
     &            AF**(5.D0/3.D0) / IINERTTOT
      JPRFLIGHT = JPRF1 * 0.4D0 * 931.49D0 * 1.16D0**2 *
     &            AIMF**(5.D0/3.D0) / IINERTTOT

      CALL EVAPORA(ZF,AF,EEIMFP,JPRFHEAVY,ZFFS,AFFS,NTOTNIMP1,NTOTPIMP1,
     &             NTOTDIMP1,NTOTTIMP1,NTOTHEIMP1,NTOTAIMP1,
     &             NTOT6HEIMP1,FF11,FIMF11,
     &             INTTYPE,INUM,TSUM,TAUC,vx1ev_imfs,vy1ev_imfs,
     &             vz1ev_imfs,tkdummy,adummy,zdummy,JPRF3,0,
     &             EV_TEMP,IEV_TAB_TEMP)


           DO IJ = 1, IEV_TAB_TEMP, 1
           EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
           EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 18/5/2010
c           EV_TAB(IJ+IEV_TAB,3) = EV_TEMP(IJ,3) + VFP1_CM(1) + VX1_IMFS
c           EV_TAB(IJ+IEV_TAB,4) = EV_TEMP(IJ,4) + VFP1_CM(2) + VY1_IMFS
c           EV_TAB(IJ+IEV_TAB,5) = EV_TEMP(IJ,5) + VFP1_CM(3) + VZ1_IMFS
           CALL LORENTZ_BOOST(vfp1_cm(1),vfp1_cm(2),vfp1_cm(3),
     &          ev_temp(ij,3),ev_temp(ij,4),ev_temp(ij,5),
     &          VXOUT,VYOUT,VZOUT)
           CALL LORENTZ_BOOST(vx1_imfs,vy1_imfs,vz1_imfs,
     &          vxout,vyout,vzout,
     &          VX2OUT,VY2OUT,VZ2OUT)
           ev_tab(ij+iev_tab,3) = VX2OUT
           ev_tab(ij+iev_tab,4) = VY2OUT
           ev_tab(ij+iev_tab,5) = VZ2OUT
           hev_tab(ij+iev_tab) = hist1
           call addtohist(step_imfh,hev_tab(ij+iev_tab))
           call addtohist(step_ev,hev_tab(ij+iev_tab))
           ENDDO
           IEV_TAB = IEV_TAB + IEV_TAB_TEMP
           hist2 = hist1
           call addtohist(step_imfh,hist1)
           if(iev_tab_temp.gt.0)
     +         call addtohist(step_er,hist1)


      TEST_DIFF = IDNINT(AF - AFFS - ADUMMY) -
     & IDNINT(NTOTNIMP1+NTOTPIMP1+2.D0*NTOTDIMP1+
     &        3.D0*NTOTTIMP1+3.D0*NTOTHEIMP1+4.D0*NTOTAIMP1+
     &        6.D0*NTOT6HEIMP1)
      IF(TEST_DIFF.NE.0)THEN
       PRINT*,'TESTING-4:',test_diff,AF,AFFS,ADUMMY,
     &             NTOTNIMP1,NTOTPIMP1,
     &             NTOTDIMP1,NTOTTIMP1,NTOTHEIMP1,NTOTAIMP1,NTOT6HEIMP1
       print*,' '
      ENDIF

C For IMF - fission and IMF emission are not allowed
      FF22 = 0
      FIMF22 = 0
C Decay of "second" IMF

      CALL EVAPORA(ZIMF,AIMF,EEIMF,JPRFLIGHT,ZFFIMFS,AFFIMFS,NTOTNIMF1,
     &             NTOTPIMF1,NTOTDIMF1,NTOTTIMF1,NTOTHEIMF1,NTOTAIMF1,
     &             NTOT6HEIMF1,FF22,FIMF22,INTTYPE,INUM,TSUM,TAUC,
     &             vx2ev_imfs,vy2ev_imfs,vz2ev_imfs,tkdummy1,adummy1,
     &             zdummy1,JPRF4,0,
     &             EV_TEMP,IEV_TAB_TEMP)


           DO IJ = 1, IEV_TAB_TEMP, 1
           EV_TAB(IJ+IEV_TAB,1) = EV_TEMP(IJ,1)
           EV_TAB(IJ+IEV_TAB,2) = EV_TEMP(IJ,2)
c Lorentz kinematics - DM 18/5/2010
c           EV_TAB(IJ+IEV_TAB,3) = EV_TEMP(IJ,3) + VFP1_CM(1) + VX2_IMFS
c           EV_TAB(IJ+IEV_TAB,4) = EV_TEMP(IJ,4) + VFP1_CM(2) + VY2_IMFS
c           EV_TAB(IJ+IEV_TAB,5) = EV_TEMP(IJ,5) + VFP1_CM(3) + VZ2_IMFS
           CALL LORENTZ_BOOST(vfp1_cm(1),vfp1_cm(2),vfp1_cm(3),
     &          ev_temp(ij,3),ev_temp(ij,4),ev_temp(ij,5),
     &          VXOUT,VYOUT,VZOUT)
           CALL LORENTZ_BOOST(vx2_imfs,vy2_imfs,vz2_imfs,
     &          vxout,vyout,vzout,
     &          VX2OUT,VY2OUT,VZ2OUT)
           ev_tab(ij+iev_tab,3) = VX2OUT
           ev_tab(ij+iev_tab,4) = VY2OUT
           ev_tab(ij+iev_tab,5) = VZ2OUT
           hev_tab(ij+iev_tab) = hist1
           call addtohist(step_imfl,hev_tab(ij+iev_tab))
           call addtohist(step_ev,hev_tab(ij+iev_tab))
           ENDDO
           IEV_TAB = IEV_TAB + IEV_TAB_TEMP
           call addtohist(step_imfl,hist2)
           if(iev_tab_temp.gt.0)
     +         call addtohist(step_er,hist2)


      TEST_DIFF = IDNINT(AIMF - AFFIMFS - ADUMMY1) -
     & IDNINT(NTOTNIMF1+NTOTPIMF1+2.D0*NTOTDIMF1+
     &        3.D0*NTOTTIMF1+3.D0*NTOTHEIMF1+4.D0*NTOTAIMF1+
     &        6.D0*NTOT6HEIMF1)
      IF(TEST_DIFF.NE.0)THEN
       PRINT*,'TESTING-5:',test_diff,AIMF,AFFIMFS,ADUMMY1,
     &             NTOTNIMF1,NTOTPIMF1,
     &             NTOTDIMF1,NTOTTIMF1,NTOTHEIMF1,NTOTAIMF1,NTOT6HEIMF1
       print*,' '
      ENDIF

      AFP1 = IDNINT(AFFS)
      ZFP1 = IDNINT(ZFFS)
      ZFP2 = IDNINT(ZFFIMFS)
      AFP2 = IDNINT(AFFIMFS)
      JPRFFIN = JPRF3

C Velocity of final heavy residue
c Lorentz kinematics - DM 18/5/2010
c        VFP1_CM(1) = V_CM(1) + VX1_IMF + VX1EV_IMF + VX1_IMFS +
c     &               VX1EV_IMFS
c        VFP1_CM(2) = V_CM(2) + VY1_IMF + VY1EV_IMF + VY1_IMFS +
c     &               VY1EV_IMFS
c        VFP1_CM(3) = V_CM(3) + VZ1_IMF + VZ1EV_IMF + VZ1_IMFS +
c     &               VZ1EV_IMFS
        CALL LORENTZ_BOOST(vx1_imf,vy1_imf,vz1_imf,
     &       v_cm(1),v_cm(2),v_cm(3),
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx1ev_imf,vy1ev_imf,vz1ev_imf,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        CALL LORENTZ_BOOST(vx1_imfs,vy1_imfs,vz1_imfs,
     &       vx2out,vy2out,vz2out,
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx1ev_imfs,vy1ev_imfs,vz1ev_imfs,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        vfp1_cm(1) = VX2OUT
        vfp1_cm(2) = VY2OUT
        vfp1_cm(3) = VZ2OUT
C Velocity of the second IMF
c Lorentz kinematics - DM 18/5/2010
c        VFP2_CM(1) = V_CM(1) + VX1_IMF + VX1EV_IMF + VX2_IMFS +
c     &               VX2EV_IMFS
c        VFP2_CM(2) = V_CM(2) + VY1_IMF + VY1EV_IMF + VY2_IMFS +
c     &               VY2EV_IMFS
c        VFP2_CM(3) = V_CM(3) + VZ1_IMF + VZ1EV_IMF + VZ2_IMFS +
c     &               VZ2EV_IMFS
        CALL LORENTZ_BOOST(vx1_imf,vy1_imf,vz1_imf,
     &       v_cm(1),v_cm(2),v_cm(3),
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx1ev_imf,vy1ev_imf,vz1ev_imf,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        CALL LORENTZ_BOOST(vx2_imfs,vy2_imfs,vz2_imfs,
     &       vx2out,vy2out,vz2out,
     &       VXOUT,VYOUT,VZOUT)
        CALL LORENTZ_BOOST(vx2ev_imfs,vy2ev_imfs,vz2ev_imfs,
     &       vxout,vyout,vzout,
     &       VX2OUT,VY2OUT,VZ2OUT)
        vfp2_cm(1) = VX2OUT
        vfp2_cm(2) = VY2OUT
        vfp2_cm(3) = VZ2OUT

      ENDIF

      ENDIF

 3456 CONTINUE

C Only evaporation of light particles
      IF(FTYPE.NE.1.AND.FTYPE.NE.21) THEN
         FEE = 0.D0
         POSTN1  = 0
         POSTP1  = 0
         POSTD1  = 0
         POSTT1  = 0
         POSTHE1 = 0
         POSTAL1 = 0
         POST6HE1 = 0

         POSTN2  = 0
         POSTP2  = 0
         POSTD2  = 0
         POSTT2  = 0
         POSTHE2 = 0
         POSTAL2 = 0
         POST6HE2 = 0

         NN_POST = 0.D0
         NP_POST = 0.D0
         ND_POST = 0.D0
         NT_POST = 0.D0
         NHE_POST = 0.D0
         NA_POST = 0.D0
         N6HE_POST = 0.D0

C     For case of fragmentation (without fission)
      NBNEVA = IDNINT(NTOTNCN+NTOTNIMP+NTOTNIMF)+NBNBU
     &       + IDNINT(NTOTNIMP1+NTOTNIMF1)

      NBPEVA = IDNINT(NTOTPCN+NTOTPIMP+NTOTPIMF)+NBPBU
     &       + IDNINT(NTOTPIMP1+NTOTPIMF1)

      NBDEVA = IDNINT(NTOTDCN+NTOTDIMP+NTOTDIMF)+NBDBU
     &       + IDNINT(NTOTDIMP1+NTOTDIMF1)

      NBTEVA = IDNINT(NTOTTCN+NTOTTIMP+NTOTTIMF)+NBTBU
     &       + IDNINT(NTOTTIMP1+NTOTTIMF1)

      NBHEEVA = IDNINT(NTOTHECN+NTOTHEIMP+NTOTHEIMF)+NBHEBU
     &        + IDNINT(NTOTHEIMP1+NTOTHEIMF1)

      NBAEVA = IDNINT(NTOTACN+NTOTAIMP+NTOTAIMF)+NBABU
     &       + IDNINT(NTOTAIMP1+NTOTAIMF1)

      NB6HEEVA = IDNINT(NTOT6HECN+NTOT6HEIMP+NTOT6HEIMF)+NB6HEBU
     &        + IDNINT(NTOT6HEIMP1+NTOT6HEIMF1)

C----------- RESOLVE UNSTABLE NUCLEI
C
       CALL UNSTABLE_NUCLEI(AFP1,ZFP1,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &                     VFP1_CM(1),VFP1_CM(2),VFP1_CM(3),
     &                     VP1X,VP1Y,VP1Z,EV_TAB_TEMP,ILOOP)

        IF(IOUNSTABLE .EQ. 1)THEN
         AFP1 = AFPNEW
         ZFP1 = ZFPNEW
         VFP1_CM(1) = VP1X
         VFP1_CM(2) = VP1Y
         VFP1_CM(3) = VP1Z
C EV_TAB_TEMP contains information on neutrons and LCPs emitted in above subroutine
C Notation is the same as in BU_TAB_TEMP, ILOOP is the size of the array.
         NBNEVA = NBNEVA + NEVA
         NBPEVA = NBPEVA + PEVA
         NBDEVA = NBDEVA + DEVA
         NBTEVA = NBTEVA + TEVA
         NBHEEVA = NBHEEVA + HEEVA
         NBAEVA = NBAEVA + AEVA
         NB6HEEVA = NB6HEEVA + HE6EVA
         DO I = 1, ILOOP
          DO IJ = 1, 5
           EV_TAB(I+IEV_TAB,IJ) = EV_TAB_TEMP(I,IJ)
          ENDDO
          hev_tab(i+iev_tab) = hist1
          call addtohist(step_unst, hev_tab(i+iev_tab))
         ENDDO
        IEV_TAB = IEV_TAB + ILOOP
        call addtohist(step_unst, hist1)
        ENDIF

      IF(FTYPE .GT. 0) THEN
       CALL UNSTABLE_NUCLEI(AFP2,ZFP2,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &                     VFP2_CM(1),VFP2_CM(2),VFP2_CM(3),
     &                     VP1X,VP1Y,VP1Z,EV_TAB_TEMP,ILOOP)

        IF(IOUNSTABLE .EQ. 1)THEN
         AFP2 = AFPNEW
         ZFP2 = ZFPNEW
         VFP2_CM(1) = VP1X
         VFP2_CM(2) = VP1Y
         VFP2_CM(3) = VP1Z
         NBNEVA = NBNEVA + NEVA
         NBPEVA = NBPEVA + PEVA
         NBDEVA = NBDEVA + DEVA
         NBTEVA = NBTEVA + TEVA
         NBHEEVA = NBHEEVA + HEEVA
         NBAEVA = NBAEVA + AEVA
         NB6HEEVA = NB6HEEVA + HE6EVA
          DO I = 1, ILOOP
           DO IJ = 1, 5
           EV_TAB(IEV_TAB+I,IJ) = EV_TAB_TEMP(I,IJ)
           ENDDO
          hev_tab(i+iev_tab) = hist2
          call addtohist(step_unst, hev_tab(i+iev_tab))
          ENDDO
          IEV_TAB = IEV_TAB + ILOOP
          call addtohist(step_unst, hist2)
         ENDIF
      ENDIF


      IF(FTYPE .GT. 1 .AND. FTYPE .NE. 10) THEN
       CALL UNSTABLE_NUCLEI(AFPIMF,ZFPIMF,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &                     VIMF_CM(1),VIMF_CM(2),VIMF_CM(3),
     &                     VP1X,VP1Y,VP1Z,EV_TAB_TEMP,ILOOP)

        IF(IOUNSTABLE .EQ. 1)THEN
         AFPIMF = AFPNEW
         ZFPIMF = ZFPNEW
         VIMF_CM(1) = VP1X
         VIMF_CM(2) = VP1Y
         VIMF_CM(3) = VP1Z

         NBNEVA = NBNEVA + NEVA
         NBPEVA = NBPEVA + PEVA
         NBDEVA = NBDEVA + DEVA
         NBTEVA = NBTEVA + TEVA
         NBHEEVA = NBHEEVA + HEEVA
         NBAEVA = NBAEVA + AEVA
         NB6HEEVA = NB6HEEVA + HE6EVA
         DO I = 1, ILOOP
          DO IJ = 1, 5
           EV_TAB(IEV_TAB+I,IJ) = EV_TAB_TEMP(I,IJ)
          ENDDO
          hev_tab(i+iev_tab) = histimf
          call addtohist(step_unst, hev_tab(i+iev_tab))
         ENDDO
        IEV_TAB = IEV_TAB + ILOOP
        call addtohist(step_unst, histimf)
        ENDIF
      ENDIF

c IEV_TAB gives multiplicities of particles created in the subroutine UNSTABLE_NUCLEI in case of
C statistical decay branch;


C IMULT = 1 means no break-up, i.e. one is entering secondary decay stage with multiplicity 1
      IMULT = IMULT + IMULTBU

      ENDIF
C
C
C----------- CALCULATE NUMBER OF ABRADED AND EVAPORATED PARTICLES
C
 300  CONTINUE

C     TRANSPARENCY
C
      IF (FTYPE.EQ.0.AND.AFP1.EQ.AP1.AND.IREACT.EQ.0) THEN
      GO TO 10
      ENDIF

      NBNABR = AP1-AABRA-(ZP1-ZABRA)
      NBPABR = ZP1-ZABRA

C     For the case of fission:
      IF(FTYPE .EQ. 1 .OR. FTYPE .EQ. 21 .OR. FTYPE .EQ. 10 .OR.
     +   FTYPE .EQ. 210)THEN
       NBNEVA = IDNINT(NTOTNCN+NN_POST) + POSTN1 + POSTN2
     +       + IDNINT(NTOTNIMP+NTOTNIMF)+NBNBU

       NBN_PRESADD = IDNINT(NTOTNCN)
       NBN_PRESCIS = IDNINT(NTOTNCN+NN_POST)

       NBPEVA = IDNINT(NTOTPCN+NP_POST) + POSTP1 + POSTP2
     +       + IDNINT(NTOTPIMP+NTOTPIMF)+NBPBU

       NBP_PRESADD = IDNINT(NTOTPCN)
       NBP_PRESCIS = IDNINT(NTOTPCN+NP_POST)

       NBDEVA = IDNINT(NTOTDCN+ND_POST) + POSTD1 + POSTD2
     +       + IDNINT(NTOTDIMP+NTOTDIMF)+NBDBU

       NBTEVA = IDNINT(NTOTTCN+NT_POST) + POSTT1 + POSTT2
     +       + IDNINT(NTOTTIMP+NTOTTIMF)+NBTBU

       NBHEEVA = IDNINT(NTOTHECN+NHE_POST) + POSTHE1 + POSTHE2
     +       + IDNINT(NTOTHEIMP+NTOTHEIMF)+NBHEBU

       NBA_PRESADD = IDNINT(NTOTACN)
       NBA_PRESCIS = IDNINT(NTOTACN+NA_POST)

       NBAEVA = IDNINT(NTOTACN+NA_POST) + POSTAL1 + POSTAL2
     +       + IDNINT(NTOTAIMP+NTOTAIMF)+NBABU

       NB6HEEVA = IDNINT(NTOT6HECN+N6HE_POST) + POST6HE1 + POST6HE2
     +       + IDNINT(NTOT6HEIMP+NTOT6HEIMF)+NB6HEBU
C----------- RESOLVE UNSTABLE NUCLEI
C
       CALL UNSTABLE_NUCLEI(AFP1,ZFP1,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &                     VFP1_CM(1),VFP1_CM(2),VFP1_CM(3),
     &                     VP1X,VP1Y,VP1Z,EV_TAB_TEMP,ILOOP)

        IF(IOUNSTABLE .EQ. 1)THEN
         AFP1 = AFPNEW
         ZFP1 = ZFPNEW
         VFP1_CM(1) = VP1X
         VFP1_CM(2) = VP1Y
         VFP1_CM(3) = VP1Z
C EV_TAB_TEMP contains information on neutrons and LCPs emitted in above subroutine
C Notation is the same as in BU_TAB_TEMP, ILOOP is the size of the array.
         NBNEVA = NBNEVA + NEVA
         NBPEVA = NBPEVA + PEVA
         NBDEVA = NBDEVA + DEVA
         NBTEVA = NBTEVA + TEVA
         NBHEEVA = NBHEEVA + HEEVA
         NBAEVA = NBAEVA + AEVA
         NB6HEEVA = NB6HEEVA + HE6EVA
         DO I = 1, ILOOP
          DO IJ = 1, 5
           EV_TAB(I+IEV_TAB,IJ) = EV_TAB_TEMP(I,IJ)
          ENDDO
          hev_tab(i+iev_tab) = hist1
          call addtohist(step_unst, hev_tab(i+iev_tab))
         ENDDO
        IEV_TAB = IEV_TAB + ILOOP
        if(iloop.gt.0) call addtohist(step_unst, hist1)
        ENDIF

      IF(FTYPE .EQ. 21) THEN
       CALL UNSTABLE_NUCLEI(AFP2,ZFP2,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &                     VFP2_CM(1),VFP2_CM(2),VFP2_CM(3),
     &                     VP1X,VP1Y,VP1Z,EV_TAB_TEMP,ILOOP)

        IF(IOUNSTABLE .EQ. 1)THEN
         AFP2 = AFPNEW
         ZFP2 = ZFPNEW
         VFP2_CM(1) = VP1X
         VFP2_CM(2) = VP1Y
         VFP2_CM(3) = VP1Z
         NBNEVA = NBNEVA + NEVA
         NBPEVA = NBPEVA + PEVA
         NBDEVA = NBDEVA + DEVA
         NBTEVA = NBTEVA + TEVA
         NBHEEVA = NBHEEVA + HEEVA
         NBAEVA = NBAEVA + AEVA
         NB6HEEVA = NB6HEEVA + HE6EVA
         DO I = 1, ILOOP
          DO IJ = 1, 5
           EV_TAB(IEV_TAB+I,IJ) = EV_TAB_TEMP(I,IJ)
          ENDDO
          hev_tab(i+iev_tab) = hist2
          call addtohist(step_unst, hev_tab(i+iev_tab))
         ENDDO
        IEV_TAB = IEV_TAB + ILOOP
        if(iloop.gt.0) call addtohist(step_unst, hist2)
        ENDIF


       CALL UNSTABLE_NUCLEI(AFPIMF,ZFPIMF,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &                     VIMF_CM(1),VIMF_CM(2),VIMF_CM(3),
     &                     VP1X,VP1Y,VP1Z,EV_TAB_TEMP,ILOOP)

        IF(IOUNSTABLE .EQ. 1)THEN
         AFPIMF = AFPNEW
         ZFPIMF = ZFPNEW
         VIMF_CM(1) = VP1X
         VIMF_CM(2) = VP1Y
         VIMF_CM(3) = VP1Z

         NBNEVA = NBNEVA + NEVA
         NBPEVA = NBPEVA + PEVA
         NBDEVA = NBDEVA + DEVA
         NBTEVA = NBTEVA + TEVA
         NBHEEVA = NBHEEVA + HEEVA
         NBAEVA = NBAEVA + AEVA
         NB6HEEVA = NB6HEEVA + HE6EVA
         DO I = 1, ILOOP
          DO IJ = 1, 5
           EV_TAB(IEV_TAB+I,IJ) = EV_TAB_TEMP(I,IJ)
          ENDDO
          hev_tab(i+iev_tab) = histimf
          call addtohist(step_unst, hev_tab(i+iev_tab))
         ENDDO
        IEV_TAB = IEV_TAB + ILOOP
        if(iloop.gt.0) call addtohist(step_unst, histimf)
        ENDIF
      ENDIF

      ENDIF

C       PRINT*,'EVENT TYPE',FTYPE, IMULTIFR

C
C Test on neutron number
       TEST_DIFF = AP1 - ZP1 - (AP1 - ZP1 -(AABRA-ZABRA)) -
     +      (IDNINT(ABU_SUM-ZBU_SUM)+
     +      NBNEVA+NBDEVA+2*NBTEVA+NBHEEVA+2*NBAEVA+4*NB6HEEVA +
     +      AFP1-ZFP1+AFP2-ZFP2+AFPIMF-ZFPIMF)
       IF (TEST_DIFF.NE.0) THEN
        WRITE(6,*)'PROBLEM IN REANUCL FOR FTYPE=, DIFF. OF NEUTRONS =',
     +     FTYPE, IMULTIFR, TEST_DIFF,APRFP,ZPRFP,AP1-ZP1,AABRA-ZABRA,
     +     IDNINT(ABU_SUM-ZBU_SUM), NBPEVA,NBDEVA,NBTEVA,NBHEEVA,NBAEVA,
     +       NB6HEEVA,AFP1-ZFP1,AFP2-ZFP2,AFPIMF-ZFPIMF
       END IF
C
C Test on proton number
       TEST_DIFF = ZP1 - (ZP1 - ZABRA)- (IDNINT(ZBU_SUM)+
     +       NBPEVA+NBDEVA+NBTEVA+2*NBHEEVA+2*NBAEVA+2*NB6HEEVA+
     +       ZFP1+ZFP2+ZFPIMF)
       IF (TEST_DIFF.NE.0)THEN
        WRITE(6,*)'PROBLEM IN REANUCL FOR FTYPE, DIFF. OF PROTONS =',
     +   FTYPE, IMULTIFR, TEST_DIFF,APRFP,ZPRFP,ZP1,ZABRA,
     +      IDNINT(ZBU_SUM), NBPEVA,NBDEVA,NBTEVA,NBHEEVA,NBAEVA,
     +       NB6HEEVA,ZFP1,ZFP2,ZFPIMF
       END IF

cDVD - start
c     Put heavy residues in the EV_TAB array

       ev_tab(iev_tab+1,1) = zfp1
       ev_tab(iev_tab+1,2) = afp1
       ev_tab(iev_tab+1,3) = vfp1_cm(1)
       ev_tab(iev_tab+1,4) = vfp1_cm(2)
       ev_tab(iev_tab+1,5) = vfp1_cm(3)
       hev_tab(iev_tab+1) = hist1
       iev_tab = iev_tab + 1

       if( afp2.ne.0 ) then

          ev_tab(iev_tab+1,1) = zfp2
          ev_tab(iev_tab+1,2) = afp2
          ev_tab(iev_tab+1,3) = vfp2_cm(1)
          ev_tab(iev_tab+1,4) = vfp2_cm(2)
          ev_tab(iev_tab+1,5) = vfp2_cm(3)
          hev_tab(iev_tab+1) = hist2
          iev_tab = iev_tab + 1

       end if

       if( afpimf.ne.0 ) then

          ev_tab(iev_tab+1,1) = zfpimf
          ev_tab(iev_tab+1,2) = afpimf
          ev_tab(iev_tab+1,3) = vimf_cm(1)
          ev_tab(iev_tab+1,4) = vimf_cm(2)
          ev_tab(iev_tab+1,5) = vimf_cm(3)
          hev_tab(iev_tab+1) = histimf
          iev_tab = iev_tab + 1

       end if
cDVD - stop


      RETURN
      END
C
C**********************************************************************
C
      DOUBLE PRECISION FUNCTION DNDE1(EGAM)
C
C     CALCULATION OF THE PHOTON-SPECTRUM E1
C     ACCORDING TO WILLIAMS-WEIZSAECKER METHOD
C
C     INPUT: EGAM - VIRTUAL GAMMA ENERGY
C            COMMON /ABRAMAIN/
C            (SEE SUBROUTINE REANUCL OR ABRAINIT FOR EXPLANATIONS)
C
C     OUTPUT: DNDE1
C
      INTEGER*4 IMAX,IRNDM
C
      REAL*8 EGAM,ddbesk0,ddbesk1,GAMMA,NGVIRT,EGVIRT
      REAL*8 ARG,BMIN
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,R_P,R_T,PI,
     &       BFPRO,SNPRO,SPPRO,SHELL
C
      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL

      EXTERNAL ddbesk0,ddbesk1

      BMIN  = BMAXNUC
      GAMMA = 1.0D0 + EAP / 931.5D0
      BETA  = DSQRT( 1.0D0 - 1.0D0 / GAMMA**2 )
      IF( BETA .LE. 1.0D-5 ) GOTO 100
C
      NGVIRT = 2.D0/PI * ZT**2 / 137.D0 / BETA**2
      EGVIRT = 197.328D0 * BETA * GAMMA / BMIN
C
C *** NUMBER OF  VIRTUAL PHOTONS (E1) PER MEV
C
      ARG = EGAM / EGVIRT
      IF( ARG .LT. 88.0D0 ) THEN
C   * WILLIAMS-WEIZAECKER-FORMEL FOR E1
        DNDE1 = NGVIRT / EGAM
     +        * (ARG * ddbesk0(ARG) * ddbesk1(ARG)
     +          - BETA**2 * ARG**2
     +          * ( ddbesk1(ARG)**2 - ddbesk0(ARG)**2 ) / 2.D0)
      ENDIF
 100  CONTINUE
      RETURN
      END
C*********************************************************************
C
      DOUBLE PRECISION FUNCTION DNDE2(EGAM)
C
C     CALCULATION OF THE PHOTON-SPECTRUM E2
C     ACCORDING TO BERTULANI-BAUR METHOD
C
C     INPUT: EGAM - VIRTUAL GAMMA ENERGY
C            COMMON /ABRAMAIN/
C            (SEE SUBROUTINE REANUCL OR ABRAINIT FOR EXPLANATIONS)
C     OUTPUT: DNDE2
C
C
      INTEGER*4 IMAX,IRNDM
C
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,R_P,R_T,PI,
     &       BFPRO,SNPRO,SPPRO,SHELL
      REAL*8 EGAM,ddbesk0,ddbesk1,GAMMA,NGVIRT,EGVIRT
      REAL*8 ARG, BMIN
C
      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
C
      EXTERNAL ddbesk0,ddbesk1
C
      BMIN = BMAXNUC
      GAMMA = 1.0D0 + EAP / 931.5D0
      BETA = DSQRT( 1.0D0 - 1.0D0 / GAMMA**2 )
      IF( BETA .LE. 1.0D-5 ) GOTO 100
C
      NGVIRT = 2.D0/PI * ZT**2 / 137.D0 / BETA**2
      EGVIRT = 197.328D0 * BETA * GAMMA / BMIN
C
C *** ANZAHL DER VIRTUELLEN PHOTONEN (E2) PRO MEV
C
      ARG = EGAM / EGVIRT
      IF( ARG .LT. 88.D0 )  THEN
C   * BERTULANI-BAUR-FORMEL FUER E2
       DNDE2 = NGVIRT / EGAM / BETA**2 *
     +        ( 2.D0 * (1.D0-BETA**2) * ddbesk1(ARG)**2 +
     +          ARG * ddbesk0(ARG) * ddbesk1(ARG)
     +          * (2.D0-BETA**2)**2 +
     +          (ARG * BETA**2)**2 * (ddbesk0(ARG)**2
     +                              - ddbesk1(ARG)**2) / 2.D0 )
      ENDIF
 100  CONTINUE
      RETURN
      END
C
C
C **********************************************************************
C
      SUBROUTINE LORRAN(XRES)
C
C     ON INPUT:  IRNDM  (IN COMMON /ABRAMAIN/)
C     ON OUTPUT: XRES
C   ____________________________________________________________________
C  /
C  /  CALCULATES A RANDOM NUMBER DISTRIBUTED ACCORDING TO A
C  /  BREIT-WIGNER (LORENTZ) FUNCTION AT CENTER 0 WITH FWHM=1
C  /  VALUES RETURNED ARE BETWEEN -10 AND +10
C  /
C  /  T.BROHM, E.HANELT; VERSION 7/07/92
C  /
C   ____________________________________________________________________
C
      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,R_P,R_T,PI,
     &       BFPRO,SNPRO,SPPRO,SHELL
      INTEGER*4 IMAX,IRNDM
C
      REAL*8 XTEST,XRES,YTEST,RLOR
      REAL*4 RNDM,haz
C
 10   CONTINUE
C
      XTEST = (DBLE(HAZ(1))-0.5D0)*40.D0
      YTEST = DBLE(HAZ(1))
      RLOR = 1.D0/((XTEST)**2+1.D0)
      IF (YTEST.GT.RLOR) GOTO 10
C
      XRES = XTEST/2.D0
C
      RETURN
      END
C
C **********************************************************************
C
      SUBROUTINE COURBE(B1,B2,Y1,Y1S,Y2,Y2S,T,U,V,W)
C
C     ON INPUT:  B1,B2,Y1,Y1S,Y2,Y2S
C     ON OUTPUT: T,U,V,W
C   ____________________________________________________________________
C  /
C  /  PROCEDURE FOR CALCULATING THE POLYNOMIAL COEFFICIENTS IN ORDER
C  /  TO CORRECT THE DEPENDENCE OF MASS OR ENERGY OF PREFRAGMENTS ON
C  /  IMPACT PARAMETER IN CRITICAL REGIONS
C  /____________________________________________________________________
C
      REAL*8 B1,B2,Y1,Y1S,Y2,Y2S,T,U,V,W,TG1,TG2
C
C-----------------------------------------------------------------------
C     B1,B2             IMPACT-PARAMETER LIMITS FOR CORRECTING
C                        MASS AND EXCITATION ENERGY OF PREFRAGMENTS
C     Y1,Y2,Y1S,Y2S     PREFRAGMENT MASS VALUES OR SURFACE-EXCESS
C                        ENERGIES CORRESPONDING TO B1,...
C     T,U,V,W           COEFFICIENTS OF CORRECTIVE POLYNOMIALS
C     TG1,TG2  LOCAL    LOCAL SLOPES OF Y1,Y2
C-----------------------------------------------------------------------
C
      TG1 = (Y1S-Y1)/0.0001D0
      TG2 = (Y2S-Y2)/0.0001D0
      T = ((B2-B1)*(TG2+TG1)-2.0D0*(Y2-Y1))/(B2-B1)**3
      U = 0.5D0*(TG2-TG1-3.0D0*T*(B2**2-B1**2))/(B2-B1)
      V = TG2-B2*(3.0D0*T*B2+2.0D0*U)
      W = Y1-B1*(T*B1**2+U*B1+V)
C
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE FBPB(B,R1,R0,A1,A,DELTA)
C
C     ON INPUT:  B,R1,R0,A1
C     ON OUTPUT: A,DELTA
C   ____________________________________________________________________
C  /
C  /  CALCUL DE LA MASSE ET DE L'ENERGIE D'EXCES DE SURFACE DU
C  /  PREFRAGMENT PAR LE MODELE D'ABRASION (METHODE APPROXIMATIVE DE
C  /  SWIATECKI, APPENDICE DE PHYS. REV. C16 (1977) 629)
C  /
C  /  PROCEDURE FOR CALCULATING MASS AND SURFACE-EXCESS ENERGY OF THE
C  /  PREFRAGMENTS ACCORDING TO THE ABRASION MODEL (APPROXIMATE METHOD
C  /  OF SWIATECKI, APPENDIX OF PHYS. REV. C16 (1977) 629)
C  /____________________________________________________________________
C
C
      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,R_P,R_T,PI,
     &       BFPRO,SNPRO,SPPRO,SHELL
      INTEGER*4 IMAX,IRNDM
C
      REAL*8 B,R1,R0,F,P,RT,U,V,W,DELTA,A1,A
      REAL*8 C1,C2,C3,C4,C5,C6,C7,C8,C9,C10,C11,C12,C13,C14
C
C-----------------------------------------------------------------------
C     B                 IMPACT PARAMETER
C     R1                RADIUS OF THE PROJECTILE NUCLEUS
C     R0                RADIUS OF THE TARGET NUCLEUS
C     F,P      LOCAL    HELP VARIABLES
C     RT       LOCAL    MAXIMUM IMPACT PARAMETER (R1+R0)
C     U,V,W    LOCAL    HELP VARIABLES
C     PI                PI VALUE
C     DELTA             SURFACE-EXCESS ENERGY CORRESSPONDING TO B
C     A1                MASS NUMBER OF THE PROJECTILE
C     A                 PREFRAGMENT MASS VALUE CORRESPONDING TO B
C     C1,...,C14  LOCAL HELP VARIABLES
C-----------------------------------------------------------------------
C
      RT = R1+R0
      U = R1/RT
      V = B/RT
      W = 1.0D0/U-1.0D0
      C1 = (1.0D0-V)/U
      C3 = DSQRT(1.0D0-U)
      C4 = 1.0D0/W-2.0D0
      C5 = DSQRT(W*U)
      IF (R1.GT.R0) THEN
        C2 = 1.0D0-W**2
        IF (B.LT.(R1-R0)) THEN
          C6 = DSQRT(1.0D0-(V/U)**2)
          F = (1.0D0-C2**1.5D0)*C6
          P = (C2**0.5D0/U-1.0D0)*C6
          ELSE
            C7 = (3.0D0*C3)/W
            C8 = 1.0D0-C2**1.5D0
            C9 = DSQRT(1.0D0-(1.0D0-W)**2)
            C10 = (C8*C9)/(W**3)
            F = 0.75D0*C3*C1**2-0.125D0*(C7-C10)*C1**3
            C11 = 0.5D0*(U/W)**0.5D0*C4
            C12 = (C2**0.5D0/U)-1.0D0
            C13 = DSQRT((2.0D0-W)*W)
            C14 = (C12*C13)/(W**3)
            P = 0.125D0*C5*C4*C1**2-0.125D0*(C11-C14)*C1**3
        END IF
        ELSE
          IF (B.LT.(R0-R1)) THEN
            F = 1.0D0
            P = -1.0D0
            ELSE
              F = 0.75D0*C3*C1**2-0.125D0*(3.0D0*C3-1.0D0)*C1**3
              P = 0.125D0*C5*C4*C1**2-0.125D0*(0.5D0*C5*C4+1.0D0)*C1**3
          END IF
      END IF
C
      A = A1*(1.0D0-F)
      DELTA = 4.0D0*PI*R1**2*(1.0D0+P-(1.0D0-F)**(2.D0/3.D0))
C
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE MASSEEX(B,R1,R0,A1,REFOPT1,B1,B2,BF1,BF2,
     +                                            AAA,BBB,CCC,DDD,AF,EE)
C
C     ON INPUT:  B,R1,R0,A1,REFOPT1,B1,B2,BF1,BF2,AAA,BBB,CCC,DDD
C     ON OUTPUT: AF,EE
C   ____________________________________________________________________
C  /
C  /  CALCUL DE LA MASSE ET DE L'ENERGIE D'EXCITATION DU PREFRAGMENT PAR
C  /  LE MODELE D'ABRASION
C  /
C  /  PROCEDURE FOR CALCULATING MASS AND EXCITATION ENERGY OF
C  /  PREFRAGMENTS WITH THE ABRASION MODEL
C  /
C  /  REFOPT1 = 0 : SURFACE-EXCESS ENERGY
C  /                (APPENDIX OF PHYS. REV. C16 (1977) 629)
C  /                WITH CORRECTIONS IN THE CRITICAL REGIONS
C  /
C  /  REFOPT1 = 1 : SURFACE-EXCESS PLUS FRICTION ENERGY
C  /                (NUCL. INSTR. METH. B18 (1987) 205)
C  /                WITH CORRECTIONS IN THE CRITICAL REGIONS
C  /
C  /  REFOPT1 = 2 : AVERAGE ENERGY OF LEVEL VACANCIES
C  /              (J.-J. GAIMARD ET K.-H. SCHMIDT, NUCL. PHYS. A (1991))
C  /____________________________________________________________________
C
C
      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,R_P,R_T,PI,
     &       BFPRO,SNPRO,SPPRO,SHELL
      INTEGER*4 IMAX,IRNDM
C
      INTEGER*4 REFOPT1
      REAL*8 B,R1,R0,A1,AF,EE,B1,B2,BF1,BF2
      REAL*8 C1,C2,C3,C4,C5,C6,C7,C8,C9,C10,C11,C12,C13,C14
      REAL*8 RT,U,V,W,F,P,DELTA,CL,CT,EFRICT,AAA(1:3),BBB(1:3),CCC(1:3),
     &       DDD(1:3)

C
C-----------------------------------------------------------------------
C     REFOPT1           OPTION (SEE ABOVE)
C     B                 IMPACT PARAMETER
C     R1                RADIUS OF THE PROJECTILE NUCLEUS
C     R0                RADIUS OF THE TARGET NUCLEUS
C     A1                MASS NUMBER OF THE PROJECTILE
C     AF                PREFRAGMENT MASS VALUE CORRESPONDING TO B
C     EE                EXCITATION ENERGY
C     PI                PI VALUE
C     B1,B2             IMPACT-PARAMETER LIMITS FOR CORRECTING
C                        MASS AND EXCITATION ENERGY OF PREFRAGMENTS
C     BF1,BF2            DITTO
C     C1,...,C14 LOCAL  HELP VARIABLES
C     RT       LOCAL    MAXIMUM IMPACT PARAMETER (R1+R0)
C     U,V,W    LOCAL    HELP VARIABLES
C     F,P      LOCAL    HELP VARIABLES
C     DELTA    LOCAL    SURFACE-EXCESS ENERGY CORRESSPONDING TO B
C     CL,CT    LOCAL    HELP VARIABLES
C     EFRICT   LOCAL    FRICTIONAL ENERGY
C     AAA( ),BBB( )     COEFFICIENTS OF CORRECTIVE POLYNOMIALS
C     CCC( ),DDD( )      DITTO
C-----------------------------------------------------------------------
C
      IF(B.LT.0.D0)THEN
       PRINT*,'WARNING - B < O'
       B = 0.D0
      ENDIF
      RT = R1+R0
      U = R1/RT
      V = B/RT
c      W = 1.D0/U-1.D0
      W = R0/R1
c      C1 = (1.D0-V)/U
      C1 = 1.D0 + R0/R1 - B/R1
c      C3 = DSQRT(1.D0-U)
      C3 = DSQRT(R0/(R0+R1))
c      C4 = 1.D0/W-2.D0
      C4 = R1/R0 - 2.D0
c      C5 = DSQRT(W*U)
       C5 = DSQRT(R0/(R0+R1))
      IF (R1.GT.R0) THEN
c        C2 = 1.D0-W**2
        C2 = 1.D0 - R0**2/R1**2
        IF (B.LT.(R1-R0)) THEN
c          C6 = DSQRT(1.D0-(V/U)**2)
          C6 = DSQRT(1.D0 - B**2/R1**2)
          F = (1.D0-C2**1.5D0)*C6
          P = (C2**0.5D0/U-1.D0)*C6
          ELSE
c            C7 = (3.D0*C3)/W
            C7 = 3.D0*DSQRT(R1**2/(R0*RT))
            C8 = 1.D0-C2**1.5D0
c            C9 = DSQRT(1.D0-(1.D0-W)**2)
            C9 = DSQRT(1.D0-(1.D0-R0/R1)**2)
            C10 = (C8*C9)/(W**3)
            F = 0.75D0*C3*C1**2-0.125D0*(C7-C10)*C1**3
c            C11 = 0.5D0*(U/W)**0.5D0*C4
            C11 = 0.5D0*R1*C4/DSQRT(R0*(R0+R1))
c            C12 = (C2**0.5D0/U)-1.D0
            C12 = RT*DSQRT((R1-R0)*(R1+R0))/R1**2-1.D0
c            C13 = DSQRT((2.D0-W)*W)
            C13 = DSQRT((2.D0*R1-R0)*R0)/R1
            C14 = (C12*C13)/(W**3)
            P = 0.125D0*C5*C4*C1**2-0.125D0*(C11-C14)*C1**3
            IF((R1**2-(R0-B)**2).LT.0.D0)THEN
             PRINT*,'MASSEX',R1,R0,B,IDNINT(AF),EE
             CL = 0.D0
            ELSE
            CL = 2.D0*DSQRT(R1**2-(R0-B)**2)
            ENDIF
            CT = R1+R0-B
            EFRICT = 6.5D0*CL+2.17D0*CL*(CT-1.5D0)
        END IF
        IF (REFOPT1.EQ.0) GO TO 40
        IF ((B.GE.BF1).AND.(B.LE.BF2)) THEN
          EFRICT = AAA(3)*B**3+BBB(3)*B**2+CCC(3)*B+DDD(3)
        END IF
        ELSE
          IF (B.LT.(R0-R1)) THEN
            F = 1.D0
            P = -1.D0
            EFRICT = 0.D0
            ELSEIF(R1.GT.(R0-B))THEN
              F = 0.75D0*C3*C1**2-0.125D0*(3.D0*C3-1.D0)*C1**3
              P = 0.125D0*C5*C4*C1**2-0.125D0*(0.5D0*C5*C4+1.D0)*C1**3
              CL = 2.D0*DSQRT(R1**2-(R0-B)**2)
              CT = R1+R0-B
              EFRICT = 6.5D0*CL+2.17D0*CL*(CT-1.5D0)
          END IF
      END IF
C
 40   CONTINUE
      AF = A1*(1.D0-F)
      DELTA = 4.D0*PI*R1**2*(1.D0+P-(1.D0-F)**0.6666666667)
      IF ((B.GE.B1).AND.(B.LE.B2)) THEN
        IF (B1.EQ.B2) GO TO 80
        DELTA = AAA(2)*B**3+BBB(2)*B**2+CCC(2)*B+DDD(2)
        IF (R1.LE.R0) GO TO 80
        AF = AAA(1)*B**3+BBB(1)*B**2+CCC(1)*B+DDD(1)
      END IF
 80   CONTINUE
C
      IF (REFOPT1.LT.1) THEN
        EE = 0.95D0*DELTA
        ELSE IF (REFOPT1.LT.2) THEN
          EE = 0.95D0*DELTA+EFRICT
        ELSE
          EE = (A1-AF)*13.3D0
      END IF
C
      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE DCHAGDR(A,Z,APF,A_B,DELTAB,VVTOTL,VV,LMI,LMA)
C
C     CALCULATION OF PREFRAGMENT-CHARGE-DISTRIBUTION ACCORDING
C     TO THE GDR-MODELL, MORRISSEY ET AL. 1978
C
C     INPUT:   A,Z,APF,A_B(1:3),DELTAB
C     OUTPUT:  VVTOTL,VV(:)
C
      INTEGER A,Z,APF,J,JMAX,LMI,LMA,FLAG
      REAL*8  A_B,DELTAB,VVTOTL,VV,EFKT,PON,ZAEHL
      REAL*8  SIDISP,SIGZ,U,DELTA,DELTAR,DELTAL,PI,COEFF,ZDA
      DIMENSION A_B(1:3),VV(0:240)
C
C     A        PROJECTILEMASS
C     Z        PROJECTILECHARGE
C     APF      PREFRAGMENTMASS
C     A_B      PREFRAGMENTMASSES AT  B-DELTAB,B,B+DELTAB
C     DELTAB   IMPACTPARAMETERDIFFERENCE TO CALCULATE DELTAA/DELTAB
C     VV(I)    PROBABILITY-DISTRIBUTION OF PREFRAGMENTCHARGE
C     VVTOTL   NORMALISATION OF  VV(I)
C     ZDA      Z/A OF  PROJECTILE
C     DELTAL   LEFTSIDED DIFFERENCEQUOTIENT
C     DELTAR   RIGHTSIDED DQ
C     DELTA    RANDOMLY SELECTED OUT OF DELTAL/R BUT CHECKED
C              FOR REASONABLE VALUE
C     COEFF    COEFFIZIENT OF GAUSSIAN DISTRIBUTION
C     J        CHARGENUMBER OF  PREFRAGMENTS
C     JMAX     MIN(APF,Z), BECAUSE  THAT IS THE MAXIMUM NUMBER
C              OF PROTONS IN THE PREFRAGMENT-NUCLEUS
C     LMI,LMA  INDEXVALUES OF IMPACTPARAMETERLOOP IN REANUCL
C     FLAG     AT SMALL IMPACT PARAMETERS, MAXIMUM ABRADED MASS,
C              THE GAUSSIAN-DISTRIBUTION BECOMES VERY SHARPLY PEAKED
C              THE EXPONENT IS ALWAYS  PON<-174.673
C              IN THIS CASE THE PROB. DIST. VV(:) IS CONST. ZERO
C              THEREFORE A FLAG IS SET AND THE VERY SHARPELY PEAKED
C              GAUSSIAN-DISTRIBUTION IS APPROXIMATED BY A DELTA-PEAK
C              AROUND THE CENTER
      PI = 3.141592653589793D0
      VVTOTL =0.D0
      DO 10 J=0,240,1
         VV(J) = 0.D0
 10   CONTINUE
      FLAG = 0
C ********** CALCULATION OF DA/DB, CHANGE OF THE ABRADED MASS
C            WITH IMPACT PARAMETER  FOR GDR-CHARGE-DISTRIBUTION ****
C
      IF ((LMI.EQ.1).AND.(LMA.EQ.2))  THEN
        DELTAL =(A_B(2) - A_B(1))/DELTAB
        DELTAR = DELTAL
      END IF
      IF ((LMI.EQ.2).AND.(LMA.EQ.3))  THEN
        DELTAR =(A_B(3) - A_B(2))/DELTAB
        DELTAL = DELTAR
      END IF
      IF ((LMI.EQ.1).AND.(LMA.EQ.3))  THEN
        DELTAR =(A_B(3) - A_B(2))/DELTAB
        DELTAL =(A_B(2) - A_B(1))/DELTAB
      END IF
      ZDA    = DFLOAT(Z)/DFLOAT(A)
C     WRITE(6,*)'DA/DB(R)= ',DELTAR
C     WRITE(6,*)'DA/DB(L)= ',DELTAL
      IF (DABS((DELTAR-DELTAL)).GT.(1.D-01)) THEN
         WRITE(6,*)'ERROR IN DCHAGDR'
         WRITE(6,*)'LEFTSIDED & RIGHTSIDED DIFFERENCE-QUOTIENT'
         WRITE(6,*)'DIFFER BY MORE THAN  0.1'
      END IF
C ************  U238 ON C12 MAXIMUM DA/DB CA. 10    *****
C               U238 ON U238 MAX. DA/DB CA. 21       *****
C               S16  ON S16 MAX.   DA/DB CA. 5.5     *****
      IF ((DABS(DELTAR).GT.25.D0).OR.(DABS(DELTAL).GT.25.D0)) THEN
         WRITE(6,*)'ERROR IN DCHAGDR'
         WRITE(6,*)'LEFT- OR RIGHTSIDED DIFFERENCEQUOTIENT TOO LARGE.'
      END IF
C ************ IF ONLY ONE OF BOTH FITS , THIS ONE IS CHOSEN  *****
C ************ DO BOTH FIT, THE LEFTSIDED IS CHOSEN     *****
      IF (DABS(DELTAR).LE.25.D0) DELTA = DELTAR
      IF (DABS(DELTAL).LE.25.D0) DELTA = DELTAL
C ************ SEE MORRISSEY ET AL. PHYS.REV C 18, #3 (1978)
      U = 6.494D0/DFLOAT(A)**(1.0D0/3.0D0)
      SIDISP = 2.619D0*U**0.5D0/((1.0D0+U)**(0.75D0)*DFLOAT(A)**0.5D0)
      SIGZ   = SIDISP*DELTA*DFLOAT(Z)/DFLOAT(A)
      COEFF  = (2.0D0*PI*SIGZ**2.0D0)**(-0.5D0)
C     WRITE(6,*)'SIGZ,COEFF',SIGZ,COEFF
C     WRITE(6,*)'0.5*SIGZ**(-2) ',0.5D0*SIGZ**(-2)
C     WRITE(6,*)'APF ',APF
C     ***********   LOOP OVER THE PREFRAGMENTCHARGE
C                   TO CALCULATE THE CHARGE-DISTRIBUTION
      JMAX = MIN(APF,Z)
      DO 20 J=0,JMAX,1
C ******* PON = EXPONENT; CHECKED AGAINST FP UNDERFLOW
C ******* EFKT = EXPONENTIAL-FCT.; == 0 IF EXPONENT < -174.673
         ZAEHL = (DFLOAT(J) - DFLOAT(APF)*ZDA)**2
         PON   = -0.5D0 * ZAEHL/SIGZ**2
         IF (DABS(PON).GT.(174.673D0)) THEN
           EFKT = 0.D0
         ELSE
C          WRITE(6,*)'J,PON,ZAEHL ',J,PON,ZAEHL
           EFKT = DEXP(PON)
           FLAG = 1
         END IF
         VV(J) = COEFF* EFKT
         VVTOTL = VVTOTL + VV(J)
 20   CONTINUE
C ******** IF FLAG=0 THEN VV(1:APF) CONST ZERO,THEREFORE
C          APPROXIMATION THROUGH DELTA-FUNCTION AROUND
C          THE CENTER   INT(DFLOAT(APF)*ZDA)
      IF (FLAG.EQ.0) THEN
         VV(INT(DFLOAT(APF)*ZDA + 0.5D0)) = 1.D0
         VVTOTL = 1.D0
         WRITE(6,*)'DELTA APPROXIMATION IN DCHAGDR'
      END IF
      RETURN
      END
C
C****************************************************************************
C
      SUBROUTINE DCHAHYP (A,Z,APF,VVTOTL,VV)
C
C     ON INPUT :  A,Z,APF
C     ON OUTPUT:  VVTOTL,VV
C
C     PROCEDURE FOR CALCULATION OF THE PREFRAGMENT-CHARGE-DISTRIBUTION
C     ACCORDING TO THE HYPERGEOMETRICAL MODELL DESCRIBED IN
C     MORRISSEY ET AL. PHYS. REV. C18 NO.3 1267 (1978)
C
C     CHANGED BY ARND JUNGHANS   AUGUST 1992
C     REPLACES THE ORIGINAL SUBROUTINES DCHAYP, DCPRF1
C     AND THE FUNCTION PROB1 FROM RN41 BY J.J. GAIMARD
C
      INTEGER A,Z,APF,N,J,AP,AN,AM
      REAL*8 VVTOTL,VV(0:240),BINO,DENOM
C
C     A      PROJECTILEMASS
C     Z      PROJECTILECHARGENUMBER
C     APF    PREFRAGMENTMASS (LOCAL)
C     N      NEUTRON-NUMBER OF PROJECTILE (LOCAL)
C     AP     PROTON NUMBER OF THE PREFRAGMENT (LOCAL)
C     AM     ABRADED MASS FROM PROJECTILE     (LOCAL)
C     J      LOOP OVER PREFRAGMENT-CHARGE-DISTRIBUTION (LOCAL)
C     VVTOTL NORMALISATION-CONSTANT FOR THE CHARGE-DISTRIBUTION
C     VV(:)  PREFRAGMENT-CHARGE-DISTRIBUTION-ARRAY
C     BINO   FUNCTION TO CALCULATE BINOMIAL-COEFFICIENTS
C     DENOM  DENOMINATOR OF HYPERGEOMETRICAL DISTRIBUTION (LOCAL)
C
      N = A-Z
      AM = A - APF

      DO 1  J = 0,240,1
      VV(J) = 0.D0
  1   CONTINUE
      VVTOTL = 0.D0
      DENOM = BINO(A,AM)
      DO 10 J = 0,MIN(AM,Z),1
      AP = J
      AN = AM - J
      VV(Z-J) = BINO(N,AN) / DENOM * BINO(Z,AP)
C     WRITE(6,*)BINO(A,AM),BINO(N,AN),BINO(Z,AP)
C     WRITE(6,*)AM,AN,AP
      VVTOTL = VVTOTL + VV(Z-J)
 10   CONTINUE
      IF (VVTOTL.GE.1.1D0) THEN
         WRITE(6,*)'ERROR IN DCHAHYP, SUM OF PROB. =',VVTOTL
         print*,'!!!!!',A,Z,APF,N,AM,DENOM
      END IF
      IF (VVTOTL.LE.0.9D0) THEN
         WRITE(6,*)'ERROR IN DCHAHYP, SUM OF PROB. =',VVTOTL
         print*,'????',A,Z,APF,N,AM,DENOM
      END IF
      RETURN
      END

C
C ************************************************************
C
      DOUBLE PRECISION FUNCTION BINO(N,K)
C
C     FUNCTION FOR CALCULATING A BINOMIAL COEFFICIENT
C     N OVER K   N,K MUST BE OF INTEGER TYPE
C     A.JUNGHANS AUGUST 1992
C
      INTEGER N,K,J,I
      REAL*8 NK
C
      I = K
      NK = 1.0D+00
      DO 10 J=N,N-K+1,-1
         NK = NK * DFLOAT(J) / DFLOAT(I)
         I  = I - 1
  10  CONTINUE
      BINO = NK
      RETURN
      END
C
C-----------------------------------------------------------------------
      SUBROUTINE LOCATE(XX,N,X,J)
C
C GIVEN AN ARRAY XX(N) AND GIVEN A VALUE X, RETURNS A VALUE KJ SUCH THAT
C X IST BETWEEN XX(J) AND XX(J+1). XX(N) MUST BE MONOTONIC EITHER INCREA
C OR DECREASING. J=0 OR J = N IS RETURNED IF X IS OUT OF RANGE
C NUMERICAL RECIPES F77 SECOND ED. P.111
C
      INTEGER J,N
      REAL*8 X,XX(N)
      INTEGER JL,JM,JU
      JL = 0
      JU = N + 1
 29   IF (JU-JL.GT.1) THEN
          JM = (JU+JL)/2
          IF ((XX(N).GE.XX(1)).EQV.(X.GE.XX(JM))) THEN
             JL = JM
          ELSE
             JU= JM
          END IF
      GO TO 29
      END IF
      IF (X.EQ.XX(1)) THEN
          J = 1
      ELSE IF (X.EQ.XX(N)) THEN
          J = N - 1
      ELSE
          J = JL
      END IF
      RETURN
      END

C***********************************************************************

      SUBROUTINE AMOMENT_ABRA(AABRA,EE,IMULTIFR,PX,PY,PZ)

C      ON INPUT  : AT, AP, AABRA, EE,IRNDM
C      ON OUTPUT : AMOM
C     (AT,AP,APRF,EE,IRNDM,beta2,SIG2,AMOM)

      IMPLICIT NONE

      REAL*8 APRF,EE,GOLDHA,AMOM,pi,BETAP,GAMMAP,AABRA
      REAL*8 FACT_PROJ,R,RNDT,CTET,RNDP,PHI,STET,RX,RY,RZ,PX,PY,PZ,
     &       PMORR,JPRFX,JPRFY,JPRFZ,PTOT,PHI_P,TETA_P,nu,GOLDHA_AB,
     &       PT,PF,TETA_J,PHI_RAT,PHI_J,teta,F,BASIS,GOLDHA_BU,GOLDHA_TH
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,BFPRO,SNPRO,SPPRO,SHELL
      INTEGER IMAX,IRNDM
      INTEGER*4 IMULTIFR,ISIGOPT,IS
      REAL*8 SIGMA_0, V0_over_VBU, ATRANS, PMORR_TRANS, PSLOPE,
     &       FACT_PROJ_1GEV, PMORR_TRANS_1GEV,T_freeze_out,Efermi,
     &       OURX
      REAL*4 RNDM, HAZ, GAUSSHAZ
      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
      COMMON /TFO/T_freeze_out
      COMMON/ISIGOPT/ISIGOPT

	pi = 3.141592653589793D0
      nu = 1.d0

      BETAP = DSQRT(1.D0 - 1.D0/DSQRT(1.D0+EAP/931.494D0))
      GAMMAP = 1.D0 / DSQRT(1.D0 - BETAP**2)
      FACT_PROJ = (GAMMAP + 1.D0) / (BETAP * GAMMAP)

      R = 1.16D0 * AABRA ** (1.D0/3.D0)

      RNDT = DBLE(HAZ(1))
      CTET = 2.D0*RNDT-1.D0
      TETA = DACOS(CTET)
      RNDP = DBLE(HAZ(1))
      PHI = RNDP*2.D0*PI
      STET = DSQRT(1.D0-CTET**2)
c      RX = R * STET * DCOS(PHI)
c      RY = R * STET * DSIN(PHI)
c      RZ = R * CTET

      RZ = 0.d0
      RY = R * DSIN(PHI)
      RX = R * DCOS(PHI)

c      GOTO 8642
C In MeV/C
c      PMORR = 8.D0 * (nu + 1.D0) * FACT_PROJ * AABRA * (1.D0 - AABRA/AP)
C Trial to include empiricaly observed acceleration effect:
C ATRANS is the mass where transition from Morriessey to acceleration occurs; it is a parameter
      ATRANS = AP - (17.D0 + 6.D0 * DBLE(1.E0 - 2.E0 * HAZ(1)))
C PMORR_TRANS is the value of shift at A = ATRANS; the idea is for masses lower than ATRANS to
C "mirror" the value of shift obtained by Morriessey.
      PMORR_TRANS = -8.D0 * (nu + 1.D0) * FACT_PROJ * ATRANS *
     &         (AP - ATRANS) / AP
      FACT_PROJ_1GEV = (2.0735D0 + 1.D0) / (0.876D0 * 2.0735D0)
      PMORR_TRANS_1GEV = -8.D0 * (nu + 1.D0) * FACT_PROJ_1GEV * ATRANS *
     &         (AP - ATRANS) / AP
C
C Parameter to describe the "strength" of the acceleration effect
      PSLOPE = 2.5D0

      IF(AABRA .GT. ATRANS)THEN
       PMORR = -8.D0 * (nu + 1.D0) * FACT_PROJ * AABRA *
     &          (AP - AABRA) / AP
      ELSE
       PMORR = PMORR_TRANS + PSLOPE * DABS(-8.D0 * (nu + 1.D0) *
     &         FACT_PROJ_1GEV * AABRA * (AP - AABRA) / AP -
     &         PMORR_TRANS_1GEV)
      ENDIF
c 8642 CONTINUE

C Parabolic parametrization - VF,MVR,KHS March 2009
c      OURX = (nu + 1.D0) * (AP - AABRA)
c      PMORR =  FACT_PROJ * AABRA / AP * (-8.D0*OURX + 0.03D0*OURX**2)

C In MeV/C
      SIGMA_0 = 118.5D0
      GOLDHA = SIGMA_0 * DSQRT((AABRA*(AP-AABRA))/(AP-1.D0))
C
      IS = 0
 123  PX = DBLE(GAUSSHAZ(1,0.E0,SNGL(GOLDHA)))
      IS = IS +1
      IF(IS.GT.100)THEN
      PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     & CALCULATING PX IN Rn07.FOR. A VALUE WILL BE FORCED.'
      PX = (AABRA-1.D0)*931.494D0
      ENDIF
      IF(DABS(PX).GE. AABRA*931.494D0) THEN
c       PRINT*,'VX > C',PX,IDNINT(APRF)
       GOTO 123
      ENDIF
      IS = 0
 456  PY = DBLE(GAUSSHAZ(1,0.E0,SNGL(GOLDHA)))
      IS = IS +1
      IF(IS.GT.100)THEN
      PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     & CALCULATING PY IN Rn07.FOR. A VALUE WILL BE FORCED.'
      PY = (AABRA-1.D0)*931.494D0
      ENDIF
      IF(DABS(PY).GE. AABRA*931.494D0) THEN
c       PRINT*,'VY > C',PY,IDNINT(APRF)
       GOTO 456
      ENDIF
      IS = 0
 789  PZ = DBLE(GAUSSHAZ(1,SNGL(PMORR),SNGL(GOLDHA)))
      IS = IS +1
      IF(IS.GT.100)THEN
      PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     & CALCULATING PZ IN Rn07.FOR. A VALUE WILL BE FORCED.'
      PZ = (AABRA-1.D0)*931.494D0
      ENDIF
      IF(DABS(PZ).GE. AABRA*931.494D0)THEN
c       PRINT*,'VZ > C',PZ,IDNINT(APRF)
       GOTO 789
      ENDIF

      PTOT = DSQRT(PX**2+PY**2+PZ**2)

C The component of the linear momentum along the prefr. radius (PR)
C doesn't contribute to angular momentum, only transversal compnenet PT
      TETA_P = DACOS(PZ/PTOT)
      IF(PX.EQ.0.D0) THEN
       IF(PY.GT.0.D0) PHI_P = PI / 2.D0
       IF(PY.LT.0.D0) PHI_P = 1.5D0 * PI
       IF(PY.EQ.0.D0) PHI_P = 0.D0
      ELSE
      PHI_P = DASIN(PY/DSQRT(PX**2+PY**2))
      IF(PHI_P .LT. 0.D0) PHI_P = 2.D0 * PI - DABS(PHI_P)
      IF(PX .LT. 0.D0) THEN
      PHI_P = PI - DASIN(PY/DSQRT(PX**2+PY**2))
      ENDIF
      ENDIF

C In MeV/C * fm
      JPRFX = RX * PZ - RZ * PY
      JPRFY = RZ * PX - RX * PZ
      JPRFZ = RX * PY - RY * PX
C In Hbar
      JPRFX = JPRFX / 197.3395D0
      JPRFY = JPRFY / 197.3395D0
      JPRFZ = JPRFZ / 197.3395D0
      AMOM = DSQRT(JPRFX**2 + JPRFY**2 + JPRFZ**2)

      RETURN

      END
C***********************************************************************

      SUBROUTINE AMOMENT(AABRA,APRF,EE,IMULTIFR,PX,PY,PZ)

C      ON INPUT  : AT, AP, APRF, EE,IRNDM
C      ON OUTPUT : AMOM
C     (AT,AP,APRF,EE,IRNDM,beta2,SIG2,AMOM)

      IMPLICIT NONE

      REAL*8 APRF,EE,GOLDHA,AMOM,pi,BETAP,GAMMAP,AABRA
      REAL*8 FACT_PROJ,R,RNDT,CTET,RNDP,PHI,STET,RX,RY,RZ,PX,PY,PZ,
     &       PMORR,JPRFX,JPRFY,JPRFZ,PTOT,PHI_P,TETA_P,nu,GOLDHA_AB,
     &       PT,PF,TETA_J,PHI_RAT,PHI_J,teta,F,BASIS,GOLDHA_BU,GOLDHA_TH
      REAL*8 AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,BFPRO,SNPRO,SPPRO,SHELL
      INTEGER IMAX,IRNDM,IS
      INTEGER*4 IMULTIFR,ISIGOPT
      REAL*8 SIGMA_0, V0_over_VBU, ATRANS, PMORR_TRANS, PSLOPE,
     &       FACT_PROJ_1GEV, PMORR_TRANS_1GEV,T_freeze_out,Efermi
      REAL*4 RNDM, HAZ, GAUSSHAZ
      COMMON /ABRAMAIN/ AP,ZP,AT,ZT,EAP,BETA,BMAXNUC,CRTOT,CRNUC,R_0,
     +                  R_P,R_T,IMAX,IRNDM,PI,BFPRO,SNPRO,SPPRO,SHELL
      COMMON /TFO/T_freeze_out
      COMMON/ISIGOPT/ISIGOPT

	pi = 3.141592653589793D0
      nu = 1.d0

      BETAP = DSQRT(1.D0 - 1.D0/DSQRT(1.D0+EAP/931.494D0))
      GAMMAP = 1.D0 / DSQRT(1.D0 - BETAP**2)
      FACT_PROJ = (GAMMAP + 1.D0) / (BETAP * GAMMAP)

      R = 1.16D0 * APRF ** (1.D0/3.D0)

      RNDT = DBLE(HAZ(1))
      CTET = 2.D0*RNDT-1.D0
      TETA = DACOS(CTET)
      RNDP = DBLE(HAZ(1))
      PHI = RNDP*2.D0*PI
      STET = DSQRT(1.D0-CTET**2)
c      RX = R * STET * DCOS(PHI)
c      RY = R * STET * DSIN(PHI)
c      RZ = R * CTET

      RZ = 0.d0
      RY = R * DSIN(PHI)
      RX = R * DCOS(PHI)

C In MeV/C
      V0_over_VBU = 1.D0 / 6.D0
      SIGMA_0 = 118.5D0
      Efermi = 5.D0 * SIGMA_0 * SIGMA_0 / (2.D0 * 931.494D0)
c      PFERMI = 281.D0 * (1.D0 - 1.D0/AABRA**0.568D0)
c      SIGMA0 = PFERMI/DSQRT(5.D0)
      IF(IMULTIFR.EQ.1) THEN
       IF(ISIGOPT .EQ. 0) THEN
C "Fermi model" picture:
C Influence of expansion:
        SIGMA_0 = SIGMA_0 * V0_over_VBU**(1.D0/3.D0)
C To take into account the influence of thermal motion of nucleons (see W. Bauer,
C PRC 51 (1995) 803)
c        Efermi = 5.D0 * SIGMA_0 * SIGMA_0 / (2.D0 * 931.49D0)

        GOLDHA_BU = SIGMA_0 * DSQRT((APRF*(AABRA-APRF))/(AABRA-1.D0))
        GOLDHA    = GOLDHA_BU*DSQRT(1.D0 +
     &              5.D0 * PI*PI / 12.D0 * (T_freeze_out / Efermi)**2)
c       PRINT*,'AFTER BU fermi:',IDNINT(AABRA),IDNINT(APRF),GOLDHA,
c     &                          GOLDHA_BU
       ELSE
C Thermal equilibrium picture (<=> to Boltzmann distribution in momentum with sigma2=M*T)
C The factor (AABRA-APRF)/AP comes from momentum conservation:
        GOLDHA_BU = DSQRT(APRF *  T_freeze_out * 931.494D0 *
     &             (AABRA - APRF) / AABRA)
        GOLDHA   = GOLDHA_BU
c       PRINT*,'AFTER BU therm:',IDNINT(AABRA),IDNINT(APRF),GOLDHA,
c     &                          GOLDHA_BU
       ENDIF
      ELSE
      GOLDHA = SIGMA_0 * DSQRT((APRF*(AABRA-APRF))/(AABRA-1.D0))
      ENDIF

c      GOLDHA = 0.D0
C
      IS = 0
 123  PX = DBLE(GAUSSHAZ(1,0.E0,SNGL(GOLDHA)))
      IS = IS +1
      IF(IS.GT.100)THEN
      PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     & CALCULATING PX IN Rn07.FOR. A VALUE WILL BE FORCED.'
      PX = (AABRA-1.D0)*931.494D0
      ENDIF
      IF(DABS(PX).GE. AABRA*931.494D0) THEN
c       PRINT*,'VX > C',PX,IDNINT(APRF)
       GOTO 123
      ENDIF
      IS = 0
 456  PY = DBLE(GAUSSHAZ(1,0.E0,SNGL(GOLDHA)))
      IS = IS +1
      IF(IS.GT.100)THEN
      PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     & CALCULATING PY IN Rn07.FOR. A VALUE WILL BE FORCED.'
      PY = (AABRA-1.D0)*931.494D0
      ENDIF
      IF(DABS(PY).GE. AABRA*931.494D0) THEN
c       PRINT*,'VY > C',PY,IDNINT(APRF)
       GOTO 456
      ENDIF
      IS = 0
 789  PZ = DBLE(GAUSSHAZ(1,0.E0,SNGL(GOLDHA)))
      IS = IS +1
      IF(IS.GT.100)THEN
      PRINT*,'WARNING: GAUSSHAZ CALLED MORE THAN 100 TIMES WHEN
     & CALCULATING PZ IN Rn07.FOR. A VALUE WILL BE FORCED.'
      PZ = (AABRA-1.D0)*931.494D0
      ENDIF
      IF(DABS(PZ).GE. AABRA*931.494D0)THEN
c       PRINT*,'VZ > C',PZ,IDNINT(APRF)
       GOTO 789
      ENDIF

      PTOT = DSQRT(PX**2+PY**2+PZ**2)

C The component of the linear momentum along the prefr. radius (PR)
C doesn't contribute to angular momentum, only transversal compnenet PT
      TETA_P = DACOS(PZ/PTOT)
      IF(PX.EQ.0.D0) THEN
       IF(PY.GT.0.D0) PHI_P = PI / 2.D0
       IF(PY.LT.0.D0) PHI_P = 1.5D0 * PI
       IF(PY.EQ.0.D0) PHI_P = 0.D0
      ELSE
      PHI_P = DASIN(PY/DSQRT(PX**2+PY**2))
      IF(PHI_P .LT. 0.D0) PHI_P = 2.D0 * PI - DABS(PHI_P)
      IF(PX .LT. 0.D0) THEN
      PHI_P = PI - DASIN(PY/DSQRT(PX**2+PY**2))
      ENDIF
      ENDIF

C In MeV/C * fm
      JPRFX = RX * PZ - RZ * PY
      JPRFY = RZ * PX - RX * PZ
      JPRFZ = RX * PY - RY * PX
C In Hbar
      JPRFX = JPRFX / 197.3395D0
      JPRFY = JPRFY / 197.3395D0
      JPRFZ = JPRFZ / 197.3395D0
      AMOM = DSQRT(JPRFX**2 + JPRFY**2 + JPRFZ**2)

      RETURN

      END

C***********************************************************************

      SUBROUTINE AMOMENT_deJong(AT,AP,APRF,EE,IRNDM,beta2,SIG2,AMOM)

C      ON INPUT  : AT, AP, APRF, EE,IRNDM
C      ON OUTPUT : AMOM

      IMPLICIT NONE

      REAL*8 AP,APRF,EE,H,SIG2,F,J,P,PJ,SIG,PMAX,AMOM,pi
      REAL*8 BASIS,PON,XRND,beta2,AT
      INTEGER*4 IRNDM
      real*4 RNDM
	pi = 3.141592653589793D0
C       WRITE(6,*) beta2
      IRNDM = IRNDM + 1

C    NUMBER OF HOLES:
      H=AP-APRF
      IF (H.GT.0.D0) THEN
C        Dependence of mean squared angular momentum on excitation energy:
          BASIS =  (1.D0-EE/(H*80.D0))
          IF (BASIS.GT.0.D0) THEN
           F=(1.D0-EE/(H*80.D0))
          ELSE
           F=0.D0
          END IF
C        sigma squared (CONSIDERING FINITE NUCLEUS ACC. GOLDHABER):
          SIG2 = 0.234D0*AP**(2.D0/3.D0)*F*(AP-H)*(H/(AP-1.D0))*(1.D0 -
     & 		2.D0*beta2 /3.D0)
	
C
      ELSE
          SIG2 = 0.D0
          AMOM = 0.D0
C          RETURN
      END IF


C MONTE CARLO METHOD FOR J

      IF (SIG2.GT.0.D0) THEN
       SIG = dsqrt(sig2)
C IF SIG SMALLER THAN 0.2 THAN AMOM IS ZERO BECAUSE WE THROW DICE ONLY UP TO 4SIG
       IF (SIG.LT.0.2D0) THEN
         SIG2 = 0.D0
         AMOM = 0.D0
	 RETURN
       END IF

C By deriving the  J distribution follows:
C The maximum of the the J distribution ist at J_max = sigma-0.5
C The probability at this J_max is PMAX:

       PMAX = 2.D0*SIG*DEXP(-(sig2-0.25D0)/(2.D0*sig2))

 10    CONTINUE

C Now consider a box around the distribution in J from 0 to J_max+4*sigma
C in P from 0 to PMAX and we are dicing a point (J,P) in this box:
       XRND = DBLE(RNDM(IRNDM))
       J = (sig-0.5D0+4.D0*sig)*XRND
       IRNDM = IRNDM +1
       XRND = DBLE(RNDM(IRNDM))
       P = PMAX*XRND
C If the point is under the J distribution we take it as proper  angular
C momentum amom, otherwise repeat the procedure
       PON=(J*(J+1.D0))/(2.D0*sig2)
       IF (DABS(PON).GT.700.D0) THEN
          PON = 700.D0*PON/DABS(PON)
       END IF
       PJ = (2.D0*J+1.D0)*DEXP(-PON)
       IF (P.GT.PJ) THEN
          GO TO 10
        ELSE
         AMOM = J
      END IF
      END IF
C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>7.11.01<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
C Introcuded by K.H. Schmidt and Beatriz Jurado to account for the fact that the
C angular momentum induced by the reaction with a hydrogen target appears to be
C twice the angular momentum obtained by appliying the Goldhaber theory. This has
C been founded by means of calculations performed with the Cougnon cascade.
         if(AT.le.2) AMOM = AMOM*2.D0
C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      RETURN
      END

C--------------------------------------------------------------------
C
      SUBROUTINE SPLINE (N,X,Y,B,C,D)
      INTEGER N,NM1,IB,I
      REAL*8 X(N),Y(N),B(N),C(N),D(N),T
C
C     Calculates the Coefficients of Cubic Spline
C     N - Number of points in the initial array (x,y)
C     b,c,d - array which contain the coefficients of the spline
C
      NM1=N-1
      IF (N.LT.2) RETURN
      IF (N.LT.3) GO TO 6
      D(1)=X(2)-X(1)
      C(2)=(Y(2)-Y(1))/D(1)
      DO 1 I=2,NM1
      D(I)=X(I+1)-X(I)
      B(I)=2.D0*(D(I-1)+D(I))
      C(I+1)=(Y(I+1)-Y(I))/D(I)
      C(I)=C(I+1)-C(I)
 1    CONTINUE
      B(1)=-D(1)
      B(N)=-D(N-1)
      C(1)=0.D0
      C(N)=0.D0
      IF (N.EQ.3) GO TO 2
      C(1)=C(3)/(X(4)-X(2))-C(2)/(X(3)-X(1))
      C(N)=C(N-1)/(X(N)-X(N-2))-C(N-2)/(X(N-1)-X(N-3))
      C(1)=C(1)*D(1)**2/(X(4)-X(1))
      C(N)=-C(N)*D(N-1)**2/(X(N)-X(N-3))
 2    DO 3 I=2,N
      T=D(I-1)/B(I-1)
      B(I)=B(I)-T*D(I-1)
      C(I)=C(I)-T*C(I-1)
 3    CONTINUE
      C(N)=C(N)/B(N)
      DO 4 IB=1,NM1
      I=N-IB
      C(I)=(C(I)-D(I)*C(I+1))/B(I)
 4    CONTINUE
      B(N)=(Y(N)-Y(NM1))/D(NM1)+D(NM1)*(C(NM1)+2.D0*C(N))
      DO 5 I=1,NM1
      B(I)=(Y(I+1)-Y(I))/D(I)-D(I)*(C(I+1)+2.D0*C(I))
      D(I)=(C(I+1)-C(I))/D(I)
      C(I)=3.D0*C(I)
 5    CONTINUE
      C(N)=3.D0*C(N)
      D(N)=D(N-1)
      RETURN
 6    B(1)=(Y(2)-Y(1))/(X(2)-X(1))
      C(1)=0.D0
      D(1)=0.D0
      B(2)=B(1)
      C(2)=0.D0
      D(2)=0.D0
      RETURN
      END
C
C--------------------------------------------------------------------
      DOUBLE PRECISION FUNCTION SEVAL (N,U,X,Y,B,C,D)
      INTEGER N,I,J,K
      REAL*8 U,X(N),Y(N),B(N),C(N),D(N),DX
C
C     Calculates the value of a cubic spline (coefficients in b,c,d)
C     in the point u
C
      DATA I/1/
      IF (I.GE.N) I=1
      IF (U.LT.X(I)) GO TO 1
      IF (U.LE.X(I+1)) GO TO 3
 1    I=1
      J=N+1
 2    K=(I+J)/2
      IF (U.LT.X(K)) J=K
      IF (U.GE.X(K)) I=K
      IF (J.GT.I+1) GO TO 2
 3    DX=U-X(I)
      SEVAL=Y(I)+DX*(B(I)+DX*(C(I)+DX*D(I)))
      RETURN
      END
C
C------------------------------------------------------------------
C
C SPLINE EVALUATION FOR EQUIDISTANT POINTS WITH INTERVAL DS
C
C
      DOUBLE PRECISION FUNCTION SPLINT (DS,N,U,X,Y,B,C,D)
      INTEGER N,I
      REAL*8 U,X(N),Y(N),B(N),C(N),D(N),DX,DS
C
C     Calculates the value of a cubic spline (coefficients in b,c,d)
C     in the point u FOR EQUIDISTANT ARRAYS
C
      I = IDINT(U/DS) + 1
      DX=U-X(I)
      SPLINT=Y(I)+DX*(B(I)+DX*(C(I)+DX*D(I)))
      RETURN
      END
C
C----------------------------------------------------------------------
C
      SUBROUTINE NORRAN(XRES)
C
C     FROM A.J.; VERSION 05/04/93;
C     CALCULATES A RANDOM NUMBER DISTRIBUTED ACCORDING TO
C     A GAUSSIAN FUNCTION NORMALIZED TO 1 AT
C     CENTER 0 WITH SIGMA = 1 ; VALUES RETURNED ARE BETWEEN
C     -10 AND +10;
C     2-DIMENSIONAL SAMPLE AND REJECTION METHOD
C
      SAVE
      INTEGER*4 I/1/
      real*8 XTEST,XRES,PI,YTEST,GAU
      real RNDM,HAZ
C
      PI = 3.141592654D0
C
10    CONTINUE
C
      I=I+1
      XTEST = (DBLE(HAZ(1))-0.5D0)*20.0D0
      I=I+1
      YTEST = DBLE(HAZ(1))
      GAU = DEXP(-0.5D0*XTEST**2)
      IF (YTEST.GT.GAU) GOTO 10
C
      XRES = XTEST
C
      END
C

C------------------------------------------------------------------
C
C  bessel function K0 nach numerical recipes
C
      FUNCTION ddbesk0(x)
      SAVE
      REAL*8 ddbesk0,x
CU    USES bessi0
      REAL*8 bessi0
      REAL*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,y
C     SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7   (kommentiert, KHS 31.1.99)
      DATA p1,p2,p3,p4,p5,p6,p7/-0.57721566d0,0.42278420d0,0.23069756d0,
     *
     *0.3488590d-1,0.262698d-2,0.10750d-3,0.74d-5/
      DATA q1,q2,q3,q4,q5,q6,q7/1.25331414d0,-0.7832358d-1,0.2189568d-1,
     *
     *-0.1062446d-1,0.587872d-2,-0.251540d-2,0.53208d-3/
      if (x.le.2.0d0) then
        y=x*x/4.0d0
       ddbesk0=(-dlog(x/2.0d0)*bessi0(x))+(p1+y*(p2+y*(p3+y*(p4+y*(p5+y*
     *(p6+y*p7))))))
      else
        y=(2.0d0/x)
        ddbesk0=(dexp(-x)/dsqrt(x))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *q7))))))
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software (52:.

C------------------------------------------------------------------
C
C bessel function K1 nach numerical recipes
C
      FUNCTION ddbesk1(x)
      SAVE
      REAL*8 ddbesk1,x
CU    USES bessi1
      REAL*8 bessi1
      REAL*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,y
C     SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7  (comm., KHS, 31.1.99)
      DATA p1,p2,p3,p4,p5,p6,p7/1.0d0,0.15443144d0,-0.67278579d0,
     &-0.18156897d0,-0.1919402d-1,-0.110404d-2,-0.4686d-4/
      DATA q1,q2,q3,q4,q5,q6,q7/1.25331414d0,0.23498619d0,-0.3655620d-1,
     &0.1504268d-1,-0.780353d-2,0.325614d-2,-0.68245d-3/
      if (x.le.2.0d0) then
        y=x*x/4.0d0
       ddbesk1=(dlog(x/2.0d0)*bessi1(x))+(1.0d0/x)*(p1+y*(p2+y*(p3+y*(p4
     &         +y*(p5+y*(p6+y*p7))))))
      else
        y=2.0d0/x
        ddbesk1=(dexp(-x)/dsqrt(x))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     &          q7))))))
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software (52:.


C------------------------------------------------------------------
C
C bessel function needed in ddbesk0
C
      FUNCTION bessi0(x)
      SAVE
      REAL*8 bessi0,x
      REAL*8 ax
      REAL*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9,y
     *
C     SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9 (comm. KHS, 31.1.99)
      DATA p1,p2,p3,p4,p5,p6,p7/1.0d0,3.5156229d0,3.0899424d0,
     *1.2067492d0,0.2659732d0,0.360768d-1,0.45813d-2/
      DATA q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,0.1328592d-1,
     *0.225319d-2,-0.157565d-2,0.916281d-2,-0.2057706d-1,0.2635537d-1,
     *-0.1647633d-1,0.392377d-2/
      if (dabs(x).lt.3.75d0) then
        y=(x/3.75d0)**2
        bessi0=p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7)))))
      else
        ax=dabs(x)
        y=3.75d0/ax
        bessi0=(dexp(ax)/dsqrt(ax))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *(q7+y*(q8+y*q9))))))))
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software (52:.

C------------------------------------------------------------------
C
C bessel function needed in ddbesk1
C
      FUNCTION bessi1(x)
      SAVE
      REAL*8 bessi1,x
      REAL*8 ax
      REAL*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9,y
     *
C     SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9  (comm. KHS, 31.1.99)
      DATA p1,p2,p3,p4,p5,p6,p7/0.5d0,0.87890594d0,0.51498869d0,
     *0.15084934d0,0.2658733d-1,0.301532d-2,0.32411d-3/
      DATA q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,-0.3988024d-1,
     *-0.362018d-2,0.163801d-2,-0.1031555d-1,0.2282967d-1,-0.2895312d-1,
     *
     *0.1787654d-1,-0.420059d-2/
      if (dabs(x).lt.3.75d0) then
        y=(x/3.75d0)**2
        bessi1=x*(p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7))))))
      else
        ax=dabs(x)
        y=3.75d0/ax
        bessi1=(dexp(ax)/dsqrt(ax))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *(q7+y*(q8+y*q9))))))))
        if(x.lt.0.d0)bessi1=-bessi1
      endif
      return
      END


C  (C) Copr. 1986-92 Numerical Recipes Software (52:.

C------------------------------------------------------------------

C------------------------------------------------------------------
C
C      FUNCTION rndm(idum)
C      INTEGER idum
C      CALL RANDOM(rndm)
C      return
C      END
C
C------------------------------------------------------------------
       REAL FUNCTION rndm(idum)
       SAVE
       INTEGER idum
       INTEGER MBIG,MSEED,MZ
c      REAL MBIG,MSEED,MZ
       REAL ran3,FAC
       PARAMETER (MBIG=1000000000,MSEED=161803398,MZ=0,
     &            FAC=1.0/MBIG)
c      PARAMETER (MBIG=4000000.,MSEED=1618033.,MZ=0.,FAC=1./MBIG)
       INTEGER i,iff,ii,inext,inextp,k
       INTEGER mj,mk,ma(55)
C      REAL mj,mk,ma(55)
C      SAVE iff,inext,inextp,ma    (kommentiert, KHS, 31.1.1999)
       DATA iff /0/
       if(idum.lt.0.or.iff.eq.0)then
         iff=1
         mj=MSEED-iabs(idum)
         mj=mod(mj,MBIG)
         ma(55)=mj
         mk=1
         do 11 i=1,54
           ii=mod(21*i,55)
           ma(ii)=mk
           mk=mj-mk
           if(mk.lt.MZ)mk=mk+MBIG
           mj=ma(ii)
 11      continue
         do 13 k=1,4
          do 12 i=1,55
            ma(i)=ma(i)-ma(1+mod(i+30,55))
             if(ma(i).lt.MZ)ma(i)=ma(i)+MBIG
 12        continue
 13      continue
         inext=0
         inextp=31
         idum=1
       endif
       inext=inext+1
       if(inext.eq.56)inext=1
       inextp=inextp+1
       if(inextp.eq.56)inextp=1
       mj=ma(inext)-ma(inextp)
       if(mj.lt.MZ)mj=mj+MBIG
       ma(inext)=mj
       ran3=mj*FAC
       rndm = ran3
       return
       END
C
C  (C) Copr. 1986-92 Numerical Recipes Software (52:.




C*********************************************************************
C
C NOW CALC. OF THE FISSION FRAGMENT DISTRIBUTION INCL.
C EVAPORATION FROM FRAGMENTS

      SUBROUTINE FISSION(AF,ZF,EE,JPRF,INUM,
     &   VX1_FISSION,VY1_FISSION,VZ1_FISSION,
     &   VX2_FISSION,VY2_FISSION,VZ2_FISSION,
     &   NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,N6HE_POST,
     &   ZFP1,AFP1,POSTN1,POSTP1,POSTD1,POSTT1,POSTHE1,POSTAL1,POST6HE1,
     &   FF1,FIMF1,TSUM1,TAUC1,TKEIMF1,AIMFF1,ZIMFF1,
     &   ZFP2,AFP2,POSTN2,POSTP2,POSTD2,POSTT2,POSTHE2,POSTAL2,POST6HE2,
     &   FF2,FIMF2,TSUM2,TAUC2,TKEIMF2,AIMFF2,ZIMFF2,imode,
     &   VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC,EV_TEMP,IEV_TAB_FIS,
     &    hev_temp,hist_fis1,hist_fis2)

C
C NN_POST .... NA_POST -> number of particles emitted between saddle and scission
C NTOTN1, NTOTN2 ... NTOTA1,NTOTA2 -> number of particles emitted from two fragments
      REAL*8 AF,ZF,EE,JPRF,EROT,
     &    VX1_FISSION,VY1_FISSION,VZ1_FISSION,
     &    VX2_FISSION,VY2_FISSION,VZ2_FISSION,
     &    NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,N6HE_POST,
     &    ZF1,AF1,NTOTN1,NTOTP1,NTOTD1,NTOTT1,NTOTHE1,NTOTA1,
     &    NTOT6HE1,TSUM1,TAUC1,TKEIMF1,AIMFF1,ZIMFF1,
     &    ZF2,AF2,NTOTN2,NTOTP2,NTOTD2,NTOTT2,NTOTHE2,NTOTA2,
     &    NTOT6HE2,TSUM2,TAUC2,TKEIMF2,AIMFF2,ZIMFF2,JPRFOUT,
     &    VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC

      REAL*8 VPERP1,ALPHA1,AFF1,
     &        ZFF1,EFF1,VFF1,AFF2,ZFF2,EFF2,VFF2,
     &    vx1_eva,vy1_eva,vz1_eva,
     &    vx2_eva,vy2_eva,vz2_eva,
     &    EV_TEMP(200,5),EV_TEMP1(200,5)
      integer hev_temp(200),hist_fis1,hist_fis2
      REAL*4 HAZ
      INTEGER*4 FF1,FIMF1,FF2,FIMF2,INTTYPE,INUM,imode,EVOPT,
     &    ZFP1,AFP1,POSTN1,POSTP1,POSTD1,POSTT1,POSTHE1,POSTAL1,
     &    ZFP2,AFP2,POSTN2,POSTP2,POSTD2,POSTT2,POSTHE2,POSTAL2,
     &    NEVA,PEVA,DEVA,TEVA,HEEVA,AEVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &    IEV_TAB_FIS, IEV_TAB_TEMP,I,K,IJ,POST6HE1,POST6HE2

      REAL*8 EV_TAB_SSC(200,5)
      INTEGER*4 IEV_TAB_SSC
      COMMON/EV_TAB_STS/EV_TAB_SSC,IEV_TAB_SSC

      real*8 vxout,vyout,vzout,vx2out,vy2out,vz2out

      integer step_ev, step_er, step_bu, step_imfl, step_imfh,
     +   step_fisl, step_fish, step_s2s, step_unst
      parameter( step_ev = 1 )
      parameter( step_er = 2 )
      parameter( step_bu = 3 )
      parameter( step_imfl = 4 )
      parameter( step_imfh = 5 )
      parameter( step_fisl = 6 )
      parameter( step_fish = 7 )
      parameter( step_s2s = 8 )
      parameter( step_unst = 9 )

      IEV_TAB_FIS = 0
      DO I = 1,200
       DO K = 1,5
        EV_TEMP(I,K) = 0.D0
       ENDDO
        hev_temp(i) = 0.D0
      ENDDO
      hist_fis1 = 0
      hist_fis2 = 0
      vx1_fission = 0.d0
      vy1_fission = 0.d0
      vz1_fission = 0.d0
      vx2_fission = 0.d0
      vy2_fission = 0.d0
      vz2_fission = 0.d0
      NTOTN1 = 0.D0
      NTOTP1 = 0.D0
      NTOTD1 = 0.D0
      NTOTT1 = 0.D0
      NTOTHE1 = 0.D0
      NTOT6HE1 = 0.D0
      NTOTA1 = 0.D0
      NTOTN2 = 0.D0
      NTOTP2 = 0.D0
      NTOTD2 = 0.D0
      NTOTT2 = 0.D0
      NTOTHE2 = 0.D0
      NTOTA2 = 0.D0
      NTOT6HE2 = 0.D0
      POSTN1 = 0
      POSTP1 = 0
      POSTD1 = 0
      POSTT1 = 0
      POSTHE1 = 0
      POSTAL1 = 0
      POST6HE1 = 0
      POSTN2 = 0
      POSTP2 = 0
      POSTD2 = 0
      POSTT2 = 0
      POSTHE2 = 0
      POSTAL2 = 0
      POST6HE2 = 0

      EROT = JPRF * JPRF * 197.D0 * 197.D0 /
     &     (2.D0 * 0.4D0 * 931.49D0 * AF**(5.D0/3.D0) * 1.16D0**2)

c     IMODE = 1 means asymmetric fission, mode 1
c     IMODE = 2 means asymmetric fission, mode 2
c     IMODE = 3 means symmetric fission

       AF = DNINT(AF)
       ZF = DNINT(ZF)

         CALL FISSION_DISTRI(AF,ZF,EE-EROT,JPRF,INUM,AFF1,
     &       ZFF1,EFF1,VFF1,AFF2,ZFF2,EFF2,VFF2,
     &       NN_POST,NP_POST,ND_POST,NT_POST,NHE_POST,NA_POST,N6HE_POST,
     &       imode,VX_EVA_SC,VY_EVA_SC,VZ_EVA_SC)

              DO IJ = 1, IEV_TAB_SSC, 1
               EV_TEMP(IJ,1) = EV_TAB_SSC(IJ,1)
               EV_TEMP(IJ,2) = EV_TAB_SSC(IJ,2)
               EV_TEMP(IJ,3) = EV_TAB_SSC(IJ,3)
               EV_TEMP(IJ,4) = EV_TAB_SSC(IJ,4)
               EV_TEMP(IJ,5) = EV_TAB_SSC(IJ,5)
               hev_temp(ij) = step_s2s
              ENDDO
               IEV_TAB_FIS = IEV_TAB_FIS + IEV_TAB_SSC
               if(iev_tab_ssc.gt.0) then
                 hist_fis1 = step_s2s
                 hist_fis2 = step_s2s
               end if

C        Velocities
         VZ1_FISSION = (2.D0 * dble(HAZ(1)) - 1.D0) * VFF1
         VPERP1 = DSQRT(VFF1**2 - VZ1_FISSION**2)
         ALPHA1 = dble(HAZ(1)) * 2.D0 * 3.142D0
         VX1_FISSION = VPERP1 * DSIN(ALPHA1)
         VY1_FISSION = VPERP1 * DCOS(ALPHA1)

         VX2_FISSION = - VX1_FISSION / VFF1 * VFF2
         VY2_FISSION = - VY1_FISSION / VFF1 * VFF2
         VZ2_FISSION = - VZ1_FISSION / VFF1 * VFF2

         if(aff1.gt.aff2) then
            call addtohist(step_fish,hist_fis1)
            call addtohist(step_fisl,hist_fis2)
         else
            call addtohist(step_fisl,hist_fis1)
            call addtohist(step_fish,hist_fis2)
         endif

         IF( (ZFF1.LE.0.D0).OR.(AFF1.LE.0.D0).OR.(AFF1.LT.ZFF1)) THEN
           WRITE(6,*) 'FF1 unphysical',ZF,AF,EE,ZFF1,AFF1
         ELSE
C fission and IMF emission are not allowed
         FF1 = 0
         FIMF1 = 0
         CALL EVAPORA(ZFF1,AFF1,EFF1,0.D0,
     &   ZF1,AF1,NTOTN1,NTOTP1,NTOTD1,NTOTT1,NTOTHE1,NTOTA1,
     &   NTOT6HE1,FF1,FIMF1,INTTYPE,INUM,TSUM1,TAUC1,
     &   vx1_eva,vy1_eva,vz1_eva,TKEIMF1,AIMFF1,ZIMFF1,JPRFOUT,1,
     &             EV_TEMP1,IEV_TAB_TEMP)

               DO IJ = 1, IEV_TAB_TEMP, 1
               EV_TEMP(IJ+IEV_TAB_FIS,1) = EV_TEMP1(IJ,1)
               EV_TEMP(IJ+IEV_TAB_FIS,2) = EV_TEMP1(IJ,2)
c Lorentz kinematics - DM 18/5/2010
c               EV_TEMP(IJ+IEV_TAB_FIS,3) = EV_TEMP1(IJ,3) + VX1_FISSION
c               EV_TEMP(IJ+IEV_TAB_FIS,4) = EV_TEMP1(IJ,4) + VY1_FISSION
c               EV_TEMP(IJ+IEV_TAB_FIS,5) = EV_TEMP1(IJ,5) + VZ1_FISSION
c Should also include the v_eva_sc contribution - DM 21/5/2010
               CALL LORENTZ_BOOST(vx1_fission,vy1_fission,vz1_fission,
     &             ev_temp1(ij,3),ev_temp1(ij,4),ev_temp1(ij,5),
     &             VXOUT,VYOUT,VZOUT)
               CALL LORENTZ_BOOST(vx_eva_sc,vy_eva_sc,vz_eva_sc,
     &             vxout,vyout,vzout,
     &             VX2OUT,VY2OUT,VZ2OUT)
               ev_temp(ij+iev_tab_fis,3) = VX2OUT
               ev_temp(ij+iev_tab_fis,4) = VY2OUT
               ev_temp(ij+iev_tab_fis,5) = VZ2OUT
               hev_temp(ij+iev_tab_fis) = hist_fis1
               call addtohist(step_ev,hev_temp(ij+iev_tab_fis))
               ENDDO
               IEV_TAB_FIS = IEV_TAB_FIS + IEV_TAB_TEMP
               if(iev_tab_temp.gt.0) call addtohist(step_er,hist_fis1)
          END IF

         IF( (ZFF2.LE.0.D0).OR.(AFF2.LE.0.D0).OR.(AFF2.LT.ZFF2)) THEN
           WRITE(6,*) 'FF2 unphysical',ZF,AF,EE,ZFF2,AFF2
         ELSE
C fission and IMF emission are not allowed
         FF2 = 0
         FIMF2 = 0
         CALL EVAPORA(ZFF2,AFF2,EFF2,0.D0,
     &   ZF2,AF2,NTOTN2,NTOTP2,NTOTD2,NTOTT2,NTOTHE2,NTOTA2,
     &   NTOT6HE2,FF2,FIMF2,INTTYPE,INUM,TSUM2,TAUC2,
     &   vx2_eva,vy2_eva,vz2_eva,TKEIMF2,AIMFF2,ZIMFF2,JPRFOUT,1,
     &             EV_TEMP1,IEV_TAB_TEMP)

               DO IJ = 1, IEV_TAB_TEMP, 1
               EV_TEMP(IJ+IEV_TAB_FIS,1) = EV_TEMP1(IJ,1)
               EV_TEMP(IJ+IEV_TAB_FIS,2) = EV_TEMP1(IJ,2)
c Lorentz kinematics - DM 18/5/2010
c               EV_TEMP(IJ+IEV_TAB_FIS,3) = EV_TEMP1(IJ,3) + VX2_FISSION
c               EV_TEMP(IJ+IEV_TAB_FIS,4) = EV_TEMP1(IJ,4) + VY2_FISSION
c               EV_TEMP(IJ+IEV_TAB_FIS,5) = EV_TEMP1(IJ,5) + VZ2_FISSION
c Should also include the v_eva_sc contribution - DM 21/5/2010
               CALL LORENTZ_BOOST(vx2_fission,vy2_fission,vz2_fission,
     &             ev_temp1(ij,3),ev_temp1(ij,4),ev_temp1(ij,5),
     &             VXOUT,VYOUT,VZOUT)
               CALL LORENTZ_BOOST(vx_eva_sc,vy_eva_sc,vz_eva_sc,
     &             vxout,vyout,vzout,
     &             VX2OUT,VY2OUT,VZ2OUT)
               ev_temp(ij+iev_tab_fis,3) = VX2OUT
               ev_temp(ij+iev_tab_fis,4) = VY2OUT
               ev_temp(ij+iev_tab_fis,5) = VZ2OUT
               hev_temp(ij+iev_tab_fis) = hist_fis2
               call addtohist(step_ev,hev_temp(ij+iev_tab_fis))
               ENDDO
               IEV_TAB_FIS = IEV_TAB_FIS + IEV_TAB_TEMP
               if(iev_tab_temp.gt.0) call addtohist(step_er,hist_fis2)

         ENDIF

c Lorentz kinematics - DM 18/5/2010
c      vx1_fission = vx1_fission + vx1_eva
c      vy1_fission = vy1_fission + vy1_eva
c      vz1_fission = vz1_fission + vz1_eva
c      vx2_fission = vx2_fission + vx2_eva
c      vy2_fission = vy2_fission + vy2_eva
c      vz2_fission = vz2_fission + vz2_eva
c The v_eva_sc contribution is considered in the calling subroutine
c DM 21/5/2010
         CALL LORENTZ_BOOST(vx1_eva,vy1_eva,vz1_eva,
     &        vx1_fission,vy1_fission,vz1_fission,
     &        VXOUT,VYOUT,VZOUT)
         vx1_fission = vxout
         vy1_fission = vyout
         vz1_fission = vzout
         CALL LORENTZ_BOOST(vx2_eva,vy2_eva,vz2_eva,
     &        vx2_fission,vy2_fission,vz2_fission,
     &        VXOUT,VYOUT,VZOUT)
         vx2_fission = vxout
         vy2_fission = vyout
         vz2_fission = vzout

         AFP1 = IDNINT(AF1)
         AFP2 = IDNINT(AF2)
         ZFP1 = IDNINT(ZF1)
         ZFP2 = IDNINT(ZF2)
C
C BILANCE OF EVAPORATED PARTICLES
C
         POSTN1  = POSTN1 + IDNINT(NTOTN1)
         POSTP1  = POSTP1 + IDNINT(NTOTP1)
         POSTD1  = POSTD1 + IDNINT(NTOTD1)
         POSTT1  = POSTT1 + IDNINT(NTOTT1)
         POSTHE1 = POSTHE1 + IDNINT(NTOTHE1)
         POSTAL1 = POSTAL1 + IDNINT(NTOTA1)
         POST6HE1 = POST6HE1 + IDNINT(NTOT6HE1)

         POSTN2  = POSTN2 + IDNINT(NTOTN2)
         POSTP2  = POSTP2 + IDNINT(NTOTP2)
         POSTD2  = POSTD2 + IDNINT(NTOTD2)
         POSTT2  = POSTT2 + IDNINT(NTOTT2)
         POSTHE2 = POSTHE2 + IDNINT(NTOTHE2)
         POSTAL2 = POSTAL2 + IDNINT(NTOTA2)
         POST6HE2 = POST6HE2 + IDNINT(NTOT6HE2)

      RETURN
      END
C***********************************************************************
      SUBROUTINE TKE_BU(Z,A,ZALL,AAL,VX,VY,VZ)

       REAL*8  Z,A,ZALL,AAL,E1,E2, RALL
       REAL*8  RMAX,TKE_MAX,TKE,R,R0,V_over_V0
       REAL*4 HAZ
       REAL*8 EKIN,V,VPERP,ALPHA1,VX,VY,VZ,RHAZ

       V_over_V0 = 6.D0
       R0 = 1.16D0

       IF(Z .LT. 1.D0) THEN
        VX = 0.D0
        VY = 0.D0
        VZ = 0.D0
        RETURN
       ENDIF

      GOTO 10
       RMAX = R0 * V_over_V0**(1.D0/3.D0) * (AAL - A)**(1.D0/3.D0)

       IF(RMAX.LT.0.01D0) THEN
        TKE_MAX = 0.D0
        RMAX = 0.01D0
       ELSE
        TKE_MAX = 1.44D0 * Z * (ZALL - Z) / RMAX
       ENDIF

       RHAZ = DBLE(HAZ(1))
       R = (RHAZ)**(1.D0/3.D0) * RMAX

       TKE = TKE_MAX * (R/RMAX)**2.5D0
 10    CONTINUE

        RALL = R0 * V_over_V0**(1.D0/3.D0) * AAL**(1.D0/3.D0)
        RHAZ = DBLE(HAZ(1))
        R = (RHAZ)**(1.D0/3.D0) * RALL
        TKE = 1.44D0 * Z * ZALL * R**2.D0 * (1.D0 - A/AAL)**2.D0
     &        / RALL**3.D0

        Ekin = TKE * (AAL - A) / AAL
c       print*,'!!!',IDNINT(AAl),IDNINT(A),IDNINT(ZALL),IDNINT(Z)
          V = DSQRT(Ekin/A) * 1.3887D0
          VZ = (2.D0 * dble(HAZ(1)) - 1.D0) * V
       VPERP = DSQRT(V**2 - VZ**2)
       ALPHA1 = dble(HAZ(1)) * 2.D0 * 3.142D0
       VX = VPERP * DSIN(ALPHA1)
       VY = VPERP * DCOS(ALPHA1)

c       if(z.eq.6.d0.or.z.eq.7.d0) write(1100,*)ekin

      RETURN
      END
C
C***********************************************************************
C
      SUBROUTINE UNSTABLE_NUCLEI(AFP,ZFP,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &        VX,VY,VZ,VP1X,VP1Y,VP1Z,BU_TAB_TEMP,ILOOP)

      INTEGER*4 AFP,ZFP,AFPNEW,ZFPNEW,NEVA,PEVA,DEVA,TEVA,HEEVA,AEVA,
     &          IOUNSTABLE,INMIN,INMAX,NDIF,I,ILOOP,IMEM,HE6EVA
      REAL*8   VX,VY,VZ,VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z
      REAL*8   BU_TAB_TEMP(200,5)

      NEVA  = 0
      PEVA  = 0
      DEVA  = 0
      TEVA  = 0
      HEEVA = 0
      AEVA  = 0
      HE6EVA = 0
      AFPNEW = AFP
      ZFPNEW = ZFP
      IOUNSTABLE = 0
      ILOOP = 0
      IMEM = 0
      DO I=1,200
      BU_TAB_TEMP(I,1) = 0.D0
      BU_TAB_TEMP(I,2) = 0.D0
      BU_TAB_TEMP(I,3) = 0.D0
      BU_TAB_TEMP(I,4) = 0.D0
      BU_TAB_TEMP(I,5) = 0.D0
      ENDDO
      VP1X = 0.D0
      VP1Y = 0.D0
      VP1Z = 0.D0

      IF(AFP.EQ.0 .AND. ZFP.EQ.0) THEN
c       PRINT*,'UNSTABLE NUCLEI, AFP=0, ZFP=0'
       RETURN
      ENDIF
      IF(AFP.EQ.1 .AND. ZFP.EQ.0) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.1 .AND. ZFP.EQ.1) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.2 .AND. ZFP.EQ.1) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.3 .AND. ZFP.EQ.1) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.3 .AND. ZFP.EQ.2) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.4 .AND. ZFP.EQ.2) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.6 .AND. ZFP.EQ.2) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.8 .AND. ZFP.EQ.2) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF


         IF ((AFP-ZFP.EQ.0).AND.ZFP.GT.1) THEN
         DO I = 0, AFP-2
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),DFLOAT(AFP-I),
     &        DFLOAT(AFP-I-1),DFLOAT(AFP-I-1),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO
           PEVA = PEVA + ZFP - 1
           AFP = 1
           ZFP = 1
           IOUNSTABLE = 1
C          RETURN
         END IF

*** Find the limits nucleus is bound :
      CALL ISOSTAB_LIM(ZFP,INMIN,INMAX)
      NDIF = AFP - ZFP
      IF(NDIF.LT.INMIN)THEN
C Proton unbound
        IOUNSTABLE = 1
       DO I = 1, 10
        CALL ISOSTAB_LIM(ZFP-I,INMIN,INMAX)
         IF(INMIN.LE.NDIF)THEN
         IMEM = I
         ZFP = ZFP - I
         AFP = ZFP + NDIF
         PEVA = I
          GOTO 10
         ENDIF
       ENDDO
 10    CONTINUE
       DO I = 0, IMEM-1
         CALL UNSTABLE_TKE(DFLOAT(NDIF+ZFP+IMEM-I),
     &        DFLOAT(ZFP+IMEM-I),
     &        DFLOAT(NDIF+ZFP+IMEM-I-1),
     &        DFLOAT(ZFP+IMEM-I-1),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(I+1+ILOOP,1) = 1.D0
         BU_TAB_TEMP(I+1+ILOOP,2) = 1.D0
         BU_TAB_TEMP(I+1+ILOOP,3) = VP2X
         BU_TAB_TEMP(I+1+ILOOP,4) = VP2Y
         BU_TAB_TEMP(I+1+ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
       ENDDO
         ILOOP = ILOOP + IMEM

      ENDIF
      IF(NDIF.GT.INMAX)THEN
C Neutron unbound
        NEVA = NDIF - INMAX
        AFP = ZFP + INMAX
        IOUNSTABLE = 1
         DO I = 0, NEVA-1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(ZFP+NDIF-I),
     &        DFLOAT(ZFP),
     &        DFLOAT(ZFP+NDIF-I-1),
     &        DFLOAT(ZFP),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO
      ENDIF

         IF ((AFP.GE.2).AND.(ZFP.EQ.0)) THEN
         DO I = 0, AFP-2
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),dfloat(zfp),
     &        DFLOAT(AFP-I-1),dfloat(zfp),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO

          NEVA = NEVA + (AFP - 1)
          AFP = 1
          ZFP = 0
          IOUNSTABLE = 1
C          RETURN
         END IF
         IF (AFP.LT.ZFP) THEN
          PRINT*,'WARNING - BU UNSTABLE: AF < ZF'
           AFP = 0
           ZFP = 0
           IOUNSTABLE = 1
C          RETURN
         END IF

         IF ((AFP.GE.4).AND.(ZFP.EQ.1)) THEN
C Heavy residue is treated as 3H and the rest of mass is emitted as neutrons:
         DO I = 0, AFP-4
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),DFLOAT(ZFP),
     &        DFLOAT(AFP-I-1),DFLOAT(ZFP),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO

         NEVA = NEVA + (AFP - 3)
         AFP = 3
         ZFP = 1
         IOUNSTABLE = 1
C          RETURN
         ENDIF



         IF ((AFP.EQ.4).AND.(ZFP.EQ.3)) THEN
C 4Li -> 3He + p  ->
          AFP = 3
          ZFP = 2
c          HEEVA = HEEVA + 1
          PEVA = PEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(4.D0,3.d0,3.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.5).AND.(ZFP.EQ.2)) THEN
C 5He -> 4He + n  ->
          AFP = 4
          ZFP = 2
          NEVA = NEVA + 1
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(5.D0,2.d0,4.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.5).AND.(ZFP.EQ.3)) THEN
C 5Li -> 4He + p
          AFP = 4
          ZFP = 2
c          AEVA = AEVA + 1
          PEVA = PEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(5.D0,3.d0,4.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
         ENDIF
         IF ((AFP.EQ.6).AND.(ZFP.EQ.4)) THEN
C 6Be -> 4He + 2p (velocity in two steps: 6Be->5Li->4He)
          AFP = 4
          ZFP = 2
c          AEVA = AEVA + 1
          PEVA = PEVA + 2
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
C 6Be -> 5Li + p
         CALL UNSTABLE_TKE(6.D0,4.d0,5.D0,3.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z

         ILOOP = ILOOP + 1
C 5Li -> 4He + p
         CALL UNSTABLE_TKE(5.D0,3.d0,4.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
C          RETURN
         ENDIF
         IF ((AFP.EQ.7).AND.(ZFP.EQ.2)) THEN
c 7He -> 6He + n
           AFP = 6
           ZFP = 2
           NEVA = NEVA + 1
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(7.D0,2.d0,6.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.7).AND.(ZFP.EQ.5)) THEN
C 7B -> 6Be + p -> 4He + 3p
         DO I = 0, AFP-5
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),DFLOAT(ZFP-I),
     &        DFLOAT(AFP-I-1),DFLOAT(ZFP-I-1),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO

          AFP = 4
          ZFP = 2
c          AEVA = AEVA + 1
          PEVA = PEVA + 3
         IOUNSTABLE = 1
C          RETURN
         ENDIF
         IF ((AFP.EQ.8).AND.(ZFP.EQ.4)) THEN
C 8Be  -> 4He + 4He
          AFP = 4
          ZFP = 2
c          AEVA = AEVA + 2
          AEVA = AEVA + 1
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(8.D0,4.d0,4.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 2.D0
         BU_TAB_TEMP(ILOOP,2) = 4.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.8).AND.(ZFP.EQ.6)) THEN
C 8C  -> 2p + 6Be
          AFP = 6
          ZFP = 4
          PEVA = PEVA + 2
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(8.D0,6.d0,7.D0,5.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(7.D0,5.d0,6.D0,4.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.9).AND.(ZFP.EQ.2)) THEN
C 9He -> 8He + n
           AFP = 8
           ZFP = 2
           NEVA = NEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(9.D0,2.d0,8.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.9).AND.(ZFP.EQ.5)) THEN
C 9B -> 4He + 4He + p  ->
          AFP = 4
          ZFP = 2
          PEVA = PEVA + 1
          AEVA = AEVA + 1
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(9.D0,5.d0,8.D0,4.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(8.D0,4.d0,4.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 2.D0
         BU_TAB_TEMP(ILOOP,2) = 4.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.10).AND.(ZFP.EQ.2)) THEN
C 10He -> 8He + 2n
           AFP = 8
           ZFP = 2
           NEVA = NEVA + 2
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
C 10He -> 9He + n
         CALL UNSTABLE_TKE(10.D0,2.d0,9.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z

         ILOOP = ILOOP + 1
C 9He -> 8He + n
         CALL UNSTABLE_TKE(9.D0,2.d0,8.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.10).AND.(ZFP.EQ.3)) THEN
C 10Li -> 9Li + n  ->
          AFP = 9
          ZFP = 3
          NEVA = NEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(10.D0,3.d0,9.D0,3.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
C          RETURN
         ENDIF
         IF ((AFP.EQ.10).AND.(ZFP.EQ.7)) THEN
C 10N -> 9C + p  ->
          AFP = 9
          ZFP = 6
          PEVA = PEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(10.D0,7.d0,9.D0,6.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
C          RETURN
         ENDIF
         IF ((AFP.EQ.11).AND.(ZFP.EQ.7)) THEN
C 11N -> 10C + p  ->
          AFP = 10
          ZFP = 6
          PEVA = PEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(11.D0,7.d0,10.D0,6.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
C          RETURN
         ENDIF
         IF ((AFP.EQ.12).AND.(ZFP.EQ.8)) THEN
C 12O -> 10C + 2p  ->
          AFP = 10
          ZFP = 6
          PEVA = PEVA + 2
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(12.D0,8.d0,11.D0,7.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(11.D0,7.d0,10.D0,6.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.15).AND.(ZFP.EQ.9)) THEN
C 15F -> 14O + p  ->
          AFP = 14
          ZFP = 8
          PEVA = PEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(15.D0,9.d0,14.D0,8.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.16).AND.(ZFP.EQ.9)) THEN
C 16F -> 15O + p  ->
          AFP = 15
          ZFP = 8
          PEVA = PEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(16.D0,9.d0,15.D0,8.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
C          RETURN
         ENDIF
         IF ((AFP.EQ.16).AND.(ZFP.EQ.10)) THEN
C 16Ne -> 14O + 2p  ->
          AFP = 14
          ZFP = 8
          PEVA = PEVA + 2
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(16.D0,10.d0,15.D0,9.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(15.D0,9.d0,14.D0,8.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.18).AND.(ZFP.EQ.11)) THEN
C 18Na -> 17Ne + p  ->
          AFP = 17
          ZFP = 10
          PEVA = PEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(18.D0,11.d0,17.D0,10.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
C          RETURN
         ENDIF
         IF ((AFP.EQ.19).AND.(ZFP.EQ.11)) THEN
C 19Na -> 18Ne + p  ->
          AFP = 18
          ZFP = 10
          PEVA = PEVA + 1
         IOUNSTABLE = 1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(19.D0,11.d0,18.D0,10.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
C          RETURN
         ENDIF

         IF ((AFP.GE.4).AND.(AFP-ZFP.EQ.1)) THEN
c Heavy residue is treated as 3He

        print*,'EVAP: AFP>4,AFP-ZFP=1',AFP,ZFP

           NEVA = AFP - 3
           PEVA = ZFP - 2

         DO I = 0, NEVA-1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),DFLOAT(ZFP),
     &        DFLOAT(AFP-I-1),DFLOAT(ZFP),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO
         DO I = 0, PEVA-1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-NEVA-I),DFLOAT(ZFP-I),
     &        DFLOAT(AFP-NEVA-I-1),DFLOAT(ZFP-I-1),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO

           AFP = 3
           ZFP = 2
c           HEEVA = HEEVA + 1
         IOUNSTABLE = 1
C          RETURN
         END IF


      AFPNEW = AFP
      ZFPNEW = ZFP


      RETURN
      END
C
C************************************************************************
C
      SUBROUTINE UNSTABLE_NUCLEI_BU(AFP,ZFP,NEVA,PEVA,DEVA,TEVA,HEEVA,
     &                     AEVA,HE6EVA,AFPNEW,ZFPNEW,IOUNSTABLE,
     &        VX,VY,VZ,VP1X,VP1Y,VP1Z,BU_TAB_TEMP,ILOOP)

      REAL*8   VX,VY,VZ,VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z
      REAL*8   BU_TAB_TEMP(200,5)
      INTEGER*4 AFP,ZFP,AFPNEW,ZFPNEW,NEVA,PEVA,DEVA,TEVA,HEEVA,AEVA,
     &          IOUNSTABLE,INMIN,INMAX,NDIF,I,ILOOP,ZFPIN,ZTEST,IMEM,
     &          HE6EVA

      NEVA  = 0
      PEVA  = 0
      DEVA  = 0
      TEVA  = 0
      HEEVA = 0
      AEVA  = 0
      HE6EVA = 0
      AFPNEW = AFP
      ZFPNEW = ZFP
      IOUNSTABLE = 0
      ILOOP = 0
      IMEM = 0
      DO I=1,200
      BU_TAB_TEMP(I,1) = 0.D0
      BU_TAB_TEMP(I,2) = 0.D0
      BU_TAB_TEMP(I,3) = 0.D0
      BU_TAB_TEMP(I,4) = 0.D0
      BU_TAB_TEMP(I,5) = 0.D0
      ENDDO
      VP1X = 0.D0
      VP1Y = 0.D0
      VP1Z = 0.D0

      ZFPIN = ZFP
      ZTEST = 0

      IF(AFP.EQ.0 .AND. ZFP.EQ.0) THEN
c       PRINT*,'UNSTABLE NUCLEI BU, AFP=0, ZFP=0'
       RETURN
      ENDIF
      IF(AFP.EQ.1 .AND. ZFP.EQ.0) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.1 .AND. ZFP.EQ.1) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.2 .AND. ZFP.EQ.1) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.3 .AND. ZFP.EQ.1) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.3 .AND. ZFP.EQ.2) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.4 .AND. ZFP.EQ.2) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.6 .AND. ZFP.EQ.2) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF
      IF(AFP.EQ.8 .AND. ZFP.EQ.2) THEN
       VP1X = VX
       VP1Y = VY
       VP1Z = VZ
       RETURN
      ENDIF

         IF ((AFP-ZFP.EQ.0).AND.ZFP.GT.1) THEN
         DO I = 0, AFP-2
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),DFLOAT(ZFP-I),
     &        DFLOAT(AFP-I-1),DFLOAT(ZFP-I-1),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO
           PEVA = PEVA + ZFP - 1
           AFP = 1
           ZFP = 1
           IOUNSTABLE = 1
C          RETURN
         END IF

*** Find the limits nucleus is bound :
      CALL ISOSTAB_LIM(ZFP,INMIN,INMAX)
      NDIF = AFP - ZFP
      IF(NDIF.LT.INMIN)THEN
C Proton unbound
        IOUNSTABLE = 1
       DO I = 1, 10
        CALL ISOSTAB_LIM(ZFP-I,INMIN,INMAX)
         IF(INMIN.LE.NDIF)THEN
         IMEM = I
         ZFP = ZFP - I
         AFP = ZFP + NDIF
         PEVA = I
          GOTO 10
         ENDIF
       ENDDO
 10    CONTINUE
       DO I = 0, IMEM-1
         CALL UNSTABLE_TKE(DFLOAT(NDIF+ZFP+IMEM-I),
     &        DFLOAT(ZFP+IMEM-I),
     &        DFLOAT(NDIF+ZFP+IMEM-I-1),
     &        DFLOAT(ZFP+IMEM-I-1),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(I+1+ILOOP,1) = 1.D0
         BU_TAB_TEMP(I+1+ILOOP,2) = 1.D0
         BU_TAB_TEMP(I+1+ILOOP,3) = VP2X
         BU_TAB_TEMP(I+1+ILOOP,4) = VP2Y
         BU_TAB_TEMP(I+1+ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
       ENDDO
         ILOOP = ILOOP + IMEM

      ENDIF
      IF(NDIF.GT.INMAX)THEN
C Neutron unbound
        NEVA = NDIF - INMAX
        AFP = ZFP + INMAX
        IOUNSTABLE = 1
         DO I = 0, NEVA-1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(ZFP+NDIF-I),DFLOAT(ZFP),
     &        DFLOAT(ZFP+NDIF-I-1),DFLOAT(ZFP),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO
      ENDIF

         IF ((AFP.GE.2).AND.(ZFP.EQ.0)) THEN
         DO I = 0, AFP-2
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),dfloat(zfp),
     &        DFLOAT(AFP-I-1),dfloat(zfp),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO

          NEVA = NEVA + (AFP - 1)
          AFP = 1
          ZFP = 0
          IOUNSTABLE = 1
C          RETURN
         END IF
         IF (AFP.LT.ZFP) THEN
          PRINT*,'WARNING - BU UNSTABLE: AF < ZF'
           AFP = 0
           ZFP = 0
           IOUNSTABLE = 1
C          RETURN
         END IF
         IF ((AFP.GE.4).AND.(ZFP.EQ.1)) THEN
C Heavy residue is treated as 3H and the rest of mass is emitted as neutrons:
         DO I = 0, AFP-4
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),DFLOAT(ZFP),
     &        DFLOAT(AFP-I-1),DFLOAT(ZFP),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO

         NEVA = NEVA + (AFP - 3)
         AFP = 3
         ZFP = 1
         IOUNSTABLE = 1
C          RETURN
         ENDIF


         IF ((AFP.EQ.5).AND.(ZFP.EQ.2)) THEN
C 5He -> 4He + n  ->
          AFP = 4
          ZFP = 2
          NEVA = NEVA + 1
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(5.D0,2.d0,4.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.7).AND.(ZFP.EQ.2)) THEN
c 7He -> 6He + n
           AFP = 6
           ZFP = 2
           NEVA = NEVA + 1
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(7.D0,2.d0,6.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.9).AND.(ZFP.EQ.2)) THEN
C 9He -> 8He + n
           AFP = 8
           ZFP = 2
           NEVA = NEVA + 1
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(9.D0,2.d0,8.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF
         IF ((AFP.EQ.10).AND.(ZFP.EQ.2)) THEN
C 10He -> 8He + 2n
           AFP = 8
           ZFP = 2
           NEVA = NEVA + 2
         IOUNSTABLE = 1

         ILOOP = ILOOP + 1
C 10He -> 9He + n
         CALL UNSTABLE_TKE(10.D0,2.d0,9.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z
         VX = VP1X
         VY = VP1Y
         VZ = VP1Z

         ILOOP = ILOOP + 1
C 9He -> 8He + n
         CALL UNSTABLE_TKE(9.D0,2.d0,8.D0,2.d0,
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)
         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

C          RETURN
         ENDIF

         IF ((AFP.GE.4).AND.(AFP-ZFP.EQ.1)) THEN
c Heavy residue is treated as 3He

        print*,'BU: AFP>4,AFP-ZFP=1',AFP,ZFP

           NEVA = AFP - 3
           PEVA = ZFP - 2

         DO I = 0, NEVA-1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-I),DFLOAT(ZFP),
     &        DFLOAT(AFP-I-1),DFLOAT(ZFP),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 0.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO
         DO I = 0, PEVA-1
         ILOOP = ILOOP + 1
         CALL UNSTABLE_TKE(DFLOAT(AFP-NEVA-I),DFLOAT(ZFP-I),
     &        DFLOAT(AFP-NEVA-I-1),DFLOAT(ZFP-I-1),
     &        VX,VY,VZ,
     &        VP1X,VP1Y,VP1Z,VP2X,VP2Y,VP2Z)

         BU_TAB_TEMP(ILOOP,1) = 1.D0
         BU_TAB_TEMP(ILOOP,2) = 1.D0
         BU_TAB_TEMP(ILOOP,3) = VP2X
         BU_TAB_TEMP(ILOOP,4) = VP2Y
         BU_TAB_TEMP(ILOOP,5) = VP2Z

         VX = VP1X
         VY = VP1Y
         VZ = VP1Z
         ENDDO

           AFP = 3
           ZFP = 2
         IOUNSTABLE = 1
C          RETURN
         END IF

      AFPNEW = AFP
      ZFPNEW = ZFP


      RETURN
      END
C
C************************************************************************
C
      SUBROUTINE UNSTABLE_TKE(AIN,ZIN,ANEW,ZNEW,VXIN,VYIN,VZIN,
     &           V1X,V1Y,V1Z,V2X,V2Y,V2Z)

      REAL*8 AIN,ANEW,VXIN,VYIN,VZIN,V1X,V1Y,V1Z,V2X,V2Y,V2Z,VBU2
      REAL*8 GAMMA_REL,C,ETOT,EKIN_P1,EKIN_P2,AMU,EKIN_TOT
      REAL*8 PXIN,PYIN,PZIN,PX1,PX2,PY1,PY2,PZ1,PZ2,PTOT
      REAL*8 RNDT,CTET1,STET1,RNDP,PHI1,ETOT_P1,ETOT_P2
      REAL*4 HAZ

      real*8 zin,znew,mass,mass1,mass2,MEXP(0:153,0:12)
      real*8 qval,b_in,b_new,b_ej
      real*8 vxout,vyout,vzout
      integer iain,izin,ianew,iznew,inin,innew
      COMMON/MEXP/MEXP

      C = 29.9792458D0        ! cm/ns
      AMU = 931.494D0         ! MeV/C^2

      VBU2 = VXIN * VXIN + VYIN * VYIN + VZIN * VZIN
      GAMMA_REL = 1.D0 / DSQRT(1.000D0 - VBU2 / (C*C))
      ETOT = AIN * AMU * GAMMA_REL
      PXIN = ETOT * VXIN / C
      PYIN = ETOT * VYIN / C
      PZIN = ETOT * VZIN / C

      iain = nint(ain)
      izin = nint(zin)
      inin = iain - izin
      ianew = nint(anew)
      iznew = nint(znew)
      innew = ianew - iznew

c      ekin_tot = eflmac(ianew,iznew,0,1) +
c     &     eflmac(iain-ianew,izin-iznew,0,1) -
c     &     eflmac(iain,izin,0,1)

      IF(IZIN.GT.12)THEN
      CALL MGLMS(AIN,ZIN,3,MASS)
      CALL MGLMS(ANEW,ZNEW,3,MASS1)
      CALL MGLMS(AIN-ANEW,ZIN-ZNEW,3,MASS2)
      ekin_tot = MASS-MASS1-MASS2
      ELSE
      ekin_tot = MEXP(ININ,IZIN) -
     &  (MEXP(INNEW,IZNEW)+MEXP(ININ-INNEW,IZIN-IZNEW))

      IF(IZIN.GT.12)write(*,*)'*** ZIN > 12 ***', IZIN

      ENDIF


      if( ekin_tot.lt.0.0d0 ) then
c         if( iain.ne.izin .and. izin.ne.0 ) then
c            print *,"Negative Q-value in UNSTABLE_TKE"
c            print *,"ekin_tot=",ekin_tot
c            print *,"ain,zin=",ain,zin,MEXP(ININ,IZIN)
c            print *,"anew,znew=",anew,znew,MEXP(INNEW,IZNEW)
c            print *
c         end if
        ekin_tot=0.0d0
      end if

      EKIN_P1 = EKIN_TOT*(AIN-ANEW)/ AIN
      ETOT_P1 = EKIN_P1 + ANEW * AMU
      PTOT = ANEW*AMU*DSQRT((EKIN_P1/(ANEW*AMU)+1.D0)**2-1.D0)  ! MeV/C
      RNDT = DBLE(HAZ(1))
      CTET1 = 2.D0*RNDT-1.D0
      STET1 = DSQRT(1.D0-CTET1**2)
      RNDP = DBLE(HAZ(1))
      PHI1 = RNDP*2.D0*3.141592654D0
      PX1 = PTOT * STET1*DCOS(PHI1)
      PY1 = PTOT * STET1*DSIN(PHI1)
      PZ1 = PTOT * CTET1
      V1X = C * PX1 / ETOT_P1
      V1Y = C * PY1 / ETOT_P1
      V1Z = C * PZ1 / ETOT_P1
      call lorentz_boost(vxin,vyin,vzin,v1x,v1y,v1z,
     &    vxout,vyout,vzout)
      v1x = vxout
      v1y = vyout
      v1z = vzout

      PX2 = - PX1
      PY2 = - PY1
      PZ2 = - PZ1
      ETOT_P2 = (EKIN_TOT - EKIN_P1) + (AIN-ANEW) * AMU
      V2X = C * PX2 / ETOT_P2
      V2Y = C * PY2 / ETOT_P2
      V2Z = C * PZ2 / ETOT_P2
      call lorentz_boost(vxin,vyin,vzin,v2x,v2y,v2z,
     &    vxout,vyout,vzout)
      v2x = vxout
      v2y = vyout
      v2z = vzout

      RETURN
      END
C
C
C**************************************************************************
C
      SUBROUTINE LORENTZ_BOOST(VXRIN,VYRIN,VZRIN,VXIN,VYIN,VZIN,
     &     VXOUT,VYOUT,VZOUT)

C Calculate velocities of a given fragment from frame 1 into frame 2.
C Frame 1 is moving with velocity v=(vxr,vyr,vzr) relative to frame 2.
C Velocity of the fragment in frame 1 -> vxin,vyin,vzin
C Velocity of the fragment in frame 2 -> vxout,vyout,vzout


      REAL*8 VXRIN,VYRIN,VZRIN,VXIN,VYIN,VZIN,VXOUT,VYOUT,VZOUT
      REAL*8 VXR,VYR,VZR
      REAL*8 GAMMA,VR,C,CC,DENO,VXNOM,VYNOM,VZNOM

      C = 29.9792458D0        ! cm/ns
      CC = C*C

C VXR,VYR,VZR are velocities of frame 1 relative to frame 2; to go from 1 to 2
C we need to multiply them by -1
      VXR = -1.D0 * VXRIN
      VYR = -1.D0 * VYRIN
      VZR = -1.D0 * VZRIN

      VR = DSQRT(VXR*VXR + VYR*VYR + VZR*VZR)
      if(vr.lt.1D-9) then
         vxout = vxin
         vyout = vyin
         vzout = vzin
         return
      end if
      GAMMA = 1.D0/DSQRT(1.D0 - VR*VR/CC)
      DENO = 1.D0 - VXR*VXIN/CC - VYR*VYIN/CC - VZR*VZIN/CC

C X component
      VXNOM = -GAMMA*VXR +
     &         (1.D0+(GAMMA-1.D0)*VXR*VXR/(VR*VR))*VXIN +
     &         (GAMMA-1.D0)*VXR*VYR/(VR*VR)*VYIN +
     &         (GAMMA-1.D0)*VXR*VZR/(VR*VR)*VZIN

      VXOUT = VXNOM / (GAMMA * DENO)

C Y component
      VYNOM = -GAMMA*VYR +
     &         (1.D0+(GAMMA-1.D0)*VYR*VYR/(VR*VR))*VYIN +
     &         (GAMMA-1.D0)*VXR*VYR/(VR*VR)*VXIN +
     &         (GAMMA-1.D0)*VYR*VZR/(VR*VR)*VZIN

      VYOUT = VYNOM / (GAMMA * DENO)

C Z component
      VZNOM = -GAMMA*VZR +
     &         (1.D0+(GAMMA-1.D0)*VZR*VZR/(VR*VR))*VZIN +
     &         (GAMMA-1.D0)*VXR*VZR/(VR*VR)*VXIN +
     &         (GAMMA-1.D0)*VYR*VZR/(VR*VR)*VYIN

      VZOUT = VZNOM / (GAMMA * DENO)



      RETURN
      END
C**************************************************************************


C-----------------------------------------------------------------------
C  E N D   O F    F I L E
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------

